# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/testhelperservice/test_helper_service__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.auth import service__client_pb2 as tecton__proto_dot_auth_dot_service____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n@tecton_proto/testhelperservice/test_helper_service__client.proto\x12\x1etecton_proto.testhelperservice\x1a\'tecton_proto/auth/service__client.proto\"K\n\x1cIngestionServerConfigRequest\x12+\n\x11\x61\x62solute_filepath\x18\x01 \x01(\tR\x10\x61\x62soluteFilepath\"\x1f\n\x1dIngestionServerConfigResponse\"b\n\x16InitializeStoreRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12*\n\x11\x66\x65\x61ture_view_name\x18\x02 \x01(\tR\x0f\x66\x65\x61tureViewName\"\x19\n\x17InitializeStoreResponse2\xf5\x02\n\x11TestHelperService\x12\xb7\x01\n\x15IngestionServerConfig\x12<.tecton_proto.testhelperservice.IngestionServerConfigRequest\x1a=.tecton_proto.testhelperservice.IngestionServerConfigResponse\"!\xa2\xbc\xe6\xc0\x05\x1b*\x19tecton/organization#admin\x12\xa5\x01\n\x0fInitializeStore\x12\x36.tecton_proto.testhelperservice.InitializeStoreRequest\x1a\x37.tecton_proto.testhelperservice.InitializeStoreResponse\"!\xa2\xbc\xe6\xc0\x05\x1b*\x19tecton/organization#adminBm\n\x1c\x63om.tecton.testhelperserviceB\x16TestHelperServiceProtoP\x01Z3github.com/tecton-ai/tecton_proto/testhelperservice')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.testhelperservice.test_helper_service__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\034com.tecton.testhelperserviceB\026TestHelperServiceProtoP\001Z3github.com/tecton-ai/tecton_proto/testhelperservice'
  _TESTHELPERSERVICE.methods_by_name['IngestionServerConfig']._options = None
  _TESTHELPERSERVICE.methods_by_name['IngestionServerConfig']._serialized_options = b'\242\274\346\300\005\033*\031tecton/organization#admin'
  _TESTHELPERSERVICE.methods_by_name['InitializeStore']._options = None
  _TESTHELPERSERVICE.methods_by_name['InitializeStore']._serialized_options = b'\242\274\346\300\005\033*\031tecton/organization#admin'
  _INGESTIONSERVERCONFIGREQUEST._serialized_start=141
  _INGESTIONSERVERCONFIGREQUEST._serialized_end=216
  _INGESTIONSERVERCONFIGRESPONSE._serialized_start=218
  _INGESTIONSERVERCONFIGRESPONSE._serialized_end=249
  _INITIALIZESTOREREQUEST._serialized_start=251
  _INITIALIZESTOREREQUEST._serialized_end=349
  _INITIALIZESTORERESPONSE._serialized_start=351
  _INITIALIZESTORERESPONSE._serialized_end=376
  _TESTHELPERSERVICE._serialized_start=379
  _TESTHELPERSERVICE._serialized_end=752
# @@protoc_insertion_point(module_scope)
