# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/principal_group__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.auditlog import metadata__client_pb2 as tecton__proto_dot_auditlog_dot_metadata____client__pb2
from tecton_proto.auth import principal__client_pb2 as tecton__proto_dot_auth_dot_principal____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n/tecton_proto/data/principal_group__client.proto\x12\x11tecton_proto.data\x1a\x1fgoogle/protobuf/timestamp.proto\x1a,tecton_proto/auditlog/metadata__client.proto\x1a)tecton_proto/auth/principal__client.proto\"\x86\x03\n\x0ePrincipalGroup\x12\x17\n\x02id\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x02id\x12\x1b\n\x04name\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x04name\x12 \n\x0b\x64\x65scription\x18\x03 \x01(\tR\x0b\x64\x65scription\x12\x33\n\x11idp_mapping_names\x18\x04 \x03(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0fidpMappingNames\x12\x34\n\x16is_membership_editable\x18\x05 \x01(\x08R\x14isMembershipEditable\x12\x39\n\ncreated_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12;\n\ncreated_by\x18\x07 \x01(\x0b\x32\x1c.tecton_proto.auth.PrincipalR\tcreatedBy\x12\x39\n\nupdated_at\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAtB\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.principal_group__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _PRINCIPALGROUP.fields_by_name['id']._options = None
  _PRINCIPALGROUP.fields_by_name['id']._serialized_options = b'\352\231\275F\002\010\002'
  _PRINCIPALGROUP.fields_by_name['name']._options = None
  _PRINCIPALGROUP.fields_by_name['name']._serialized_options = b'\352\231\275F\002\010\002'
  _PRINCIPALGROUP.fields_by_name['idp_mapping_names']._options = None
  _PRINCIPALGROUP.fields_by_name['idp_mapping_names']._serialized_options = b'\352\231\275F\002\010\002'
  _PRINCIPALGROUP._serialized_start=193
  _PRINCIPALGROUP._serialized_end=583
# @@protoc_insertion_point(module_scope)
