# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/user_defined_function__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.args import diff_options__client_pb2 as tecton__proto_dot_args_dot_diff__options____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5tecton_proto/args/user_defined_function__client.proto\x12\x11tecton_proto.args\x1a,tecton_proto/args/diff_options__client.proto\"J\n\x13UserDefinedFunction\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x19\n\x04\x62ody\x18\x03 \x01(\tB\x05\x92M\x02\x18\x02R\x04\x62odyJ\x04\x08\x02\x10\x03\x42;\n\x0f\x63om.tecton.argsP\x01Z&github.com/tecton-ai/tecton_proto/args')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.user_defined_function__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001Z&github.com/tecton-ai/tecton_proto/args'
  _USERDEFINEDFUNCTION.fields_by_name['body']._options = None
  _USERDEFINEDFUNCTION.fields_by_name['body']._serialized_options = b'\222M\002\030\002'
  _USERDEFINEDFUNCTION._serialized_start=122
  _USERDEFINEDFUNCTION._serialized_end=196
# @@protoc_insertion_point(module_scope)
