# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/api/featureservice/feature_service_request__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nEtecton_proto/api/featureservice/feature_service_request__client.proto\x12 tecton_proto.api.feature_service\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xa5\x02\n\x0fMetadataOptions\x12#\n\rinclude_names\x18\x01 \x01(\x08R\x0cincludeNames\x12\x36\n\x17include_effective_times\x18\x02 \x01(\x08R\x15includeEffectiveTimes\x12\'\n\rinclude_types\x18\x03 \x01(\x08\x42\x02\x18\x01R\x0cincludeTypes\x12,\n\x12include_data_types\x18\x05 \x01(\x08R\x10includeDataTypes\x12(\n\x10include_slo_info\x18\x04 \x01(\x08R\x0eincludeSloInfo\x12\x34\n\x16include_serving_status\x18\x06 \x01(\x08R\x14includeServingStatus\"\xbe\x01\n\x0eRequestOptions\x12,\n\x0fread_from_cache\x18\x01 \x01(\x08:\x04trueR\rreadFromCache\x12*\n\x0ewrite_to_cache\x18\x02 \x01(\x08:\x04trueR\x0cwriteToCache\x12R\n\x17high_watermark_override\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x15highWatermarkOverride\"e\n\x12GetFeaturesRequest\x12O\n\x06params\x18\x01 \x01(\x0b\x32\x37.tecton_proto.api.feature_service.GetFeaturesParametersR\x06params\"o\n\x17GetFeaturesBatchRequest\x12T\n\x06params\x18\x01 \x01(\x0b\x32<.tecton_proto.api.feature_service.GetFeaturesBatchParametersR\x06params\"\xba\x08\n\x15GetFeaturesParameters\x12.\n\x12\x66\x65\x61ture_service_id\x18\x01 \x01(\tH\x00R\x10\x66\x65\x61tureServiceId\x12\x32\n\x14\x66\x65\x61ture_service_name\x18\x02 \x01(\tH\x00R\x12\x66\x65\x61tureServiceName\x12.\n\x12\x66\x65\x61ture_package_id\x18\x0c \x01(\tH\x00R\x10\x66\x65\x61turePackageId\x12\x32\n\x14\x66\x65\x61ture_package_name\x18\r \x01(\tH\x00R\x12\x66\x65\x61turePackageName\x12(\n\x0f\x66\x65\x61ture_view_id\x18\x0f \x01(\tH\x00R\rfeatureViewId\x12,\n\x11\x66\x65\x61ture_view_name\x18\x10 \x01(\tH\x00R\x0f\x66\x65\x61tureViewName\x12%\n\x0eworkspace_name\x18\n \x01(\tR\rworkspaceName\x12i\n\x0cjoin_key_map\x18\x08 \x03(\x0b\x32G.tecton_proto.api.feature_service.GetFeaturesParameters.JoinKeyMapEntryR\njoinKeyMap\x12~\n\x13request_context_map\x18\t \x03(\x0b\x32N.tecton_proto.api.feature_service.GetFeaturesParameters.RequestContextMapEntryR\x11requestContextMap\x12\\\n\x10metadata_options\x18\x07 \x01(\x0b\x32\x31.tecton_proto.api.feature_service.MetadataOptionsR\x0fmetadataOptions\x12\x32\n\x15\x61llow_partial_results\x18\x0b \x01(\x08R\x13\x61llowPartialResults\x12Y\n\x0frequest_options\x18\x12 \x01(\x0b\x32\x30.tecton_proto.api.feature_service.RequestOptionsR\x0erequestOptions\x12$\n\risCallerBatch\x18\x11 \x01(\x08R\risCallerBatch\x1aU\n\x0fJoinKeyMapEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x16.google.protobuf.ValueR\x05value:\x02\x38\x01\x1a\\\n\x16RequestContextMapEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x16.google.protobuf.ValueR\x05value:\x02\x38\x01\x42\t\n\x07locatorJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06J\x04\x08\x06\x10\x07J\x04\x08\x0e\x10\x0f\"\xf2\x02\n\x1aGetFeaturesBatchParameters\x12.\n\x12\x66\x65\x61ture_service_id\x18\x01 \x01(\tH\x00R\x10\x66\x65\x61tureServiceId\x12\x32\n\x14\x66\x65\x61ture_service_name\x18\x02 \x01(\tH\x00R\x12\x66\x65\x61tureServiceName\x12%\n\x0eworkspace_name\x18\x03 \x01(\tR\rworkspaceName\x12`\n\x0crequest_data\x18\x04 \x03(\x0b\x32=.tecton_proto.api.feature_service.GetFeaturesBatchRequestDataR\x0brequestData\x12\\\n\x10metadata_options\x18\x05 \x01(\x0b\x32\x31.tecton_proto.api.feature_service.MetadataOptionsR\x0fmetadataOptionsB\t\n\x07locator\"\xca\x03\n\x1bGetFeaturesBatchRequestData\x12o\n\x0cjoin_key_map\x18\x01 \x03(\x0b\x32M.tecton_proto.api.feature_service.GetFeaturesBatchRequestData.JoinKeyMapEntryR\njoinKeyMap\x12\x84\x01\n\x13request_context_map\x18\x02 \x03(\x0b\x32T.tecton_proto.api.feature_service.GetFeaturesBatchRequestData.RequestContextMapEntryR\x11requestContextMap\x1aU\n\x0fJoinKeyMapEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x16.google.protobuf.ValueR\x05value:\x02\x38\x01\x1a\\\n\x16RequestContextMapEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x16.google.protobuf.ValueR\x05value:\x02\x38\x01\x42\x36Z4github.com/tecton-ai/tecton_proto/api/featureservice')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.api.featureservice.feature_service_request__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z4github.com/tecton-ai/tecton_proto/api/featureservice'
  _METADATAOPTIONS.fields_by_name['include_types']._options = None
  _METADATAOPTIONS.fields_by_name['include_types']._serialized_options = b'\030\001'
  _GETFEATURESPARAMETERS_JOINKEYMAPENTRY._options = None
  _GETFEATURESPARAMETERS_JOINKEYMAPENTRY._serialized_options = b'8\001'
  _GETFEATURESPARAMETERS_REQUESTCONTEXTMAPENTRY._options = None
  _GETFEATURESPARAMETERS_REQUESTCONTEXTMAPENTRY._serialized_options = b'8\001'
  _GETFEATURESBATCHREQUESTDATA_JOINKEYMAPENTRY._options = None
  _GETFEATURESBATCHREQUESTDATA_JOINKEYMAPENTRY._serialized_options = b'8\001'
  _GETFEATURESBATCHREQUESTDATA_REQUESTCONTEXTMAPENTRY._options = None
  _GETFEATURESBATCHREQUESTDATA_REQUESTCONTEXTMAPENTRY._serialized_options = b'8\001'
  _METADATAOPTIONS._serialized_start=171
  _METADATAOPTIONS._serialized_end=464
  _REQUESTOPTIONS._serialized_start=467
  _REQUESTOPTIONS._serialized_end=657
  _GETFEATURESREQUEST._serialized_start=659
  _GETFEATURESREQUEST._serialized_end=760
  _GETFEATURESBATCHREQUEST._serialized_start=762
  _GETFEATURESBATCHREQUEST._serialized_end=873
  _GETFEATURESPARAMETERS._serialized_start=876
  _GETFEATURESPARAMETERS._serialized_end=1958
  _GETFEATURESPARAMETERS_JOINKEYMAPENTRY._serialized_start=1738
  _GETFEATURESPARAMETERS_JOINKEYMAPENTRY._serialized_end=1823
  _GETFEATURESPARAMETERS_REQUESTCONTEXTMAPENTRY._serialized_start=1825
  _GETFEATURESPARAMETERS_REQUESTCONTEXTMAPENTRY._serialized_end=1917
  _GETFEATURESBATCHPARAMETERS._serialized_start=1961
  _GETFEATURESBATCHPARAMETERS._serialized_end=2331
  _GETFEATURESBATCHREQUESTDATA._serialized_start=2334
  _GETFEATURESBATCHREQUESTDATA._serialized_end=2792
  _GETFEATURESBATCHREQUESTDATA_JOINKEYMAPENTRY._serialized_start=1738
  _GETFEATURESBATCHREQUESTDATA_JOINKEYMAPENTRY._serialized_end=1823
  _GETFEATURESBATCHREQUESTDATA_REQUESTCONTEXTMAPENTRY._serialized_start=1825
  _GETFEATURESBATCHREQUESTDATA_REQUESTCONTEXTMAPENTRY._serialized_end=1917
# @@protoc_insertion_point(module_scope)
