# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/materializationjobservice/materialization_job_service__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.args import feature_view__client_pb2 as tecton__proto_dot_args_dot_feature__view____client__pb2
from tecton_proto.auth import service__client_pb2 as tecton__proto_dot_auth_dot_service____client__pb2
from tecton_proto.common import compute_mode__client_pb2 as tecton__proto_dot_common_dot_compute__mode____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import schema__client_pb2 as tecton__proto_dot_common_dot_schema____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nPtecton_proto/materializationjobservice/materialization_job_service__client.proto\x12&tecton_proto.materializationjobservice\x1a google/protobuf/descriptor.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a,tecton_proto/args/feature_view__client.proto\x1a\'tecton_proto/auth/service__client.proto\x1a.tecton_proto/common/compute_mode__client.proto\x1a$tecton_proto/common/id__client.proto\x1a(tecton_proto/common/schema__client.proto\"\xdb\x02\n\x19MaterializationJobRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12!\n\x0c\x66\x65\x61ture_view\x18\x02 \x01(\tR\x0b\x66\x65\x61tureView\x12\x39\n\nstart_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tstartTime\x12\x35\n\x08\x65nd_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x07\x65ndTime\x12\x16\n\x06online\x18\x05 \x01(\x08R\x06online\x12\x18\n\x07offline\x18\x06 \x01(\x08R\x07offline\x12;\n\x1ause_tecton_managed_retries\x18\x07 \x01(\x08R\x17useTectonManagedRetries\x12\x1c\n\toverwrite\x18\x08 \x01(\x08R\toverwrite\"j\n\x1aMaterializationJobResponse\x12L\n\x03job\x18\x01 \x01(\x0b\x32:.tecton_proto.materializationjobservice.MaterializationJobR\x03job\"\xe5\x04\n\x12MaterializationJob\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x1c\n\tworkspace\x18\x02 \x01(\tR\tworkspace\x12!\n\x0c\x66\x65\x61ture_view\x18\x03 \x01(\tR\x0b\x66\x65\x61tureView\x12\x39\n\nstart_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tstartTime\x12\x35\n\x08\x65nd_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x07\x65ndTime\x12\x39\n\ncreated_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12\x14\n\x05state\x18\t \x01(\tR\x05state\x12N\n\x08\x61ttempts\x18\n \x03(\x0b\x32\x32.tecton_proto.materializationjobservice.JobAttemptR\x08\x61ttempts\x12\x42\n\x0fnext_attempt_at\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.TimestampR\rnextAttemptAt\x12\x16\n\x06online\x18\x0c \x01(\x08R\x06online\x12\x18\n\x07offline\x18\r \x01(\x08R\x07offline\x12\x19\n\x08job_type\x18\x0e \x01(\tR\x07jobType\x12\x1f\n\x0bingest_path\x18\x0f \x01(\tR\ningestPath\"\xc1\x01\n\nJobAttempt\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x39\n\ncreated_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12\x14\n\x05state\x18\x04 \x01(\tR\x05state\x12\x17\n\x07run_url\x18\x05 \x01(\tR\x06runUrl\"R\n\x0fListJobsRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12!\n\x0c\x66\x65\x61ture_view\x18\x02 \x01(\tR\x0b\x66\x65\x61tureView\"b\n\x10ListJobsResponse\x12N\n\x04jobs\x18\x01 \x03(\x0b\x32:.tecton_proto.materializationjobservice.MaterializationJobR\x04jobs\"g\n\rGetJobRequest\x12\x15\n\x06job_id\x18\x01 \x01(\tR\x05jobId\x12\x1c\n\tworkspace\x18\x02 \x01(\tR\tworkspace\x12!\n\x0c\x66\x65\x61ture_view\x18\x03 \x01(\tR\x0b\x66\x65\x61tureView\"^\n\x0eGetJobResponse\x12L\n\x03job\x18\x01 \x01(\x0b\x32:.tecton_proto.materializationjobservice.MaterializationJobR\x03job\"j\n\x10\x43\x61ncelJobRequest\x12\x15\n\x06job_id\x18\x01 \x01(\tR\x05jobId\x12\x1c\n\tworkspace\x18\x02 \x01(\tR\tworkspace\x12!\n\x0c\x66\x65\x61ture_view\x18\x03 \x01(\tR\x0b\x66\x65\x61tureView\"a\n\x11\x43\x61ncelJobResponse\x12L\n\x03job\x18\x01 \x01(\x0b\x32:.tecton_proto.materializationjobservice.MaterializationJobR\x03job\"\x85\x01\n\x19GetLatestReadyTimeRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12!\n\x0c\x66\x65\x61ture_view\x18\x02 \x01(\tR\x0b\x66\x65\x61tureView\x12\'\n\x0f\x66\x65\x61ture_service\x18\x03 \x01(\tR\x0e\x66\x65\x61tureService\"\xc8\x01\n\x1aGetLatestReadyTimeResponse\x12S\n\x18online_latest_ready_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x15onlineLatestReadyTime\x12U\n\x19offline_latest_ready_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x16offlineLatestReadyTime\"\xc3\x03\n+TestOnlyGetMaterializationTaskParamsRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12*\n\x11\x66\x65\x61ture_view_name\x18\x02 \x01(\tR\x0f\x66\x65\x61tureViewName\x12\x61\n\x08job_type\x18\x03 \x01(\x0e\x32\x46.tecton_proto.materializationjobservice.TestOnlyMaterializationJobTypeR\x07jobType\x12\'\n\x0f\x64isable_offline\x18\x04 \x01(\x08R\x0e\x64isableOffline\x12%\n\x0e\x64isable_online\x18\x05 \x01(\x08R\rdisableOnline\x12@\n\x0ejob_start_time\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0cjobStartTime\x12<\n\x0cjob_end_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\njobEndTime\x12\x17\n\x07\x64\x66_path\x18\x07 \x01(\tR\x06\x64\x66Path\"t\n,TestOnlyGetMaterializationTaskParamsResponse\x12\x44\n\x1e\x65ncoded_materialization_params\x18\x01 \x01(\tR\x1c\x65ncodedMaterializationParams\"Z\n\x17GetDataframeInfoRequest\x12!\n\x0c\x66\x65\x61ture_view\x18\x01 \x01(\tR\x0b\x66\x65\x61tureView\x12\x1c\n\tworkspace\x18\x02 \x01(\tR\tworkspace\"k\n\x18GetDataframeInfoResponse\x12\x17\n\x07\x64\x66_path\x18\x01 \x01(\tR\x06\x64\x66Path\x12\x36\n\x18signed_url_for_df_upload\x18\x02 \x01(\tR\x14signedUrlForDfUpload\"\xb5\x01\n\x1cIngestDataframeFromS3Request\x12!\n\x0c\x66\x65\x61ture_view\x18\x01 \x01(\tR\x0b\x66\x65\x61tureView\x12\x17\n\x07\x64\x66_path\x18\x02 \x01(\tR\x06\x64\x66Path\x12\x1c\n\tworkspace\x18\x03 \x01(\tR\tworkspace\x12;\n\x1ause_tecton_managed_retries\x18\x04 \x01(\x08R\x17useTectonManagedRetries\"m\n\x1dIngestDataframeFromS3Response\x12L\n\x03job\x18\x01 \x01(\x0b\x32:.tecton_proto.materializationjobservice.MaterializationJobR\x03job\"\xcf\x02\n\nDatasetJob\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x1c\n\tworkspace\x18\x02 \x01(\tR\tworkspace\x12\x37\n\x18saved_feature_data_frame\x18\x03 \x01(\tR\x15savedFeatureDataFrame\x12\x39\n\ncreated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12\x14\n\x05state\x18\x06 \x01(\tR\x05state\x12N\n\x08\x61ttempts\x18\x07 \x03(\x0b\x32\x32.tecton_proto.materializationjobservice.JobAttemptR\x08\x61ttempts\"\x90\n\n\x16StartDatasetJobRequest\x12H\n\x0c\x63ompute_mode\x18\x01 \x01(\x0e\x32%.tecton_proto.common.BatchComputeModeR\x0b\x63omputeMode\x12\x1f\n\x0b\x66rom_source\x18\x02 \x01(\x08R\nfromSource\x12\x1c\n\tworkspace\x18\x03 \x01(\tR\tworkspace\x12G\n\x12\x66\x65\x61ture_service_id\x18\x04 \x01(\x0b\x32\x17.tecton_proto.common.IdH\x00R\x10\x66\x65\x61tureServiceId\x12\x41\n\x0f\x66\x65\x61ture_view_id\x18\x05 \x01(\x0b\x32\x17.tecton_proto.common.IdH\x00R\rfeatureViewId\x12\x61\n\x05spine\x18\x06 \x01(\x0b\x32I.tecton_proto.materializationjobservice.StartDatasetJobRequest.SpineInputH\x01R\x05spine\x12z\n\x0e\x64\x61tetime_range\x18\x07 \x01(\x0b\x32Q.tecton_proto.materializationjobservice.StartDatasetJobRequest.DateTimeRangeInputH\x01R\rdatetimeRange\x12!\n\x0c\x64\x61taset_name\x18\t \x01(\tR\x0b\x64\x61tasetName\x12G\n\x0e\x63luster_config\x18\n \x01(\x0b\x32 .tecton_proto.args.ClusterConfigR\rclusterConfig\x12%\n\x0etecton_runtime\x18\x0b \x01(\tR\rtectonRuntime\x12 \n\x0b\x65nvironment\x18\x0c \x01(\tR\x0b\x65nvironment\x12r\n\x0c\x65xtra_config\x18\r \x03(\x0b\x32O.tecton_proto.materializationjobservice.StartDatasetJobRequest.ExtraConfigEntryR\x0b\x65xtraConfig\x12\x44\n\x0f\x65xpected_schema\x18\x0e \x01(\x0b\x32\x1b.tecton_proto.common.SchemaR\x0e\x65xpectedSchema\x1ah\n\nSpineInput\x12\x12\n\x04path\x18\x01 \x01(\tR\x04path\x12#\n\rtimestamp_key\x18\x02 \x01(\tR\x0ctimestampKey\x12!\n\x0c\x63olumn_names\x18\x03 \x03(\tR\x0b\x63olumnNames\x1a\xd8\x01\n\x12\x44\x61teTimeRangeInput\x12\x30\n\x05start\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x05start\x12,\n\x03\x65nd\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x03\x65nd\x12=\n\x0cmax_lookback\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0bmaxLookback\x12#\n\rentities_path\x18\x04 \x01(\tR\x0c\x65ntitiesPath\x1a>\n\x10\x45xtraConfigEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x05\n\x03\x66\x63oB\x07\n\x05input\"_\n\x17StartDatasetJobResponse\x12\x44\n\x03job\x18\x01 \x01(\x0b\x32\x32.tecton_proto.materializationjobservice.DatasetJobR\x03job\"\x87\x01\n\x17\x43\x61ncelDatasetJobRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12\x37\n\x18saved_feature_data_frame\x18\x02 \x01(\tR\x15savedFeatureDataFrame\x12\x15\n\x06job_id\x18\x03 \x01(\tR\x05jobId\"`\n\x18\x43\x61ncelDatasetJobResponse\x12\x44\n\x03job\x18\x01 \x01(\x0b\x32\x32.tecton_proto.materializationjobservice.DatasetJobR\x03job\"\x84\x01\n\x14GetDatasetJobRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12\x37\n\x18saved_feature_data_frame\x18\x02 \x01(\tR\x15savedFeatureDataFrame\x12\x15\n\x06job_id\x18\x03 \x01(\tR\x05jobId\"]\n\x15GetDatasetJobResponse\x12\x44\n\x03job\x18\x01 \x01(\x0b\x32\x32.tecton_proto.materializationjobservice.DatasetJobR\x03job\"V\n\'TestOnlyWriteFeatureServerConfigRequest\x12+\n\x11\x61\x62solute_filepath\x18\x01 \x01(\tR\x10\x61\x62soluteFilepath\"*\n(TestOnlyWriteFeatureServerConfigResponse\"\xa4\x01\n\x1eTestOnlyOnlineTableNameRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12*\n\x11\x66\x65\x61ture_view_name\x18\x02 \x01(\tR\x0f\x66\x65\x61tureViewName\x12\x38\n\twatermark\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\twatermark\"@\n\x1fTestOnlyOnlineTableNameResponse\x12\x1d\n\ntable_name\x18\x01 \x01(\tR\ttableName*\x94\x02\n\x1eTestOnlyMaterializationJobType\x12\x32\n.TEST_ONLY_MATERIALIZATION_JOB_TYPE_UNSPECIFIED\x10\x00\x12,\n(TEST_ONLY_MATERIALIZATION_JOB_TYPE_BATCH\x10\x01\x12-\n)TEST_ONLY_MATERIALIZATION_JOB_TYPE_STREAM\x10\x02\x12-\n)TEST_ONLY_MATERIALIZATION_JOB_TYPE_INGEST\x10\x03\x12\x32\n.TEST_ONLY_MATERIALIZATION_JOB_TYPE_MAINTENANCE\x10\x04\x32\xfc\x14\n\x19MaterializationJobService\x12\xe7\x01\n\x18SubmitMaterializationJob\x12\x41.tecton_proto.materializationjobservice.MaterializationJobRequest\x1a\x42.tecton_proto.materializationjobservice.MaterializationJobResponse\"D\xa2\xbc\xe6\xc0\x05>\x1a\r\x08\x01\x12\tworkspace*-tecton/workspace#update_materialization_state\x12\xd0\x01\n\x17ListMaterializationJobs\x12\x37.tecton_proto.materializationjobservice.ListJobsRequest\x1a\x38.tecton_proto.materializationjobservice.ListJobsResponse\"B\xa2\xbc\xe6\xc0\x05<\x1a\r\x08\x01\x12\tworkspace*+tecton/workspace#read_materialization_state\x12\x90\x01\n\x15GetMaterializationJob\x12\x35.tecton_proto.materializationjobservice.GetJobRequest\x1a\x36.tecton_proto.materializationjobservice.GetJobResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\x99\x01\n\x18\x43\x61ncelMaterializationJob\x12\x38.tecton_proto.materializationjobservice.CancelJobRequest\x1a\x39.tecton_proto.materializationjobservice.CancelJobResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\xdf\x01\n\x12GetLatestReadyTime\x12\x41.tecton_proto.materializationjobservice.GetLatestReadyTimeRequest\x1a\x42.tecton_proto.materializationjobservice.GetLatestReadyTimeResponse\"B\xa2\xbc\xe6\xc0\x05<\x1a\r\x08\x01\x12\tworkspace*+tecton/workspace#read_materialization_state\x12\xdb\x01\n\x10GetDataframeInfo\x12?.tecton_proto.materializationjobservice.GetDataframeInfoRequest\x1a@.tecton_proto.materializationjobservice.GetDataframeInfoResponse\"D\xa2\xbc\xe6\xc0\x05>\x1a\r\x08\x01\x12\tworkspace*-tecton/workspace#update_materialization_state\x12\xea\x01\n\x15IngestDataframeFromS3\x12\x44.tecton_proto.materializationjobservice.IngestDataframeFromS3Request\x1a\x45.tecton_proto.materializationjobservice.IngestDataframeFromS3Response\"D\xa2\xbc\xe6\xc0\x05>\x1a\r\x08\x01\x12\tworkspace*-tecton/workspace#update_materialization_state\x12\xd8\x01\n\x0fStartDatasetJob\x12>.tecton_proto.materializationjobservice.StartDatasetJobRequest\x1a?.tecton_proto.materializationjobservice.StartDatasetJobResponse\"D\xa2\xbc\xe6\xc0\x05>\x1a\r\x08\x01\x12\tworkspace*-tecton/workspace#update_materialization_state\x12\x9f\x01\n\x10\x43\x61ncelDatasetJob\x12?.tecton_proto.materializationjobservice.CancelDatasetJobRequest\x1a@.tecton_proto.materializationjobservice.CancelDatasetJobResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\x96\x01\n\rGetDatasetJob\x12<.tecton_proto.materializationjobservice.GetDatasetJobRequest\x1a=.tecton_proto.materializationjobservice.GetDatasetJobResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\xf4\x01\n$TestOnlyGetMaterializationTaskParams\x12S.tecton_proto.materializationjobservice.TestOnlyGetMaterializationTaskParamsRequest\x1aT.tecton_proto.materializationjobservice.TestOnlyGetMaterializationTaskParamsResponse\"!\xa2\xbc\xe6\xc0\x05\x1b*\x19tecton/organization#admin\x12\xe8\x01\n TestOnlyWriteFeatureServerConfig\x12O.tecton_proto.materializationjobservice.TestOnlyWriteFeatureServerConfigRequest\x1aP.tecton_proto.materializationjobservice.TestOnlyWriteFeatureServerConfigResponse\"!\xa2\xbc\xe6\xc0\x05\x1b*\x19tecton/organization#admin\x12\xcd\x01\n\x17TestOnlyOnlineTableName\x12\x46.tecton_proto.materializationjobservice.TestOnlyOnlineTableNameRequest\x1aG.tecton_proto.materializationjobservice.TestOnlyOnlineTableNameResponse\"!\xa2\xbc\xe6\xc0\x05\x1b*\x19tecton/organization#adminB\x85\x01\n$com.tecton.materializationjobserviceB\x1eMaterializationJobServiceProtoP\x01Z;github.com/tecton-ai/tecton_proto/materializationjobservice')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.materializationjobservice.materialization_job_service__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n$com.tecton.materializationjobserviceB\036MaterializationJobServiceProtoP\001Z;github.com/tecton-ai/tecton_proto/materializationjobservice'
  _STARTDATASETJOBREQUEST_EXTRACONFIGENTRY._options = None
  _STARTDATASETJOBREQUEST_EXTRACONFIGENTRY._serialized_options = b'8\001'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['SubmitMaterializationJob']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['SubmitMaterializationJob']._serialized_options = b'\242\274\346\300\005>\032\r\010\001\022\tworkspace*-tecton/workspace#update_materialization_state'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['ListMaterializationJobs']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['ListMaterializationJobs']._serialized_options = b'\242\274\346\300\005<\032\r\010\001\022\tworkspace*+tecton/workspace#read_materialization_state'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetMaterializationJob']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetMaterializationJob']._serialized_options = b'\242\274\346\300\005\002@\001'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['CancelMaterializationJob']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['CancelMaterializationJob']._serialized_options = b'\242\274\346\300\005\002@\001'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetLatestReadyTime']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetLatestReadyTime']._serialized_options = b'\242\274\346\300\005<\032\r\010\001\022\tworkspace*+tecton/workspace#read_materialization_state'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetDataframeInfo']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetDataframeInfo']._serialized_options = b'\242\274\346\300\005>\032\r\010\001\022\tworkspace*-tecton/workspace#update_materialization_state'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['IngestDataframeFromS3']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['IngestDataframeFromS3']._serialized_options = b'\242\274\346\300\005>\032\r\010\001\022\tworkspace*-tecton/workspace#update_materialization_state'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['StartDatasetJob']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['StartDatasetJob']._serialized_options = b'\242\274\346\300\005>\032\r\010\001\022\tworkspace*-tecton/workspace#update_materialization_state'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['CancelDatasetJob']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['CancelDatasetJob']._serialized_options = b'\242\274\346\300\005\002@\001'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetDatasetJob']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetDatasetJob']._serialized_options = b'\242\274\346\300\005\002@\001'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['TestOnlyGetMaterializationTaskParams']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['TestOnlyGetMaterializationTaskParams']._serialized_options = b'\242\274\346\300\005\033*\031tecton/organization#admin'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['TestOnlyWriteFeatureServerConfig']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['TestOnlyWriteFeatureServerConfig']._serialized_options = b'\242\274\346\300\005\033*\031tecton/organization#admin'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['TestOnlyOnlineTableName']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['TestOnlyOnlineTableName']._serialized_options = b'\242\274\346\300\005\033*\031tecton/organization#admin'
  _TESTONLYMATERIALIZATIONJOBTYPE._serialized_start=6241
  _TESTONLYMATERIALIZATIONJOBTYPE._serialized_end=6517
  _MATERIALIZATIONJOBREQUEST._serialized_start=407
  _MATERIALIZATIONJOBREQUEST._serialized_end=754
  _MATERIALIZATIONJOBRESPONSE._serialized_start=756
  _MATERIALIZATIONJOBRESPONSE._serialized_end=862
  _MATERIALIZATIONJOB._serialized_start=865
  _MATERIALIZATIONJOB._serialized_end=1478
  _JOBATTEMPT._serialized_start=1481
  _JOBATTEMPT._serialized_end=1674
  _LISTJOBSREQUEST._serialized_start=1676
  _LISTJOBSREQUEST._serialized_end=1758
  _LISTJOBSRESPONSE._serialized_start=1760
  _LISTJOBSRESPONSE._serialized_end=1858
  _GETJOBREQUEST._serialized_start=1860
  _GETJOBREQUEST._serialized_end=1963
  _GETJOBRESPONSE._serialized_start=1965
  _GETJOBRESPONSE._serialized_end=2059
  _CANCELJOBREQUEST._serialized_start=2061
  _CANCELJOBREQUEST._serialized_end=2167
  _CANCELJOBRESPONSE._serialized_start=2169
  _CANCELJOBRESPONSE._serialized_end=2266
  _GETLATESTREADYTIMEREQUEST._serialized_start=2269
  _GETLATESTREADYTIMEREQUEST._serialized_end=2402
  _GETLATESTREADYTIMERESPONSE._serialized_start=2405
  _GETLATESTREADYTIMERESPONSE._serialized_end=2605
  _TESTONLYGETMATERIALIZATIONTASKPARAMSREQUEST._serialized_start=2608
  _TESTONLYGETMATERIALIZATIONTASKPARAMSREQUEST._serialized_end=3059
  _TESTONLYGETMATERIALIZATIONTASKPARAMSRESPONSE._serialized_start=3061
  _TESTONLYGETMATERIALIZATIONTASKPARAMSRESPONSE._serialized_end=3177
  _GETDATAFRAMEINFOREQUEST._serialized_start=3179
  _GETDATAFRAMEINFOREQUEST._serialized_end=3269
  _GETDATAFRAMEINFORESPONSE._serialized_start=3271
  _GETDATAFRAMEINFORESPONSE._serialized_end=3378
  _INGESTDATAFRAMEFROMS3REQUEST._serialized_start=3381
  _INGESTDATAFRAMEFROMS3REQUEST._serialized_end=3562
  _INGESTDATAFRAMEFROMS3RESPONSE._serialized_start=3564
  _INGESTDATAFRAMEFROMS3RESPONSE._serialized_end=3673
  _DATASETJOB._serialized_start=3676
  _DATASETJOB._serialized_end=4011
  _STARTDATASETJOBREQUEST._serialized_start=4014
  _STARTDATASETJOBREQUEST._serialized_end=5310
  _STARTDATASETJOBREQUEST_SPINEINPUT._serialized_start=4907
  _STARTDATASETJOBREQUEST_SPINEINPUT._serialized_end=5011
  _STARTDATASETJOBREQUEST_DATETIMERANGEINPUT._serialized_start=5014
  _STARTDATASETJOBREQUEST_DATETIMERANGEINPUT._serialized_end=5230
  _STARTDATASETJOBREQUEST_EXTRACONFIGENTRY._serialized_start=5232
  _STARTDATASETJOBREQUEST_EXTRACONFIGENTRY._serialized_end=5294
  _STARTDATASETJOBRESPONSE._serialized_start=5312
  _STARTDATASETJOBRESPONSE._serialized_end=5407
  _CANCELDATASETJOBREQUEST._serialized_start=5410
  _CANCELDATASETJOBREQUEST._serialized_end=5545
  _CANCELDATASETJOBRESPONSE._serialized_start=5547
  _CANCELDATASETJOBRESPONSE._serialized_end=5643
  _GETDATASETJOBREQUEST._serialized_start=5646
  _GETDATASETJOBREQUEST._serialized_end=5778
  _GETDATASETJOBRESPONSE._serialized_start=5780
  _GETDATASETJOBRESPONSE._serialized_end=5873
  _TESTONLYWRITEFEATURESERVERCONFIGREQUEST._serialized_start=5875
  _TESTONLYWRITEFEATURESERVERCONFIGREQUEST._serialized_end=5961
  _TESTONLYWRITEFEATURESERVERCONFIGRESPONSE._serialized_start=5963
  _TESTONLYWRITEFEATURESERVERCONFIGRESPONSE._serialized_end=6005
  _TESTONLYONLINETABLENAMEREQUEST._serialized_start=6008
  _TESTONLYONLINETABLENAMEREQUEST._serialized_end=6172
  _TESTONLYONLINETABLENAMERESPONSE._serialized_start=6174
  _TESTONLYONLINETABLENAMERESPONSE._serialized_end=6238
  _MATERIALIZATIONJOBSERVICE._serialized_start=6520
  _MATERIALIZATIONJOBSERVICE._serialized_end=9204
# @@protoc_insertion_point(module_scope)
