# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/materialization/materialization_task__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.args import diff_options__client_pb2 as tecton__proto_dot_args_dot_diff__options____client__pb2
from tecton_proto.args import feature_view__client_pb2 as tecton__proto_dot_args_dot_feature__view____client__pb2
from tecton_proto.common import compute_mode__client_pb2 as tecton__proto_dot_common_dot_compute__mode____client__pb2
from tecton_proto.common import fco_locator__client_pb2 as tecton__proto_dot_common_dot_fco__locator____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.data import batch_data_source__client_pb2 as tecton__proto_dot_data_dot_batch__data__source____client__pb2
from tecton_proto.materialization import materialization_states__client_pb2 as tecton__proto_dot_materialization_dot_materialization__states____client__pb2
from tecton_proto.materialization import spark_cluster__client_pb2 as tecton__proto_dot_materialization_dot_spark__cluster____client__pb2
from tecton_proto.online_store_writer import config__client_pb2 as tecton__proto_dot_online__store__writer_dot_config____client__pb2
from tecton_proto.snowflake import location__client_pb2 as tecton__proto_dot_snowflake_dot_location____client__pb2
from tecton_proto.spark_api import jobs__client_pb2 as tecton__proto_dot_spark__api_dot_jobs____client__pb2
from tecton_proto.spark_common import libraries__client_pb2 as tecton__proto_dot_spark__common_dot_libraries____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n?tecton_proto/materialization/materialization_task__client.proto\x12\x1ctecton_proto.materialization\x1a\x1fgoogle/protobuf/timestamp.proto\x1a,tecton_proto/args/diff_options__client.proto\x1a,tecton_proto/args/feature_view__client.proto\x1a.tecton_proto/common/compute_mode__client.proto\x1a-tecton_proto/common/fco_locator__client.proto\x1a$tecton_proto/common/id__client.proto\x1a\x31tecton_proto/data/batch_data_source__client.proto\x1a\x41tecton_proto/materialization/materialization_states__client.proto\x1a\x38tecton_proto/materialization/spark_cluster__client.proto\x1a\x35tecton_proto/online_store_writer/config__client.proto\x1a-tecton_proto/snowflake/location__client.proto\x1a)tecton_proto/spark_api/jobs__client.proto\x1a\x31tecton_proto/spark_common/libraries__client.proto\"\xb5\x01\n\"MaterializationTaskStateTransition\x12U\n\ntask_state\x18\x01 \x01(\x0e\x32\x36.tecton_proto.materialization.MaterializationTaskStateR\ttaskState\x12\x38\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp\"\x88\x01\n\x16MaterializationTaskDep\x12n\n\x18offline_store_dependency\x18\x01 \x01(\x0b\x32\x34.tecton_proto.materialization.OfflineStoreDependencyR\x16offlineStoreDependency\"\xa8\x01\n\x16OfflineStoreDependency\x12H\n\x12\x66\x65\x61ture_start_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x10\x66\x65\x61tureStartTime\x12\x44\n\x10\x66\x65\x61ture_end_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0e\x66\x65\x61tureEndTime\"\xab\x08\n\x1e\x42\x61tchMaterializationParameters\x12\x46\n\x11window_start_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0fwindowStartTime\x12\x42\n\x0fwindow_end_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\rwindowEndTime\x12H\n\x12\x66\x65\x61ture_start_time\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x10\x66\x65\x61tureStartTime\x12\x44\n\x10\x66\x65\x61ture_end_time\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0e\x66\x65\x61tureEndTime\x12\x1d\n\ntile_count\x18\x05 \x01(\x05R\ttileCount\x12@\n\x1dwrite_to_online_feature_store\x18\x08 \x01(\x08R\x19writeToOnlineFeatureStore\x12\x42\n\x1ewrite_to_offline_feature_store\x18\x06 \x01(\x08R\x1awriteToOfflineFeatureStore\x12\x32\n\x15is_overwrite_backfill\x18\t \x01(\x08R\x13isOverwriteBackfill\x12\x36\n\x17\x62ootstraps_online_store\x18\x0f \x01(\x08R\x15\x62ootstrapsOnlineStore\x12T\n(read_from_offline_store_for_online_write\x18\r \x01(\x08R\"readFromOfflineStoreForOnlineWrite\x12.\n\x13\x63reate_online_table\x18\x0e \x01(\x08R\x11\x63reateOnlineTable\x12~\n\x1e\x63reate_online_table_parameters\x18\x12 \x01(\x0b\x32\x39.tecton_proto.materialization.CreateOnlineTableParametersR\x1b\x63reateOnlineTableParameters\x12\x38\n\x18\x62\x61tch_compaction_enabled\x18\x13 \x01(\x08R\x16\x62\x61tchCompactionEnabled\x12!\n\x0cis_overwrite\x18\x10 \x01(\x08R\x0bisOverwrite\x12\x61\n\x11task_dependencies\x18\x11 \x01(\x0b\x32\x34.tecton_proto.materialization.MaterializationTaskDepR\x10taskDependenciesJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03J\x04\x08\x07\x10\x08J\x04\x08\n\x10\x0b\"K\n\x1b\x43reateOnlineTableParameters\x12,\n\x12import_path_prefix\x18\x01 \x01(\tR\x10importPathPrefix\"\xc8\x01\n\x1fIngestMaterializationParameters\x12@\n\x1dwrite_to_online_feature_store\x18\x08 \x01(\x08R\x19writeToOnlineFeatureStore\x12\x42\n\x1ewrite_to_offline_feature_store\x18\x06 \x01(\x08R\x1awriteToOfflineFeatureStore\x12\x1f\n\x0bingest_path\x18\r \x01(\tR\ningestPath\"\xfa\x01\n\x12\x44\x65letionParameters\x12\x33\n\x15online_join_keys_path\x18\x01 \x01(\tH\x00R\x12onlineJoinKeysPath\x12<\n\x1aonline_join_keys_full_path\x18\x05 \x01(\tH\x00R\x16onlineJoinKeysFullPath\x12\x33\n\x16offline_join_keys_path\x18\x02 \x01(\tR\x13offlineJoinKeysPath\x12\x16\n\x06online\x18\x03 \x01(\x08R\x06online\x12\x18\n\x07offline\x18\x04 \x01(\x08R\x07offlineB\n\n\x08location\"\xaa\x02\n\x1a\x44\x65ltaMaintenanceParameters\x12+\n\x11generate_manifest\x18\x01 \x01(\x08R\x10generateManifest\x12\x39\n\nperiod_end\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tperiodEnd\x12-\n\x12\x65xecute_compaction\x18\x03 \x01(\x08R\x11\x65xecuteCompaction\x12\x16\n\x06vacuum\x18\x04 \x01(\x08R\x06vacuum\x12\'\n\x0f\x65xecute_sorting\x18\x05 \x01(\x08R\x0e\x65xecuteSorting\x12\x34\n\x16vacuum_retention_hours\x18\x06 \x01(\x05R\x14vacuumRetentionHours\"\x94\x01\n\x1fStreamMaterializationParameters\x12\x65\n\x15stream_handoff_config\x18\x03 \x01(\x0b\x32\x31.tecton_proto.materialization.StreamHandoffConfigR\x13streamHandoffConfigJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03\"\xb3\x02\n\x17\x46\x65\x61tureExportParameters\x12\\\n\x1eparent_materialization_task_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x1bparentMaterializationTaskId\x12H\n\x12\x66\x65\x61ture_start_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x10\x66\x65\x61tureStartTime\x12\x44\n\x10\x66\x65\x61ture_end_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0e\x66\x65\x61tureEndTime\x12*\n\x11\x65xport_store_path\x18\x04 \x01(\tR\x0f\x65xportStorePath\"\xde\n\n\x1b\x44\x61tasetGenerationParameters\x12\x1f\n\x0b\x66rom_source\x18\x02 \x01(\x08R\nfromSource\x12L\n\x0f\x66\x65\x61ture_service\x18\x03 \x01(\x0b\x32!.tecton_proto.common.IdFcoLocatorH\x00R\x0e\x66\x65\x61tureService\x12\x46\n\x0c\x66\x65\x61ture_view\x18\x04 \x01(\x0b\x32!.tecton_proto.common.IdFcoLocatorH\x00R\x0b\x66\x65\x61tureView\x12;\n\x07\x64\x61taset\x18\x0b \x01(\x0b\x32!.tecton_proto.common.IdFcoLocatorR\x07\x64\x61taset\x12\\\n\x05spine\x18\x05 \x01(\x0b\x32\x44.tecton_proto.materialization.DatasetGenerationParameters.SpineInputH\x01R\x05spine\x12u\n\x0e\x64\x61tetime_range\x18\x06 \x01(\x0b\x32L.tecton_proto.materialization.DatasetGenerationParameters.DateTimeRangeInputH\x01R\rdatetimeRange\x12o\n\x0c\x62\x61tch_config\x18\x07 \x01(\x0b\x32J.tecton_proto.materialization.DatasetGenerationParameters.BatchConfigInputH\x01R\x0b\x62\x61tchConfig\x12\x1f\n\x0bresult_path\x18\x08 \x01(\tR\nresultPath\x12G\n\x0e\x63luster_config\x18\t \x01(\x0b\x32 .tecton_proto.args.ClusterConfigR\rclusterConfig\x12m\n\x0c\x65xtra_config\x18\n \x03(\x0b\x32J.tecton_proto.materialization.DatasetGenerationParameters.ExtraConfigEntryR\x0b\x65xtraConfig\x1a\x45\n\nSpineInput\x12\x12\n\x04path\x18\x01 \x01(\tR\x04path\x12#\n\rtimestamp_key\x18\x02 \x01(\tR\x0ctimestampKey\x1a\xd8\x01\n\x12\x44\x61teTimeRangeInput\x12\x30\n\x05start\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x05start\x12,\n\x03\x65nd\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x03\x65nd\x12=\n\x0cmax_lookback\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0bmaxLookback\x12#\n\rentities_path\x18\x04 \x01(\tR\x0c\x65ntitiesPath\x1a\xb9\x01\n\x10\x42\x61tchConfigInput\x12\x45\n\x0c\x62\x61tch_source\x18\x01 \x01(\x0b\x32\".tecton_proto.data.BatchDataSourceR\x0b\x62\x61tchSource\x12\x30\n\x05start\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x05start\x12,\n\x03\x65nd\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x03\x65nd\x1a>\n\x10\x45xtraConfigEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x05\n\x03\x66\x63oB\x07\n\x05input\"r\n\x13StreamHandoffConfig\x12\x18\n\x07\x65nabled\x18\x01 \x01(\x08R\x07\x65nabled\x12\x41\n\x10previous_task_id\x18\x02 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x0epreviousTaskId\"\xdc\x10\n\x13MaterializationTask\x12O\n\x17materialization_task_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x15materializationTaskId\x12\x30\n\x07plan_id\x18\x1c \x01(\x0b\x32\x17.tecton_proto.common.IdR\x06planId\x12X\n\x17id_feature_view_locator\x18\x10 \x01(\x0b\x32!.tecton_proto.common.IdFcoLocatorR\x14idFeatureViewLocator\x12^\n\x1aid_feature_service_locator\x18\x1e \x01(\x0b\x32!.tecton_proto.common.IdFcoLocatorR\x17idFeatureServiceLocator\x12\x44\n\x1ematerialization_serial_version\x18\x0e \x01(\x05R\x1cmaterializationSerialVersion\x12\x34\n\x16tecton_runtime_version\x18\x1a \x01(\tR\x14tectonRuntimeVersion\x12-\n\x12tecton_environment\x18\x1f \x01(\tR\x11tectonEnvironment\x12H\n\x0c\x63ompute_mode\x18  \x01(\x0e\x32%.tecton_proto.common.BatchComputeModeR\x0b\x63omputeMode\x12i\n\x10\x62\x61tch_parameters\x18\x03 \x01(\x0b\x32<.tecton_proto.materialization.BatchMaterializationParametersH\x00R\x0f\x62\x61tchParameters\x12l\n\x11stream_parameters\x18\x04 \x01(\x0b\x32=.tecton_proto.materialization.StreamMaterializationParametersH\x00R\x10streamParameters\x12l\n\x11ingest_parameters\x18\x11 \x01(\x0b\x32=.tecton_proto.materialization.IngestMaterializationParametersH\x00R\x10ingestParameters\x12\x63\n\x13\x64\x65letion_parameters\x18\x13 \x01(\x0b\x32\x30.tecton_proto.materialization.DeletionParametersH\x00R\x12\x64\x65letionParameters\x12|\n\x1c\x64\x65lta_maintenance_parameters\x18\x18 \x01(\x0b\x32\x38.tecton_proto.materialization.DeltaMaintenanceParametersH\x00R\x1a\x64\x65ltaMaintenanceParameters\x12s\n\x19\x66\x65\x61ture_export_parameters\x18\x1b \x01(\x0b\x32\x35.tecton_proto.materialization.FeatureExportParametersH\x00R\x17\x66\x65\x61tureExportParameters\x12\x7f\n\x1d\x64\x61taset_generation_parameters\x18\x1d \x01(\x0b\x32\x39.tecton_proto.materialization.DatasetGenerationParametersH\x00R\x1b\x64\x61tasetGenerationParameters\x12I\n!spark_cluster_environment_version\x18\r \x01(\x05R\x1esparkClusterEnvironmentVersion\x12m\n\x11state_transitions\x18\x08 \x03(\x0b\x32@.tecton_proto.materialization.MaterializationTaskStateTransitionR\x10stateTransitions\x12Q\n&number_of_attempts_from_old_executions\x18\x12 \x01(\x05R!numberOfAttemptsFromOldExecutions\x12#\n\rerror_message\x18\t \x01(\tR\x0c\x65rrorMessage\x12\x39\n\ncreated_at\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12\x41\n\x08\x61ttempts\x18\x14 \x03(\x0b\x32%.tecton_proto.materialization.AttemptR\x08\x61ttempts\x12O\n\rcanary_params\x18\x15 \x01(\x0b\x32*.tecton_proto.materialization.CanaryParamsR\x0c\x63\x61naryParams\x12-\n\x12manually_triggered\x18\x16 \x01(\x08R\x11manuallyTriggered\x12\'\n\x0fmanaged_retries\x18\x17 \x01(\x08R\x0emanagedRetries\x12\x34\n\x16\x61ttempt_status_message\x18\x19 \x01(\tR\x14\x61ttemptStatusMessageB\x0b\n\ttask_typeJ\x04\x08\x02\x10\x03J\x04\x08\x06\x10\x07J\x04\x08\x07\x10\x08J\x04\x08\n\x10\x0bJ\x04\x08\x0f\x10\x10\"\xee\x03\n\x0c\x43\x61naryParams\x12\x1b\n\tcanary_id\x18\x01 \x01(\tR\x08\x63\x61naryId\x12r\n\x14\x63\x61nary_online_config\x18\x02 \x01(\x0b\x32@.tecton_proto.online_store_writer.OnlineStoreWriterConfigurationR\x12\x63\x61naryOnlineConfig\x12G\n canary_streaming_checkpoint_path\x18\x03 \x01(\tR\x1d\x63\x61naryStreamingCheckpointPath\x12\\\n\x18\x63\x61nary_library_overrides\x18\x04 \x03(\x0b\x32\".tecton_proto.spark_common.LibraryR\x16\x63\x61naryLibraryOverrides\x12~\n canary_spark_cluster_environment\x18\x05 \x01(\x0b\x32\x35.tecton_proto.materialization.SparkClusterEnvironmentR\x1d\x63\x61narySparkClusterEnvironment\x12&\n\x0f\x63\x61nary_run_name\x18\x06 \x01(\tR\rcanaryRunName\"\x99\x01\n\x07\x41ttempt\x12\x38\n\x0bworkflow_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\nworkflowId\x12T\n\'has_legacy_materialization_task_attempt\x18\x02 \x01(\x08R#hasLegacyMaterializationTaskAttempt\"\xcb\x02\n)MaterializationTaskAttemptStateTransition\x12\x62\n\rattempt_state\x18\x01 \x01(\x0e\x32=.tecton_proto.materialization.MaterializationTaskAttemptStateR\x0c\x61ttemptState\x12#\n\rstate_message\x18\x03 \x01(\tR\x0cstateMessage\x12[\n\x12termination_reason\x18\x04 \x01(\x0e\x32,.tecton_proto.spark_api.RunTerminationReasonR\x11terminationReason\x12\x38\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp\"\xc8\x08\n\x1aMaterializationTaskAttempt\x12^\n\x1fmaterialization_task_attempt_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x1cmaterializationTaskAttemptId\x12O\n\x17materialization_task_id\x18\x02 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x15materializationTaskId\x12l\n\x15\x65xecution_environment\x18\x0e \x01(\x0e\x32\x37.tecton_proto.materialization.SparkExecutionEnvironmentR\x14\x65xecutionEnvironment\x12\x15\n\x06run_id\x18\r \x01(\tR\x05runId\x12 \n\x0crun_page_url\x18\x0c \x01(\tR\nrunPageUrl\x12t\n\x11state_transitions\x18\x04 \x03(\x0b\x32G.tecton_proto.materialization.MaterializationTaskAttemptStateTransitionR\x10stateTransitions\x12%\n\x0e\x61ttempt_number\x18\x0b \x01(\x05R\rattemptNumber\x12\x39\n\ncreated_at\x18\t \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\n \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12G\n\x0e\x63luster_config\x18\x0f \x01(\x0b\x32 .tecton_proto.args.ClusterConfigR\rclusterConfig\x12\x8f\x01\n\x19online_store_copier_tasks\x18\x11 \x03(\x0b\x32T.tecton_proto.materialization.MaterializationTaskAttempt.OnlineStoreCopierTasksEntryR\x16onlineStoreCopierTasks\x12R\n\x0esnowflake_data\x18\x12 \x01(\x0b\x32+.tecton_proto.materialization.SnowflakeDataR\rsnowflakeData\x1a~\n\x1bOnlineStoreCopierTasksEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12I\n\x05value\x18\x02 \x01(\x0b\x32\x33.tecton_proto.materialization.OnlineStoreCopierTaskR\x05value:\x02\x38\x01J\x04\x08\x03\x10\x04J\x04\x08\x05\x10\tJ\x04\x08\x10\x10\x11\"\xd5\x01\n$OnlineStoreCopierTaskStateTransition\x12N\n\x05state\x18\x01 \x01(\x0e\x32\x38.tecton_proto.materialization.OnlineStoreCopierTaskStateR\x05state\x12#\n\rstate_message\x18\x03 \x01(\tR\x0cstateMessage\x12\x38\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp\"\xc7\x02\n\x15OnlineStoreCopierTask\x12V\n\x08\x61ttempts\x18\x02 \x03(\x0b\x32:.tecton_proto.materialization.OnlineStoreCopierTaskAttemptR\x08\x61ttempts\x12\x1d\n\ntotal_rows\x18\x04 \x01(\x03R\ttotalRows\x12%\n\x0e\x63ompleted_rows\x18\x03 \x01(\x03R\rcompletedRows\x12\x37\n\x18last_progress_request_id\x18\x05 \x01(\tR\x15lastProgressRequestId\x12Q\n\x17last_progress_send_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x14lastProgressSendTimeJ\x04\x08\x01\x10\x02\"\x8f\x01\n\x1cOnlineStoreCopierTaskAttempt\x12o\n\x11state_transitions\x18\x01 \x03(\x0b\x32\x42.tecton_proto.materialization.OnlineStoreCopierTaskStateTransitionR\x10stateTransitions\"\xbb\x01\n\rSnowflakeData\x12J\n\rbase_location\x18\x02 \x01(\x0b\x32%.tecton_proto.snowflake.StageLocationR\x0c\x62\x61seLocation\x12*\n\x11total_online_rows\x18\x03 \x01(\x03R\x0ftotalOnlineRows\x12,\n\x12total_offline_rows\x18\x04 \x01(\x03R\x10totalOfflineRowsJ\x04\x08\x01\x10\x02\x42\x1e\n\x1a\x63om.tecton.materializationP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.materialization.materialization_task__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\032com.tecton.materializationP\001'
  _DATASETGENERATIONPARAMETERS_EXTRACONFIGENTRY._options = None
  _DATASETGENERATIONPARAMETERS_EXTRACONFIGENTRY._serialized_options = b'8\001'
  _MATERIALIZATIONTASKATTEMPT_ONLINESTORECOPIERTASKSENTRY._options = None
  _MATERIALIZATIONTASKATTEMPT_ONLINESTORECOPIERTASKSENTRY._serialized_options = b'8\001'
  _MATERIALIZATIONTASKSTATETRANSITION._serialized_start=728
  _MATERIALIZATIONTASKSTATETRANSITION._serialized_end=909
  _MATERIALIZATIONTASKDEP._serialized_start=912
  _MATERIALIZATIONTASKDEP._serialized_end=1048
  _OFFLINESTOREDEPENDENCY._serialized_start=1051
  _OFFLINESTOREDEPENDENCY._serialized_end=1219
  _BATCHMATERIALIZATIONPARAMETERS._serialized_start=1222
  _BATCHMATERIALIZATIONPARAMETERS._serialized_end=2289
  _CREATEONLINETABLEPARAMETERS._serialized_start=2291
  _CREATEONLINETABLEPARAMETERS._serialized_end=2366
  _INGESTMATERIALIZATIONPARAMETERS._serialized_start=2369
  _INGESTMATERIALIZATIONPARAMETERS._serialized_end=2569
  _DELETIONPARAMETERS._serialized_start=2572
  _DELETIONPARAMETERS._serialized_end=2822
  _DELTAMAINTENANCEPARAMETERS._serialized_start=2825
  _DELTAMAINTENANCEPARAMETERS._serialized_end=3123
  _STREAMMATERIALIZATIONPARAMETERS._serialized_start=3126
  _STREAMMATERIALIZATIONPARAMETERS._serialized_end=3274
  _FEATUREEXPORTPARAMETERS._serialized_start=3277
  _FEATUREEXPORTPARAMETERS._serialized_end=3584
  _DATASETGENERATIONPARAMETERS._serialized_start=3587
  _DATASETGENERATIONPARAMETERS._serialized_end=4961
  _DATASETGENERATIONPARAMETERS_SPINEINPUT._serialized_start=4405
  _DATASETGENERATIONPARAMETERS_SPINEINPUT._serialized_end=4474
  _DATASETGENERATIONPARAMETERS_DATETIMERANGEINPUT._serialized_start=4477
  _DATASETGENERATIONPARAMETERS_DATETIMERANGEINPUT._serialized_end=4693
  _DATASETGENERATIONPARAMETERS_BATCHCONFIGINPUT._serialized_start=4696
  _DATASETGENERATIONPARAMETERS_BATCHCONFIGINPUT._serialized_end=4881
  _DATASETGENERATIONPARAMETERS_EXTRACONFIGENTRY._serialized_start=4883
  _DATASETGENERATIONPARAMETERS_EXTRACONFIGENTRY._serialized_end=4945
  _STREAMHANDOFFCONFIG._serialized_start=4963
  _STREAMHANDOFFCONFIG._serialized_end=5077
  _MATERIALIZATIONTASK._serialized_start=5080
  _MATERIALIZATIONTASK._serialized_end=7220
  _CANARYPARAMS._serialized_start=7223
  _CANARYPARAMS._serialized_end=7717
  _ATTEMPT._serialized_start=7720
  _ATTEMPT._serialized_end=7873
  _MATERIALIZATIONTASKATTEMPTSTATETRANSITION._serialized_start=7876
  _MATERIALIZATIONTASKATTEMPTSTATETRANSITION._serialized_end=8207
  _MATERIALIZATIONTASKATTEMPT._serialized_start=8210
  _MATERIALIZATIONTASKATTEMPT._serialized_end=9306
  _MATERIALIZATIONTASKATTEMPT_ONLINESTORECOPIERTASKSENTRY._serialized_start=9162
  _MATERIALIZATIONTASKATTEMPT_ONLINESTORECOPIERTASKSENTRY._serialized_end=9288
  _ONLINESTORECOPIERTASKSTATETRANSITION._serialized_start=9309
  _ONLINESTORECOPIERTASKSTATETRANSITION._serialized_end=9522
  _ONLINESTORECOPIERTASK._serialized_start=9525
  _ONLINESTORECOPIERTASK._serialized_end=9852
  _ONLINESTORECOPIERTASKATTEMPT._serialized_start=9855
  _ONLINESTORECOPIERTASKATTEMPT._serialized_end=9998
  _SNOWFLAKEDATA._serialized_start=10001
  _SNOWFLAKEDATA._serialized_end=10188
# @@protoc_insertion_point(module_scope)
