# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/data_source__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from tecton_proto.args import data_source_config__client_pb2 as tecton__proto_dot_args_dot_data__source__config____client__pb2
from tecton_proto.args import diff_options__client_pb2 as tecton__proto_dot_args_dot_diff__options____client__pb2
from tecton_proto.args import transformation__client_pb2 as tecton__proto_dot_args_dot_transformation____client__pb2
from tecton_proto.args import user_defined_function__client_pb2 as tecton__proto_dot_args_dot_user__defined__function____client__pb2
from tecton_proto.common import schema__client_pb2 as tecton__proto_dot_common_dot_schema____client__pb2
from tecton_proto.common import secret__client_pb2 as tecton__proto_dot_common_dot_secret____client__pb2
from tecton_proto.common import spark_schema__client_pb2 as tecton__proto_dot_common_dot_spark__schema____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+tecton_proto/args/data_source__client.proto\x12\x11tecton_proto.args\x1a\x1egoogle/protobuf/duration.proto\x1a\x32tecton_proto/args/data_source_config__client.proto\x1a,tecton_proto/args/diff_options__client.proto\x1a.tecton_proto/args/transformation__client.proto\x1a\x35tecton_proto/args/user_defined_function__client.proto\x1a(tecton_proto/common/schema__client.proto\x1a(tecton_proto/common/secret__client.proto\x1a.tecton_proto/common/spark_schema__client.proto\"\xa0\x01\n\x1b\x44\x61tetimePartitionColumnArgs\x12\x1f\n\x0b\x63olumn_name\x18\x01 \x01(\tR\ncolumnName\x12\x1a\n\x08\x64\x61tepart\x18\x02 \x01(\tR\x08\x64\x61tepart\x12\x1f\n\x0bzero_padded\x18\x03 \x01(\x08R\nzeroPadded\x12#\n\rformat_string\x18\x04 \x01(\tR\x0c\x66ormatString\"\xcd\x01\n\x19\x42\x61tchDataSourceCommonArgs\x12\'\n\x0ftimestamp_field\x18\x01 \x01(\tR\x0etimestampField\x12M\n\x0epost_processor\x18\x02 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\rpostProcessor\x12\x38\n\ndata_delay\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\tdataDelay\"\xae\x02\n\x1aStreamDataSourceCommonArgs\x12\'\n\x0ftimestamp_field\x18\x01 \x01(\tR\x0etimestampField\x12\\\n\x19watermark_delay_threshold\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationB\x05\x92M\x02\x08\x08R\x17watermarkDelayThreshold\x12T\n\x0epost_processor\x18\x03 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionB\x05\x92M\x02\x08\x08R\rpostProcessor\x12\x33\n\x15\x64\x65\x64uplication_columns\x18\x04 \x03(\tR\x14\x64\x65\x64uplicationColumns\"\xcc\x02\n\x12HiveDataSourceArgs\x12\x14\n\x05table\x18\x01 \x01(\tR\x05table\x12\x1a\n\x08\x64\x61tabase\x18\x02 \x01(\tR\x08\x64\x61tabase\x12)\n\x10timestamp_format\x18\x05 \x01(\tR\x0ftimestampFormat\x12l\n\x1a\x64\x61tetime_partition_columns\x18\x07 \x03(\x0b\x32..tecton_proto.args.DatetimePartitionColumnArgsR\x18\x64\x61tetimePartitionColumns\x12M\n\x0b\x63ommon_args\x18\n \x01(\x0b\x32,.tecton_proto.args.BatchDataSourceCommonArgsR\ncommonArgsJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05J\x04\x08\x06\x10\x07J\x04\x08\x08\x10\tJ\x04\x08\t\x10\n\"\x98\x03\n\x13UnityDataSourceArgs\x12\x18\n\x07\x63\x61talog\x18\x01 \x01(\tR\x07\x63\x61talog\x12\x16\n\x06schema\x18\x02 \x01(\tR\x06schema\x12\x14\n\x05table\x18\x03 \x01(\tR\x05table\x12M\n\x0b\x63ommon_args\x18\x04 \x01(\x0b\x32,.tecton_proto.args.BatchDataSourceCommonArgsR\ncommonArgs\x12)\n\x10timestamp_format\x18\x05 \x01(\tR\x0ftimestampFormat\x12l\n\x1a\x64\x61tetime_partition_columns\x18\x06 \x03(\x0b\x32..tecton_proto.args.DatetimePartitionColumnArgsR\x18\x64\x61tetimePartitionColumns\x12Q\n\x0b\x61\x63\x63\x65ss_mode\x18\x07 \x01(\x0e\x32).tecton_proto.args.UnityCatalogAccessModeB\x05\x92M\x02 \x0eR\naccessMode\"\x80\x03\n\x12\x46ileDataSourceArgs\x12\x10\n\x03uri\x18\x01 \x01(\tR\x03uri\x12\x1f\n\x0b\x66ile_format\x18\x02 \x01(\tR\nfileFormat\x12\x33\n\x16\x63onvert_to_glue_format\x18\x03 \x01(\x08R\x13\x63onvertToGlueFormat\x12$\n\nschema_uri\x18\x05 \x01(\tB\x05\x92M\x02\x08\x01R\tschemaUri\x12)\n\x10timestamp_format\x18\x07 \x01(\tR\x0ftimestampFormat\x12I\n\x0fschema_override\x18\t \x01(\x0b\x32 .tecton_proto.common.SparkSchemaR\x0eschemaOverride\x12T\n\x0b\x63ommon_args\x18\n \x01(\x0b\x32,.tecton_proto.args.BatchDataSourceCommonArgsB\x05\x92M\x02\x10\x01R\ncommonArgsJ\x04\x08\x04\x10\x05J\x04\x08\x06\x10\x07J\x04\x08\x08\x10\t\"7\n\x06Option\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x1b\n\x05value\x18\x02 \x01(\tB\x05\x92M\x02 \x02R\x05value\"\xe9\x02\n\x15KinesisDataSourceArgs\x12\x1f\n\x0bstream_name\x18\x01 \x01(\tR\nstreamName\x12\x16\n\x06region\x18\x02 \x01(\tR\x06region\x12g\n\x17initial_stream_position\x18\x0b \x01(\x0e\x32(.tecton_proto.args.InitialStreamPositionB\x05\x92M\x02\x08\x08R\x15initialStreamPosition\x12\x33\n\x07options\x18\x08 \x03(\x0b\x32\x19.tecton_proto.args.OptionR\x07options\x12U\n\x0b\x63ommon_args\x18\n \x01(\x0b\x32-.tecton_proto.args.StreamDataSourceCommonArgsB\x05\x92M\x02\x10\x01R\ncommonArgsJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06J\x04\x08\x06\x10\x07J\x04\x08\x07\x10\x08J\x04\x08\t\x10\n\"\xcf\x04\n\x13KafkaDataSourceArgs\x12\x36\n\x17kafka_bootstrap_servers\x18\x01 \x01(\tR\x15kafkaBootstrapServers\x12\x16\n\x06topics\x18\x02 \x01(\tR\x06topics\x12\x33\n\x07options\x18\x06 \x03(\x0b\x32\x19.tecton_proto.args.OptionR\x07options\x12\x39\n\x15ssl_keystore_location\x18\x07 \x01(\tB\x05\x92M\x02\x08\x01R\x13sslKeystoreLocation\x12K\n\x1fssl_keystore_password_secret_id\x18\x08 \x01(\tB\x05\x92M\x02\x08\x01R\x1bsslKeystorePasswordSecretId\x12=\n\x17ssl_truststore_location\x18\t \x01(\tB\x05\x92M\x02\x08\x01R\x15sslTruststoreLocation\x12O\n!ssl_truststore_password_secret_id\x18\n \x01(\tB\x05\x92M\x02\x08\x01R\x1dsslTruststorePasswordSecretId\x12\x32\n\x11security_protocol\x18\x0b \x01(\tB\x05\x92M\x02\x08\x01R\x10securityProtocol\x12U\n\x0b\x63ommon_args\x18\x0c \x01(\x0b\x32-.tecton_proto.args.StreamDataSourceCommonArgsB\x05\x92M\x02\x10\x01R\ncommonArgsJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06\"\xac\x02\n\x0ePushSourceArgs\x12\x1f\n\x0blog_offline\x18\x01 \x01(\x08R\nlogOffline\x12T\n\x0epost_processor\x18\x02 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionB\x05\x92M\x02\x08\x06R\rpostProcessor\x12\x45\n\x0cinput_schema\x18\x03 \x01(\x0b\x32\x1b.tecton_proto.common.SchemaB\x05\x92M\x02\x08\x06R\x0binputSchema\x12\\\n\x13post_processor_mode\x18\x04 \x01(\x0e\x32%.tecton_proto.args.TransformationModeB\x05\x92M\x02\x08\x06R\x11postProcessorMode\"\xdd\x01\n\x16RedshiftDataSourceArgs\x12\x1a\n\x08\x65ndpoint\x18\x01 \x01(\tR\x08\x65ndpoint\x12\x16\n\x05table\x18\x02 \x01(\tH\x00R\x05table\x12\x1d\n\x05query\x18\x05 \x01(\tB\x05\x92M\x02\x18\x03H\x00R\x05query\x12T\n\x0b\x63ommon_args\x18\x07 \x01(\x0b\x32,.tecton_proto.args.BatchDataSourceCommonArgsB\x05\x92M\x02\x10\x01R\ncommonArgsB\x08\n\x06sourceJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05J\x04\x08\x06\x10\x07\"\xcc\x03\n\x17SnowflakeDataSourceArgs\x12\x10\n\x03url\x18\x01 \x01(\tR\x03url\x12\x19\n\x04role\x18\x02 \x01(\tB\x05\x92M\x02\x08\x01R\x04role\x12\x1a\n\x08\x64\x61tabase\x18\x03 \x01(\tR\x08\x64\x61tabase\x12\x16\n\x06schema\x18\x04 \x01(\tR\x06schema\x12#\n\twarehouse\x18\x05 \x01(\tB\x05\x92M\x02\x08\x01R\twarehouse\x12\x16\n\x05table\x18\x06 \x01(\tH\x00R\x05table\x12\x1d\n\x05query\x18\x07 \x01(\tB\x05\x92M\x02\x18\x03H\x00R\x05query\x12T\n\x0b\x63ommon_args\x18\x0c \x01(\x0b\x32,.tecton_proto.args.BatchDataSourceCommonArgsB\x05\x92M\x02\x10\x01R\ncommonArgs\x12?\n\x04user\x18\r \x01(\x0b\x32$.tecton_proto.common.SecretReferenceB\x05\x92M\x02\x08\x01R\x04user\x12G\n\x08password\x18\x0e \x01(\x0b\x32$.tecton_proto.common.SecretReferenceB\x05\x92M\x02\x08\x01R\x08passwordB\x08\n\x06sourceJ\x04\x08\x08\x10\tJ\x04\x08\t\x10\n\"\xe2\x01\n\x14SparkBatchConfigArgs\x12X\n\x14\x64\x61ta_source_function\x18\x01 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x12\x64\x61taSourceFunction\x12\x38\n\ndata_delay\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationR\tdataDelay\x12\x36\n\x17supports_time_filtering\x18\x03 \x01(\x08R\x15supportsTimeFiltering\"q\n\x15SparkStreamConfigArgs\x12X\n\x14\x64\x61ta_source_function\x18\x01 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x12\x64\x61taSourceFunction\"\xa9\x03\n\x15PandasBatchConfigArgs\x12X\n\x14\x64\x61ta_source_function\x18\x01 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x12\x64\x61taSourceFunction\x12\x38\n\ndata_delay\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationR\tdataDelay\x12\x36\n\x17supports_time_filtering\x18\x03 \x01(\x08R\x15supportsTimeFiltering\x12V\n\x07secrets\x18\x06 \x03(\x0b\x32\x35.tecton_proto.args.PandasBatchConfigArgs.SecretsEntryB\x05\x92M\x02\x08\x01R\x07secrets\x1a`\n\x0cSecretsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12:\n\x05value\x18\x02 \x01(\x0b\x32$.tecton_proto.common.SecretReferenceR\x05value:\x02\x38\x01J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06\"\xda\x02\n\x16\x42igqueryDataSourceArgs\x12\x1d\n\nproject_id\x18\x01 \x01(\tR\tprojectId\x12\x18\n\x07\x64\x61taset\x18\x02 \x01(\tR\x07\x64\x61taset\x12!\n\x08location\x18\x03 \x01(\tB\x05\x92M\x02\x08\x01R\x08location\x12\x16\n\x05table\x18\x04 \x01(\tH\x00R\x05table\x12\x1d\n\x05query\x18\x05 \x01(\tB\x05\x92M\x02\x18\x03H\x00R\x05query\x12T\n\x0b\x63ommon_args\x18\x06 \x01(\x0b\x32,.tecton_proto.args.BatchDataSourceCommonArgsB\x05\x92M\x02\x10\x01R\ncommonArgs\x12M\n\x0b\x63redentials\x18\x07 \x01(\x0b\x32$.tecton_proto.common.SecretReferenceB\x05\x92M\x02\x08\x01R\x0b\x63redentialsB\x08\n\x06source*\xc9\x01\n\x16UnityCatalogAccessMode\x12)\n%UNITY_CATALOG_ACCESS_MODE_UNSPECIFIED\x10\x00\x12)\n%UNITY_CATALOG_ACCESS_MODE_SINGLE_USER\x10\x01\x12\x33\n/UNITY_CATALOG_ACCESS_MODE_SINGLE_USER_WITH_FGAC\x10\x03\x12$\n UNITY_CATALOG_ACCESS_MODE_SHARED\x10\x02\x42\x13\n\x0f\x63om.tecton.argsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.data_source__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001'
  _STREAMDATASOURCECOMMONARGS.fields_by_name['watermark_delay_threshold']._options = None
  _STREAMDATASOURCECOMMONARGS.fields_by_name['watermark_delay_threshold']._serialized_options = b'\222M\002\010\010'
  _STREAMDATASOURCECOMMONARGS.fields_by_name['post_processor']._options = None
  _STREAMDATASOURCECOMMONARGS.fields_by_name['post_processor']._serialized_options = b'\222M\002\010\010'
  _UNITYDATASOURCEARGS.fields_by_name['access_mode']._options = None
  _UNITYDATASOURCEARGS.fields_by_name['access_mode']._serialized_options = b'\222M\002 \016'
  _FILEDATASOURCEARGS.fields_by_name['schema_uri']._options = None
  _FILEDATASOURCEARGS.fields_by_name['schema_uri']._serialized_options = b'\222M\002\010\001'
  _FILEDATASOURCEARGS.fields_by_name['common_args']._options = None
  _FILEDATASOURCEARGS.fields_by_name['common_args']._serialized_options = b'\222M\002\020\001'
  _OPTION.fields_by_name['value']._options = None
  _OPTION.fields_by_name['value']._serialized_options = b'\222M\002 \002'
  _KINESISDATASOURCEARGS.fields_by_name['initial_stream_position']._options = None
  _KINESISDATASOURCEARGS.fields_by_name['initial_stream_position']._serialized_options = b'\222M\002\010\010'
  _KINESISDATASOURCEARGS.fields_by_name['common_args']._options = None
  _KINESISDATASOURCEARGS.fields_by_name['common_args']._serialized_options = b'\222M\002\020\001'
  _KAFKADATASOURCEARGS.fields_by_name['ssl_keystore_location']._options = None
  _KAFKADATASOURCEARGS.fields_by_name['ssl_keystore_location']._serialized_options = b'\222M\002\010\001'
  _KAFKADATASOURCEARGS.fields_by_name['ssl_keystore_password_secret_id']._options = None
  _KAFKADATASOURCEARGS.fields_by_name['ssl_keystore_password_secret_id']._serialized_options = b'\222M\002\010\001'
  _KAFKADATASOURCEARGS.fields_by_name['ssl_truststore_location']._options = None
  _KAFKADATASOURCEARGS.fields_by_name['ssl_truststore_location']._serialized_options = b'\222M\002\010\001'
  _KAFKADATASOURCEARGS.fields_by_name['ssl_truststore_password_secret_id']._options = None
  _KAFKADATASOURCEARGS.fields_by_name['ssl_truststore_password_secret_id']._serialized_options = b'\222M\002\010\001'
  _KAFKADATASOURCEARGS.fields_by_name['security_protocol']._options = None
  _KAFKADATASOURCEARGS.fields_by_name['security_protocol']._serialized_options = b'\222M\002\010\001'
  _KAFKADATASOURCEARGS.fields_by_name['common_args']._options = None
  _KAFKADATASOURCEARGS.fields_by_name['common_args']._serialized_options = b'\222M\002\020\001'
  _PUSHSOURCEARGS.fields_by_name['post_processor']._options = None
  _PUSHSOURCEARGS.fields_by_name['post_processor']._serialized_options = b'\222M\002\010\006'
  _PUSHSOURCEARGS.fields_by_name['input_schema']._options = None
  _PUSHSOURCEARGS.fields_by_name['input_schema']._serialized_options = b'\222M\002\010\006'
  _PUSHSOURCEARGS.fields_by_name['post_processor_mode']._options = None
  _PUSHSOURCEARGS.fields_by_name['post_processor_mode']._serialized_options = b'\222M\002\010\006'
  _REDSHIFTDATASOURCEARGS.fields_by_name['query']._options = None
  _REDSHIFTDATASOURCEARGS.fields_by_name['query']._serialized_options = b'\222M\002\030\003'
  _REDSHIFTDATASOURCEARGS.fields_by_name['common_args']._options = None
  _REDSHIFTDATASOURCEARGS.fields_by_name['common_args']._serialized_options = b'\222M\002\020\001'
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['role']._options = None
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['role']._serialized_options = b'\222M\002\010\001'
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['warehouse']._options = None
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['warehouse']._serialized_options = b'\222M\002\010\001'
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['query']._options = None
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['query']._serialized_options = b'\222M\002\030\003'
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['common_args']._options = None
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['common_args']._serialized_options = b'\222M\002\020\001'
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['user']._options = None
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['user']._serialized_options = b'\222M\002\010\001'
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['password']._options = None
  _SNOWFLAKEDATASOURCEARGS.fields_by_name['password']._serialized_options = b'\222M\002\010\001'
  _PANDASBATCHCONFIGARGS_SECRETSENTRY._options = None
  _PANDASBATCHCONFIGARGS_SECRETSENTRY._serialized_options = b'8\001'
  _PANDASBATCHCONFIGARGS.fields_by_name['secrets']._options = None
  _PANDASBATCHCONFIGARGS.fields_by_name['secrets']._serialized_options = b'\222M\002\010\001'
  _BIGQUERYDATASOURCEARGS.fields_by_name['location']._options = None
  _BIGQUERYDATASOURCEARGS.fields_by_name['location']._serialized_options = b'\222M\002\010\001'
  _BIGQUERYDATASOURCEARGS.fields_by_name['query']._options = None
  _BIGQUERYDATASOURCEARGS.fields_by_name['query']._serialized_options = b'\222M\002\030\003'
  _BIGQUERYDATASOURCEARGS.fields_by_name['common_args']._options = None
  _BIGQUERYDATASOURCEARGS.fields_by_name['common_args']._serialized_options = b'\222M\002\020\001'
  _BIGQUERYDATASOURCEARGS.fields_by_name['credentials']._options = None
  _BIGQUERYDATASOURCEARGS.fields_by_name['credentials']._serialized_options = b'\222M\002\010\001'
  _UNITYCATALOGACCESSMODE._serialized_start=5367
  _UNITYCATALOGACCESSMODE._serialized_end=5568
  _DATETIMEPARTITIONCOLUMNARGS._serialized_start=432
  _DATETIMEPARTITIONCOLUMNARGS._serialized_end=592
  _BATCHDATASOURCECOMMONARGS._serialized_start=595
  _BATCHDATASOURCECOMMONARGS._serialized_end=800
  _STREAMDATASOURCECOMMONARGS._serialized_start=803
  _STREAMDATASOURCECOMMONARGS._serialized_end=1105
  _HIVEDATASOURCEARGS._serialized_start=1108
  _HIVEDATASOURCEARGS._serialized_end=1440
  _UNITYDATASOURCEARGS._serialized_start=1443
  _UNITYDATASOURCEARGS._serialized_end=1851
  _FILEDATASOURCEARGS._serialized_start=1854
  _FILEDATASOURCEARGS._serialized_end=2238
  _OPTION._serialized_start=2240
  _OPTION._serialized_end=2295
  _KINESISDATASOURCEARGS._serialized_start=2298
  _KINESISDATASOURCEARGS._serialized_end=2659
  _KAFKADATASOURCEARGS._serialized_start=2662
  _KAFKADATASOURCEARGS._serialized_end=3253
  _PUSHSOURCEARGS._serialized_start=3256
  _PUSHSOURCEARGS._serialized_end=3556
  _REDSHIFTDATASOURCEARGS._serialized_start=3559
  _REDSHIFTDATASOURCEARGS._serialized_end=3780
  _SNOWFLAKEDATASOURCEARGS._serialized_start=3783
  _SNOWFLAKEDATASOURCEARGS._serialized_end=4243
  _SPARKBATCHCONFIGARGS._serialized_start=4246
  _SPARKBATCHCONFIGARGS._serialized_end=4472
  _SPARKSTREAMCONFIGARGS._serialized_start=4474
  _SPARKSTREAMCONFIGARGS._serialized_end=4587
  _PANDASBATCHCONFIGARGS._serialized_start=4590
  _PANDASBATCHCONFIGARGS._serialized_end=5015
  _PANDASBATCHCONFIGARGS_SECRETSENTRY._serialized_start=4907
  _PANDASBATCHCONFIGARGS_SECRETSENTRY._serialized_end=5003
  _BIGQUERYDATASOURCEARGS._serialized_start=5018
  _BIGQUERYDATASOURCEARGS._serialized_end=5364
# @@protoc_insertion_point(module_scope)
