# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/common/time_window__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-tecton_proto/common/time_window__client.proto\x12\x13tecton_proto.common\x1a\x1egoogle/protobuf/duration.proto\"\x9a\x02\n\nTimeWindow\x12[\n\x14relative_time_window\x18\x01 \x01(\x0b\x32\'.tecton_proto.common.RelativeTimeWindowH\x00R\x12relativeTimeWindow\x12N\n\x0flifetime_window\x18\x02 \x01(\x0b\x32#.tecton_proto.common.LifetimeWindowH\x00R\x0elifetimeWindow\x12U\n\x12time_window_series\x18\x03 \x01(\x0b\x32%.tecton_proto.common.TimeWindowSeriesH\x00R\x10timeWindowSeriesB\x08\n\x06window\"^\n\x10TimeWindowSeries\x12J\n\x0ctime_windows\x18\x01 \x03(\x0b\x32\'.tecton_proto.common.RelativeTimeWindowR\x0btimeWindows\"\x8c\x01\n\x12RelativeTimeWindow\x12<\n\x0cwindow_start\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0bwindowStart\x12\x38\n\nwindow_end\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationR\twindowEnd\"\x10\n\x0eLifetimeWindowB\x15\n\x11\x63om.tecton.commonP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.common.time_window__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021com.tecton.commonP\001'
  _TIMEWINDOW._serialized_start=103
  _TIMEWINDOW._serialized_end=385
  _TIMEWINDOWSERIES._serialized_start=387
  _TIMEWINDOWSERIES._serialized_end=481
  _RELATIVETIMEWINDOW._serialized_start=484
  _RELATIVETIMEWINDOW._serialized_end=624
  _LIFETIMEWINDOW._serialized_start=626
  _LIFETIMEWINDOW._serialized_end=642
# @@protoc_insertion_point(module_scope)
