# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/spark_common/libraries__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.spark_common import clusters__client_pb2 as tecton__proto_dot_spark__common_dot_clusters____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n1tecton_proto/spark_common/libraries__client.proto\x12\x19tecton_proto.spark_common\x1a\x30tecton_proto/spark_common/clusters__client.proto\"\xca\x03\n\x07Library\x12\x12\n\x03jar\x18\x01 \x01(\tH\x00R\x03jar\x12\x12\n\x03\x65gg\x18\x02 \x01(\tH\x00R\x03\x65gg\x12\x12\n\x03whl\x18\x03 \x01(\tH\x00R\x03whl\x12P\n\x0cjar_resource\x18\x06 \x01(\x0b\x32+.tecton_proto.spark_common.ResourceLocationH\x00R\x0bjarResource\x12P\n\x0c\x65gg_resource\x18\x07 \x01(\x0b\x32+.tecton_proto.spark_common.ResourceLocationH\x00R\x0b\x65ggResource\x12P\n\x0cwhl_resource\x18\x08 \x01(\x0b\x32+.tecton_proto.spark_common.ResourceLocationH\x00R\x0bwhlResource\x12?\n\x05maven\x18\x04 \x01(\x0b\x32\'.tecton_proto.spark_common.MavenLibraryH\x00R\x05maven\x12<\n\x04pypi\x18\x05 \x01(\x0b\x32&.tecton_proto.spark_common.PyPiLibraryH\x00R\x04pypiB\x0e\n\x0clibrary_type\";\n\x0bPyPiLibrary\x12\x18\n\x07package\x18\x01 \x01(\tR\x07package\x12\x12\n\x04repo\x18\x02 \x01(\tR\x04repo\"d\n\x0cMavenLibrary\x12 \n\x0b\x63oordinates\x18\x01 \x01(\tR\x0b\x63oordinates\x12\x12\n\x04repo\x18\x02 \x01(\tR\x04repo\x12\x1e\n\nexclusions\x18\x03 \x03(\tR\nexclusionsB\x1b\n\x17\x63om.tecton.spark_commonP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.spark_common.libraries__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\027com.tecton.spark_commonP\001'
  _LIBRARY._serialized_start=131
  _LIBRARY._serialized_end=589
  _PYPILIBRARY._serialized_start=591
  _PYPILIBRARY._serialized_end=650
  _MAVENLIBRARY._serialized_start=652
  _MAVENLIBRARY._serialized_end=752
# @@protoc_insertion_point(module_scope)
