# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/common/framework_version__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n3tecton_proto/common/framework_version__client.proto\x12\x13tecton_proto.common*I\n\x10\x46rameworkVersion\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x08\n\x04\x46WV3\x10\x03\x12\x08\n\x04\x46WV5\x10\x05\"\x04\x08\x01\x10\x01\"\x04\x08\x02\x10\x02\"\x04\x08\x04\x10\x04\x42\x15\n\x11\x63om.tecton.commonP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.common.framework_version__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021com.tecton.commonP\001'
  _FRAMEWORKVERSION._serialized_start=76
  _FRAMEWORKVERSION._serialized_end=149
# @@protoc_insertion_point(module_scope)
