# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/common/id__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.auditlog import metadata__client_pb2 as tecton__proto_dot_auditlog_dot_metadata____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$tecton_proto/common/id__client.proto\x12\x13tecton_proto.common\x1a,tecton_proto/auditlog/metadata__client.proto\"\x80\x01\n\x02Id\x12;\n\x15most_significant_bits\x18\x01 \x01(\x06\x42\x07\xea\x99\xbd\x46\x02\x08\x02R\x13mostSignificantBits\x12=\n\x16least_significant_bits\x18\x02 \x01(\x06\x42\x07\xea\x99\xbd\x46\x02\x08\x02R\x14leastSignificantBitsB\x15\n\x11\x63om.tecton.commonP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.common.id__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021com.tecton.commonP\001'
  _ID.fields_by_name['most_significant_bits']._options = None
  _ID.fields_by_name['most_significant_bits']._serialized_options = b'\352\231\275F\002\010\002'
  _ID.fields_by_name['least_significant_bits']._options = None
  _ID.fields_by_name['least_significant_bits']._serialized_options = b'\352\231\275F\002\010\002'
  _ID._serialized_start=108
  _ID._serialized_end=236
# @@protoc_insertion_point(module_scope)
