# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/feature_view__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.args import feature_view__client_pb2 as tecton__proto_dot_args_dot_feature__view____client__pb2
from tecton_proto.args import pipeline__client_pb2 as tecton__proto_dot_args_dot_pipeline____client__pb2
from tecton_proto.common import aggregation_function__client_pb2 as tecton__proto_dot_common_dot_aggregation__function____client__pb2
from tecton_proto.common import compute_mode__client_pb2 as tecton__proto_dot_common_dot_compute__mode____client__pb2
from tecton_proto.common import data_source_type__client_pb2 as tecton__proto_dot_common_dot_data__source__type____client__pb2
from tecton_proto.common import framework_version__client_pb2 as tecton__proto_dot_common_dot_framework__version____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import schema__client_pb2 as tecton__proto_dot_common_dot_schema____client__pb2
from tecton_proto.common import time_window__client_pb2 as tecton__proto_dot_common_dot_time__window____client__pb2
from tecton_proto.data import fco_metadata__client_pb2 as tecton__proto_dot_data_dot_fco__metadata____client__pb2
from tecton_proto.data import fv_materialization__client_pb2 as tecton__proto_dot_data_dot_fv__materialization____client__pb2
from tecton_proto.data import realtime_compute__client_pb2 as tecton__proto_dot_data_dot_realtime__compute____client__pb2
from tecton_proto.modelartifactservice import model_artifact_service__client_pb2 as tecton__proto_dot_modelartifactservice_dot_model__artifact__service____client__pb2
from tecton_proto.validation import validator__client_pb2 as tecton__proto_dot_validation_dot_validator____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,tecton_proto/data/feature_view__client.proto\x12\x11tecton_proto.data\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a,tecton_proto/args/feature_view__client.proto\x1a(tecton_proto/args/pipeline__client.proto\x1a\x36tecton_proto/common/aggregation_function__client.proto\x1a.tecton_proto/common/compute_mode__client.proto\x1a\x32tecton_proto/common/data_source_type__client.proto\x1a\x33tecton_proto/common/framework_version__client.proto\x1a$tecton_proto/common/id__client.proto\x1a(tecton_proto/common/schema__client.proto\x1a-tecton_proto/common/time_window__client.proto\x1a,tecton_proto/data/fco_metadata__client.proto\x1a\x32tecton_proto/data/fv_materialization__client.proto\x1a\x30tecton_proto/data/realtime_compute__client.proto\x1a\x46tecton_proto/modelartifactservice/model_artifact_service__client.proto\x1a/tecton_proto/validation/validator__client.proto\"\xe2\x10\n\x0b\x46\x65\x61tureView\x12?\n\x0f\x66\x65\x61ture_view_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rfeatureViewId\x12\x41\n\x0c\x66\x63o_metadata\x18\x02 \x01(\x0b\x32\x1e.tecton_proto.data.FcoMetadataR\x0b\x66\x63oMetadata\x12\x36\n\nentity_ids\x18\x03 \x03(\x0b\x32\x17.tecton_proto.common.IdR\tentityIds\x12\x1b\n\tjoin_keys\x18\x04 \x03(\tR\x08joinKeys\x12?\n\x07schemas\x18\x0b \x01(\x0b\x32%.tecton_proto.data.FeatureViewSchemasR\x07schemas\x12P\n\x0b\x65nrichments\x18\xe8\x07 \x01(\x0b\x32).tecton_proto.data.FeatureViewEnrichmentsB\x02\x18\x01R\x0b\x65nrichments\x12U\n\x12temporal_aggregate\x18\x07 \x01(\x0b\x32$.tecton_proto.data.TemporalAggregateH\x00R\x11temporalAggregate\x12\x39\n\x08temporal\x18\x08 \x01(\x0b\x32\x1b.tecton_proto.data.TemporalH\x00R\x08temporal\x12\\\n\x15realtime_feature_view\x18\x11 \x01(\x0b\x32&.tecton_proto.data.RealtimeFeatureViewH\x00R\x13realtimeFeatureView\x12\x46\n\rfeature_table\x18\x10 \x01(\x0b\x32\x1f.tecton_proto.data.FeatureTableH\x00R\x0c\x66\x65\x61tureTable\x12#\n\rtimestamp_key\x18\n \x01(\tR\x0ctimestampKey\x12W\n\x14online_serving_index\x18\x05 \x01(\x0b\x32%.tecton_proto.data.OnlineServingIndexR\x12onlineServingIndex\x12\x37\n\x08pipeline\x18\x06 \x01(\x0b\x32\x1b.tecton_proto.args.PipelineR\x08pipeline\x12\x62\n\x16materialization_params\x18\t \x01(\x0b\x32+.tecton_proto.data.NewMaterializationParamsR\x15materializationParams\x12\x37\n\x17materialization_enabled\x18\x0c \x01(\x08R\x16materializationEnabled\x12}\n!materialization_state_transitions\x18\r \x03(\x0b\x32\x31.tecton_proto.data.MaterializationStateTransitionR\x1fmaterializationStateTransitions\x12P\n\x11monitoring_params\x18\x0e \x01(\x0b\x32#.tecton_proto.data.MonitoringParamsR\x10monitoringParams\x12?\n\x1c\x66\x65\x61ture_store_format_version\x18\x17 \x01(\x05R\x19\x66\x65\x61tureStoreFormatVersion\x12G\n\x0esnowflake_data\x18\x18 \x01(\x0b\x32 .tecton_proto.data.SnowflakeDataR\rsnowflakeData\x12/\n\x11\x66ramework_version\x18\x13 \x01(\x05\x42\x02\x18\x01R\x10\x66rameworkVersion\x12H\n\nfw_version\x18\x1a \x01(\x0e\x32%.tecton_proto.common.FrameworkVersionB\x02\x18\x01R\tfwVersion\x12\x17\n\x07web_url\x18\x19 \x01(\tR\x06webUrl\x12H\n\rbatch_trigger\x18\x1b \x01(\x0e\x32#.tecton_proto.args.BatchTriggerTypeR\x0c\x62\x61tchTrigger\x12[\n\x0fvalidation_args\x18\x1c \x01(\x0b\x32\x32.tecton_proto.validation.FeatureViewValidationArgsR\x0evalidationArgs\x12T\n\x13\x64\x61ta_quality_config\x18\x1d \x01(\x0b\x32$.tecton_proto.data.DataQualityConfigR\x11\x64\x61taQualityConfig\x12\x45\n\x07options\x18\x1e \x03(\x0b\x32+.tecton_proto.data.FeatureView.OptionsEntryR\x07options\x12S\n\x12\x62\x61tch_compute_mode\x18\x1f \x01(\x0e\x32%.tecton_proto.common.BatchComputeModeR\x10\x62\x61tchComputeMode\x12L\n\x0c\x63\x61\x63he_config\x18  \x01(\x0b\x32).tecton_proto.data.FeatureViewCacheConfigR\x0b\x63\x61\x63heConfig\x12\x34\n\x16\x63ontext_parameter_name\x18! \x01(\tR\x14\x63ontextParameterName\x1a:\n\x0cOptionsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x13\n\x11\x66\x65\x61ture_view_type\"\xe8\x03\n\x11TemporalAggregate\x12@\n\x0eslide_interval\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationR\rslideInterval\x12\x32\n\x15slide_interval_string\x18\x05 \x01(\tR\x13slideIntervalString\x12?\n\x08\x66\x65\x61tures\x18\x02 \x03(\x0b\x32#.tecton_proto.data.AggregateFeatureR\x08\x66\x65\x61tures\x12#\n\ris_continuous\x18\x03 \x01(\x08R\x0cisContinuous\x12M\n\x10\x64\x61ta_source_type\x18\x04 \x01(\x0e\x32#.tecton_proto.common.DataSourceTypeR\x0e\x64\x61taSourceType\x12:\n\x19\x61ggregation_secondary_key\x18\x06 \x01(\tR\x17\x61ggregationSecondaryKey\x12l\n\x1csecondary_key_output_columns\x18\x07 \x03(\x0b\x32+.tecton_proto.data.SecondaryKeyOutputColumnR\x19secondaryKeyOutputColumns\"\xd8\x03\n\x10\x41ggregateFeature\x12,\n\x12input_feature_name\x18\x01 \x01(\tR\x10inputFeatureName\x12.\n\x13output_feature_name\x18\x02 \x01(\tR\x11outputFeatureName\x12\x44\n\x08\x66unction\x18\x03 \x01(\x0e\x32(.tecton_proto.common.AggregationFunctionR\x08\x66unction\x12W\n\x0f\x66unction_params\x18\x05 \x01(\x0b\x32..tecton_proto.common.AggregationFunctionParamsR\x0e\x66unctionParams\x12\x31\n\x06window\x18\x04 \x01(\x0b\x32\x19.google.protobuf.DurationR\x06window\x12@\n\x0btime_window\x18\x06 \x01(\x0b\x32\x1f.tecton_proto.common.TimeWindowR\ntimeWindow\x12R\n\x18\x62\x61tch_sawtooth_tile_size\x18\x07 \x01(\x0b\x32\x19.google.protobuf.DurationR\x15\x62\x61tchSawtoothTileSize\"\x82\x01\n\x10\x45mbeddingFeature\x12*\n\x11input_column_name\x18\x01 \x01(\tR\x0finputColumnName\x12,\n\x12output_column_name\x18\x02 \x01(\tR\x10outputColumnName\x12\x14\n\x05model\x18\x05 \x01(\tR\x05model\"\xf3\x01\n\x10InferenceFeature\x12@\n\rinput_columns\x18\x01 \x03(\x0b\x32\x1b.tecton_proto.common.ColumnR\x0cinputColumns\x12@\n\routput_column\x18\x02 \x01(\x0b\x32\x1b.tecton_proto.common.ColumnR\x0coutputColumn\x12[\n\x0emodel_artifact\x18\x03 \x01(\x0b\x32\x34.tecton_proto.modelartifactservice.ModelArtifactInfoR\rmodelArtifact\"\xf5\x01\n\x1dTrailingTimeWindowAggregation\x12\x19\n\x08time_key\x18\x01 \x01(\tR\x07timeKey\x12S\n\x18\x61ggregation_slide_period\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationR\x16\x61ggregationSlidePeriod\x12?\n\x08\x66\x65\x61tures\x18\x03 \x03(\x0b\x32#.tecton_proto.data.AggregateFeatureR\x08\x66\x65\x61tures\x12#\n\ris_continuous\x18\x04 \x01(\x08R\x0cisContinuous\"\xc5\x03\n\x08Temporal\x12:\n\x0bserving_ttl\x18\x10 \x01(\x0b\x32\x19.google.protobuf.DurationR\nservingTtl\x12M\n\x10\x64\x61ta_source_type\x18\x11 \x01(\x0e\x32#.tecton_proto.common.DataSourceTypeR\x0e\x64\x61taSourceType\x12J\n\x0f\x62\x61\x63kfill_config\x18\x12 \x01(\x0b\x32!.tecton_proto.args.BackfillConfigR\x0e\x62\x61\x63kfillConfig\x12\x33\n\x15incremental_backfills\x18\x13 \x01(\x08R\x14incrementalBackfills\x12#\n\ris_continuous\x18\x14 \x01(\x08R\x0cisContinuous\x12\x43\n\nembeddings\x18\x15 \x03(\x0b\x32#.tecton_proto.data.EmbeddingFeatureR\nembeddings\x12\x43\n\ninferences\x18\x16 \x03(\x0b\x32#.tecton_proto.data.InferenceFeatureR\ninferences\"\x9a\x01\n\x0c\x46\x65\x61tureTable\x12%\n\x0eonline_enabled\x18\x01 \x01(\x08R\ronlineEnabled\x12\'\n\x0foffline_enabled\x18\x02 \x01(\x08R\x0eofflineEnabled\x12:\n\x0bserving_ttl\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\nservingTtl\"\x97\x01\n\x13RealtimeFeatureView\x12\x13\n\x05no_op\x18\x01 \x01(\x08R\x04noOp\x12\x65\n\x16supported_environments\x18\x03 \x03(\x0b\x32..tecton_proto.data.RemoteFunctionComputeConfigR\x15supportedEnvironmentsJ\x04\x08\x02\x10\x03\"\xc5\x02\n\x12\x46\x65\x61tureViewSchemas\x12<\n\x0bview_schema\x18\x01 \x01(\x0b\x32\x1b.tecton_proto.common.SchemaR\nviewSchema\x12\x35\n\x17is_explicit_view_schema\x18\x04 \x01(\x08R\x14isExplicitViewSchema\x12R\n\x16materialization_schema\x18\x02 \x01(\x0b\x32\x1b.tecton_proto.common.SchemaR\x15materializationSchema\x12\x66\n\x19online_batch_table_format\x18\x03 \x01(\x0b\x32+.tecton_proto.common.OnlineBatchTableFormatR\x16onlineBatchTableFormat\"1\n\x12OnlineServingIndex\x12\x1b\n\tjoin_keys\x18\x01 \x03(\tR\x08joinKeys\"\xb3\x03\n\x1eMaterializationStateTransition\x12\x38\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp\x12%\n\x0eonline_enabled\x18\x02 \x01(\x08R\ronlineEnabled\x12\'\n\x0foffline_enabled\x18\x03 \x01(\x08R\x0eofflineEnabled\x12R\n\x17\x66\x65\x61ture_start_timestamp\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x15\x66\x65\x61tureStartTimestamp\x12\x44\n\x1ematerialization_serial_version\x18\x05 \x01(\x05R\x1cmaterializationSerialVersion\x12\x37\n\x18\x66orce_stream_job_restart\x18\x06 \x01(\x08R\x15\x66orceStreamJobRestart\x12\x34\n\x16tecton_runtime_version\x18\x07 \x01(\tR\x14tectonRuntimeVersion\"\xaf\x01\n\x19ParquetOfflineStoreParams\x12I\n\x13time_partition_size\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationR\x11timePartitionSize\x12G\n\x07version\x18\x02 \x01(\x0e\x32-.tecton_proto.data.ParquetOfflineStoreVersionR\x07version\"\xab\x01\n\x17\x44\x65ltaOfflineStoreParams\x12I\n\x13time_partition_size\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationR\x11timePartitionSize\x12\x45\n\x07version\x18\x02 \x01(\x0e\x32+.tecton_proto.data.DeltaOfflineStoreVersionR\x07version\"\xac\x01\n\x12OfflineStoreParams\x12H\n\x07parquet\x18\x01 \x01(\x0b\x32,.tecton_proto.data.ParquetOfflineStoreParamsH\x00R\x07parquet\x12\x42\n\x05\x64\x65lta\x18\x02 \x01(\x0b\x32*.tecton_proto.data.DeltaOfflineStoreParamsH\x00R\x05\x64\x65ltaB\x08\n\x06params\"\xa0\x01\n FeaturePublishOfflineStoreConfig\x12\x32\n\x15publish_full_features\x18\x01 \x01(\x08R\x13publishFullFeatures\x12H\n\x12publish_start_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x10publishStartTime\"\xf7\r\n\x18NewMaterializationParams\x12\x46\n\x11schedule_interval\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationR\x10scheduleInterval\x12\x62\n\x1fmaterialization_start_timestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x1dmaterializationStartTimestamp\x12R\n\x17\x66\x65\x61ture_start_timestamp\x18\r \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x15\x66\x65\x61tureStartTimestamp\x12l\n%manual_trigger_backfill_end_timestamp\x18\x0f \x01(\x0b\x32\x1a.google.protobuf.TimestampR!manualTriggerBackfillEndTimestamp\x12M\n\x15max_backfill_interval\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\x13maxBackfillInterval\x12\x33\n\x16writes_to_online_store\x18\x04 \x01(\x08R\x13writesToOnlineStore\x12\x35\n\x17writes_to_offline_store\x18\x05 \x01(\x08R\x14writesToOfflineStore\x12^\n\x14offline_store_config\x18\x06 \x01(\x0b\x32,.tecton_proto.args.OfflineFeatureStoreConfigR\x12offlineStoreConfig\x12W\n\x14offline_store_params\x18\x0e \x01(\x0b\x32%.tecton_proto.data.OfflineStoreParamsR\x12offlineStoreParams\x12U\n\x15\x62\x61tch_materialization\x18\x07 \x01(\x0b\x32 .tecton_proto.args.ClusterConfigR\x14\x62\x61tchMaterialization\x12W\n\x16stream_materialization\x18\x08 \x01(\x0b\x32 .tecton_proto.args.ClusterConfigR\x15streamMaterialization\x12L\n\x15max_source_data_delay\x18\t \x01(\x0b\x32\x19.google.protobuf.DurationR\x12maxSourceDataDelay\x12T\n\x13online_store_params\x18\n \x01(\x0b\x32$.tecton_proto.data.OnlineStoreParamsR\x11onlineStoreParams\x12\x44\n\routput_stream\x18\x0b \x01(\x0b\x32\x1f.tecton_proto.args.OutputStreamR\x0coutputStream\x12]\n\x11time_range_policy\x18\x0c \x01(\x0e\x32\x31.tecton_proto.data.MaterializationTimeRangePolicyR\x0ftimeRangePolicy\x12\x64\n\x19online_backfill_load_type\x18\x10 \x01(\x0e\x32).tecton_proto.data.OnlineBackfillLoadTypeR\x16onlineBackfillLoadType\x12\x44\n\x1etecton_materialization_runtime\x18\x11 \x01(\tR\x1ctectonMaterializationRuntime\x12\x83\x01\n$feature_publish_offline_store_config\x18\x13 \x01(\x0b\x32\x33.tecton_proto.data.FeaturePublishOfflineStoreConfigR featurePublishOfflineStoreConfig\x12-\n\x12\x63ompaction_enabled\x18\x14 \x01(\x08R\x11\x63ompactionEnabled\x12\x32\n\x15stream_tiling_enabled\x18\x17 \x01(\x08R\x13streamTilingEnabled\x12 \n\x0b\x65nvironment\x18\x15 \x01(\tR\x0b\x65nvironment\x12\x43\n\x10stream_tile_size\x18\x16 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0estreamTileSizeJ\x04\x08\x12\x10\x13\"\xec\x01\n\x11OnlineStoreParams\x12@\n\x06\x64ynamo\x18\x02 \x01(\x0b\x32&.tecton_proto.data.DynamoDbOnlineStoreH\x00R\x06\x64ynamo\x12;\n\x05redis\x18\x03 \x01(\x0b\x32#.tecton_proto.data.RedisOnlineStoreH\x00R\x05redis\x12\x44\n\x08\x62igtable\x18\x04 \x01(\x0b\x32&.tecton_proto.data.BigtableOnlineStoreH\x00R\x08\x62igtableB\x0c\n\nstore_typeJ\x04\x08\x01\x10\x02\"\xa1\x02\n\x13\x44ynamoDbOnlineStore\x12\x33\n\x16\x63ross_account_role_arn\x18\x01 \x01(\tR\x13\x63rossAccountRoleArn\x12\x39\n\x19\x63ross_account_external_id\x18\x02 \x01(\tR\x16\x63rossAccountExternalId\x12L\n#cross_account_intermediate_role_arn\x18\x05 \x01(\tR\x1f\x63rossAccountIntermediateRoleArn\x12\x18\n\x07\x65nabled\x18\x03 \x01(\x08R\x07\x65nabled\x12\x32\n\x15\x64\x62\x66s_credentials_path\x18\x04 \x01(\tR\x13\x64\x62\x66sCredentialsPath\"\xeb\x01\n\x10RedisOnlineStore\x12)\n\x10primary_endpoint\x18\x02 \x01(\tR\x0fprimaryEndpoint\x12\x31\n\x14\x61uthentication_token\x18\x04 \x01(\tR\x13\x61uthenticationToken\x12\x1f\n\x0btls_enabled\x18\x06 \x01(\x08R\ntlsEnabled\x12\x18\n\x07\x65nabled\x18\x05 \x01(\x08R\x07\x65nabled\x12&\n\x0finject_host_sni\x18\t \x01(\x08R\rinjectHostSniJ\x04\x08\x01\x10\x02J\x04\x08\x03\x10\x04J\x04\x08\x07\x10\x08J\x04\x08\x08\x10\t\"o\n\x13\x42igtableOnlineStore\x12\x18\n\x07\x65nabled\x18\x01 \x01(\x08R\x07\x65nabled\x12\x1d\n\nproject_id\x18\x02 \x01(\tR\tprojectId\x12\x1f\n\x0binstance_id\x18\x03 \x01(\tR\ninstanceId\"\x92\x02\n\x10MonitoringParams\x12%\n\x0euser_specified\x18\x01 \x01(\x08R\ruserSpecified\x12+\n\x11monitor_freshness\x18\x02 \x01(\x08R\x10monitorFreshness\x12W\n\x1a\x65xpected_feature_freshness\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\x18\x65xpectedFeatureFreshness\x12\x1f\n\x0b\x61lert_email\x18\x04 \x01(\tR\nalertEmail\x12\x30\n\x14grace_period_seconds\x18\x05 \x01(\x05R\x12gracePeriodSeconds\"\x85\x01\n\x16\x46\x65\x61tureViewEnrichments\x12S\n\x12\x66p_materialization\x18\x04 \x01(\x0b\x32$.tecton_proto.data.FvMaterializationR\x11\x66pMaterializationJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04J\x04\x08\x05\x10\x06\"?\n\rSnowflakeData\x12.\n\x13snowflake_view_name\x18\x01 \x01(\tR\x11snowflakeViewName\"\x81\x01\n\x11\x44\x61taQualityConfig\x12\x30\n\x14\x64\x61ta_quality_enabled\x18\x01 \x01(\x08R\x12\x64\x61taQualityEnabled\x12:\n\x19skip_default_expectations\x18\x02 \x01(\x08R\x17skipDefaultExpectations\"p\n\x18SecondaryKeyOutputColumn\x12@\n\x0btime_window\x18\x01 \x01(\x0b\x32\x1f.tecton_proto.common.TimeWindowR\ntimeWindow\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\"\x92\x02\n\x16\x46\x65\x61tureViewCacheConfig\x12\x1c\n\tnamespace\x18\x01 \x01(\tR\tnamespace\x12(\n\x10\x63\x61\x63he_group_name\x18\x02 \x01(\tR\x0e\x63\x61\x63heGroupName\x12\x41\n\x0fmax_age_seconds\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\rmaxAgeSeconds\x12?\n\x0emax_age_jitter\x18\x04 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0cmaxAgeJitter\x12,\n\x12remapped_join_keys\x18\x05 \x03(\tR\x10remappedJoinKeys*\x95\x01\n\x1aParquetOfflineStoreVersion\x12-\n)PARQUET_OFFLINE_STORE_VERSION_UNSPECIFIED\x10\x00\x12#\n\x1fPARQUET_OFFLINE_STORE_VERSION_1\x10\x01\x12#\n\x1fPARQUET_OFFLINE_STORE_VERSION_2\x10\x02*\x8d\x01\n\x18\x44\x65ltaOfflineStoreVersion\x12+\n\'DELTA_OFFLINE_STORE_VERSION_UNSPECIFIED\x10\x00\x12!\n\x1d\x44\x45LTA_OFFLINE_STORE_VERSION_1\x10\x01\x12!\n\x1d\x44\x45LTA_OFFLINE_STORE_VERSION_2\x10\x02*\xc6\x01\n\x1eMaterializationTimeRangePolicy\x12\x31\n-MATERIALIZATION_TIME_RANGE_POLICY_UNSPECIFIED\x10\x00\x12:\n6MATERIALIZATION_TIME_RANGE_POLICY_FAIL_IF_OUT_OF_RANGE\x10\x01\x12\x35\n1MATERIALIZATION_TIME_RANGE_POLICY_FILTER_TO_RANGE\x10\x02\x42\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.feature_view__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _FEATUREVIEW_OPTIONSENTRY._options = None
  _FEATUREVIEW_OPTIONSENTRY._serialized_options = b'8\001'
  _FEATUREVIEW.fields_by_name['enrichments']._options = None
  _FEATUREVIEW.fields_by_name['enrichments']._serialized_options = b'\030\001'
  _FEATUREVIEW.fields_by_name['framework_version']._options = None
  _FEATUREVIEW.fields_by_name['framework_version']._serialized_options = b'\030\001'
  _FEATUREVIEW.fields_by_name['fw_version']._options = None
  _FEATUREVIEW.fields_by_name['fw_version']._serialized_options = b'\030\001'
  _PARQUETOFFLINESTOREVERSION._serialized_start=10511
  _PARQUETOFFLINESTOREVERSION._serialized_end=10660
  _DELTAOFFLINESTOREVERSION._serialized_start=10663
  _DELTAOFFLINESTOREVERSION._serialized_end=10804
  _MATERIALIZATIONTIMERANGEPOLICY._serialized_start=10807
  _MATERIALIZATIONTIMERANGEPOLICY._serialized_end=11005
  _FEATUREVIEW._serialized_start=826
  _FEATUREVIEW._serialized_end=2972
  _FEATUREVIEW_OPTIONSENTRY._serialized_start=2893
  _FEATUREVIEW_OPTIONSENTRY._serialized_end=2951
  _TEMPORALAGGREGATE._serialized_start=2975
  _TEMPORALAGGREGATE._serialized_end=3463
  _AGGREGATEFEATURE._serialized_start=3466
  _AGGREGATEFEATURE._serialized_end=3938
  _EMBEDDINGFEATURE._serialized_start=3941
  _EMBEDDINGFEATURE._serialized_end=4071
  _INFERENCEFEATURE._serialized_start=4074
  _INFERENCEFEATURE._serialized_end=4317
  _TRAILINGTIMEWINDOWAGGREGATION._serialized_start=4320
  _TRAILINGTIMEWINDOWAGGREGATION._serialized_end=4565
  _TEMPORAL._serialized_start=4568
  _TEMPORAL._serialized_end=5021
  _FEATURETABLE._serialized_start=5024
  _FEATURETABLE._serialized_end=5178
  _REALTIMEFEATUREVIEW._serialized_start=5181
  _REALTIMEFEATUREVIEW._serialized_end=5332
  _FEATUREVIEWSCHEMAS._serialized_start=5335
  _FEATUREVIEWSCHEMAS._serialized_end=5660
  _ONLINESERVINGINDEX._serialized_start=5662
  _ONLINESERVINGINDEX._serialized_end=5711
  _MATERIALIZATIONSTATETRANSITION._serialized_start=5714
  _MATERIALIZATIONSTATETRANSITION._serialized_end=6149
  _PARQUETOFFLINESTOREPARAMS._serialized_start=6152
  _PARQUETOFFLINESTOREPARAMS._serialized_end=6327
  _DELTAOFFLINESTOREPARAMS._serialized_start=6330
  _DELTAOFFLINESTOREPARAMS._serialized_end=6501
  _OFFLINESTOREPARAMS._serialized_start=6504
  _OFFLINESTOREPARAMS._serialized_end=6676
  _FEATUREPUBLISHOFFLINESTORECONFIG._serialized_start=6679
  _FEATUREPUBLISHOFFLINESTORECONFIG._serialized_end=6839
  _NEWMATERIALIZATIONPARAMS._serialized_start=6842
  _NEWMATERIALIZATIONPARAMS._serialized_end=8625
  _ONLINESTOREPARAMS._serialized_start=8628
  _ONLINESTOREPARAMS._serialized_end=8864
  _DYNAMODBONLINESTORE._serialized_start=8867
  _DYNAMODBONLINESTORE._serialized_end=9156
  _REDISONLINESTORE._serialized_start=9159
  _REDISONLINESTORE._serialized_end=9394
  _BIGTABLEONLINESTORE._serialized_start=9396
  _BIGTABLEONLINESTORE._serialized_end=9507
  _MONITORINGPARAMS._serialized_start=9510
  _MONITORINGPARAMS._serialized_end=9784
  _FEATUREVIEWENRICHMENTS._serialized_start=9787
  _FEATUREVIEWENRICHMENTS._serialized_end=9920
  _SNOWFLAKEDATA._serialized_start=9922
  _SNOWFLAKEDATA._serialized_end=9985
  _DATAQUALITYCONFIG._serialized_start=9988
  _DATAQUALITYCONFIG._serialized_end=10117
  _SECONDARYKEYOUTPUTCOLUMN._serialized_start=10119
  _SECONDARYKEYOUTPUTCOLUMN._serialized_end=10231
  _FEATUREVIEWCACHECONFIG._serialized_start=10234
  _FEATUREVIEWCACHECONFIG._serialized_end=10508
# @@protoc_insertion_point(module_scope)
