# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/common/data_type__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+tecton_proto/common/data_type__client.proto\x12\x13tecton_proto.common\"\xdb\x02\n\x08\x44\x61taType\x12\x35\n\x04type\x18\x01 \x01(\x0e\x32!.tecton_proto.common.DataTypeEnumR\x04type\x12K\n\x12\x61rray_element_type\x18\x02 \x01(\x0b\x32\x1d.tecton_proto.common.DataTypeR\x10\x61rrayElementType\x12\x45\n\rstruct_fields\x18\x03 \x03(\x0b\x32 .tecton_proto.common.StructFieldR\x0cstructFields\x12?\n\x0cmap_key_type\x18\x04 \x01(\x0b\x32\x1d.tecton_proto.common.DataTypeR\nmapKeyType\x12\x43\n\x0emap_value_type\x18\x05 \x01(\x0b\x32\x1d.tecton_proto.common.DataTypeR\x0cmapValueType\"]\n\x0bStructField\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12:\n\tdata_type\x18\x02 \x01(\x0b\x32\x1d.tecton_proto.common.DataTypeR\x08\x64\x61taType*\x9d\x02\n\x0c\x44\x61taTypeEnum\x12\x15\n\x11\x44\x41TA_TYPE_UNKNOWN\x10\x00\x12\x13\n\x0f\x44\x41TA_TYPE_INT32\x10\x01\x12\x13\n\x0f\x44\x41TA_TYPE_INT64\x10\x02\x12\x15\n\x11\x44\x41TA_TYPE_FLOAT32\x10\x03\x12\x15\n\x11\x44\x41TA_TYPE_FLOAT64\x10\x04\x12\x14\n\x10\x44\x41TA_TYPE_STRING\x10\x05\x12\x12\n\x0e\x44\x41TA_TYPE_BOOL\x10\x06\x12\x17\n\x13\x44\x41TA_TYPE_TIMESTAMP\x10\x07\x12\x13\n\x0f\x44\x41TA_TYPE_ARRAY\x10\x08\x12\x14\n\x10\x44\x41TA_TYPE_STRUCT\x10\t\x12\x1d\n\x19\x44\x41TA_TYPE_TIMESTAMP_INT96\x10\n\x12\x11\n\rDATA_TYPE_MAP\x10\x0b\x42\x15\n\x11\x63om.tecton.commonP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.common.data_type__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021com.tecton.commonP\001'
  _DATATYPEENUM._serialized_start=514
  _DATATYPEENUM._serialized_end=799
  _DATATYPE._serialized_start=69
  _DATATYPE._serialized_end=416
  _STRUCTFIELD._serialized_start=418
  _STRUCTFIELD._serialized_end=511
# @@protoc_insertion_point(module_scope)
