# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/saved_feature_data_frame__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import schema__client_pb2 as tecton__proto_dot_common_dot_schema____client__pb2
from tecton_proto.common import spark_schema__client_pb2 as tecton__proto_dot_common_dot_spark__schema____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n8tecton_proto/data/saved_feature_data_frame__client.proto\x12\x11tecton_proto.data\x1a\x1fgoogle/protobuf/timestamp.proto\x1a$tecton_proto/common/id__client.proto\x1a(tecton_proto/common/schema__client.proto\x1a.tecton_proto/common/spark_schema__client.proto\"\x95\x02\n\x19SavedFeatureDataFrameInfo\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x39\n\ncreated_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x1f\n\x0bis_archived\x18\x03 \x01(\x08R\nisArchived\x12\x1c\n\tworkspace\x18\x04 \x01(\tR\tworkspace\x12&\n\x0fis_data_deleted\x18\x05 \x01(\x08R\risDataDeleted\x12\x42\n\x0f\x64\x61ta_deleted_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\rdataDeletedAt\"\xda\x08\n\x15SavedFeatureDataFrame\x12T\n\x1asaved_feature_dataframe_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x17savedFeatureDataframeId\x12@\n\x04info\x18\x02 \x01(\x0b\x32,.tecton_proto.data.SavedFeatureDataFrameInfoR\x04info\x12-\n\x12\x64\x61taframe_location\x18\x03 \x01(\tR\x11\x64\x61taframeLocation\x12G\n\x12\x66\x65\x61ture_package_id\x18\x04 \x01(\x0b\x32\x17.tecton_proto.common.IdH\x00R\x10\x66\x65\x61turePackageId\x12G\n\x12\x66\x65\x61ture_service_id\x18\x05 \x01(\x0b\x32\x17.tecton_proto.common.IdH\x00R\x10\x66\x65\x61tureServiceId\x12\x32\n\x14\x66\x65\x61ture_package_name\x18\x06 \x01(\tH\x01R\x12\x66\x65\x61turePackageName\x12\x32\n\x14\x66\x65\x61ture_service_name\x18\x07 \x01(\tH\x01R\x12\x66\x65\x61tureServiceName\x12\x42\n\x1cstate_update_entry_commit_id\x18\x08 \x01(\tB\x02\x18\x01R\x18stateUpdateEntryCommitId\x12\x35\n\x15join_key_column_names\x18\t \x03(\tB\x02\x18\x01R\x12joinKeyColumnNames\x12\x36\n\x15timestamp_column_name\x18\n \x01(\tB\x02\x18\x01R\x13timestampColumnName\x12<\n\x06schema\x18\x0b \x01(\x0b\x32 .tecton_proto.common.SparkSchemaB\x02\x18\x01R\x06schema\x12\x44\n\x04type\x18\x0c \x01(\x0e\x32,.tecton_proto.data.SavedFeatureDataFrameTypeB\x02\x18\x01R\x04type\x12\x46\n\x06\x66ormat\x18\r \x01(\x0e\x32..tecton_proto.data.SavedFeatureDataFrameFormatR\x06\x66ormat\x12\x42\n\x0eunified_schema\x18\x0e \x01(\x0b\x32\x1b.tecton_proto.common.SchemaR\runifiedSchema\x12\x46\n\rsaved_dataset\x18\x0f \x01(\x0b\x32\x1f.tecton_proto.data.SavedDatasetH\x02R\x0csavedDataset\x12I\n\x0elogged_dataset\x18\x10 \x01(\x0b\x32 .tecton_proto.data.LoggedDatasetH\x02R\rloggedDatasetB\x0f\n\rsource_fco_idB\x11\n\x0fsource_fco_nameB\x06\n\x04kind\"\xb3\x01\n\x0cSavedDataset\x12\x41\n\x10\x63reation_task_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x0e\x63reationTaskId\x12,\n\x12\x65vent_column_names\x18\x02 \x03(\tR\x10\x65ventColumnNames\x12\x32\n\x15timestamp_column_name\x18\x03 \x01(\tR\x13timestampColumnName\"v\n\rLoggedDataset\x12\x31\n\x15join_key_column_names\x18\x01 \x03(\tR\x12joinKeyColumnNames\x12\x32\n\x15timestamp_column_name\x18\x02 \x01(\tR\x13timestampColumnName*?\n\x19SavedFeatureDataFrameType\x12\x0b\n\x07NOT_SET\x10\x00\x12\t\n\x05SAVED\x10\x01\x12\n\n\x06LOGGED\x10\x02*\xaf\x01\n\x1bSavedFeatureDataFrameFormat\x12/\n+SAVED_FEATURED_DATAFRAME_FORMAT_UNSPECIFIED\x10\x00\x12*\n&SAVED_FEATURED_DATAFRAME_FORMAT_LEGACY\x10\x01\x12\x33\n/SAVED_FEATURED_DATAFRAME_FORMAT_UNIFIED_STORAGE\x10\x02\x42\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.saved_feature_data_frame__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _SAVEDFEATUREDATAFRAME.fields_by_name['state_update_entry_commit_id']._options = None
  _SAVEDFEATUREDATAFRAME.fields_by_name['state_update_entry_commit_id']._serialized_options = b'\030\001'
  _SAVEDFEATUREDATAFRAME.fields_by_name['join_key_column_names']._options = None
  _SAVEDFEATUREDATAFRAME.fields_by_name['join_key_column_names']._serialized_options = b'\030\001'
  _SAVEDFEATUREDATAFRAME.fields_by_name['timestamp_column_name']._options = None
  _SAVEDFEATUREDATAFRAME.fields_by_name['timestamp_column_name']._serialized_options = b'\030\001'
  _SAVEDFEATUREDATAFRAME.fields_by_name['schema']._options = None
  _SAVEDFEATUREDATAFRAME.fields_by_name['schema']._serialized_options = b'\030\001'
  _SAVEDFEATUREDATAFRAME.fields_by_name['type']._options = None
  _SAVEDFEATUREDATAFRAME.fields_by_name['type']._serialized_options = b'\030\001'
  _SAVEDFEATUREDATAFRAMETYPE._serialized_start=1939
  _SAVEDFEATUREDATAFRAMETYPE._serialized_end=2002
  _SAVEDFEATUREDATAFRAMEFORMAT._serialized_start=2005
  _SAVEDFEATUREDATAFRAMEFORMAT._serialized_end=2180
  _SAVEDFEATUREDATAFRAMEINFO._serialized_start=241
  _SAVEDFEATUREDATAFRAMEINFO._serialized_end=518
  _SAVEDFEATUREDATAFRAME._serialized_start=521
  _SAVEDFEATUREDATAFRAME._serialized_end=1635
  _SAVEDDATASET._serialized_start=1638
  _SAVEDDATASET._serialized_end=1817
  _LOGGEDDATASET._serialized_start=1819
  _LOGGEDDATASET._serialized_end=1937
# @@protoc_insertion_point(module_scope)
