# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/feature_analytics/feature_analytics__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.common import fco_locator__client_pb2 as tecton__proto_dot_common_dot_fco__locator____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n>tecton_proto/feature_analytics/feature_analytics__client.proto\x12\x1etecton_proto.feature_analytics\x1a\x1fgoogle/protobuf/timestamp.proto\x1a-tecton_proto/common/fco_locator__client.proto\x1a$tecton_proto/common/id__client.proto\"\x8e\x03\n\x19\x46\x65\x61tureViewSimilarityPair\x12K\n\x11\x66v_locator_second\x18\x01 \x01(\x0b\x32\x1f.tecton_proto.common.FcoLocatorR\x0f\x66vLocatorSecond\x12R\n\x19workspace_state_id_second\x18\x02 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x16workspaceStateIdSecond\x12\x1e\n\nsimilarity\x18\x03 \x01(\x02R\nsimilarity\x12M\n\x04type\x18\x04 \x01(\x0e\x32\x39.tecton_proto.feature_analytics.FeatureViewSimilarityTypeR\x04type\x12*\n\x11\x66v_details_second\x18\x05 \x01(\tR\x0f\x66vDetailsSecond\x12\x35\n\x16similarity_description\x18\x06 \x01(\tR\x15similarityDescription\"\xb4\x02\n\x1a\x46\x65\x61tureViewSimilarityGroup\x12I\n\x10\x66v_locator_first\x18\x01 \x01(\x0b\x32\x1f.tecton_proto.common.FcoLocatorR\x0e\x66vLocatorFirst\x12P\n\x18workspace_state_id_first\x18\x02 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x15workspaceStateIdFirst\x12(\n\x10\x66v_details_first\x18\x03 \x01(\tR\x0e\x66vDetailsFirst\x12O\n\x05pairs\x18\x04 \x03(\x0b\x32\x39.tecton_proto.feature_analytics.FeatureViewSimilarityPairR\x05pairs\"\xbb\x01\n\x1f\x46\x65\x61tureSimilarityAnalysisResult\x12?\n\ranalysis_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0c\x61nalysisTime\x12W\n\tfv_groups\x18\x02 \x03(\x0b\x32:.tecton_proto.feature_analytics.FeatureViewSimilarityGroupR\x08\x66vGroups*\x93\x01\n\x19\x46\x65\x61tureViewSimilarityType\x12#\n\x1f\x46\x45\x41TURE_VIEW_SIMILARITY_UNKNOWN\x10\x00\x12%\n!FEATURE_VIEW_SIMILARITY_DUPLICATE\x10\x01\x12*\n&FEATURE_VIEW_SIMILARITY_NEAR_DUPLICATE\x10\x02\x42 \n\x1c\x63om.tecton.feature_analyticsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.feature_analytics.feature_analytics__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\034com.tecton.feature_analyticsP\001'
  _FEATUREVIEWSIMILARITYTYPE._serialized_start=1119
  _FEATUREVIEWSIMILARITYTYPE._serialized_end=1266
  _FEATUREVIEWSIMILARITYPAIR._serialized_start=217
  _FEATUREVIEWSIMILARITYPAIR._serialized_end=615
  _FEATUREVIEWSIMILARITYGROUP._serialized_start=618
  _FEATUREVIEWSIMILARITYGROUP._serialized_end=926
  _FEATURESIMILARITYANALYSISRESULT._serialized_start=929
  _FEATURESIMILARITYANALYSISRESULT._serialized_end=1116
# @@protoc_insertion_point(module_scope)
