# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/auditlog/metadata__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,tecton_proto/auditlog/metadata__client.proto\x12\x15tecton_proto.auditlog\x1a google/protobuf/descriptor.proto\"\x96\x01\n\x10\x41uditLogMetadata\x12+\n\x12write_to_audit_log\x18\x01 \x01(\x08R\x0fwriteToAuditLog\x12;\n\x1a\x63ustomer_facing_event_name\x18\x02 \x01(\tR\x17\x63ustomerFacingEventName\x12\x18\n\x07version\x18\x03 \x01(\tR\x07version\"\x9c\x01\n\x0f\x41uditLogOptions\x12\x41\n\nvisibility\x18\x01 \x01(\x0e\x32!.tecton_proto.auditlog.VisibilityR\nvisibility\x12\x46\n\ttransform\x18\x02 \x01(\x0e\x32(.tecton_proto.auditlog.AuditLogTransformR\ttransform*U\n\nVisibility\x12\x1a\n\x16VISIBILITY_UNSPECIFIED\x10\x00\x12\x13\n\x0fVISIBILITY_OMIT\x10\x01\x12\x16\n\x12VISIBILITY_VISIBLE\x10\x02*g\n\x11\x41uditLogTransform\x12\x19\n\x15TRANSFORM_UNSPECIFIED\x10\x00\x12\x1a\n\x16TRANSFORM_LEGACY_ROLES\x10\x01\x12\x1b\n\x17TRANSFORM_REDACT_STRING\x10\x02:b\n\x07options\x12\x1d.google.protobuf.FieldOptions\x18\x9d\xd3\xe7\x08 \x01(\x0b\x32&.tecton_proto.auditlog.AuditLogOptionsR\x07options:x\n\x12\x61udit_log_metadata\x12\x1e.google.protobuf.MethodOptions\x18\x9e\xd3\xe7\x08 \x01(\x0b\x32\'.tecton_proto.auditlog.AuditLogMetadataR\x10\x61uditLogMetadataBC\n\x13\x63om.tecton.auditlogP\x01Z*github.com/tecton-ai/tecton_proto/auditlog')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.auditlog.metadata__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(options)
  google_dot_protobuf_dot_descriptor__pb2.MethodOptions.RegisterExtension(audit_log_metadata)

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023com.tecton.auditlogP\001Z*github.com/tecton-ai/tecton_proto/auditlog'
  _VISIBILITY._serialized_start=417
  _VISIBILITY._serialized_end=502
  _AUDITLOGTRANSFORM._serialized_start=504
  _AUDITLOGTRANSFORM._serialized_end=607
  _AUDITLOGMETADATA._serialized_start=106
  _AUDITLOGMETADATA._serialized_end=256
  _AUDITLOGOPTIONS._serialized_start=259
  _AUDITLOGOPTIONS._serialized_end=415
# @@protoc_insertion_point(module_scope)
