# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/remoteenvironmentservice/remote_environment_service__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.auth import principal__client_pb2 as tecton__proto_dot_auth_dot_principal____client__pb2
from tecton_proto.auth import service__client_pb2 as tecton__proto_dot_auth_dot_service____client__pb2
from tecton_proto.common import container_image__client_pb2 as tecton__proto_dot_common_dot_container__image____client__pb2
from tecton_proto.data import remote_compute_environment__client_pb2 as tecton__proto_dot_data_dot_remote__compute__environment____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nNtecton_proto/remoteenvironmentservice/remote_environment_service__client.proto\x12%tecton_proto.remoteenvironmentservice\x1a)tecton_proto/auth/principal__client.proto\x1a\'tecton_proto/auth/service__client.proto\x1a\x31tecton_proto/common/container_image__client.proto\x1a:tecton_proto/data/remote_compute_environment__client.proto\"\xb5\x04\n\x1e\x43reateRemoteEnvironmentRequest\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12 \n\x0b\x64\x65scription\x18\x02 \x01(\tR\x0b\x64\x65scription\x12\x42\n\nimage_info\x18\x03 \x01(\x0b\x32#.tecton_proto.common.ContainerImageR\timageInfo\x12\x0e\n\x02id\x18\x04 \x01(\tR\x02id\x12%\n\x0epython_version\x18\x05 \x01(\tR\rpythonVersion\x12\"\n\x0crequirements\x18\x06 \x01(\tR\x0crequirements\x12\x33\n\x15resolved_requirements\x18\x07 \x01(\tR\x14resolvedRequirements\x12,\n\x12s3_wheels_location\x18\x0e \x01(\tR\x10s3WheelsLocation\x12:\n\x19transform_runtime_version\x18\x0f \x01(\tR\x17transformRuntimeVersion\x12O\n$rift_materialization_runtime_version\x18\x10 \x01(\tR!riftMaterializationRuntimeVersion\x12\x1f\n\x0bsdk_version\x18\x11 \x01(\tR\nsdkVersion\x12-\n\x12online_provisioned\x18\x12 \x01(\x08R\x11onlineProvisioned\"}\n\x1f\x43reateRemoteEnvironmentResponse\x12Z\n\x12remote_environment\x18\x01 \x01(\x0b\x32+.tecton_proto.data.RemoteComputeEnvironmentR\x11remoteEnvironment\"\xac\x01\n\x1eUpdateRemoteEnvironmentRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x36\n\x17remote_function_version\x18\x02 \x01(\tR\x15remoteFunctionVersion\x12\x42\n\x06status\x18\x03 \x01(\x0e\x32*.tecton_proto.data.RemoteEnvironmentStatusR\x06status\"}\n\x1fUpdateRemoteEnvironmentResponse\x12Z\n\x12remote_environment\x18\x01 \x01(\x0b\x32+.tecton_proto.data.RemoteComputeEnvironmentR\x11remoteEnvironment\"3\n\x1f\x44\x65leteRemoteEnvironmentsRequest\x12\x10\n\x03ids\x18\x01 \x03(\tR\x03ids\"\"\n DeleteRemoteEnvironmentsResponse\"\x1f\n\x1dListRemoteEnvironmentsRequest\"~\n\x1eListRemoteEnvironmentsResponse\x12\\\n\x13remote_environments\x18\x01 \x03(\x0b\x32+.tecton_proto.data.RemoteComputeEnvironmentR\x12remoteEnvironments\"-\n\x1bGetRemoteEnvironmentRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"z\n\x1cGetRemoteEnvironmentResponse\x12Z\n\x12remote_environment\x18\x01 \x01(\x0b\x32+.tecton_proto.data.RemoteComputeEnvironmentR\x11remoteEnvironment\"\x8f\x01\n\x1bGetPackagesUploadUrlRequest\x12%\n\x0e\x65nvironment_id\x18\x01 \x01(\tR\renvironmentId\x12I\n\x0bupload_part\x18\x02 \x01(\x0b\x32(.tecton_proto.data.ObjectStoreUploadPartR\nuploadPart\"=\n\x1cGetPackagesUploadUrlResponse\x12\x1d\n\nupload_url\x18\x01 \x01(\tR\tuploadUrl\"C\n\x1aStartPackagesUploadRequest\x12%\n\x0e\x65nvironment_id\x18\x01 \x01(\tR\renvironmentId\"n\n\x1bStartPackagesUploadResponse\x12O\n\x0bupload_info\x18\x01 \x01(\x0b\x32..tecton_proto.data.RemoteEnvironmentUploadInfoR\nuploadInfo\"p\n\x1d\x43ompletePackagesUploadRequest\x12O\n\x0bupload_info\x18\x01 \x01(\x0b\x32..tecton_proto.data.RemoteEnvironmentUploadInfoR\nuploadInfo\"K\n\x1e\x43ompletePackagesUploadResponse\x12)\n\x10storage_location\x18\x01 \x01(\tR\x0fstorageLocation\"K\n\"GetDependentFeatureServicesRequest\x12%\n\x0e\x65nvironment_id\x18\x01 \x01(\tR\renvironmentId\"\x8f\x01\n#GetDependentFeatureServicesResponse\x12h\n\x1a\x64\x65pendent_feature_services\x18\x01 \x03(\x0b\x32*.tecton_proto.data.DependentFeatureServiceR\x18\x64\x65pendentFeatureServices2\xb8\x0f\n\x18RemoteEnvironmentService\x12\xda\x01\n\x17\x43reateRemoteEnvironment\x12\x45.tecton_proto.remoteenvironmentservice.CreateRemoteEnvironmentRequest\x1a\x46.tecton_proto.remoteenvironmentservice.CreateRemoteEnvironmentResponse\"0\xa2\xbc\xe6\xc0\x05**(tecton/organization#scale_feature_server\x12\xda\x01\n\x17UpdateRemoteEnvironment\x12\x45.tecton_proto.remoteenvironmentservice.UpdateRemoteEnvironmentRequest\x1a\x46.tecton_proto.remoteenvironmentservice.UpdateRemoteEnvironmentResponse\"0\xa2\xbc\xe6\xc0\x05**(tecton/organization#scale_feature_server\x12\xdd\x01\n\x18\x44\x65leteRemoteEnvironments\x12\x46.tecton_proto.remoteenvironmentservice.DeleteRemoteEnvironmentsRequest\x1aG.tecton_proto.remoteenvironmentservice.DeleteRemoteEnvironmentsResponse\"0\xa2\xbc\xe6\xc0\x05**(tecton/organization#scale_feature_server\x12\xd4\x01\n\x16ListRemoteEnvironments\x12\x44.tecton_proto.remoteenvironmentservice.ListRemoteEnvironmentsRequest\x1a\x45.tecton_proto.remoteenvironmentservice.ListRemoteEnvironmentsResponse\"-\xa2\xbc\xe6\xc0\x05\'*%tecton/organization#read_organization\x12\xce\x01\n\x14GetRemoteEnvironment\x12\x42.tecton_proto.remoteenvironmentservice.GetRemoteEnvironmentRequest\x1a\x43.tecton_proto.remoteenvironmentservice.GetRemoteEnvironmentResponse\"-\xa2\xbc\xe6\xc0\x05\'*%tecton/organization#read_organization\x12\xce\x01\n\x14GetPackagesUploadUrl\x12\x42.tecton_proto.remoteenvironmentservice.GetPackagesUploadUrlRequest\x1a\x43.tecton_proto.remoteenvironmentservice.GetPackagesUploadUrlResponse\"-\xa2\xbc\xe6\xc0\x05\'*%tecton/organization#read_organization\x12\xcb\x01\n\x13StartPackagesUpload\x12\x41.tecton_proto.remoteenvironmentservice.StartPackagesUploadRequest\x1a\x42.tecton_proto.remoteenvironmentservice.StartPackagesUploadResponse\"-\xa2\xbc\xe6\xc0\x05\'*%tecton/organization#read_organization\x12\xd4\x01\n\x16\x43ompletePackagesUpload\x12\x44.tecton_proto.remoteenvironmentservice.CompletePackagesUploadRequest\x1a\x45.tecton_proto.remoteenvironmentservice.CompletePackagesUploadResponse\"-\xa2\xbc\xe6\xc0\x05\'*%tecton/organization#read_organization\x12\xe3\x01\n\x1bGetDependentFeatureServices\x12I.tecton_proto.remoteenvironmentservice.GetDependentFeatureServicesRequest\x1aJ.tecton_proto.remoteenvironmentservice.GetDependentFeatureServicesResponse\"-\xa2\xbc\xe6\xc0\x05\'*%tecton/organization#read_organizationB\x82\x01\n#com.tecton.remoteenvironmentserviceB\x1dRemoteEnvironmentServiceProtoP\x01Z:github.com/tecton-ai/tecton_proto/remoteenvironmentservice')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.remoteenvironmentservice.remote_environment_service__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n#com.tecton.remoteenvironmentserviceB\035RemoteEnvironmentServiceProtoP\001Z:github.com/tecton-ai/tecton_proto/remoteenvironmentservice'
  _REMOTEENVIRONMENTSERVICE.methods_by_name['CreateRemoteEnvironment']._options = None
  _REMOTEENVIRONMENTSERVICE.methods_by_name['CreateRemoteEnvironment']._serialized_options = b'\242\274\346\300\005**(tecton/organization#scale_feature_server'
  _REMOTEENVIRONMENTSERVICE.methods_by_name['UpdateRemoteEnvironment']._options = None
  _REMOTEENVIRONMENTSERVICE.methods_by_name['UpdateRemoteEnvironment']._serialized_options = b'\242\274\346\300\005**(tecton/organization#scale_feature_server'
  _REMOTEENVIRONMENTSERVICE.methods_by_name['DeleteRemoteEnvironments']._options = None
  _REMOTEENVIRONMENTSERVICE.methods_by_name['DeleteRemoteEnvironments']._serialized_options = b'\242\274\346\300\005**(tecton/organization#scale_feature_server'
  _REMOTEENVIRONMENTSERVICE.methods_by_name['ListRemoteEnvironments']._options = None
  _REMOTEENVIRONMENTSERVICE.methods_by_name['ListRemoteEnvironments']._serialized_options = b'\242\274\346\300\005\'*%tecton/organization#read_organization'
  _REMOTEENVIRONMENTSERVICE.methods_by_name['GetRemoteEnvironment']._options = None
  _REMOTEENVIRONMENTSERVICE.methods_by_name['GetRemoteEnvironment']._serialized_options = b'\242\274\346\300\005\'*%tecton/organization#read_organization'
  _REMOTEENVIRONMENTSERVICE.methods_by_name['GetPackagesUploadUrl']._options = None
  _REMOTEENVIRONMENTSERVICE.methods_by_name['GetPackagesUploadUrl']._serialized_options = b'\242\274\346\300\005\'*%tecton/organization#read_organization'
  _REMOTEENVIRONMENTSERVICE.methods_by_name['StartPackagesUpload']._options = None
  _REMOTEENVIRONMENTSERVICE.methods_by_name['StartPackagesUpload']._serialized_options = b'\242\274\346\300\005\'*%tecton/organization#read_organization'
  _REMOTEENVIRONMENTSERVICE.methods_by_name['CompletePackagesUpload']._options = None
  _REMOTEENVIRONMENTSERVICE.methods_by_name['CompletePackagesUpload']._serialized_options = b'\242\274\346\300\005\'*%tecton/organization#read_organization'
  _REMOTEENVIRONMENTSERVICE.methods_by_name['GetDependentFeatureServices']._options = None
  _REMOTEENVIRONMENTSERVICE.methods_by_name['GetDependentFeatureServices']._serialized_options = b'\242\274\346\300\005\'*%tecton/organization#read_organization'
  _CREATEREMOTEENVIRONMENTREQUEST._serialized_start=317
  _CREATEREMOTEENVIRONMENTREQUEST._serialized_end=882
  _CREATEREMOTEENVIRONMENTRESPONSE._serialized_start=884
  _CREATEREMOTEENVIRONMENTRESPONSE._serialized_end=1009
  _UPDATEREMOTEENVIRONMENTREQUEST._serialized_start=1012
  _UPDATEREMOTEENVIRONMENTREQUEST._serialized_end=1184
  _UPDATEREMOTEENVIRONMENTRESPONSE._serialized_start=1186
  _UPDATEREMOTEENVIRONMENTRESPONSE._serialized_end=1311
  _DELETEREMOTEENVIRONMENTSREQUEST._serialized_start=1313
  _DELETEREMOTEENVIRONMENTSREQUEST._serialized_end=1364
  _DELETEREMOTEENVIRONMENTSRESPONSE._serialized_start=1366
  _DELETEREMOTEENVIRONMENTSRESPONSE._serialized_end=1400
  _LISTREMOTEENVIRONMENTSREQUEST._serialized_start=1402
  _LISTREMOTEENVIRONMENTSREQUEST._serialized_end=1433
  _LISTREMOTEENVIRONMENTSRESPONSE._serialized_start=1435
  _LISTREMOTEENVIRONMENTSRESPONSE._serialized_end=1561
  _GETREMOTEENVIRONMENTREQUEST._serialized_start=1563
  _GETREMOTEENVIRONMENTREQUEST._serialized_end=1608
  _GETREMOTEENVIRONMENTRESPONSE._serialized_start=1610
  _GETREMOTEENVIRONMENTRESPONSE._serialized_end=1732
  _GETPACKAGESUPLOADURLREQUEST._serialized_start=1735
  _GETPACKAGESUPLOADURLREQUEST._serialized_end=1878
  _GETPACKAGESUPLOADURLRESPONSE._serialized_start=1880
  _GETPACKAGESUPLOADURLRESPONSE._serialized_end=1941
  _STARTPACKAGESUPLOADREQUEST._serialized_start=1943
  _STARTPACKAGESUPLOADREQUEST._serialized_end=2010
  _STARTPACKAGESUPLOADRESPONSE._serialized_start=2012
  _STARTPACKAGESUPLOADRESPONSE._serialized_end=2122
  _COMPLETEPACKAGESUPLOADREQUEST._serialized_start=2124
  _COMPLETEPACKAGESUPLOADREQUEST._serialized_end=2236
  _COMPLETEPACKAGESUPLOADRESPONSE._serialized_start=2238
  _COMPLETEPACKAGESUPLOADRESPONSE._serialized_end=2313
  _GETDEPENDENTFEATURESERVICESREQUEST._serialized_start=2315
  _GETDEPENDENTFEATURESERVICESREQUEST._serialized_end=2390
  _GETDEPENDENTFEATURESERVICESRESPONSE._serialized_start=2393
  _GETDEPENDENTFEATURESERVICESRESPONSE._serialized_end=2536
  _REMOTEENVIRONMENTSERVICE._serialized_start=2539
  _REMOTEENVIRONMENTSERVICE._serialized_end=4515
# @@protoc_insertion_point(module_scope)
