# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/state_update__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.args import diff_options__client_pb2 as tecton__proto_dot_args_dot_diff__options____client__pb2
from tecton_proto.args import fco_args__client_pb2 as tecton__proto_dot_args_dot_fco__args____client__pb2
from tecton_proto.args import repo_metadata__client_pb2 as tecton__proto_dot_args_dot_repo__metadata____client__pb2
from tecton_proto.auth import principal__client_pb2 as tecton__proto_dot_auth_dot_principal____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,tecton_proto/data/state_update__client.proto\x12\x11tecton_proto.data\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a,tecton_proto/args/diff_options__client.proto\x1a(tecton_proto/args/fco_args__client.proto\x1a-tecton_proto/args/repo_metadata__client.proto\x1a)tecton_proto/auth/principal__client.proto\x1a$tecton_proto/common/id__client.proto\"\xa2\x02\n\x0f\x46\x63oPropertyDiff\x12#\n\rproperty_name\x18\x01 \x01(\tR\x0cpropertyName\x12!\n\x0cval_existing\x18\x02 \x01(\tR\x0bvalExisting\x12!\n\x0cval_declared\x18\x03 \x01(\tR\x0bvalDeclared\x12R\n\x0erendering_type\x18\x04 \x01(\x0e\x32+.tecton_proto.args.FcoPropertyRenderingTypeR\rrenderingType\x12P\n\x11\x63ustom_comparator\x18\x05 \x01(\x0e\x32#.tecton_proto.args.CustomComparatorR\x10\x63ustomComparator\"\x87\x01\n\x18\x46\x63oTransitionSideEffects\x12k\n\x13stream_restart_type\x18\x01 \x01(\x0e\x32;.tecton_proto.data.FcoTransitionSideEffectStreamRestartTypeR\x11streamRestartType\"\xe6\x01\n\x1c\x42\x61tchMaterializationTaskDiff\x12%\n\x0e\x64isplay_string\x18\x01 \x01(\tR\rdisplayString\x12\x46\n\x11schedule_interval\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationR\x10scheduleInterval\x12W\n\x0b\x64\x65stination\x18\x03 \x01(\x0e\x32\x35.tecton_proto.data.MaterializationTaskDiffDestinationR\x0b\x64\x65stination\"F\n\x1dStreamMaterializationTaskDiff\x12%\n\x0e\x64isplay_string\x18\x01 \x01(\tR\rdisplayString\"\xd7\x02\n\x1f\x42\x61\x63kfillMaterializationTaskDiff\x12%\n\x0e\x64isplay_string\x18\x01 \x01(\tR\rdisplayString\x12H\n\x12\x66\x65\x61ture_start_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x10\x66\x65\x61tureStartTime\x12\x44\n\x10\x66\x65\x61ture_end_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0e\x66\x65\x61tureEndTime\x12$\n\x0enumber_of_jobs\x18\x04 \x01(\x05R\x0cnumberOfJobs\x12W\n\x0b\x64\x65stination\x18\x05 \x01(\x0e\x32\x35.tecton_proto.data.MaterializationTaskDiffDestinationR\x0b\x64\x65stination\"D\n\x1bPlanIntegrationTestTaskDiff\x12%\n\x0e\x64isplay_string\x18\x01 \x01(\tR\rdisplayString\"\x8f\x04\n\x13MaterializationInfo\x12\x62\n\x13\x62\x61\x63kfill_task_diffs\x18\x01 \x03(\x0b\x32\x32.tecton_proto.data.BackfillMaterializationTaskDiffR\x11\x62\x61\x63kfillTaskDiffs\x12W\n\x0f\x62\x61tch_task_diff\x18\x02 \x01(\x0b\x32/.tecton_proto.data.BatchMaterializationTaskDiffR\rbatchTaskDiff\x12Z\n\x10stream_task_diff\x18\x03 \x01(\x0b\x32\x30.tecton_proto.data.StreamMaterializationTaskDiffR\x0estreamTaskDiff\x12p\n\x1b\x62\x61\x63kfill_publish_task_diffs\x18\x04 \x03(\x0b\x32\x31.tecton_proto.data.BackfillFeaturePublishTaskDiffR\x18\x62\x61\x63kfillPublishTaskDiffs\x12m\n\x1bintegration_test_task_diffs\x18\x05 \x03(\x0b\x32..tecton_proto.data.PlanIntegrationTestTaskDiffR\x18integrationTestTaskDiffs\"\xfd\x01\n\x1e\x42\x61\x63kfillFeaturePublishTaskDiff\x12%\n\x0e\x64isplay_string\x18\x01 \x01(\tR\rdisplayString\x12H\n\x12\x66\x65\x61ture_start_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x10\x66\x65\x61tureStartTime\x12\x44\n\x10\x66\x65\x61ture_end_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0e\x66\x65\x61tureEndTime\x12$\n\x0enumber_of_jobs\x18\x04 \x01(\x05R\x0cnumberOfJobs\"\xc3\x03\n\x07\x46\x63oDiff\x12\x38\n\x04type\x18\x01 \x01(\x0e\x32$.tecton_proto.data.FcoTransitionTypeR\x04type\x12\x63\n\x17transition_side_effects\x18\x0c \x01(\x0b\x32+.tecton_proto.data.FcoTransitionSideEffectsR\x15transitionSideEffects\x12?\n\rexisting_args\x18\x07 \x01(\x0b\x32\x1a.tecton_proto.args.FcoArgsR\x0c\x65xistingArgs\x12?\n\rdeclared_args\x18\x08 \x01(\x0b\x32\x1a.tecton_proto.args.FcoArgsR\x0c\x64\x65\x63laredArgs\x12\x36\n\x04\x64iff\x18\x04 \x03(\x0b\x32\".tecton_proto.data.FcoPropertyDiffR\x04\x64iff\x12Y\n\x14materialization_info\x18\x0b \x01(\x0b\x32&.tecton_proto.data.MaterializationInfoR\x13materializationInfoJ\x04\x08\t\x10\n\"C\n\x0b\x46\x63oFieldRef\x12.\n\x06\x66\x63o_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x05\x66\x63oIdJ\x04\x08\x02\x10\x03\"h\n\x11ValidationMessage\x12\x18\n\x07message\x18\x01 \x01(\tR\x07message\x12\x39\n\x08\x66\x63o_refs\x18\x02 \x03(\x0b\x32\x1e.tecton_proto.data.FcoFieldRefR\x07\x66\x63oRefs\"\x92\x01\n\x10ValidationResult\x12<\n\x06\x65rrors\x18\x01 \x03(\x0b\x32$.tecton_proto.data.ValidationMessageR\x06\x65rrors\x12@\n\x08warnings\x18\x02 \x03(\x0b\x32$.tecton_proto.data.ValidationMessageR\x08warnings\"\xa6\x02\n\x14SuccessfulPlanOutput\x12#\n\rstring_output\x18\x01 \x01(\tR\x0cstringOutput\x12\x1f\n\x0bjson_output\x18\x02 \x01(\tR\njsonOutput\x12%\n\x0e\x61pply_warnings\x18\x03 \x01(\tR\rapplyWarnings\x12(\n\x10num_fcos_changed\x18\x04 \x01(\x05R\x0enumFcosChanged\x12!\n\x0cnum_warnings\x18\x05 \x01(\x05R\x0bnumWarnings\x12T\n\x0etest_summaries\x18\x06 \x03(\x0b\x32-.tecton_proto.data.PlanIntegrationTestSummaryR\rtestSummaries\"`\n\x19IntegrationTestJobSummary\x12\x43\n\x06status\x18\x01 \x01(\x0e\x32+.tecton_proto.data.IntegrationTestJobStatusR\x06status\"\xb0\x01\n\x1aPlanIntegrationTestSummary\x12?\n\x0f\x66\x65\x61ture_view_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rfeatureViewId\x12Q\n\rjob_summaries\x18\x02 \x03(\x0b\x32,.tecton_proto.data.IntegrationTestJobSummaryR\x0cjobSummaries\"\x9d\x04\n\x12StateUpdateRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12\x35\n\x08\x66\x63o_args\x18\x02 \x03(\x0b\x32\x1a.tecton_proto.args.FcoArgsR\x07\x66\x63oArgs\x12R\n\x10repo_source_info\x18\x03 \x01(\x0b\x32(.tecton_proto.args.FeatureRepoSourceInfoR\x0erepoSourceInfo\x12-\n\x12suppress_recreates\x18\n \x01(\x08R\x11suppressRecreates\x12\x1f\n\x0bupgrade_all\x18\x04 \x01(\x08R\nupgradeAll\x12!\n\x0crequested_by\x18\x05 \x01(\tR\x0brequestedBy\x12R\n\x16requested_by_principal\x18\x0b \x01(\x0b\x32\x1c.tecton_proto.auth.PrincipalR\x14requestedByPrincipal\x12\x1f\n\x0bsdk_version\x18\t \x01(\tR\nsdkVersion\x12\x64\n\x15plan_integration_type\x18\x0c \x01(\x0e\x32\x30.tecton_proto.data.PlanIntegrationTestSelectTypeR\x13planIntegrationTypeJ\x04\x08\x06\x10\x07J\x04\x08\x07\x10\x08J\x04\x08\x08\x10\t\"\xa1\x04\n\x10StateUpdateEntry\x12\x1b\n\tcommit_id\x18\x01 \x01(\tR\x08\x63ommitId\x12\x1d\n\napplied_by\x18\x02 \x01(\tR\tappliedBy\x12S\n\x14\x61pplied_by_principal\x18\x06 \x01(\x0b\x32!.tecton_proto.auth.PrincipalBasicR\x12\x61ppliedByPrincipal\x12\x39\n\napplied_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tappliedAt\x12\x1c\n\tworkspace\x18\x04 \x01(\tR\tworkspace\x12\x1f\n\x0bsdk_version\x18\x05 \x01(\tR\nsdkVersion\x12\x39\n\ncreated_at\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x42\n\x0bstatus_type\x18\x08 \x01(\x0e\x32!.tecton_proto.data.PlanStatusTypeR\nstatusType\x12\x16\n\x05\x65rror\x18\t \x01(\tH\x00R\x05\x65rror\x12_\n\x16successful_plan_output\x18\n \x01(\x0b\x32\'.tecton_proto.data.SuccessfulPlanOutputH\x00R\x14successfulPlanOutputB\n\n\x08response\"\xae\x03\n\x16StateUpdatePlanSummary\x12L\n\ndiff_items\x18\x01 \x03(\x0b\x32-.tecton_proto.data.StateUpdatePlanSummaryDiffR\tdiffItems\x12\x1d\n\napplied_by\x18\x02 \x01(\tR\tappliedBy\x12S\n\x14\x61pplied_by_principal\x18\x08 \x01(\x0b\x32!.tecton_proto.auth.PrincipalBasicR\x12\x61ppliedByPrincipal\x12\x1d\n\ncreated_by\x18\x03 \x01(\tR\tcreatedBy\x12\x39\n\napplied_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tappliedAt\x12\x39\n\ncreated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x1c\n\tworkspace\x18\x06 \x01(\tR\tworkspace\x12\x1f\n\x0bsdk_version\x18\x07 \x01(\tR\nsdkVersion\"\xa1\x03\n\x1aStateUpdatePlanSummaryDiff\x12\x19\n\x08\x66\x63o_type\x18\x01 \x01(\tR\x07\x66\x63oType\x12\x38\n\x04type\x18\x02 \x01(\x0e\x32$.tecton_proto.data.FcoTransitionTypeR\x04type\x12\x63\n\x17transition_side_effects\x18\x07 \x01(\x0b\x32+.tecton_proto.data.FcoTransitionSideEffectsR\x15transitionSideEffects\x12\x38\n\x05\x64iffs\x18\x03 \x03(\x0b\x32\".tecton_proto.data.FcoPropertyDiffR\x05\x64iffs\x12\x12\n\x04name\x18\x04 \x01(\tR\x04name\x12 \n\x0b\x64\x65scription\x18\x05 \x01(\tR\x0b\x64\x65scription\x12Y\n\x14materialization_info\x18\x06 \x01(\x0b\x32&.tecton_proto.data.MaterializationInfoR\x13materializationInfo*n\n\x11\x46\x63oTransitionType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\n\n\x06\x43REATE\x10\x01\x12\n\n\x06\x44\x45LETE\x10\x02\x12\x0b\n\x07UPGRADE\x10\x05\x12\n\n\x06UPDATE\x10\x03\x12\x0c\n\x08RECREATE\x10\x04\x12\r\n\tUNCHANGED\x10\x06*\x95\x01\n(FcoTransitionSideEffectStreamRestartType\x12\x17\n\x13RESTART_STREAM_NONE\x10\x00\x12$\n RESTART_STREAM_REUSE_CHECKPOINTS\x10\x01\x12*\n&RESTART_STREAM_CHECKPOINTS_INVALIDATED\x10\x02*\xba\x02\n\"MaterializationTaskDiffDestination\x12\x35\n1MATERIALIZATION_TASK_DIFF_DESTINATION_UNSPECIFIED\x10\x00\x12\x31\n-MATERIALIZATION_TASK_DIFF_DESTINATION_OFFLINE\x10\x01\x12\x30\n,MATERIALIZATION_TASK_DIFF_DESTINATION_ONLINE\x10\x02\x12<\n8MATERIALIZATION_TASK_DIFF_DESTINATION_ONLINE_AND_OFFLINE\x10\x03\x12:\n6MATERIALIZATION_TASK_DIFF_DESTINATION_BULK_LOAD_ONLINE\x10\x04*\xb3\x01\n\x18IntegrationTestJobStatus\x12\x1a\n\x16JOB_STATUS_UNSPECIFIED\x10\x00\x12\x1a\n\x16JOB_STATUS_NOT_STARTED\x10\x01\x12\x16\n\x12JOB_STATUS_RUNNING\x10\x02\x12\x18\n\x14JOB_STATUS_CANCELLED\x10\x03\x12\x16\n\x12JOB_STATUS_SUCCEED\x10\x04\x12\x15\n\x11JOB_STATUS_FAILED\x10\x05*D\n\x1dPlanIntegrationTestSelectType\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x08\n\x04\x41UTO\x10\x01\x12\x08\n\x04NONE\x10\x02*\xa7\x02\n\x0ePlanStatusType\x12\x14\n\x10PLAN_UNSPECIFIED\x10\x00\x12\x10\n\x0cPLAN_CREATED\x10\x01\x12\"\n\x1ePLAN_INTEGRATION_TESTS_SKIPPED\x10\x02\x12&\n\"PLAN_INTEGRATION_TESTS_NOT_STARTED\x10\x03\x12\"\n\x1ePLAN_INTEGRATION_TESTS_RUNNING\x10\x04\x12$\n PLAN_INTEGRATION_TESTS_CANCELLED\x10\x05\x12\"\n\x1ePLAN_INTEGRATION_TESTS_SUCCEED\x10\x06\x12!\n\x1dPLAN_INTEGRATION_TESTS_FAILED\x10\x07\x12\x10\n\x0cPLAN_APPLIED\x10\x08\x42\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.state_update__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _FCOTRANSITIONTYPE._serialized_start=5583
  _FCOTRANSITIONTYPE._serialized_end=5693
  _FCOTRANSITIONSIDEEFFECTSTREAMRESTARTTYPE._serialized_start=5696
  _FCOTRANSITIONSIDEEFFECTSTREAMRESTARTTYPE._serialized_end=5845
  _MATERIALIZATIONTASKDIFFDESTINATION._serialized_start=5848
  _MATERIALIZATIONTASKDIFFDESTINATION._serialized_end=6162
  _INTEGRATIONTESTJOBSTATUS._serialized_start=6165
  _INTEGRATIONTESTJOBSTATUS._serialized_end=6344
  _PLANINTEGRATIONTESTSELECTTYPE._serialized_start=6346
  _PLANINTEGRATIONTESTSELECTTYPE._serialized_end=6414
  _PLANSTATUSTYPE._serialized_start=6417
  _PLANSTATUSTYPE._serialized_end=6712
  _FCOPROPERTYDIFF._serialized_start=349
  _FCOPROPERTYDIFF._serialized_end=639
  _FCOTRANSITIONSIDEEFFECTS._serialized_start=642
  _FCOTRANSITIONSIDEEFFECTS._serialized_end=777
  _BATCHMATERIALIZATIONTASKDIFF._serialized_start=780
  _BATCHMATERIALIZATIONTASKDIFF._serialized_end=1010
  _STREAMMATERIALIZATIONTASKDIFF._serialized_start=1012
  _STREAMMATERIALIZATIONTASKDIFF._serialized_end=1082
  _BACKFILLMATERIALIZATIONTASKDIFF._serialized_start=1085
  _BACKFILLMATERIALIZATIONTASKDIFF._serialized_end=1428
  _PLANINTEGRATIONTESTTASKDIFF._serialized_start=1430
  _PLANINTEGRATIONTESTTASKDIFF._serialized_end=1498
  _MATERIALIZATIONINFO._serialized_start=1501
  _MATERIALIZATIONINFO._serialized_end=2028
  _BACKFILLFEATUREPUBLISHTASKDIFF._serialized_start=2031
  _BACKFILLFEATUREPUBLISHTASKDIFF._serialized_end=2284
  _FCODIFF._serialized_start=2287
  _FCODIFF._serialized_end=2738
  _FCOFIELDREF._serialized_start=2740
  _FCOFIELDREF._serialized_end=2807
  _VALIDATIONMESSAGE._serialized_start=2809
  _VALIDATIONMESSAGE._serialized_end=2913
  _VALIDATIONRESULT._serialized_start=2916
  _VALIDATIONRESULT._serialized_end=3062
  _SUCCESSFULPLANOUTPUT._serialized_start=3065
  _SUCCESSFULPLANOUTPUT._serialized_end=3359
  _INTEGRATIONTESTJOBSUMMARY._serialized_start=3361
  _INTEGRATIONTESTJOBSUMMARY._serialized_end=3457
  _PLANINTEGRATIONTESTSUMMARY._serialized_start=3460
  _PLANINTEGRATIONTESTSUMMARY._serialized_end=3636
  _STATEUPDATEREQUEST._serialized_start=3639
  _STATEUPDATEREQUEST._serialized_end=4180
  _STATEUPDATEENTRY._serialized_start=4183
  _STATEUPDATEENTRY._serialized_end=4728
  _STATEUPDATEPLANSUMMARY._serialized_start=4731
  _STATEUPDATEPLANSUMMARY._serialized_end=5161
  _STATEUPDATEPLANSUMMARYDIFF._serialized_start=5164
  _STATEUPDATEPLANSUMMARYDIFF._serialized_end=5581
# @@protoc_insertion_point(module_scope)
