# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/materialization/materialization_states__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nAtecton_proto/materialization/materialization_states__client.proto\x12\x1ctecton_proto.materialization*\xc0\x03\n\x18MaterializationTaskState\x12+\n\'MATERIALIZATION_TASK_STATUS_UNSPECIFIED\x10\x00\x12\'\n#MATERIALIZATION_TASK_STATUS_RUNNING\x10\x01\x12(\n$MATERIALIZATION_TASK_STATUS_DRAINING\x10\x02\x12\'\n#MATERIALIZATION_TASK_STATUS_DRAINED\x10\x05\x12=\n9MATERIALIZATION_TASK_STATUS_MANUAL_CANCELLATION_REQUESTED\x10\x07\x12\x32\n.MATERIALIZATION_TASK_STATUS_MANUALLY_CANCELLED\x10\x08\x12\'\n#MATERIALIZATION_TASK_STATUS_SUCCESS\x10\x03\x12\x31\n-MATERIALIZATION_TASK_STATUS_PERMANENT_FAILURE\x10\x04\x12,\n(MATERIALIZATION_TASK_STATUS_MANUAL_RETRY\x10\x06*\xad\x02\n\x1fMaterializationTaskAttemptState\x12.\n*MATERIALIZATION_ATTEMPT_STATUS_UNSPECIFIED\x10\x00\x12*\n&MATERIALIZATION_ATTEMPT_STATUS_PENDING\x10\x01\x12*\n&MATERIALIZATION_ATTEMPT_STATUS_RUNNING\x10\x02\x12*\n&MATERIALIZATION_ATTEMPT_STATUS_SUCCESS\x10\x03\x12(\n$MATERIALIZATION_ATTEMPT_STATUS_ERROR\x10\x04\x12,\n(MATERIALIZATION_ATTEMPT_STATUS_CANCELLED\x10\x05*\xa5\x01\n\x1aOnlineStoreCopierTaskState\x12\x1d\n\x19\x43OPIER_STATUS_UNSPECIFIED\x10\x00\x12\x19\n\x15\x43OPIER_STATUS_PENDING\x10\x01\x12\x19\n\x15\x43OPIER_STATUS_RUNNING\x10\x02\x12\x19\n\x15\x43OPIER_STATUS_SUCCESS\x10\x03\x12\x17\n\x13\x43OPIER_STATUS_ERROR\x10\x04\x42\x1e\n\x1a\x63om.tecton.materializationP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.materialization.materialization_states__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\032com.tecton.materializationP\001'
  _MATERIALIZATIONTASKSTATE._serialized_start=100
  _MATERIALIZATIONTASKSTATE._serialized_end=548
  _MATERIALIZATIONTASKATTEMPTSTATE._serialized_start=551
  _MATERIALIZATIONTASKATTEMPTSTATE._serialized_end=852
  _ONLINESTORECOPIERTASKSTATE._serialized_start=855
  _ONLINESTORECOPIERTASKSTATE._serialized_end=1020
# @@protoc_insertion_point(module_scope)
