# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/entity__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import schema__client_pb2 as tecton__proto_dot_common_dot_schema____client__pb2
from tecton_proto.data import fco_metadata__client_pb2 as tecton__proto_dot_data_dot_fco__metadata____client__pb2
from tecton_proto.validation import validator__client_pb2 as tecton__proto_dot_validation_dot_validator____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&tecton_proto/data/entity__client.proto\x12\x11tecton_proto.data\x1a$tecton_proto/common/id__client.proto\x1a(tecton_proto/common/schema__client.proto\x1a,tecton_proto/data/fco_metadata__client.proto\x1a/tecton_proto/validation/validator__client.proto\"\xd9\x03\n\x06\x45ntity\x12\x34\n\tentity_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x08\x65ntityId\x12(\n\x10join_keys_legacy\x18\x03 \x03(\tR\x0ejoinKeysLegacy\x12\x41\n\x0c\x66\x63o_metadata\x18\x07 \x01(\x0b\x32\x1e.tecton_proto.data.FcoMetadataR\x0b\x66\x63oMetadata\x12V\n\x0fvalidation_args\x18\t \x01(\x0b\x32-.tecton_proto.validation.EntityValidationArgsR\x0evalidationArgs\x12@\n\x07options\x18\n \x03(\x0b\x32&.tecton_proto.data.Entity.OptionsEntryR\x07options\x12\x38\n\tjoin_keys\x18\x0b \x03(\x0b\x32\x1b.tecton_proto.common.ColumnR\x08joinKeys\x1a:\n\x0cOptionsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01J\x04\x08\x02\x10\x03J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06J\x04\x08\x06\x10\x07J\x04\x08\x08\x10\tB\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.entity__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _ENTITY_OPTIONSENTRY._options = None
  _ENTITY_OPTIONSENTRY._serialized_options = b'8\001'
  _ENTITY._serialized_start=237
  _ENTITY._serialized_end=710
  _ENTITY_OPTIONSENTRY._serialized_start=622
  _ENTITY_OPTIONSENTRY._serialized_end=680
# @@protoc_insertion_point(module_scope)
