# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/workspace__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.auditlog import metadata__client_pb2 as tecton__proto_dot_auditlog_dot_metadata____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n)tecton_proto/data/workspace__client.proto\x12\x11tecton_proto.data\x1a\x1fgoogle/protobuf/timestamp.proto\x1a,tecton_proto/auditlog/metadata__client.proto\"\xa5\x01\n\x15WorkspaceCapabilities\x12/\n\x0ematerializable\x18\x01 \x01(\x08\x42\x07\xea\x99\xbd\x46\x02\x08\x02R\x0ematerializable\x12[\n\"offline_store_subdirectory_enabled\x18\x02 \x01(\x08:\x05\x66\x61lseB\x07\xea\x99\xbd\x46\x02\x08\x01R\x1fofflineStoreSubdirectoryEnabled\"\xa8\x01\n\tWorkspace\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12L\n\x0c\x63\x61pabilities\x18\x02 \x01(\x0b\x32(.tecton_proto.data.WorkspaceCapabilitiesR\x0c\x63\x61pabilities\x12\x39\n\ncreated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAtB\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.workspace__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _WORKSPACECAPABILITIES.fields_by_name['materializable']._options = None
  _WORKSPACECAPABILITIES.fields_by_name['materializable']._serialized_options = b'\352\231\275F\002\010\002'
  _WORKSPACECAPABILITIES.fields_by_name['offline_store_subdirectory_enabled']._options = None
  _WORKSPACECAPABILITIES.fields_by_name['offline_store_subdirectory_enabled']._serialized_options = b'\352\231\275F\002\010\001'
  _WORKSPACECAPABILITIES._serialized_start=144
  _WORKSPACECAPABILITIES._serialized_end=309
  _WORKSPACE._serialized_start=312
  _WORKSPACE._serialized_end=480
# @@protoc_insertion_point(module_scope)
