# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/common/aggregation_function__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n6tecton_proto/common/aggregation_function__client.proto\x12\x13tecton_proto.common\"\x1b\n\x0bLastNParams\x12\x0c\n\x01n\x18\x01 \x01(\x05R\x01n\"\x1c\n\x0c\x46irstNParams\x12\x0c\n\x01n\x18\x01 \x01(\x05R\x01n\"9\n\x19\x41pproxCountDistinctParams\x12\x1c\n\tprecision\x18\x01 \x01(\x05R\tprecision\"V\n\x16\x41pproxPercentileParams\x12\x1e\n\npercentile\x18\x01 \x01(\x01R\npercentile\x12\x1c\n\tprecision\x18\x02 \x01(\x05R\tprecision\"\xe0\x02\n\x19\x41ggregationFunctionParams\x12\x39\n\x06last_n\x18\x01 \x01(\x0b\x32 .tecton_proto.common.LastNParamsH\x00R\x05lastN\x12<\n\x07\x66irst_n\x18\x02 \x01(\x0b\x32!.tecton_proto.common.FirstNParamsH\x00R\x06\x66irstN\x12\x64\n\x15\x61pprox_count_distinct\x18\x03 \x01(\x0b\x32..tecton_proto.common.ApproxCountDistinctParamsH\x00R\x13\x61pproxCountDistinct\x12Z\n\x11\x61pprox_percentile\x18\x04 \x01(\x0b\x32+.tecton_proto.common.ApproxPercentileParamsH\x00R\x10\x61pproxPercentileB\x08\n\x06params*\x99\x05\n\x13\x41ggregationFunction\x12 \n\x1c\x41GGREGATION_FUNCTION_UNKNOWN\x10\x00\x12\x1e\n\x1a\x41GGREGATION_FUNCTION_COUNT\x10\x01\x12\x1c\n\x18\x41GGREGATION_FUNCTION_SUM\x10\x02\x12\x1d\n\x19\x41GGREGATION_FUNCTION_MEAN\x10\x03\x12\x1d\n\x19\x41GGREGATION_FUNCTION_LAST\x10\x05\x12\x1c\n\x18\x41GGREGATION_FUNCTION_MIN\x10\x06\x12\x1c\n\x18\x41GGREGATION_FUNCTION_MAX\x10\x07\x12(\n$AGGREGATION_FUNCTION_LAST_DISTINCT_N\x10\t\x12!\n\x1d\x41GGREGATION_FUNCTION_VAR_SAMP\x10\n\x12 \n\x1c\x41GGREGATION_FUNCTION_VAR_POP\x10\x0b\x12$\n AGGREGATION_FUNCTION_STDDEV_SAMP\x10\x0c\x12#\n\x1f\x41GGREGATION_FUNCTION_STDDEV_POP\x10\r\x12,\n(AGGREGATION_FUNCTION_LAST_NON_DISTINCT_N\x10\x0e\x12-\n)AGGREGATION_FUNCTION_FIRST_NON_DISTINCT_N\x10\x0f\x12)\n%AGGREGATION_FUNCTION_FIRST_DISTINCT_N\x10\x10\x12.\n*AGGREGATION_FUNCTION_APPROX_COUNT_DISTINCT\x10\x11\x12*\n&AGGREGATION_FUNCTION_APPROX_PERCENTILE\x10\x12\"\x04\x08\x04\x10\x04\"\x04\x08\x08\x10\x08\x42?\n\x11\x63om.tecton.commonP\x01Z(github.com/tecton-ai/tecton_proto/common')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.common.aggregation_function__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021com.tecton.commonP\001Z(github.com/tecton-ai/tecton_proto/common'
  _AGGREGATIONFUNCTION._serialized_start=641
  _AGGREGATIONFUNCTION._serialized_end=1306
  _LASTNPARAMS._serialized_start=79
  _LASTNPARAMS._serialized_end=106
  _FIRSTNPARAMS._serialized_start=108
  _FIRSTNPARAMS._serialized_end=136
  _APPROXCOUNTDISTINCTPARAMS._serialized_start=138
  _APPROXCOUNTDISTINCTPARAMS._serialized_end=195
  _APPROXPERCENTILEPARAMS._serialized_start=197
  _APPROXPERCENTILEPARAMS._serialized_end=283
  _AGGREGATIONFUNCTIONPARAMS._serialized_start=286
  _AGGREGATIONFUNCTIONPARAMS._serialized_end=638
# @@protoc_insertion_point(module_scope)
