# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/consumption/consumption__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2tecton_proto/consumption/consumption__client.proto\x12\x18tecton_proto.consumption\x1a google/protobuf/descriptor.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a$tecton_proto/common/id__client.proto\"\x83\x04\n\x0f\x43onsumptionInfo\x12\x46\n\x11time_bucket_start\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0ftimeBucketStart\x12%\n\x0eunits_consumed\x18\x02 \x01(\x03R\runitsConsumed\x12\x16\n\x06metric\x18\x03 \x01(\tR\x06metric\x12P\n\x07\x64\x65tails\x18\x04 \x03(\x0b\x32\x36.tecton_proto.consumption.ConsumptionInfo.DetailsEntryR\x07\x64\x65tails\x12\x1b\n\tsource_id\x18\x05 \x01(\tR\x08sourceId\x12?\n\x0f\x66\x65\x61ture_view_id\x18\x06 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rfeatureViewId\x12*\n\x11\x66\x65\x61ture_view_name\x18\x08 \x01(\tR\x0f\x66\x65\x61tureViewName\x12\x1c\n\tworkspace\x18\t \x01(\tR\tworkspace\x12\x33\n\x16online_read_aws_region\x18\x07 \x01(\tR\x13onlineReadAwsRegion\x1a:\n\x0c\x44\x65tailsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xff\x01\n\x17\x45nrichedConsumptionInfo\x12T\n\x10\x63onsumption_info\x18\x01 \x01(\x0b\x32).tecton_proto.consumption.ConsumptionInfoR\x0f\x63onsumptionInfo\x12\x34\n\x16\x66\x65\x61ture_view_workspace\x18\x03 \x01(\tR\x14\x66\x65\x61tureViewWorkspace\x12*\n\x11\x66\x65\x61ture_view_name\x18\x04 \x01(\tR\x0f\x66\x65\x61tureViewName\x12&\n\x0f\x66\x65\x61ture_view_id\x18\x05 \x01(\tR\rfeatureViewIdJ\x04\x08\x02\x10\x03\"\x9f\x01\n\x14\x42illableUsageOptions\x12\x44\n\nvisibility\x18\x01 \x01(\x0e\x32$.tecton_proto.consumption.VisibilityR\nvisibility\x12\x41\n\x08required\x18\x02 \x01(\x0e\x32%.tecton_proto.consumption.RequirementR\x08required\"\xd0\x03\n&MaterializationJobOnlineWritesMetadata\x12*\n\x11online_store_type\x18\x01 \x01(\tR\x0fonlineStoreType\x12\"\n\rtecton_job_id\x18\x02 \x01(\tR\x0btectonJobId\x12%\n\tworkspace\x18\x03 \x01(\tB\x07\xf2\x99\xbd\x46\x02\x08\x01R\tworkspace\x12,\n\x12workspace_state_id\x18\x04 \x01(\tR\x10workspaceStateId\x12\x35\n\x12tecton_object_name\x18\x05 \x01(\tB\x07\xf2\x99\xbd\x46\x02\x08\x01R\x10tectonObjectName\x12(\n\x10tecton_object_id\x18\x06 \x01(\tR\x0etectonObjectId\x12g\n\x04tags\x18\x07 \x03(\x0b\x32J.tecton_proto.consumption.MaterializationJobOnlineWritesMetadata.TagsEntryB\x07\xf2\x99\xbd\x46\x02\x08\x01R\x04tags\x1a\x37\n\tTagsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xa6\x03\n\'MaterializationJobOfflineWritesMetadata\x12\"\n\rtecton_job_id\x18\x01 \x01(\tR\x0btectonJobId\x12%\n\tworkspace\x18\x02 \x01(\tB\x07\xf2\x99\xbd\x46\x02\x08\x01R\tworkspace\x12,\n\x12workspace_state_id\x18\x03 \x01(\tR\x10workspaceStateId\x12\x35\n\x12tecton_object_name\x18\x04 \x01(\tB\x07\xf2\x99\xbd\x46\x02\x08\x01R\x10tectonObjectName\x12(\n\x10tecton_object_id\x18\x05 \x01(\tR\x0etectonObjectId\x12h\n\x04tags\x18\x06 \x03(\x0b\x32K.tecton_proto.consumption.MaterializationJobOfflineWritesMetadata.TagsEntryB\x07\xf2\x99\xbd\x46\x02\x08\x01R\x04tags\x1a\x37\n\tTagsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xa5\x05\n\x1dTectonJobComputeHoursMetadata\x12+\n\rtecton_job_id\x18\x01 \x01(\tB\x07\xf2\x99\xbd\x46\x02\x08\x01R\x0btectonJobId\x12,\n\rinstance_type\x18\x02 \x01(\tB\x07\xf2\x99\xbd\x46\x02\x08\x01R\x0cinstanceType\x12\x16\n\x06region\x18\x08 \x01(\tR\x06region\x12(\n\x0bnum_workers\x18\x0b \x01(\rB\x07\xf2\x99\xbd\x46\x02\x10\x01R\nnumWorkers\x12Q\n\x0c\x63ompute_type\x18\t \x01(\x0e\x32%.tecton_proto.consumption.ComputeTypeB\x07\xf2\x99\xbd\x46\x02\x08\x01R\x0b\x63omputeType\x12\x45\n\x08job_type\x18\n \x01(\x0e\x32!.tecton_proto.consumption.JobTypeB\x07\xf2\x99\xbd\x46\x02\x08\x01R\x07jobType\x12%\n\tworkspace\x18\x03 \x01(\tB\x07\xf2\x99\xbd\x46\x02\x08\x01R\tworkspace\x12,\n\x12workspace_state_id\x18\x04 \x01(\tR\x10workspaceStateId\x12\x35\n\x12tecton_object_name\x18\x05 \x01(\tB\x07\xf2\x99\xbd\x46\x02\x08\x01R\x10tectonObjectName\x12(\n\x10tecton_object_id\x18\x06 \x01(\tR\x0etectonObjectId\x12^\n\x04tags\x18\x07 \x03(\x0b\x32\x41.tecton_proto.consumption.TectonJobComputeHoursMetadata.TagsEntryB\x07\xf2\x99\xbd\x46\x02\x08\x01R\x04tags\x1a\x37\n\tTagsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xb3\x04\n\x1dIngestApiComputeHoursMetadata\x12H\n\x0c\x63ompute_type\x18\x01 \x01(\x0e\x32%.tecton_proto.consumption.ComputeTypeR\x0b\x63omputeType\x12<\n\x08job_type\x18\x02 \x01(\x0e\x32!.tecton_proto.consumption.JobTypeR\x07jobType\x12\x1c\n\toperation\x18\x03 \x01(\tR\toperation\x12\x1d\n\nmemory_mib\x18\t \x01(\x04R\tmemoryMib\x12%\n\tworkspace\x18\x04 \x01(\tB\x07\xf2\x99\xbd\x46\x02\x08\x01R\tworkspace\x12,\n\x12workspace_state_id\x18\x05 \x01(\tR\x10workspaceStateId\x12\x35\n\x12tecton_object_name\x18\x06 \x01(\tB\x07\xf2\x99\xbd\x46\x02\x08\x01R\x10tectonObjectName\x12(\n\x10tecton_object_id\x18\x07 \x01(\tR\x0etectonObjectId\x12^\n\x04tags\x18\x08 \x03(\x0b\x32\x41.tecton_proto.consumption.IngestApiComputeHoursMetadata.TagsEntryB\x07\xf2\x99\xbd\x46\x02\x08\x01R\x04tags\x1a\x37\n\tTagsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\x9d\x01\n\x1e\x46\x65\x61tureServerNodeHoursMetadata\x12\x1f\n\x06region\x18\x01 \x01(\tB\x07\xf2\x99\xbd\x46\x02\x08\x01R\x06region\x12\x17\n\x07pod_cpu\x18\x02 \x01(\x01R\x06podCpu\x12$\n\x0epod_memory_mib\x18\x03 \x01(\rR\x0cpodMemoryMib\x12\x1b\n\tpod_count\x18\x04 \x01(\rR\x08podCount\"\xad\x01\n\x1a\x46\x65\x61tureServerReadsMetadata\x12%\n\tworkspace\x18\x01 \x01(\tB\x07\xf2\x99\xbd\x46\x02\x08\x01R\tworkspace\x12\x35\n\x12tecton_object_name\x18\x02 \x01(\tB\x07\xf2\x99\xbd\x46\x02\x08\x01R\x10tectonObjectName\x12\x31\n\x10tecton_object_id\x18\x03 \x01(\tB\x07\xf2\x99\xbd\x46\x02\x10\x01R\x0etectonObjectId\"\x94\x03\n\x17RealTimeComputeMetadata\x12\x30\n\x14memory_allocated_mib\x18\x01 \x01(\x04R\x12memoryAllocatedMib\x12%\n\tworkspace\x18\x02 \x01(\tB\x07\xf2\x99\xbd\x46\x02\x08\x01R\tworkspace\x12,\n\x12workspace_state_id\x18\x03 \x01(\tR\x10workspaceStateId\x12\x35\n\x12tecton_object_name\x18\x04 \x01(\tB\x07\xf2\x99\xbd\x46\x02\x08\x01R\x10tectonObjectName\x12(\n\x10tecton_object_id\x18\x05 \x01(\tR\x0etectonObjectId\x12X\n\x04tags\x18\x06 \x03(\x0b\x32;.tecton_proto.consumption.RealTimeComputeMetadata.TagsEntryB\x07\xf2\x99\xbd\x46\x02\x08\x01R\x04tags\x1a\x37\n\tTagsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xfb\t\n\x11\x43onsumptionRecord\x12\x41\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x07\xf2\x99\xbd\x46\x02\x08\x01R\ttimestamp\x12M\n\x14\x63ollection_timestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x13\x63ollectionTimestamp\x12>\n\x08\x64uration\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationB\x07\xf2\x99\xbd\x46\x02\x10\x01R\x08\x64uration\x12*\n\x0c\x61\x63\x63ount_name\x18\x04 \x01(\tB\x07\xf2\x99\xbd\x46\x02\x08\x01R\x0b\x61\x63\x63ountName\x12\x89\x01\n*materialization_job_online_writes_metadata\x18\x07 \x01(\x0b\x32@.tecton_proto.consumption.MaterializationJobOnlineWritesMetadataB\x07\xf2\x99\xbd\x46\x02\x08\x01H\x00R\x08metadata\x12\x8b\x01\n+materialization_job_offline_writes_metadata\x18\x08 \x01(\x0b\x32\x41.tecton_proto.consumption.MaterializationJobOfflineWritesMetadataB\x07\xf2\x99\xbd\x46\x02\x08\x01H\x00R\x08metadata\x12y\n\"feature_server_node_hours_metadata\x18\n \x01(\x0b\x32\x38.tecton_proto.consumption.FeatureServerNodeHoursMetadataB\x07\xf2\x99\xbd\x46\x02\x08\x01H\x00R\x08metadata\x12p\n\x1d\x66\x65\x61ture_server_reads_metadata\x18\x0e \x01(\x0b\x32\x34.tecton_proto.consumption.FeatureServerReadsMetadataB\x07\xf2\x99\xbd\x46\x02\x08\x01H\x00R\x08metadata\x12j\n\x1areal_time_compute_metadata\x18\x0c \x01(\x0b\x32\x31.tecton_proto.consumption.RealTimeComputeMetadataB\x07\xf2\x99\xbd\x46\x02\x08\x01H\x00R\x08metadata\x12w\n!tecton_job_compute_hours_metadata\x18\r \x01(\x0b\x32\x37.tecton_proto.consumption.TectonJobComputeHoursMetadataB\x07\xf2\x99\xbd\x46\x02\x08\x01H\x00R\x08metadata\x12w\n!ingest_api_compute_hours_metadata\x18\x0f \x01(\x0b\x32\x37.tecton_proto.consumption.IngestApiComputeHoursMetadataB\x07\xf2\x99\xbd\x46\x02\x08\x01H\x00R\x08metadata\x12#\n\x08quantity\x18\x05 \x01(\x01\x42\x07\xf2\x99\xbd\x46\x02\x08\x01R\x08quantity\x12\x46\n\x04unit\x18\x06 \x01(\x0e\x32).tecton_proto.consumption.ConsumptionUnitB\x07\xf2\x99\xbd\x46\x02\x08\x01R\x04unitB\n\n\x08metadataJ\x04\x08\t\x10\nJ\x04\x08\x0b\x10\x0c*\xb6\x01\n\x07JobType\x12\x18\n\x14JOB_TYPE_UNSPECIFIED\x10\x00\x12\x1a\n\x16STREAM_MATERIALIZATION\x10\x01\x12\x19\n\x15\x42\x41TCH_MATERIALIZATION\x10\x02\x12\x13\n\x0f\x45NTITY_DELETION\x10\x03\x12\x13\n\x0f\x46\x45\x41TURE_PUBLISH\x10\x04\x12\x18\n\x14\x46\x45\x41TURE_TABLE_INGEST\x10\x05\x12\x16\n\x12\x44\x41TASET_GENERATION\x10\x06*N\n\x0b\x43omputeType\x12\x1c\n\x18\x43OMPUTE_TYPE_UNSPECIFIED\x10\x00\x12\x07\n\x03\x45MR\x10\x01\x12\x0e\n\nDATABRICKS\x10\x02\x12\x08\n\x04RIFT\x10\x03*\xf3\x02\n\x0f\x43onsumptionUnit\x12!\n\x1d\x43ONSUMPTION_UNITS_UNSPECIFIED\x10\x00\x12\x15\n\x11ONLINE_WRITE_ROWS\x10\x01\x12\x16\n\x12OFFLINE_WRITE_ROWS\x10\x02\x12\x18\n\x14OFFLINE_WRITE_VALUES\x10\x03\x12\x1c\n\x18TECTON_JOB_COMPUTE_HOURS\x10\x04\x12\x1d\n\x19\x46\x45\x41TURE_SERVER_NODE_HOURS\x10\x05\x12$\n REAL_TIME_COMPUTE_DURATION_HOURS\x10\t\x12\"\n\x1e\x46\x45\x41TURE_SERVICE_VECTORS_SERVED\x10\x06\x12#\n\x1f\x46\x45\x41TURE_SERVICE_ONLINE_REQUESTS\x10\x07\x12\x1d\n\x19\x46\x45\x41TURE_VIEW_ONLINE_READS\x10\x08\x12)\n%FEATURE_SERVICE_ONLINE_VECTORS_SERVED\x10\n*@\n\nVisibility\x12\x1a\n\x16VISIBILITY_UNSPECIFIED\x10\x00\x12\x16\n\x12VISIBILITY_VISIBLE\x10\x01*b\n\x0bRequirement\x12\x1b\n\x17REQUIREMENT_UNSPECIFIED\x10\x00\x12\x1c\n\x18REQUIREMENT_NOT_REQUIRED\x10\x01\x12\x18\n\x14REQUIREMENT_REQUIRED\x10\x02:\x86\x01\n\x16\x62illable_usage_options\x12\x1d.google.protobuf.FieldOptions\x18\x9e\xd3\xe7\x08 \x01(\x0b\x32..tecton_proto.consumption.BillableUsageOptionsR\x14\x62illableUsageOptionsB\x1a\n\x16\x63om.tecton.consumptionP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.consumption.consumption__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(billable_usage_options)

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.tecton.consumptionP\001'
  _CONSUMPTIONINFO_DETAILSENTRY._options = None
  _CONSUMPTIONINFO_DETAILSENTRY._serialized_options = b'8\001'
  _MATERIALIZATIONJOBONLINEWRITESMETADATA_TAGSENTRY._options = None
  _MATERIALIZATIONJOBONLINEWRITESMETADATA_TAGSENTRY._serialized_options = b'8\001'
  _MATERIALIZATIONJOBONLINEWRITESMETADATA.fields_by_name['workspace']._options = None
  _MATERIALIZATIONJOBONLINEWRITESMETADATA.fields_by_name['workspace']._serialized_options = b'\362\231\275F\002\010\001'
  _MATERIALIZATIONJOBONLINEWRITESMETADATA.fields_by_name['tecton_object_name']._options = None
  _MATERIALIZATIONJOBONLINEWRITESMETADATA.fields_by_name['tecton_object_name']._serialized_options = b'\362\231\275F\002\010\001'
  _MATERIALIZATIONJOBONLINEWRITESMETADATA.fields_by_name['tags']._options = None
  _MATERIALIZATIONJOBONLINEWRITESMETADATA.fields_by_name['tags']._serialized_options = b'\362\231\275F\002\010\001'
  _MATERIALIZATIONJOBOFFLINEWRITESMETADATA_TAGSENTRY._options = None
  _MATERIALIZATIONJOBOFFLINEWRITESMETADATA_TAGSENTRY._serialized_options = b'8\001'
  _MATERIALIZATIONJOBOFFLINEWRITESMETADATA.fields_by_name['workspace']._options = None
  _MATERIALIZATIONJOBOFFLINEWRITESMETADATA.fields_by_name['workspace']._serialized_options = b'\362\231\275F\002\010\001'
  _MATERIALIZATIONJOBOFFLINEWRITESMETADATA.fields_by_name['tecton_object_name']._options = None
  _MATERIALIZATIONJOBOFFLINEWRITESMETADATA.fields_by_name['tecton_object_name']._serialized_options = b'\362\231\275F\002\010\001'
  _MATERIALIZATIONJOBOFFLINEWRITESMETADATA.fields_by_name['tags']._options = None
  _MATERIALIZATIONJOBOFFLINEWRITESMETADATA.fields_by_name['tags']._serialized_options = b'\362\231\275F\002\010\001'
  _TECTONJOBCOMPUTEHOURSMETADATA_TAGSENTRY._options = None
  _TECTONJOBCOMPUTEHOURSMETADATA_TAGSENTRY._serialized_options = b'8\001'
  _TECTONJOBCOMPUTEHOURSMETADATA.fields_by_name['tecton_job_id']._options = None
  _TECTONJOBCOMPUTEHOURSMETADATA.fields_by_name['tecton_job_id']._serialized_options = b'\362\231\275F\002\010\001'
  _TECTONJOBCOMPUTEHOURSMETADATA.fields_by_name['instance_type']._options = None
  _TECTONJOBCOMPUTEHOURSMETADATA.fields_by_name['instance_type']._serialized_options = b'\362\231\275F\002\010\001'
  _TECTONJOBCOMPUTEHOURSMETADATA.fields_by_name['num_workers']._options = None
  _TECTONJOBCOMPUTEHOURSMETADATA.fields_by_name['num_workers']._serialized_options = b'\362\231\275F\002\020\001'
  _TECTONJOBCOMPUTEHOURSMETADATA.fields_by_name['compute_type']._options = None
  _TECTONJOBCOMPUTEHOURSMETADATA.fields_by_name['compute_type']._serialized_options = b'\362\231\275F\002\010\001'
  _TECTONJOBCOMPUTEHOURSMETADATA.fields_by_name['job_type']._options = None
  _TECTONJOBCOMPUTEHOURSMETADATA.fields_by_name['job_type']._serialized_options = b'\362\231\275F\002\010\001'
  _TECTONJOBCOMPUTEHOURSMETADATA.fields_by_name['workspace']._options = None
  _TECTONJOBCOMPUTEHOURSMETADATA.fields_by_name['workspace']._serialized_options = b'\362\231\275F\002\010\001'
  _TECTONJOBCOMPUTEHOURSMETADATA.fields_by_name['tecton_object_name']._options = None
  _TECTONJOBCOMPUTEHOURSMETADATA.fields_by_name['tecton_object_name']._serialized_options = b'\362\231\275F\002\010\001'
  _TECTONJOBCOMPUTEHOURSMETADATA.fields_by_name['tags']._options = None
  _TECTONJOBCOMPUTEHOURSMETADATA.fields_by_name['tags']._serialized_options = b'\362\231\275F\002\010\001'
  _INGESTAPICOMPUTEHOURSMETADATA_TAGSENTRY._options = None
  _INGESTAPICOMPUTEHOURSMETADATA_TAGSENTRY._serialized_options = b'8\001'
  _INGESTAPICOMPUTEHOURSMETADATA.fields_by_name['workspace']._options = None
  _INGESTAPICOMPUTEHOURSMETADATA.fields_by_name['workspace']._serialized_options = b'\362\231\275F\002\010\001'
  _INGESTAPICOMPUTEHOURSMETADATA.fields_by_name['tecton_object_name']._options = None
  _INGESTAPICOMPUTEHOURSMETADATA.fields_by_name['tecton_object_name']._serialized_options = b'\362\231\275F\002\010\001'
  _INGESTAPICOMPUTEHOURSMETADATA.fields_by_name['tags']._options = None
  _INGESTAPICOMPUTEHOURSMETADATA.fields_by_name['tags']._serialized_options = b'\362\231\275F\002\010\001'
  _FEATURESERVERNODEHOURSMETADATA.fields_by_name['region']._options = None
  _FEATURESERVERNODEHOURSMETADATA.fields_by_name['region']._serialized_options = b'\362\231\275F\002\010\001'
  _FEATURESERVERREADSMETADATA.fields_by_name['workspace']._options = None
  _FEATURESERVERREADSMETADATA.fields_by_name['workspace']._serialized_options = b'\362\231\275F\002\010\001'
  _FEATURESERVERREADSMETADATA.fields_by_name['tecton_object_name']._options = None
  _FEATURESERVERREADSMETADATA.fields_by_name['tecton_object_name']._serialized_options = b'\362\231\275F\002\010\001'
  _FEATURESERVERREADSMETADATA.fields_by_name['tecton_object_id']._options = None
  _FEATURESERVERREADSMETADATA.fields_by_name['tecton_object_id']._serialized_options = b'\362\231\275F\002\020\001'
  _REALTIMECOMPUTEMETADATA_TAGSENTRY._options = None
  _REALTIMECOMPUTEMETADATA_TAGSENTRY._serialized_options = b'8\001'
  _REALTIMECOMPUTEMETADATA.fields_by_name['workspace']._options = None
  _REALTIMECOMPUTEMETADATA.fields_by_name['workspace']._serialized_options = b'\362\231\275F\002\010\001'
  _REALTIMECOMPUTEMETADATA.fields_by_name['tecton_object_name']._options = None
  _REALTIMECOMPUTEMETADATA.fields_by_name['tecton_object_name']._serialized_options = b'\362\231\275F\002\010\001'
  _REALTIMECOMPUTEMETADATA.fields_by_name['tags']._options = None
  _REALTIMECOMPUTEMETADATA.fields_by_name['tags']._serialized_options = b'\362\231\275F\002\010\001'
  _CONSUMPTIONRECORD.fields_by_name['timestamp']._options = None
  _CONSUMPTIONRECORD.fields_by_name['timestamp']._serialized_options = b'\362\231\275F\002\010\001'
  _CONSUMPTIONRECORD.fields_by_name['duration']._options = None
  _CONSUMPTIONRECORD.fields_by_name['duration']._serialized_options = b'\362\231\275F\002\020\001'
  _CONSUMPTIONRECORD.fields_by_name['account_name']._options = None
  _CONSUMPTIONRECORD.fields_by_name['account_name']._serialized_options = b'\362\231\275F\002\010\001'
  _CONSUMPTIONRECORD.fields_by_name['materialization_job_online_writes_metadata']._options = None
  _CONSUMPTIONRECORD.fields_by_name['materialization_job_online_writes_metadata']._serialized_options = b'\362\231\275F\002\010\001'
  _CONSUMPTIONRECORD.fields_by_name['materialization_job_offline_writes_metadata']._options = None
  _CONSUMPTIONRECORD.fields_by_name['materialization_job_offline_writes_metadata']._serialized_options = b'\362\231\275F\002\010\001'
  _CONSUMPTIONRECORD.fields_by_name['feature_server_node_hours_metadata']._options = None
  _CONSUMPTIONRECORD.fields_by_name['feature_server_node_hours_metadata']._serialized_options = b'\362\231\275F\002\010\001'
  _CONSUMPTIONRECORD.fields_by_name['feature_server_reads_metadata']._options = None
  _CONSUMPTIONRECORD.fields_by_name['feature_server_reads_metadata']._serialized_options = b'\362\231\275F\002\010\001'
  _CONSUMPTIONRECORD.fields_by_name['real_time_compute_metadata']._options = None
  _CONSUMPTIONRECORD.fields_by_name['real_time_compute_metadata']._serialized_options = b'\362\231\275F\002\010\001'
  _CONSUMPTIONRECORD.fields_by_name['tecton_job_compute_hours_metadata']._options = None
  _CONSUMPTIONRECORD.fields_by_name['tecton_job_compute_hours_metadata']._serialized_options = b'\362\231\275F\002\010\001'
  _CONSUMPTIONRECORD.fields_by_name['ingest_api_compute_hours_metadata']._options = None
  _CONSUMPTIONRECORD.fields_by_name['ingest_api_compute_hours_metadata']._serialized_options = b'\362\231\275F\002\010\001'
  _CONSUMPTIONRECORD.fields_by_name['quantity']._options = None
  _CONSUMPTIONRECORD.fields_by_name['quantity']._serialized_options = b'\362\231\275F\002\010\001'
  _CONSUMPTIONRECORD.fields_by_name['unit']._options = None
  _CONSUMPTIONRECORD.fields_by_name['unit']._serialized_options = b'\362\231\275F\002\010\001'
  _JOBTYPE._serialized_start=5315
  _JOBTYPE._serialized_end=5497
  _COMPUTETYPE._serialized_start=5499
  _COMPUTETYPE._serialized_end=5577
  _CONSUMPTIONUNIT._serialized_start=5580
  _CONSUMPTIONUNIT._serialized_end=5951
  _VISIBILITY._serialized_start=5953
  _VISIBILITY._serialized_end=6017
  _REQUIREMENT._serialized_start=6019
  _REQUIREMENT._serialized_end=6117
  _CONSUMPTIONINFO._serialized_start=218
  _CONSUMPTIONINFO._serialized_end=733
  _CONSUMPTIONINFO_DETAILSENTRY._serialized_start=675
  _CONSUMPTIONINFO_DETAILSENTRY._serialized_end=733
  _ENRICHEDCONSUMPTIONINFO._serialized_start=736
  _ENRICHEDCONSUMPTIONINFO._serialized_end=991
  _BILLABLEUSAGEOPTIONS._serialized_start=994
  _BILLABLEUSAGEOPTIONS._serialized_end=1153
  _MATERIALIZATIONJOBONLINEWRITESMETADATA._serialized_start=1156
  _MATERIALIZATIONJOBONLINEWRITESMETADATA._serialized_end=1620
  _MATERIALIZATIONJOBONLINEWRITESMETADATA_TAGSENTRY._serialized_start=1565
  _MATERIALIZATIONJOBONLINEWRITESMETADATA_TAGSENTRY._serialized_end=1620
  _MATERIALIZATIONJOBOFFLINEWRITESMETADATA._serialized_start=1623
  _MATERIALIZATIONJOBOFFLINEWRITESMETADATA._serialized_end=2045
  _MATERIALIZATIONJOBOFFLINEWRITESMETADATA_TAGSENTRY._serialized_start=1565
  _MATERIALIZATIONJOBOFFLINEWRITESMETADATA_TAGSENTRY._serialized_end=1620
  _TECTONJOBCOMPUTEHOURSMETADATA._serialized_start=2048
  _TECTONJOBCOMPUTEHOURSMETADATA._serialized_end=2725
  _TECTONJOBCOMPUTEHOURSMETADATA_TAGSENTRY._serialized_start=1565
  _TECTONJOBCOMPUTEHOURSMETADATA_TAGSENTRY._serialized_end=1620
  _INGESTAPICOMPUTEHOURSMETADATA._serialized_start=2728
  _INGESTAPICOMPUTEHOURSMETADATA._serialized_end=3291
  _INGESTAPICOMPUTEHOURSMETADATA_TAGSENTRY._serialized_start=1565
  _INGESTAPICOMPUTEHOURSMETADATA_TAGSENTRY._serialized_end=1620
  _FEATURESERVERNODEHOURSMETADATA._serialized_start=3294
  _FEATURESERVERNODEHOURSMETADATA._serialized_end=3451
  _FEATURESERVERREADSMETADATA._serialized_start=3454
  _FEATURESERVERREADSMETADATA._serialized_end=3627
  _REALTIMECOMPUTEMETADATA._serialized_start=3630
  _REALTIMECOMPUTEMETADATA._serialized_end=4034
  _REALTIMECOMPUTEMETADATA_TAGSENTRY._serialized_start=1565
  _REALTIMECOMPUTEMETADATA_TAGSENTRY._serialized_end=1620
  _CONSUMPTIONRECORD._serialized_start=4037
  _CONSUMPTIONRECORD._serialized_end=5312
# @@protoc_insertion_point(module_scope)
