# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/fco_args__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.args import entity__client_pb2 as tecton__proto_dot_args_dot_entity____client__pb2
from tecton_proto.args import feature_service__client_pb2 as tecton__proto_dot_args_dot_feature__service____client__pb2
from tecton_proto.args import feature_view__client_pb2 as tecton__proto_dot_args_dot_feature__view____client__pb2
from tecton_proto.args import transformation__client_pb2 as tecton__proto_dot_args_dot_transformation____client__pb2
from tecton_proto.args import virtual_data_source__client_pb2 as tecton__proto_dot_args_dot_virtual__data__source____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(tecton_proto/args/fco_args__client.proto\x12\x11tecton_proto.args\x1a&tecton_proto/args/entity__client.proto\x1a/tecton_proto/args/feature_service__client.proto\x1a,tecton_proto/args/feature_view__client.proto\x1a.tecton_proto/args/transformation__client.proto\x1a\x33tecton_proto/args/virtual_data_source__client.proto\"\x9e\x03\n\x07\x46\x63oArgs\x12Z\n\x13virtual_data_source\x18\x01 \x01(\x0b\x32(.tecton_proto.args.VirtualDataSourceArgsH\x00R\x11virtualDataSource\x12\x37\n\x06\x65ntity\x18\x02 \x01(\x0b\x32\x1d.tecton_proto.args.EntityArgsH\x00R\x06\x65ntity\x12G\n\x0c\x66\x65\x61ture_view\x18\x06 \x01(\x0b\x32\".tecton_proto.args.FeatureViewArgsH\x00R\x0b\x66\x65\x61tureView\x12P\n\x0f\x66\x65\x61ture_service\x18\x04 \x01(\x0b\x32%.tecton_proto.args.FeatureServiceArgsH\x00R\x0e\x66\x65\x61tureService\x12O\n\x0etransformation\x18\x07 \x01(\x0b\x32%.tecton_proto.args.TransformationArgsH\x00R\x0etransformationB\x06\n\x04\x61rgsJ\x04\x08\x03\x10\x04J\x04\x08\x05\x10\x06\x42\x13\n\x0f\x63om.tecton.argsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.fco_args__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001'
  _FCOARGS._serialized_start=300
  _FCOARGS._serialized_end=714
# @@protoc_insertion_point(module_scope)
