# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/common/analytics_options__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n3tecton_proto/common/analytics_options__client.proto\x12\x13tecton_proto.common\x1a google/protobuf/descriptor.proto\"V\n\tAnalytics\x12I\n\x0b\x66ilter_type\x18\x01 \x01(\x0e\x32(.tecton_proto.common.AnalyticsFilterTypeR\nfilterType*8\n\x13\x41nalyticsFilterType\x12\x0f\n\x0bNOOP_FILTER\x10\x00\x12\x10\n\x0cPSEUDONYMIZE\x10\x01:\\\n\tanalytics\x12\x1d.google.protobuf.FieldOptions\x18\xae, \x01(\x0b\x32\x1e.tecton_proto.common.AnalyticsR\tanalyticsB\x15\n\x11\x63om.tecton.commonP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.common.analytics_options__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(analytics)

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021com.tecton.commonP\001'
  _ANALYTICSFILTERTYPE._serialized_start=198
  _ANALYTICSFILTERTYPE._serialized_end=254
  _ANALYTICS._serialized_start=110
  _ANALYTICS._serialized_end=196
# @@protoc_insertion_point(module_scope)
