# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/entity__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.args import basic_info__client_pb2 as tecton__proto_dot_args_dot_basic__info____client__pb2
from tecton_proto.args import diff_options__client_pb2 as tecton__proto_dot_args_dot_diff__options____client__pb2
from tecton_proto.common import framework_version__client_pb2 as tecton__proto_dot_common_dot_framework__version____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import schema__client_pb2 as tecton__proto_dot_common_dot_schema____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&tecton_proto/args/entity__client.proto\x12\x11tecton_proto.args\x1a*tecton_proto/args/basic_info__client.proto\x1a,tecton_proto/args/diff_options__client.proto\x1a\x33tecton_proto/common/framework_version__client.proto\x1a$tecton_proto/common/id__client.proto\x1a(tecton_proto/common/schema__client.proto\"\x92\x04\n\nEntityArgs\x12\x34\n\tentity_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x08\x65ntityId\x12\x37\n\x04info\x18\x02 \x01(\x0b\x32\x1c.tecton_proto.args.BasicInfoB\x05\x92M\x02\x10\x01R\x04info\x12\x46\n\x07version\x18\x04 \x01(\x0e\x32%.tecton_proto.common.FrameworkVersionB\x05\x92M\x02\x08\x05R\x07version\x12.\n\x0fprevent_destroy\x18\x05 \x01(\x08\x42\x05\x92M\x02\x08\x01R\x0epreventDestroy\x12K\n\x07options\x18\x06 \x03(\x0b\x32*.tecton_proto.args.EntityArgs.OptionsEntryB\x05\x92M\x02\x08\x01R\x07options\x12N\n\x10join_keys_legacy\x18\x03 \x03(\tB$\x92M\r*\x0b\n\tjoin_keys\x92M\x0c*\n\x12\x08\x30.10.0b0\x92M\x02 \x0fR\x0ejoinKeysLegacy\x12\x44\n\tjoin_keys\x18\x07 \x03(\x0b\x32\x1b.tecton_proto.common.ColumnB\n\x92M\x02 \x0f\x92M\x02\x08\x06R\x08joinKeys\x1a:\n\x0cOptionsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x13\n\x0f\x63om.tecton.argsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.entity__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001'
  _ENTITYARGS_OPTIONSENTRY._options = None
  _ENTITYARGS_OPTIONSENTRY._serialized_options = b'8\001'
  _ENTITYARGS.fields_by_name['info']._options = None
  _ENTITYARGS.fields_by_name['info']._serialized_options = b'\222M\002\020\001'
  _ENTITYARGS.fields_by_name['version']._options = None
  _ENTITYARGS.fields_by_name['version']._serialized_options = b'\222M\002\010\005'
  _ENTITYARGS.fields_by_name['prevent_destroy']._options = None
  _ENTITYARGS.fields_by_name['prevent_destroy']._serialized_options = b'\222M\002\010\001'
  _ENTITYARGS.fields_by_name['options']._options = None
  _ENTITYARGS.fields_by_name['options']._serialized_options = b'\222M\002\010\001'
  _ENTITYARGS.fields_by_name['join_keys_legacy']._options = None
  _ENTITYARGS.fields_by_name['join_keys_legacy']._serialized_options = b'\222M\r*\013\n\tjoin_keys\222M\014*\n\022\0100.10.0b0\222M\002 \017'
  _ENTITYARGS.fields_by_name['join_keys']._options = None
  _ENTITYARGS.fields_by_name['join_keys']._serialized_options = b'\222M\002 \017\222M\002\010\006'
  _ENTITYARGS._serialized_start=285
  _ENTITYARGS._serialized_end=815
  _ENTITYARGS_OPTIONSENTRY._serialized_start=757
  _ENTITYARGS_OPTIONSENTRY._serialized_end=815
# @@protoc_insertion_point(module_scope)
