# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/dataobs/validation__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.dataobs import expectation__client_pb2 as tecton__proto_dot_dataobs_dot_expectation____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-tecton_proto/dataobs/validation__client.proto\x12\x14tecton_proto.dataobs\x1a\x1fgoogle/protobuf/timestamp.proto\x1a$tecton_proto/common/id__client.proto\x1a.tecton_proto/dataobs/expectation__client.proto\"\xd0\x06\n\x11\x45xpectationResult\x12\x43\n\x11validation_job_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x0fvalidationJobId\x12\x1c\n\tworkspace\x18\x02 \x01(\tR\tworkspace\x12*\n\x11\x66\x65\x61ture_view_name\x18\t \x01(\tR\x0f\x66\x65\x61tureViewName\x12\x45\n\x12\x66\x65\x61ture_package_id\x18\x03 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x10\x66\x65\x61turePackageId\x12\x43\n\x0fvalidation_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0evalidationTime\x12Y\n\x1b\x66\x65\x61ture_interval_start_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x18\x66\x65\x61tureIntervalStartTime\x12U\n\x19\x66\x65\x61ture_interval_end_time\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x16\x66\x65\x61tureIntervalEndTime\x12t\n\x1c\x66\x65\x61ture_expectation_metadata\x18\x06 \x01(\x0b\x32\x30.tecton_proto.dataobs.FeatureExpectationMetadataH\x00R\x1a\x66\x65\x61tureExpectationMetadata\x12q\n\x1bmetric_expectation_metadata\x18\x07 \x01(\x0b\x32/.tecton_proto.dataobs.MetricExpectationMetadataH\x00R\x19metricExpectationMetadata\x12\x43\n\x06result\x18\x08 \x01(\x0e\x32+.tecton_proto.dataobs.ExpectationResultEnumR\x06result\x12\x34\n\tresult_id\x18\n \x01(\x0b\x32\x17.tecton_proto.common.IdR\x08resultIdB\n\n\x08metadata\"\xeb\x01\n\x1a\x46\x65\x61tureExpectationMetadata\x12J\n\x0b\x65xpectation\x18\x01 \x01(\x0b\x32(.tecton_proto.dataobs.FeatureExpectationR\x0b\x65xpectation\x12\x1b\n\talert_msg\x18\x03 \x01(\tR\x08\x61lertMsg\x12-\n\x12\x66\x61ilure_percentage\x18\x04 \x01(\x01R\x11\x66\x61ilurePercentage\x12\x35\n\x17\x66\x61iled_join_key_samples\x18\x05 \x03(\tR\x14\x66\x61iledJoinKeySamples\"\x81\x03\n\x19MetricExpectationMetadata\x12I\n\x0b\x65xpectation\x18\x01 \x01(\x0b\x32\'.tecton_proto.dataobs.MetricExpectationR\x0b\x65xpectation\x12\x1b\n\talert_msg\x18\x03 \x01(\tR\x08\x61lertMsg\x12]\n\x0cparam_values\x18\x04 \x03(\x0b\x32:.tecton_proto.dataobs.MetricExpectationMetadata.ParamValueR\x0bparamValues\x1a\x9c\x01\n\nParamValue\x12\x1f\n\x0bmetric_name\x18\x01 \x01(\tR\nmetricName\x12!\n\x0c\x61\x63tual_value\x18\x02 \x01(\tR\x0b\x61\x63tualValue\x12J\n\x13interval_start_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x11intervalStartTime\"o\n\rResultSummary\x12\x16\n\x06passed\x18\x01 \x01(\x05R\x06passed\x12\x16\n\x06\x66\x61iled\x18\x02 \x01(\x05R\x06\x66\x61iled\x12\x14\n\x05\x65rror\x18\x03 \x01(\x05R\x05\x65rror\x12\x18\n\x07unknown\x18\x04 \x01(\x05R\x07unknown\"\xd7\x01\n\x16WorkspaceResultSummary\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12=\n\x07summary\x18\x02 \x01(\x0b\x32#.tecton_proto.dataobs.ResultSummaryR\x07summary\x12`\n\x14\x66\x65\x61ture_view_summary\x18\x03 \x03(\x0b\x32..tecton_proto.dataobs.FeatureViewResultSummaryR\x12\x66\x65\x61tureViewSummary\"\xe6\x01\n\x18\x46\x65\x61tureViewResultSummary\x12*\n\x11\x66\x65\x61ture_view_name\x18\x01 \x01(\tR\x0f\x66\x65\x61tureViewName\x12=\n\x07summary\x18\x02 \x01(\x0b\x32#.tecton_proto.dataobs.ResultSummaryR\x07summary\x12_\n\x13\x65xpectation_summary\x18\x03 \x03(\x0b\x32..tecton_proto.dataobs.ExpectationResultSummaryR\x12\x65xpectationSummary\"\x84\x01\n\x18\x45xpectationResultSummary\x12)\n\x10\x65xpectation_name\x18\x01 \x01(\tR\x0f\x65xpectationName\x12=\n\x07summary\x18\x02 \x01(\x0b\x32#.tecton_proto.dataobs.ResultSummaryR\x07summary*c\n\x15\x45xpectationResultEnum\x12\x12\n\x0eRESULT_UNKNOWN\x10\x00\x12\x11\n\rRESULT_PASSED\x10\x01\x12\x11\n\rRESULT_FAILED\x10\x02\x12\x10\n\x0cRESULT_ERROR\x10\x03\x42\x16\n\x12\x63om.tecton.dataobsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.dataobs.validation__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\022com.tecton.dataobsP\001'
  _EXPECTATIONRESULTENUM._serialized_start=2366
  _EXPECTATIONRESULTENUM._serialized_end=2465
  _EXPECTATIONRESULT._serialized_start=191
  _EXPECTATIONRESULT._serialized_end=1039
  _FEATUREEXPECTATIONMETADATA._serialized_start=1042
  _FEATUREEXPECTATIONMETADATA._serialized_end=1277
  _METRICEXPECTATIONMETADATA._serialized_start=1280
  _METRICEXPECTATIONMETADATA._serialized_end=1665
  _METRICEXPECTATIONMETADATA_PARAMVALUE._serialized_start=1509
  _METRICEXPECTATIONMETADATA_PARAMVALUE._serialized_end=1665
  _RESULTSUMMARY._serialized_start=1667
  _RESULTSUMMARY._serialized_end=1778
  _WORKSPACERESULTSUMMARY._serialized_start=1781
  _WORKSPACERESULTSUMMARY._serialized_end=1996
  _FEATUREVIEWRESULTSUMMARY._serialized_start=1999
  _FEATUREVIEWRESULTSUMMARY._serialized_end=2229
  _EXPECTATIONRESULTSUMMARY._serialized_start=2232
  _EXPECTATIONRESULTSUMMARY._serialized_end=2364
# @@protoc_insertion_point(module_scope)
