# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/secrets/secrets_service__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.auditlog import metadata__client_pb2 as tecton__proto_dot_auditlog_dot_metadata____client__pb2
from tecton_proto.auth import service__client_pb2 as tecton__proto_dot_auth_dot_service____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2tecton_proto/secrets/secrets_service__client.proto\x12\x14tecton_proto.secrets\x1a,tecton_proto/auditlog/metadata__client.proto\x1a\'tecton_proto/auth/service__client.proto\"!\n\x0bSecretScope\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\"\x1f\n\tSecretKey\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\"\x19\n\x17ListSecretScopesRequest\"U\n\x18ListSecretScopesResponse\x12\x39\n\x06scopes\x18\x01 \x03(\x0b\x32!.tecton_proto.secrets.SecretScopeR\x06scopes\"9\n\x18\x43reateSecretScopeRequest\x12\x1d\n\x05scope\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x05scope\"\x1b\n\x19\x43reateSecretScopeResponse\"9\n\x18\x44\x65leteSecretScopeRequest\x12\x1d\n\x05scope\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x05scope\"\x1b\n\x19\x44\x65leteSecretScopeResponse\"*\n\x12ListSecretsRequest\x12\x14\n\x05scope\x18\x01 \x01(\tR\x05scope\"J\n\x13ListSecretsResponse\x12\x33\n\x04keys\x18\x02 \x03(\x0b\x32\x1f.tecton_proto.secrets.SecretKeyR\x04keys\"?\n\x15GetSecretValueRequest\x12\x14\n\x05scope\x18\x01 \x01(\tR\x05scope\x12\x10\n\x03key\x18\x02 \x01(\tR\x03key\".\n\x16GetSecretValueResponse\x12\x14\n\x05value\x18\x01 \x01(\tR\x05value\"w\n\x15PutSecretValueRequest\x12\x1d\n\x05scope\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x05scope\x12\x19\n\x03key\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x03key\x12$\n\x05value\x18\x03 \x01(\tB\x0e\xea\x99\xbd\x46\x02\x08\x02\xea\x99\xbd\x46\x02\x10\x02R\x05value\"\x18\n\x16PutSecretValueResponse\"O\n\x13\x44\x65leteSecretRequest\x12\x1d\n\x05scope\x18\x01 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x05scope\x12\x19\n\x03key\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x03key\"\x16\n\x14\x44\x65leteSecretResponse2\xee\t\n\x0eSecretsService\x12{\n\x10ListSecretScopes\x12-.tecton_proto.secrets.ListSecretScopesRequest\x1a..tecton_proto.secrets.ListSecretScopesResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\xc4\x01\n\x11\x43reateSecretScope\x12..tecton_proto.secrets.CreateSecretScopeRequest\x1a/.tecton_proto.secrets.CreateSecretScopeResponse\"N\xa2\xbc\xe6\xc0\x05)*\'tecton/organization#create_secret_scope\xf2\x99\xbd\x46\x1a\x08\x01\x12\x13\x63reate_secret_scope\x1a\x01\x31\x12\xcf\x01\n\x11\x44\x65leteSecretScope\x12..tecton_proto.secrets.DeleteSecretScopeRequest\x1a/.tecton_proto.secrets.DeleteSecretScopeResponse\"Y\xa2\xbc\xe6\xc0\x05\x34\x1a\t\x08\x04\x12\x05scope*\'tecton/secret_scope#delete_secret_scope\xf2\x99\xbd\x46\x1a\x08\x01\x12\x13\x64\x65lete_secret_scope\x1a\x01\x31\x12\x9a\x01\n\x0bListSecrets\x12(.tecton_proto.secrets.ListSecretsRequest\x1a).tecton_proto.secrets.ListSecretsResponse\"6\xa2\xbc\xe6\xc0\x05\x30\x1a\t\x08\x04\x12\x05scope*#tecton/secret_scope#list_secret_key\x12\xa5\x01\n\x0eGetSecretValue\x12+.tecton_proto.secrets.GetSecretValueRequest\x1a,.tecton_proto.secrets.GetSecretValueResponse\"8\xa2\xbc\xe6\xc0\x05\x32\x1a\t\x08\x04\x12\x05scope*%tecton/secret_scope#read_secret_value\x12\xc3\x01\n\x0ePutSecretValue\x12+.tecton_proto.secrets.PutSecretValueRequest\x1a,.tecton_proto.secrets.PutSecretValueResponse\"V\xa2\xbc\xe6\xc0\x05\x34\x1a\t\x08\x04\x12\x05scope*\'tecton/secret_scope#manage_secret_value\xf2\x99\xbd\x46\x17\x08\x01\x12\x10put_secret_value\x1a\x01\x31\x12\xba\x01\n\x0c\x44\x65leteSecret\x12).tecton_proto.secrets.DeleteSecretRequest\x1a*.tecton_proto.secrets.DeleteSecretResponse\"S\xa2\xbc\xe6\xc0\x05\x34\x1a\t\x08\x04\x12\x05scope*\'tecton/secret_scope#manage_secret_value\xf2\x99\xbd\x46\x14\x08\x01\x12\rdelete_secret\x1a\x01\x31\x42V\n\x12\x63om.tecton.secretsB\x13SecretsServiceProtoP\x01Z)github.com/tecton-ai/tecton_proto/secrets')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.secrets.secrets_service__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\022com.tecton.secretsB\023SecretsServiceProtoP\001Z)github.com/tecton-ai/tecton_proto/secrets'
  _CREATESECRETSCOPEREQUEST.fields_by_name['scope']._options = None
  _CREATESECRETSCOPEREQUEST.fields_by_name['scope']._serialized_options = b'\352\231\275F\002\010\002'
  _DELETESECRETSCOPEREQUEST.fields_by_name['scope']._options = None
  _DELETESECRETSCOPEREQUEST.fields_by_name['scope']._serialized_options = b'\352\231\275F\002\010\002'
  _PUTSECRETVALUEREQUEST.fields_by_name['scope']._options = None
  _PUTSECRETVALUEREQUEST.fields_by_name['scope']._serialized_options = b'\352\231\275F\002\010\002'
  _PUTSECRETVALUEREQUEST.fields_by_name['key']._options = None
  _PUTSECRETVALUEREQUEST.fields_by_name['key']._serialized_options = b'\352\231\275F\002\010\002'
  _PUTSECRETVALUEREQUEST.fields_by_name['value']._options = None
  _PUTSECRETVALUEREQUEST.fields_by_name['value']._serialized_options = b'\352\231\275F\002\010\002\352\231\275F\002\020\002'
  _DELETESECRETREQUEST.fields_by_name['scope']._options = None
  _DELETESECRETREQUEST.fields_by_name['scope']._serialized_options = b'\352\231\275F\002\010\002'
  _DELETESECRETREQUEST.fields_by_name['key']._options = None
  _DELETESECRETREQUEST.fields_by_name['key']._serialized_options = b'\352\231\275F\002\010\002'
  _SECRETSSERVICE.methods_by_name['ListSecretScopes']._options = None
  _SECRETSSERVICE.methods_by_name['ListSecretScopes']._serialized_options = b'\242\274\346\300\005\002@\001'
  _SECRETSSERVICE.methods_by_name['CreateSecretScope']._options = None
  _SECRETSSERVICE.methods_by_name['CreateSecretScope']._serialized_options = b'\242\274\346\300\005)*\'tecton/organization#create_secret_scope\362\231\275F\032\010\001\022\023create_secret_scope\032\0011'
  _SECRETSSERVICE.methods_by_name['DeleteSecretScope']._options = None
  _SECRETSSERVICE.methods_by_name['DeleteSecretScope']._serialized_options = b'\242\274\346\300\0054\032\t\010\004\022\005scope*\'tecton/secret_scope#delete_secret_scope\362\231\275F\032\010\001\022\023delete_secret_scope\032\0011'
  _SECRETSSERVICE.methods_by_name['ListSecrets']._options = None
  _SECRETSSERVICE.methods_by_name['ListSecrets']._serialized_options = b'\242\274\346\300\0050\032\t\010\004\022\005scope*#tecton/secret_scope#list_secret_key'
  _SECRETSSERVICE.methods_by_name['GetSecretValue']._options = None
  _SECRETSSERVICE.methods_by_name['GetSecretValue']._serialized_options = b'\242\274\346\300\0052\032\t\010\004\022\005scope*%tecton/secret_scope#read_secret_value'
  _SECRETSSERVICE.methods_by_name['PutSecretValue']._options = None
  _SECRETSSERVICE.methods_by_name['PutSecretValue']._serialized_options = b'\242\274\346\300\0054\032\t\010\004\022\005scope*\'tecton/secret_scope#manage_secret_value\362\231\275F\027\010\001\022\020put_secret_value\032\0011'
  _SECRETSSERVICE.methods_by_name['DeleteSecret']._options = None
  _SECRETSSERVICE.methods_by_name['DeleteSecret']._serialized_options = b'\242\274\346\300\0054\032\t\010\004\022\005scope*\'tecton/secret_scope#manage_secret_value\362\231\275F\024\010\001\022\rdelete_secret\032\0011'
  _SECRETSCOPE._serialized_start=163
  _SECRETSCOPE._serialized_end=196
  _SECRETKEY._serialized_start=198
  _SECRETKEY._serialized_end=229
  _LISTSECRETSCOPESREQUEST._serialized_start=231
  _LISTSECRETSCOPESREQUEST._serialized_end=256
  _LISTSECRETSCOPESRESPONSE._serialized_start=258
  _LISTSECRETSCOPESRESPONSE._serialized_end=343
  _CREATESECRETSCOPEREQUEST._serialized_start=345
  _CREATESECRETSCOPEREQUEST._serialized_end=402
  _CREATESECRETSCOPERESPONSE._serialized_start=404
  _CREATESECRETSCOPERESPONSE._serialized_end=431
  _DELETESECRETSCOPEREQUEST._serialized_start=433
  _DELETESECRETSCOPEREQUEST._serialized_end=490
  _DELETESECRETSCOPERESPONSE._serialized_start=492
  _DELETESECRETSCOPERESPONSE._serialized_end=519
  _LISTSECRETSREQUEST._serialized_start=521
  _LISTSECRETSREQUEST._serialized_end=563
  _LISTSECRETSRESPONSE._serialized_start=565
  _LISTSECRETSRESPONSE._serialized_end=639
  _GETSECRETVALUEREQUEST._serialized_start=641
  _GETSECRETVALUEREQUEST._serialized_end=704
  _GETSECRETVALUERESPONSE._serialized_start=706
  _GETSECRETVALUERESPONSE._serialized_end=752
  _PUTSECRETVALUEREQUEST._serialized_start=754
  _PUTSECRETVALUEREQUEST._serialized_end=873
  _PUTSECRETVALUERESPONSE._serialized_start=875
  _PUTSECRETVALUERESPONSE._serialized_end=899
  _DELETESECRETREQUEST._serialized_start=901
  _DELETESECRETREQUEST._serialized_end=980
  _DELETESECRETRESPONSE._serialized_start=982
  _DELETESECRETRESPONSE._serialized_end=1004
  _SECRETSSERVICE._serialized_start=1007
  _SECRETSSERVICE._serialized_end=2269
# @@protoc_insertion_point(module_scope)
