# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/feature_service__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.args import basic_info__client_pb2 as tecton__proto_dot_args_dot_basic__info____client__pb2
from tecton_proto.args import diff_options__client_pb2 as tecton__proto_dot_args_dot_diff__options____client__pb2
from tecton_proto.common import framework_version__client_pb2 as tecton__proto_dot_common_dot_framework__version____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n/tecton_proto/args/feature_service__client.proto\x12\x11tecton_proto.args\x1a*tecton_proto/args/basic_info__client.proto\x1a,tecton_proto/args/diff_options__client.proto\x1a\x33tecton_proto/common/framework_version__client.proto\x1a$tecton_proto/common/id__client.proto\"d\n\x11LoggingConfigArgs\x12\x1f\n\x0bsample_rate\x18\x01 \x01(\x02R\nsampleRate\x12.\n\x13log_effective_times\x18\x02 \x01(\x08R\x11logEffectiveTimes\"V\n\nColumnPair\x12!\n\x0cspine_column\x18\x01 \x01(\tR\x0bspineColumn\x12%\n\x0e\x66\x65\x61ture_column\x18\x02 \x01(\tR\rfeatureColumn\"\xe1\x01\n\x10\x46\x65\x61tureReference\x12?\n\x0f\x66\x65\x61ture_view_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rfeatureViewId\x12R\n\x12override_join_keys\x18\x02 \x03(\x0b\x32\x1d.tecton_proto.args.ColumnPairB\x05\x92M\x02 \x07R\x10overrideJoinKeys\x12\x1c\n\tnamespace\x18\x03 \x01(\tR\tnamespace\x12\x1a\n\x08\x66\x65\x61tures\x18\x04 \x03(\tR\x08\x66\x65\x61tures\"\xe2\x05\n\x12\x46\x65\x61tureServiceArgs\x12\x45\n\x12\x66\x65\x61ture_service_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x10\x66\x65\x61tureServiceId\x12\x37\n\x04info\x18\x02 \x01(\x0b\x32\x1c.tecton_proto.args.BasicInfoB\x05\x92M\x02\x10\x01R\x04info\x12\x46\n\x07version\x18\x06 \x01(\x0e\x32%.tecton_proto.common.FrameworkVersionB\x05\x92M\x02\x08\x05R\x07version\x12.\n\x0fprevent_destroy\x18\x07 \x01(\x08\x42\x05\x92M\x02\x08\x01R\x0epreventDestroy\x12S\n\x07options\x18\n \x03(\x0b\x32\x32.tecton_proto.args.FeatureServiceArgs.OptionsEntryB\x05\x92M\x02\x08\x01R\x07options\x12\x39\n\x15\x65nable_online_caching\x18\x0b \x01(\x08\x42\x05\x92M\x02\x08\x01R\x13\x65nableOnlineCaching\x12R\n\x12\x66\x65\x61ture_references\x18\x03 \x03(\x0b\x32#.tecton_proto.args.FeatureReferenceR\x11\x66\x65\x61tureReferences\x12;\n\x16online_serving_enabled\x18\x04 \x01(\x08\x42\x05\x92M\x02\x08\x01R\x14onlineServingEnabled\x12>\n\x07logging\x18\x05 \x01(\x0b\x32$.tecton_proto.args.LoggingConfigArgsR\x07logging\x12\x31\n\x14realtime_environment\x18\t \x01(\tR\x13realtimeEnvironment\x1a:\n\x0cOptionsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01J\x04\x08\x08\x10\tB\x13\n\x0f\x63om.tecton.argsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.feature_service__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001'
  _FEATUREREFERENCE.fields_by_name['override_join_keys']._options = None
  _FEATUREREFERENCE.fields_by_name['override_join_keys']._serialized_options = b'\222M\002 \007'
  _FEATURESERVICEARGS_OPTIONSENTRY._options = None
  _FEATURESERVICEARGS_OPTIONSENTRY._serialized_options = b'8\001'
  _FEATURESERVICEARGS.fields_by_name['info']._options = None
  _FEATURESERVICEARGS.fields_by_name['info']._serialized_options = b'\222M\002\020\001'
  _FEATURESERVICEARGS.fields_by_name['version']._options = None
  _FEATURESERVICEARGS.fields_by_name['version']._serialized_options = b'\222M\002\010\005'
  _FEATURESERVICEARGS.fields_by_name['prevent_destroy']._options = None
  _FEATURESERVICEARGS.fields_by_name['prevent_destroy']._serialized_options = b'\222M\002\010\001'
  _FEATURESERVICEARGS.fields_by_name['options']._options = None
  _FEATURESERVICEARGS.fields_by_name['options']._serialized_options = b'\222M\002\010\001'
  _FEATURESERVICEARGS.fields_by_name['enable_online_caching']._options = None
  _FEATURESERVICEARGS.fields_by_name['enable_online_caching']._serialized_options = b'\222M\002\010\001'
  _FEATURESERVICEARGS.fields_by_name['online_serving_enabled']._options = None
  _FEATURESERVICEARGS.fields_by_name['online_serving_enabled']._serialized_options = b'\222M\002\010\001'
  _LOGGINGCONFIGARGS._serialized_start=251
  _LOGGINGCONFIGARGS._serialized_end=351
  _COLUMNPAIR._serialized_start=353
  _COLUMNPAIR._serialized_end=439
  _FEATUREREFERENCE._serialized_start=442
  _FEATUREREFERENCE._serialized_end=667
  _FEATURESERVICEARGS._serialized_start=670
  _FEATURESERVICEARGS._serialized_end=1408
  _FEATURESERVICEARGS_OPTIONSENTRY._serialized_start=1344
  _FEATURESERVICEARGS_OPTIONSENTRY._serialized_end=1402
# @@protoc_insertion_point(module_scope)
