# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/fv_materialization__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.common import fco_locator__client_pb2 as tecton__proto_dot_common_dot_fco__locator____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2tecton_proto/data/fv_materialization__client.proto\x12\x11tecton_proto.data\x1a\x1fgoogle/protobuf/timestamp.proto\x1a-tecton_proto/common/fco_locator__client.proto\"\"\n\x0c\x46ileLocation\x12\x12\n\x04path\x18\x01 \x01(\tR\x04path\"\xa5\x08\n\x11\x46vMaterialization\x12X\n\x17id_feature_view_locator\x18\t \x01(\x0b\x32!.tecton_proto.common.IdFcoLocatorR\x14idFeatureViewLocator\x12\x44\n\x1ematerialization_serial_version\x18\x03 \x01(\x05R\x1cmaterializationSerialVersion\x12]\n\x1amaterialized_data_location\x18\x05 \x01(\x0b\x32\x1f.tecton_proto.data.FileLocationR\x18materializedDataLocation\x12\x65\n\x1estreaming_checkpoint_locations\x18\x06 \x03(\x0b\x32\x1f.tecton_proto.data.FileLocationR\x1cstreamingCheckpointLocations\x12\x32\n\x15has_materialized_data\x18\x07 \x01(\x08R\x13hasMaterializedData\x12?\n\x1c\x66\x65\x61ture_store_format_version\x18\x08 \x01(\x05R\x19\x66\x65\x61tureStoreFormatVersion\x12\\\n\x1amisc_offline_data_location\x18\x0b \x01(\x0b\x32\x1f.tecton_proto.data.FileLocationR\x17miscOfflineDataLocation\x12V\n(online_table_imported_by_materialization\x18\x0c \x01(\x08R$onlineTableImportedByMaterialization\x12?\n\x1conline_table_import_complete\x18\r \x01(\x08R\x19onlineTableImportComplete\x12`\n\x1c\x66\x65\x61ture_export_data_location\x18\x0e \x01(\x0b\x32\x1f.tecton_proto.data.FileLocationR\x19\x66\x65\x61tureExportDataLocation\x12P\n\x12online_data_tables\x18\x0f \x03(\x0b\x32\".tecton_proto.data.OnlineDataTableR\x10onlineDataTables\x12*\n\x11has_been_revisted\x18\x10 \x01(\x08R\x0fhasBeenRevisted\x12\x46\n\x11last_revisited_ts\x18\x11 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0flastRevisitedTsJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03J\x04\x08\x04\x10\x05J\x04\x08\n\x10\x0b\"\xbb\x02\n\x0fOnlineDataTable\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12P\n\x16\x66\x65\x61ture_data_watermark\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x14\x66\x65\x61tureDataWatermark\x12^\n\x11state_transitions\x18\x03 \x03(\x0b\x32\x31.tecton_proto.data.OnlineDataTableStateTransitionR\x10stateTransitions\x12\x62\n\x1a\x62\x61tch_table_format_version\x18\x04 \x01(\x0e\x32%.tecton_proto.data.TableFormatVersionR\x17\x62\x61tchTableFormatVersion\"\x99\x01\n\x1eOnlineDataTableStateTransition\x12=\n\x05state\x18\x01 \x01(\x0e\x32\'.tecton_proto.data.OnlineDataTableStateR\x05state\x12\x38\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp*\xb5\x01\n\x16OnlineBackfillLoadType\x12)\n%ONLINE_BACKFILL_LOAD_TYPE_UNSPECIFIED\x10\x00\x12\"\n\x1eONLINE_BACKFILL_LOAD_TYPE_TASK\x10\x01\x12\"\n\x1eONLINE_BACKFILL_LOAD_TYPE_BULK\x10\x02\x12(\n$ONLINE_BACKFILL_LOAD_TYPE_COMPACTION\x10\x03*|\n\x12TableFormatVersion\x12,\n(TABLE_FORMAT_VERSION_DEFAULT_UNSPECIFIED\x10\x00\x12\x1b\n\x17TABLE_FORMAT_VERSION_V2\x10\x01\x12\x1b\n\x17TABLE_FORMAT_VERSION_V3\x10\x02*\xc1\x01\n\x14OnlineDataTableState\x12(\n$DATA_TABLE_STATE_UNKNOWN_UNSPECIFIED\x10\x00\x12\x1e\n\x1a\x44\x41TA_TABLE_STATE_NOT_READY\x10\x01\x12\x1a\n\x16\x44\x41TA_TABLE_STATE_READY\x10\x02\x12%\n!DATA_TABLE_STATE_PENDING_DELETION\x10\x03\x12\x1c\n\x18\x44\x41TA_TABLE_STATE_DELETED\x10\x04\x42\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.fv_materialization__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _ONLINEBACKFILLLOADTYPE._serialized_start=1728
  _ONLINEBACKFILLLOADTYPE._serialized_end=1909
  _TABLEFORMATVERSION._serialized_start=1911
  _TABLEFORMATVERSION._serialized_end=2035
  _ONLINEDATATABLESTATE._serialized_start=2038
  _ONLINEDATATABLESTATE._serialized_end=2231
  _FILELOCATION._serialized_start=153
  _FILELOCATION._serialized_end=187
  _FVMATERIALIZATION._serialized_start=190
  _FVMATERIALIZATION._serialized_end=1251
  _ONLINEDATATABLE._serialized_start=1254
  _ONLINEDATATABLE._serialized_end=1569
  _ONLINEDATATABLESTATETRANSITION._serialized_start=1572
  _ONLINEDATATABLESTATETRANSITION._serialized_end=1725
# @@protoc_insertion_point(module_scope)
