# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/feature_view__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.args import basic_info__client_pb2 as tecton__proto_dot_args_dot_basic__info____client__pb2
from tecton_proto.args import data_source__client_pb2 as tecton__proto_dot_args_dot_data__source____client__pb2
from tecton_proto.args import diff_options__client_pb2 as tecton__proto_dot_args_dot_diff__options____client__pb2
from tecton_proto.args import pipeline__client_pb2 as tecton__proto_dot_args_dot_pipeline____client__pb2
from tecton_proto.common import analytics_options__client_pb2 as tecton__proto_dot_common_dot_analytics__options____client__pb2
from tecton_proto.common import compute_mode__client_pb2 as tecton__proto_dot_common_dot_compute__mode____client__pb2
from tecton_proto.common import data_source_type__client_pb2 as tecton__proto_dot_common_dot_data__source__type____client__pb2
from tecton_proto.common import data_type__client_pb2 as tecton__proto_dot_common_dot_data__type____client__pb2
from tecton_proto.common import framework_version__client_pb2 as tecton__proto_dot_common_dot_framework__version____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import schema__client_pb2 as tecton__proto_dot_common_dot_schema____client__pb2
from tecton_proto.common import spark_schema__client_pb2 as tecton__proto_dot_common_dot_spark__schema____client__pb2
from tecton_proto.common import time_window__client_pb2 as tecton__proto_dot_common_dot_time__window____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,tecton_proto/args/feature_view__client.proto\x12\x11tecton_proto.args\x1a\x1egoogle/protobuf/duration.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a*tecton_proto/args/basic_info__client.proto\x1a+tecton_proto/args/data_source__client.proto\x1a,tecton_proto/args/diff_options__client.proto\x1a(tecton_proto/args/pipeline__client.proto\x1a\x33tecton_proto/common/analytics_options__client.proto\x1a.tecton_proto/common/compute_mode__client.proto\x1a\x32tecton_proto/common/data_source_type__client.proto\x1a+tecton_proto/common/data_type__client.proto\x1a\x33tecton_proto/common/framework_version__client.proto\x1a$tecton_proto/common/id__client.proto\x1a(tecton_proto/common/schema__client.proto\x1a.tecton_proto/common/spark_schema__client.proto\x1a-tecton_proto/common/time_window__client.proto\"\x84\r\n\x0f\x46\x65\x61tureViewArgs\x12?\n\x0f\x66\x65\x61ture_view_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rfeatureViewId\x12N\n\x11\x66\x65\x61ture_view_type\x18\x02 \x01(\x0e\x32\".tecton_proto.args.FeatureViewTypeR\x0f\x66\x65\x61tureViewType\x12\x37\n\x04info\x18\x03 \x01(\x0b\x32\x1c.tecton_proto.args.BasicInfoB\x05\x92M\x02\x10\x01R\x04info\x12\x46\n\x07version\x18\x1d \x01(\x0e\x32%.tecton_proto.common.FrameworkVersionB\x05\x92M\x02\x08\x05R\x07version\x12.\n\x0fprevent_destroy\x18  \x01(\x08\x42\x05\x92M\x02\x08\x01R\x0epreventDestroy\x12P\n\x07options\x18\" \x03(\x0b\x32/.tecton_proto.args.FeatureViewArgs.OptionsEntryB\x05\x92M\x02\x08\x01R\x07options\x12H\n\x0c\x63\x61\x63he_config\x18$ \x01(\x0b\x32\x1e.tecton_proto.args.CacheConfigB\x05\x92M\x02\x08\x01R\x0b\x63\x61\x63heConfig\x12G\n\x08\x65ntities\x18\x04 \x03(\x0b\x32$.tecton_proto.args.EntityKeyOverrideB\x05\x92M\x02\x08\x06R\x08\x65ntities\x12|\n\x1ematerialized_feature_view_args\x18\x1c \x01(\x0b\x32..tecton_proto.args.MaterializedFeatureViewArgsB\x05\x92M\x02\x10\x01H\x00R\x1bmaterializedFeatureViewArgs\x12M\n\rrealtime_args\x18\x19 \x01(\x0b\x32\x1f.tecton_proto.args.RealtimeArgsB\x05\x92M\x02\x10\x01H\x00R\x0crealtimeArgs\x12Z\n\x12\x66\x65\x61ture_table_args\x18\x1a \x01(\x0b\x32#.tecton_proto.args.FeatureTableArgsB\x05\x92M\x02\x10\x01H\x00R\x10\x66\x65\x61tureTableArgs\x12\x34\n\x16\x63ontext_parameter_name\x18% \x01(\tR\x14\x63ontextParameterName\x12\x30\n\x14online_serving_index\x18\x05 \x03(\tR\x12onlineServingIndex\x12,\n\x0eonline_enabled\x18\x0e \x01(\x08\x42\x05\x92M\x02 \x0bR\ronlineEnabled\x12.\n\x0foffline_enabled\x18\x0f \x01(\x08\x42\x05\x92M\x02 \x0bR\x0eofflineEnabled\x12Z\n\x12\x62\x61tch_compute_mode\x18# \x01(\x0e\x32%.tecton_proto.common.BatchComputeModeB\x05\x92M\x02\x08\tR\x10\x62\x61tchComputeMode\x12>\n\x08pipeline\x18\x06 \x01(\x0b\x32\x1b.tecton_proto.args.PipelineB\x05\x92M\x02\x08\x06R\x08pipeline\x12`\n\x13\x64\x61ta_quality_config\x18! \x01(\x0b\x32$.tecton_proto.args.DataQualityConfigB\n\x92M\x02\x08\x01\x92M\x02\x10\x01R\x11\x64\x61taQualityConfig\x12N\n\x12\x66orced_view_schema\x18\x1e \x01(\x0b\x32 .tecton_proto.common.SparkSchemaR\x10\x66orcedViewSchema\x12^\n\x1a\x66orced_materialized_schema\x18\x1f \x01(\x0b\x32 .tecton_proto.common.SparkSchemaR\x18\x66orcedMaterializedSchema\x1a:\n\x0cOptionsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x0b\n\ttype_argsJ\x04\x08\x07\x10\x08J\x04\x08\x08\x10\tJ\x04\x08\t\x10\nJ\x04\x08\n\x10\x0bJ\x04\x08\x0b\x10\x0cJ\x04\x08\x0c\x10\rJ\x04\x08\r\x10\x0eJ\x04\x08\x10\x10\x11J\x04\x08\x11\x10\x12J\x04\x08\x12\x10\x13J\x04\x08\x13\x10\x14J\x04\x08\x14\x10\x15J\x04\x08\x15\x10\x16J\x04\x08\x16\x10\x17J\x04\x08\x17\x10\x18J\x04\x08\x18\x10\x19J\x04\x08\x1b\x10\x1c\"f\n\x11\x45ntityKeyOverride\x12\x34\n\tentity_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x08\x65ntityId\x12\x1b\n\tjoin_keys\x18\x02 \x03(\tR\x08joinKeys\"K\n\x0e\x42\x61\x63kfillConfig\x12\x39\n\x04mode\x18\x01 \x01(\x0e\x32%.tecton_proto.args.BackfillConfigModeR\x04mode\"\xce\x01\n\x0cOutputStream\x12)\n\x10include_features\x18\x01 \x01(\x08R\x0fincludeFeatures\x12\x44\n\x07kinesis\x18\x02 \x01(\x0b\x32(.tecton_proto.args.KinesisDataSourceArgsH\x00R\x07kinesis\x12>\n\x05kafka\x18\x03 \x01(\x0b\x32&.tecton_proto.args.KafkaDataSourceArgsH\x00R\x05kafkaB\r\n\x0bstream_sink\"\xab\x13\n\x1bMaterializedFeatureViewArgs\x12.\n\x0ftimestamp_field\x18\x01 \x01(\tB\x05\x92M\x02 \x10R\x0etimestampField\x12G\n\x0e\x62\x61tch_schedule\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationB\x05\x92M\x02 \x08R\rbatchSchedule\x12O\n\x12\x66\x65\x61ture_start_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\x92M\x02\x08\x01R\x10\x66\x65\x61tureStartTime\x12i\n manual_trigger_backfill_end_time\x18\x13 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\x92M\x02\x08\x01R\x1cmanualTriggerBackfillEndTime\x12\x85\x01\n\x15max_backfill_interval\x18\x04 \x01(\x0b\x32\x19.google.protobuf.DurationB6\x92M\"* \n\x1emax_batch_aggregation_interval\x92M\t*\x07\x12\x05\x30.7.0\x92M\x02\x08\x01R\x13maxBackfillInterval\x12\x41\n\x0bserving_ttl\x18\x05 \x01(\x0b\x32\x19.google.protobuf.DurationB\x05\x92M\x02\x08\x06R\nservingTtl\x12\x85\x01\n\x14offline_store_legacy\x18\x06 \x01(\x0b\x32,.tecton_proto.args.OfflineFeatureStoreConfigB%\x92M\x02 \r\x92M\x11*\x0f\n\roffline_store\x92M\t*\x07\x12\x05\x30.8.0R\x12offlineStoreLegacy\x12Q\n\roffline_store\x18\x19 \x01(\x0b\x32%.tecton_proto.args.OfflineStoreConfigB\x05\x92M\x02 \x0cR\x0cofflineStore\x12\x45\n\rbatch_compute\x18\x07 \x01(\x0b\x32 .tecton_proto.args.ClusterConfigR\x0c\x62\x61tchCompute\x12G\n\x0estream_compute\x18\x08 \x01(\x0b\x32 .tecton_proto.args.ClusterConfigR\rstreamCompute\x12O\n\nmonitoring\x18\t \x01(\x0b\x32#.tecton_proto.args.MonitoringConfigB\n\x92M\x02\x08\x01\x92M\x02\x10\x01R\nmonitoring\x12M\n\x10\x64\x61ta_source_type\x18\n \x01(\x0e\x32#.tecton_proto.common.DataSourceTypeR\x0e\x64\x61taSourceType\x12G\n\x0conline_store\x18\x0b \x01(\x0b\x32$.tecton_proto.args.OnlineStoreConfigR\x0bonlineStore\x12\x33\n\x15incremental_backfills\x18\x0c \x01(\x08R\x14incrementalBackfills\x12L\n\x14\x61ggregation_interval\x18\r \x01(\x0b\x32\x19.google.protobuf.DurationR\x13\x61ggregationInterval\x12\x64\n\x16stream_processing_mode\x18\x0f \x01(\x0e\x32\'.tecton_proto.args.StreamProcessingModeB\x05\x92M\x02 \x06R\x14streamProcessingMode\x12I\n\x0c\x61ggregations\x18\x0e \x03(\x0b\x32%.tecton_proto.args.FeatureAggregationR\x0c\x61ggregations\x12\x44\n\routput_stream\x18\x10 \x01(\x0b\x32\x1f.tecton_proto.args.OutputStreamR\x0coutputStream\x12O\n\rbatch_trigger\x18\x11 \x01(\x0e\x32#.tecton_proto.args.BatchTriggerTypeB\x05\x92M\x02\x08\x01R\x0c\x62\x61tchTrigger\x12?\n\x06schema\x18\x12 \x01(\x0b\x32\x1b.tecton_proto.common.SchemaB\n\x92M\x02\x08\x06\x92M\x02 \x10R\x06schema\x12:\n\x19\x61ggregation_secondary_key\x18\x14 \x01(\tR\x17\x61ggregationSecondaryKey\x12s\n\x1csecondary_key_output_columns\x18\x15 \x03(\x0b\x32+.tecton_proto.args.SecondaryKeyOutputColumnB\x05\x92M\x02\x18\x05R\x19secondaryKeyOutputColumns\x12I\n\x1drun_transformation_validation\x18\x16 \x01(\x08\x42\x05\x92M\x02\x08\x01R\x1brunTransformationValidation\x12K\n\x1etecton_materialization_runtime\x18\x17 \x01(\tB\x05\x92M\x02\x08\x01R\x1ctectonMaterializationRuntime\x12J\n\x13lifetime_start_time\x18\x18 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x11lifetimeStartTime\x12-\n\x12\x63ompaction_enabled\x18\x1a \x01(\x08R\x11\x63ompactionEnabled\x12\x32\n\x15stream_tiling_enabled\x18\x1c \x01(\x08R\x13streamTilingEnabled\x12J\n\x10stream_tile_size\x18\x1d \x01(\x0b\x32\x19.google.protobuf.DurationB\x05\x92M\x02\x18\x05R\x0estreamTileSize\x12\'\n\x0b\x65nvironment\x18\x1b \x01(\tB\x05\x92M\x02\x08\x01R\x0b\x65nvironment\x12\x43\n\nattributes\x18\x1e \x03(\x0b\x32\x1c.tecton_proto.args.AttributeB\x05\x92M\x02 \x10R\nattributes\x12<\n\nembeddings\x18\x1f \x03(\x0b\x32\x1c.tecton_proto.args.EmbeddingR\nembeddings\x12<\n\ninferences\x18  \x03(\x0b\x32\x1c.tecton_proto.args.InferenceR\ninferences\"e\n\tAttribute\x12\x16\n\x06\x63olumn\x18\x01 \x01(\tR\x06\x63olumn\x12@\n\x0c\x63olumn_dtype\x18\x02 \x01(\x0b\x32\x1d.tecton_proto.common.DataTypeR\x0b\x63olumnDtype\"w\n\tInference\x12@\n\rinput_columns\x18\x01 \x03(\x0b\x32\x1b.tecton_proto.common.ColumnR\x0cinputColumns\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12\x14\n\x05model\x18\x03 \x01(\tR\x05model\"\x8f\x01\n\tEmbedding\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x16\n\x06\x63olumn\x18\x02 \x01(\tR\x06\x63olumn\x12@\n\x0c\x63olumn_dtype\x18\x03 \x01(\x0b\x32\x1d.tecton_proto.common.DataTypeR\x0b\x63olumnDtype\x12\x14\n\x05model\x18\x04 \x01(\tR\x05model\"\xcb\x01\n\x0cRealtimeArgs\x12?\n\x06schema\x18\x02 \x01(\x0b\x32 .tecton_proto.common.SparkSchemaB\x05\x92M\x02 \x10R\x06schema\x12)\n\x0c\x65nvironments\x18\x04 \x03(\tB\x05\x92M\x02\x08\x01R\x0c\x65nvironments\x12\x43\n\nattributes\x18\x05 \x03(\x0b\x32\x1c.tecton_proto.args.AttributeB\x05\x92M\x02 \x10R\nattributesJ\x04\x08\x01\x10\x02J\x04\x08\x03\x10\x04\"\xac\x06\n\x10\x46\x65\x61tureTableArgs\x12?\n\x06schema\x18\x06 \x01(\x0b\x32 .tecton_proto.common.SparkSchemaB\x05\x92M\x02 \x10R\x06schema\x12\x41\n\x0bserving_ttl\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationB\x05\x92M\x02\x08\x06R\nservingTtl\x12\x85\x01\n\x14offline_store_legacy\x18\x07 \x01(\x0b\x32,.tecton_proto.args.OfflineFeatureStoreConfigB%\x92M\x02 \r\x92M\x11*\x0f\n\roffline_store\x92M\t*\x07\x12\x05\x30.8.0R\x12offlineStoreLegacy\x12Q\n\roffline_store\x18\x0c \x01(\x0b\x32%.tecton_proto.args.OfflineStoreConfigB\x05\x92M\x02 \x0cR\x0cofflineStore\x12G\n\x0conline_store\x18\x08 \x01(\x0b\x32$.tecton_proto.args.OnlineStoreConfigR\x0bonlineStore\x12\x45\n\rbatch_compute\x18\t \x01(\x0b\x32 .tecton_proto.args.ClusterConfigR\x0c\x62\x61tchCompute\x12O\n\nmonitoring\x18\n \x01(\x0b\x32#.tecton_proto.args.MonitoringConfigB\n\x92M\x02\x08\x01\x92M\x02\x10\x01R\nmonitoring\x12K\n\x1etecton_materialization_runtime\x18\x0b \x01(\tB\x05\x92M\x02\x08\x01R\x1ctectonMaterializationRuntime\x12\x43\n\nattributes\x18\r \x03(\x0b\x32\x1c.tecton_proto.args.AttributeB\x05\x92M\x02 \x10R\nattributes\x12.\n\x0ftimestamp_field\x18\x0e \x01(\tB\x05\x92M\x02 \x10R\x0etimestampFieldJ\x04\x08\x01\x10\x02J\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06\"\xbf\x06\n\x12\x46\x65\x61tureAggregation\x12\x16\n\x06\x63olumn\x18\x01 \x01(\tR\x06\x63olumn\x12\x1a\n\x08\x66unction\x18\x02 \x01(\tR\x08\x66unction\x12\x62\n\x0f\x66unction_params\x18\x05 \x03(\x0b\x32\x39.tecton_proto.args.FeatureAggregation.FunctionParamsEntryR\x0e\x66unctionParams\x12l\n\x12time_window_legacy\x18\x06 \x01(\x0b\x32\x19.google.protobuf.DurationB#\x92M\x0f*\r\n\x0btime_window\x92M\t*\x07\x12\x05\x30.8.0\x92M\x02 \tR\x10timeWindowLegacy\x12\x19\n\x04name\x18\x07 \x01(\tB\x05\x92M\x02 \x01R\x04name\x12G\n\x0btime_window\x18\x08 \x01(\x0b\x32\x1d.tecton_proto.args.TimeWindowB\x05\x92M\x02 \nH\x00R\ntimeWindow\x12N\n\x0flifetime_window\x18\t \x01(\x0b\x32#.tecton_proto.common.LifetimeWindowH\x00R\x0elifetimeWindow\x12S\n\x12time_window_series\x18\n \x01(\x0b\x32#.tecton_proto.args.TimeWindowSeriesH\x00R\x10timeWindowSeries\x12G\n\x0c\x63olumn_dtype\x18\x0b \x01(\x0b\x32\x1d.tecton_proto.common.DataTypeB\x05\x92M\x02 \x10R\x0b\x63olumnDtype\x12Y\n\x18\x62\x61tch_sawtooth_tile_size\x18\x0c \x01(\x0b\x32\x19.google.protobuf.DurationB\x05\x92M\x02\x18\x05R\x15\x62\x61tchSawtoothTileSize\x1a`\n\x13\x46unctionParamsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x33\n\x05value\x18\x02 \x01(\x0b\x32\x1d.tecton_proto.args.ParamValueR\x05value:\x02\x38\x01\x42\x08\n\x06windowJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05\"]\n\nParamValue\x12!\n\x0bint64_value\x18\x01 \x01(\x03H\x00R\nint64Value\x12#\n\x0c\x64ouble_value\x18\x02 \x01(\x01H\x00R\x0b\x64oubleValueB\x07\n\x05value\"\x8f\x01\n\x11\x44\x61taQualityConfig\x12\x37\n\x14\x64\x61ta_quality_enabled\x18\x01 \x01(\x08\x42\x05\x92M\x02 \x03R\x12\x64\x61taQualityEnabled\x12\x41\n\x19skip_default_expectations\x18\x02 \x01(\x08\x42\x05\x92M\x02 \x03R\x17skipDefaultExpectations\"\xac\x05\n\rClusterConfig\x12\\\n\x10\x65xisting_cluster\x18\x01 \x01(\x0b\x32(.tecton_proto.args.ExistingClusterConfigB\x05\x92M\x02\x08\x01H\x00R\x0f\x65xistingCluster\x12S\n\x0enew_databricks\x18\x02 \x01(\x0b\x32#.tecton_proto.args.NewClusterConfigB\x05\x92M\x02\x08\x01H\x00R\rnewDatabricks\x12\x45\n\x07new_emr\x18\x03 \x01(\x0b\x32#.tecton_proto.args.NewClusterConfigB\x05\x92M\x02\x08\x01H\x00R\x06newEmr\x12^\n\x0fimplicit_config\x18\x04 \x01(\x0b\x32\'.tecton_proto.args.DefaultClusterConfigB\n\x92M\x02\x08\x01\x92M\x02\x18\x05H\x00R\x0eimplicitConfig\x12V\n\x0fjson_databricks\x18\x05 \x01(\x0b\x32$.tecton_proto.args.JsonClusterConfigB\x05\x92M\x02\x08\x01H\x00R\x0ejsonDatabricks\x12H\n\x08json_emr\x18\x06 \x01(\x0b\x32$.tecton_proto.args.JsonClusterConfigB\x05\x92M\x02\x08\x01H\x00R\x07jsonEmr\x12R\n\rjson_dataproc\x18\x07 \x01(\x0b\x32$.tecton_proto.args.JsonClusterConfigB\x05\x92M\x02\x08\x01H\x00R\x0cjsonDataproc\x12\x41\n\x04rift\x18\x08 \x01(\x0b\x32$.tecton_proto.args.RiftClusterConfigB\x05\x92M\x02\x08\x01H\x00R\x04riftB\x08\n\x06\x63onfig\"@\n\x11JsonClusterConfig\x12+\n\x04json\x18\x01 \x01(\x0b\x32\x17.google.protobuf.StructR\x04json\"G\n\x15\x45xistingClusterConfig\x12.\n\x13\x65xisting_cluster_id\x18\x01 \x01(\tR\x11\x65xistingClusterId\"\x9f\x03\n\x10NewClusterConfig\x12#\n\rinstance_type\x18\x01 \x01(\tR\x0cinstanceType\x12\x33\n\x15instance_availability\x18\x06 \x01(\tR\x14instanceAvailability\x12*\n\x11number_of_workers\x18\x02 \x01(\x05R\x0fnumberOfWorkers\x12\x32\n\x16root_volume_size_in_gb\x18\x03 \x01(\x05R\x12rootVolumeSizeInGb\x12\x34\n\x16\x65xtra_pip_dependencies\x18\x04 \x03(\tR\x14\x65xtraPipDependencies\x12\x41\n\x0cspark_config\x18\x05 \x01(\x0b\x32\x1e.tecton_proto.args.SparkConfigR\x0bsparkConfig\x12&\n\x0f\x66irst_on_demand\x18\x07 \x01(\x05R\rfirstOnDemand\x12\x30\n\x14pinned_spark_version\x18\x08 \x01(\tR\x12pinnedSparkVersion\"8\n\x11RiftClusterConfig\x12#\n\rinstance_type\x18\x01 \x01(\tR\x0cinstanceType\"\xbd\x01\n\x14\x44\x65\x66\x61ultClusterConfig\x12\x38\n\x18\x64\x61tabricks_spark_version\x18\x01 \x01(\tR\x16\x64\x61tabricksSparkVersion\x12*\n\x11\x65mr_spark_version\x18\x02 \x01(\tR\x0f\x65mrSparkVersion\x12?\n\x1ctecton_compute_instance_type\x18\x03 \x01(\tR\x19tectonComputeInstanceType\"\x83\x03\n\x0bSparkConfig\x12.\n\x13spark_driver_memory\x18\x01 \x01(\tR\x11sparkDriverMemory\x12\x32\n\x15spark_executor_memory\x18\x02 \x01(\tR\x13sparkExecutorMemory\x12?\n\x1cspark_driver_memory_overhead\x18\x03 \x01(\tR\x19sparkDriverMemoryOverhead\x12\x43\n\x1espark_executor_memory_overhead\x18\x04 \x01(\tR\x1bsparkExecutorMemoryOverhead\x12L\n\nspark_conf\x18\x05 \x03(\x0b\x32-.tecton_proto.args.SparkConfig.SparkConfEntryR\tsparkConf\x1a<\n\x0eSparkConfEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xe6\x01\n\x11OnlineStoreConfig\x12@\n\x06\x64ynamo\x18\x01 \x01(\x0b\x32&.tecton_proto.args.DynamoDbOnlineStoreH\x00R\x06\x64ynamo\x12;\n\x05redis\x18\x02 \x01(\x0b\x32#.tecton_proto.args.RedisOnlineStoreH\x00R\x05redis\x12\x44\n\x08\x62igtable\x18\x03 \x01(\x0b\x32&.tecton_proto.args.BigtableOnlineStoreH\x00R\x08\x62igtableB\x0c\n\nstore_type\"G\n\x13\x44ynamoDbOnlineStore\x12\x18\n\x07\x65nabled\x18\x03 \x01(\x08R\x07\x65nabledJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06\"\xc8\x01\n\x10RedisOnlineStore\x12\x30\n\x10primary_endpoint\x18\x02 \x01(\tB\x05\x92M\x02\x08\x06R\x0fprimaryEndpoint\x12=\n\x14\x61uthentication_token\x18\x04 \x01(\tB\n\x92M\x02\x08\x01\x92M\x02\x18\x06R\x13\x61uthenticationToken\x12\x1f\n\x07\x65nabled\x18\x05 \x01(\x08\x42\x05\x92M\x02\x18\x05R\x07\x65nabledJ\x04\x08\x01\x10\x02J\x04\x08\x03\x10\x04J\x04\x08\x06\x10\x07J\x04\x08\x07\x10\x08J\x04\x08\x08\x10\tJ\x04\x08\t\x10\n\"o\n\x13\x42igtableOnlineStore\x12\x18\n\x07\x65nabled\x18\x01 \x01(\x08R\x07\x65nabled\x12\x1d\n\nproject_id\x18\x02 \x01(\tR\tprojectId\x12\x1f\n\x0binstance_id\x18\x03 \x01(\tR\ninstanceId\"\xdb\x01\n\x19OfflineFeatureStoreConfig\x12<\n\x07parquet\x18\x01 \x01(\x0b\x32 .tecton_proto.args.ParquetConfigH\x00R\x07parquet\x12\x36\n\x05\x64\x65lta\x18\x02 \x01(\x0b\x32\x1e.tecton_proto.args.DeltaConfigH\x00R\x05\x64\x65lta\x12:\n\x15subdirectory_override\x18\x03 \x01(\tB\x05\x92M\x02\x08\x01R\x14subdirectoryOverrideB\x0c\n\nstore_type\"\x0f\n\rParquetConfig\"X\n\x0b\x44\x65ltaConfig\x12I\n\x13time_partition_size\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationR\x11timePartitionSize\"\x80\x02\n\x12OfflineStoreConfig\x12^\n\x14staging_table_format\x18\x01 \x01(\x0b\x32,.tecton_proto.args.OfflineFeatureStoreConfigR\x12stagingTableFormat\x12\x39\n\x15publish_full_features\x18\x02 \x01(\x08\x42\x05\x92M\x02\x08\x01R\x13publishFullFeatures\x12O\n\x12publish_start_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\x92M\x02\x08\x01R\x10publishStartTime\"\xb8\x01\n\x10MonitoringConfig\x12+\n\x11monitor_freshness\x18\x01 \x01(\x08R\x10monitorFreshness\x12H\n\x12\x65xpected_freshness\x18\x04 \x01(\x0b\x32\x19.google.protobuf.DurationR\x11\x65xpectedFreshness\x12\'\n\x0b\x61lert_email\x18\x03 \x01(\tB\x06\xf2\xe2\x02\x02\x08\x01R\nalertEmailJ\x04\x08\x02\x10\x03\"\x9f\x02\n\x18SecondaryKeyOutputColumn\x12@\n\x0btime_window\x18\x01 \x01(\x0b\x32\x1d.tecton_proto.args.TimeWindowH\x00R\ntimeWindow\x12N\n\x0flifetime_window\x18\x03 \x01(\x0b\x32#.tecton_proto.common.LifetimeWindowH\x00R\x0elifetimeWindow\x12S\n\x12time_window_series\x18\x04 \x01(\x0b\x32#.tecton_proto.args.TimeWindowSeriesH\x00R\x10timeWindowSeries\x12\x12\n\x04name\x18\x02 \x01(\tR\x04nameB\x08\n\x06window\"\x83\x01\n\nTimeWindow\x12\x42\n\x0fwindow_duration\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0ewindowDuration\x12\x31\n\x06offset\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationR\x06offset\"\x86\x02\n\x10TimeWindowSeries\x12<\n\x0cseries_start\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0bseriesStart\x12\x38\n\nseries_end\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationR\tseriesEnd\x12\x36\n\tstep_size\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\x08stepSize\x12\x42\n\x0fwindow_duration\x18\x04 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0ewindowDuration\"5\n\x0b\x43\x61\x63heConfig\x12&\n\x0fmax_age_seconds\x18\x01 \x01(\x03R\rmaxAgeSeconds*\xae\x01\n\x0f\x46\x65\x61tureViewType\x12!\n\x1d\x46\x45\x41TURE_VIEW_TYPE_UNSPECIFIED\x10\x00\x12\x1e\n\x1a\x46\x45\x41TURE_VIEW_TYPE_REALTIME\x10\x03\x12#\n\x1f\x46\x45\x41TURE_VIEW_TYPE_FEATURE_TABLE\x10\x04\x12\'\n#FEATURE_VIEW_TYPE_FWV5_FEATURE_VIEW\x10\x05\"\x04\x08\x01\x10\x01\"\x04\x08\x02\x10\x02*\xbf\x01\n\x12\x42\x61\x63kfillConfigMode\x12$\n BACKFILL_CONFIG_MODE_UNSPECIFIED\x10\x00\x12?\n;BACKFILL_CONFIG_MODE_SINGLE_BATCH_SCHEDULE_INTERVAL_PER_JOB\x10\x01\x12\x42\n>BACKFILL_CONFIG_MODE_MULTIPLE_BATCH_SCHEDULE_INTERVALS_PER_JOB\x10\x02*\x8f\x01\n\x14StreamProcessingMode\x12&\n\"STREAM_PROCESSING_MODE_UNSPECIFIED\x10\x00\x12(\n$STREAM_PROCESSING_MODE_TIME_INTERVAL\x10\x01\x12%\n!STREAM_PROCESSING_MODE_CONTINUOUS\x10\x02*\xa8\x01\n\x10\x42\x61tchTriggerType\x12\"\n\x1e\x42\x41TCH_TRIGGER_TYPE_UNSPECIFIED\x10\x00\x12 \n\x1c\x42\x41TCH_TRIGGER_TYPE_SCHEDULED\x10\x01\x12\x1d\n\x19\x42\x41TCH_TRIGGER_TYPE_MANUAL\x10\x02\x12/\n+BATCH_TRIGGER_TYPE_NO_BATCH_MATERIALIZATION\x10\x03\x42\x13\n\x0f\x63om.tecton.argsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.feature_view__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001'
  _FEATUREVIEWARGS_OPTIONSENTRY._options = None
  _FEATUREVIEWARGS_OPTIONSENTRY._serialized_options = b'8\001'
  _FEATUREVIEWARGS.fields_by_name['info']._options = None
  _FEATUREVIEWARGS.fields_by_name['info']._serialized_options = b'\222M\002\020\001'
  _FEATUREVIEWARGS.fields_by_name['version']._options = None
  _FEATUREVIEWARGS.fields_by_name['version']._serialized_options = b'\222M\002\010\005'
  _FEATUREVIEWARGS.fields_by_name['prevent_destroy']._options = None
  _FEATUREVIEWARGS.fields_by_name['prevent_destroy']._serialized_options = b'\222M\002\010\001'
  _FEATUREVIEWARGS.fields_by_name['options']._options = None
  _FEATUREVIEWARGS.fields_by_name['options']._serialized_options = b'\222M\002\010\001'
  _FEATUREVIEWARGS.fields_by_name['cache_config']._options = None
  _FEATUREVIEWARGS.fields_by_name['cache_config']._serialized_options = b'\222M\002\010\001'
  _FEATUREVIEWARGS.fields_by_name['entities']._options = None
  _FEATUREVIEWARGS.fields_by_name['entities']._serialized_options = b'\222M\002\010\006'
  _FEATUREVIEWARGS.fields_by_name['materialized_feature_view_args']._options = None
  _FEATUREVIEWARGS.fields_by_name['materialized_feature_view_args']._serialized_options = b'\222M\002\020\001'
  _FEATUREVIEWARGS.fields_by_name['realtime_args']._options = None
  _FEATUREVIEWARGS.fields_by_name['realtime_args']._serialized_options = b'\222M\002\020\001'
  _FEATUREVIEWARGS.fields_by_name['feature_table_args']._options = None
  _FEATUREVIEWARGS.fields_by_name['feature_table_args']._serialized_options = b'\222M\002\020\001'
  _FEATUREVIEWARGS.fields_by_name['online_enabled']._options = None
  _FEATUREVIEWARGS.fields_by_name['online_enabled']._serialized_options = b'\222M\002 \013'
  _FEATUREVIEWARGS.fields_by_name['offline_enabled']._options = None
  _FEATUREVIEWARGS.fields_by_name['offline_enabled']._serialized_options = b'\222M\002 \013'
  _FEATUREVIEWARGS.fields_by_name['batch_compute_mode']._options = None
  _FEATUREVIEWARGS.fields_by_name['batch_compute_mode']._serialized_options = b'\222M\002\010\t'
  _FEATUREVIEWARGS.fields_by_name['pipeline']._options = None
  _FEATUREVIEWARGS.fields_by_name['pipeline']._serialized_options = b'\222M\002\010\006'
  _FEATUREVIEWARGS.fields_by_name['data_quality_config']._options = None
  _FEATUREVIEWARGS.fields_by_name['data_quality_config']._serialized_options = b'\222M\002\010\001\222M\002\020\001'
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['timestamp_field']._options = None
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['timestamp_field']._serialized_options = b'\222M\002 \020'
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['batch_schedule']._options = None
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['batch_schedule']._serialized_options = b'\222M\002 \010'
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['feature_start_time']._options = None
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['feature_start_time']._serialized_options = b'\222M\002\010\001'
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['manual_trigger_backfill_end_time']._options = None
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['manual_trigger_backfill_end_time']._serialized_options = b'\222M\002\010\001'
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['max_backfill_interval']._options = None
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['max_backfill_interval']._serialized_options = b'\222M\"* \n\036max_batch_aggregation_interval\222M\t*\007\022\0050.7.0\222M\002\010\001'
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['serving_ttl']._options = None
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['serving_ttl']._serialized_options = b'\222M\002\010\006'
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['offline_store_legacy']._options = None
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['offline_store_legacy']._serialized_options = b'\222M\002 \r\222M\021*\017\n\roffline_store\222M\t*\007\022\0050.8.0'
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['offline_store']._options = None
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['offline_store']._serialized_options = b'\222M\002 \014'
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['monitoring']._options = None
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['monitoring']._serialized_options = b'\222M\002\010\001\222M\002\020\001'
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['stream_processing_mode']._options = None
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['stream_processing_mode']._serialized_options = b'\222M\002 \006'
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['batch_trigger']._options = None
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['batch_trigger']._serialized_options = b'\222M\002\010\001'
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['schema']._options = None
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['schema']._serialized_options = b'\222M\002\010\006\222M\002 \020'
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['secondary_key_output_columns']._options = None
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['secondary_key_output_columns']._serialized_options = b'\222M\002\030\005'
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['run_transformation_validation']._options = None
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['run_transformation_validation']._serialized_options = b'\222M\002\010\001'
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['tecton_materialization_runtime']._options = None
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['tecton_materialization_runtime']._serialized_options = b'\222M\002\010\001'
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['stream_tile_size']._options = None
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['stream_tile_size']._serialized_options = b'\222M\002\030\005'
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['environment']._options = None
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['environment']._serialized_options = b'\222M\002\010\001'
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['attributes']._options = None
  _MATERIALIZEDFEATUREVIEWARGS.fields_by_name['attributes']._serialized_options = b'\222M\002 \020'
  _REALTIMEARGS.fields_by_name['schema']._options = None
  _REALTIMEARGS.fields_by_name['schema']._serialized_options = b'\222M\002 \020'
  _REALTIMEARGS.fields_by_name['environments']._options = None
  _REALTIMEARGS.fields_by_name['environments']._serialized_options = b'\222M\002\010\001'
  _REALTIMEARGS.fields_by_name['attributes']._options = None
  _REALTIMEARGS.fields_by_name['attributes']._serialized_options = b'\222M\002 \020'
  _FEATURETABLEARGS.fields_by_name['schema']._options = None
  _FEATURETABLEARGS.fields_by_name['schema']._serialized_options = b'\222M\002 \020'
  _FEATURETABLEARGS.fields_by_name['serving_ttl']._options = None
  _FEATURETABLEARGS.fields_by_name['serving_ttl']._serialized_options = b'\222M\002\010\006'
  _FEATURETABLEARGS.fields_by_name['offline_store_legacy']._options = None
  _FEATURETABLEARGS.fields_by_name['offline_store_legacy']._serialized_options = b'\222M\002 \r\222M\021*\017\n\roffline_store\222M\t*\007\022\0050.8.0'
  _FEATURETABLEARGS.fields_by_name['offline_store']._options = None
  _FEATURETABLEARGS.fields_by_name['offline_store']._serialized_options = b'\222M\002 \014'
  _FEATURETABLEARGS.fields_by_name['monitoring']._options = None
  _FEATURETABLEARGS.fields_by_name['monitoring']._serialized_options = b'\222M\002\010\001\222M\002\020\001'
  _FEATURETABLEARGS.fields_by_name['tecton_materialization_runtime']._options = None
  _FEATURETABLEARGS.fields_by_name['tecton_materialization_runtime']._serialized_options = b'\222M\002\010\001'
  _FEATURETABLEARGS.fields_by_name['attributes']._options = None
  _FEATURETABLEARGS.fields_by_name['attributes']._serialized_options = b'\222M\002 \020'
  _FEATURETABLEARGS.fields_by_name['timestamp_field']._options = None
  _FEATURETABLEARGS.fields_by_name['timestamp_field']._serialized_options = b'\222M\002 \020'
  _FEATUREAGGREGATION_FUNCTIONPARAMSENTRY._options = None
  _FEATUREAGGREGATION_FUNCTIONPARAMSENTRY._serialized_options = b'8\001'
  _FEATUREAGGREGATION.fields_by_name['time_window_legacy']._options = None
  _FEATUREAGGREGATION.fields_by_name['time_window_legacy']._serialized_options = b'\222M\017*\r\n\013time_window\222M\t*\007\022\0050.8.0\222M\002 \t'
  _FEATUREAGGREGATION.fields_by_name['name']._options = None
  _FEATUREAGGREGATION.fields_by_name['name']._serialized_options = b'\222M\002 \001'
  _FEATUREAGGREGATION.fields_by_name['time_window']._options = None
  _FEATUREAGGREGATION.fields_by_name['time_window']._serialized_options = b'\222M\002 \n'
  _FEATUREAGGREGATION.fields_by_name['column_dtype']._options = None
  _FEATUREAGGREGATION.fields_by_name['column_dtype']._serialized_options = b'\222M\002 \020'
  _FEATUREAGGREGATION.fields_by_name['batch_sawtooth_tile_size']._options = None
  _FEATUREAGGREGATION.fields_by_name['batch_sawtooth_tile_size']._serialized_options = b'\222M\002\030\005'
  _DATAQUALITYCONFIG.fields_by_name['data_quality_enabled']._options = None
  _DATAQUALITYCONFIG.fields_by_name['data_quality_enabled']._serialized_options = b'\222M\002 \003'
  _DATAQUALITYCONFIG.fields_by_name['skip_default_expectations']._options = None
  _DATAQUALITYCONFIG.fields_by_name['skip_default_expectations']._serialized_options = b'\222M\002 \003'
  _CLUSTERCONFIG.fields_by_name['existing_cluster']._options = None
  _CLUSTERCONFIG.fields_by_name['existing_cluster']._serialized_options = b'\222M\002\010\001'
  _CLUSTERCONFIG.fields_by_name['new_databricks']._options = None
  _CLUSTERCONFIG.fields_by_name['new_databricks']._serialized_options = b'\222M\002\010\001'
  _CLUSTERCONFIG.fields_by_name['new_emr']._options = None
  _CLUSTERCONFIG.fields_by_name['new_emr']._serialized_options = b'\222M\002\010\001'
  _CLUSTERCONFIG.fields_by_name['implicit_config']._options = None
  _CLUSTERCONFIG.fields_by_name['implicit_config']._serialized_options = b'\222M\002\010\001\222M\002\030\005'
  _CLUSTERCONFIG.fields_by_name['json_databricks']._options = None
  _CLUSTERCONFIG.fields_by_name['json_databricks']._serialized_options = b'\222M\002\010\001'
  _CLUSTERCONFIG.fields_by_name['json_emr']._options = None
  _CLUSTERCONFIG.fields_by_name['json_emr']._serialized_options = b'\222M\002\010\001'
  _CLUSTERCONFIG.fields_by_name['json_dataproc']._options = None
  _CLUSTERCONFIG.fields_by_name['json_dataproc']._serialized_options = b'\222M\002\010\001'
  _CLUSTERCONFIG.fields_by_name['rift']._options = None
  _CLUSTERCONFIG.fields_by_name['rift']._serialized_options = b'\222M\002\010\001'
  _SPARKCONFIG_SPARKCONFENTRY._options = None
  _SPARKCONFIG_SPARKCONFENTRY._serialized_options = b'8\001'
  _REDISONLINESTORE.fields_by_name['primary_endpoint']._options = None
  _REDISONLINESTORE.fields_by_name['primary_endpoint']._serialized_options = b'\222M\002\010\006'
  _REDISONLINESTORE.fields_by_name['authentication_token']._options = None
  _REDISONLINESTORE.fields_by_name['authentication_token']._serialized_options = b'\222M\002\010\001\222M\002\030\006'
  _REDISONLINESTORE.fields_by_name['enabled']._options = None
  _REDISONLINESTORE.fields_by_name['enabled']._serialized_options = b'\222M\002\030\005'
  _OFFLINEFEATURESTORECONFIG.fields_by_name['subdirectory_override']._options = None
  _OFFLINEFEATURESTORECONFIG.fields_by_name['subdirectory_override']._serialized_options = b'\222M\002\010\001'
  _OFFLINESTORECONFIG.fields_by_name['publish_full_features']._options = None
  _OFFLINESTORECONFIG.fields_by_name['publish_full_features']._serialized_options = b'\222M\002\010\001'
  _OFFLINESTORECONFIG.fields_by_name['publish_start_time']._options = None
  _OFFLINESTORECONFIG.fields_by_name['publish_start_time']._serialized_options = b'\222M\002\010\001'
  _MONITORINGCONFIG.fields_by_name['alert_email']._options = None
  _MONITORINGCONFIG.fields_by_name['alert_email']._serialized_options = b'\362\342\002\002\010\001'
  _FEATUREVIEWTYPE._serialized_start=11796
  _FEATUREVIEWTYPE._serialized_end=11970
  _BACKFILLCONFIGMODE._serialized_start=11973
  _BACKFILLCONFIGMODE._serialized_end=12164
  _STREAMPROCESSINGMODE._serialized_start=12167
  _STREAMPROCESSINGMODE._serialized_end=12310
  _BATCHTRIGGERTYPE._serialized_start=12313
  _BATCHTRIGGERTYPE._serialized_end=12481
  _FEATUREVIEWARGS._serialized_start=766
  _FEATUREVIEWARGS._serialized_end=2434
  _FEATUREVIEWARGS_OPTIONSENTRY._serialized_start=2261
  _FEATUREVIEWARGS_OPTIONSENTRY._serialized_end=2319
  _ENTITYKEYOVERRIDE._serialized_start=2436
  _ENTITYKEYOVERRIDE._serialized_end=2538
  _BACKFILLCONFIG._serialized_start=2540
  _BACKFILLCONFIG._serialized_end=2615
  _OUTPUTSTREAM._serialized_start=2618
  _OUTPUTSTREAM._serialized_end=2824
  _MATERIALIZEDFEATUREVIEWARGS._serialized_start=2827
  _MATERIALIZEDFEATUREVIEWARGS._serialized_end=5302
  _ATTRIBUTE._serialized_start=5304
  _ATTRIBUTE._serialized_end=5405
  _INFERENCE._serialized_start=5407
  _INFERENCE._serialized_end=5526
  _EMBEDDING._serialized_start=5529
  _EMBEDDING._serialized_end=5672
  _REALTIMEARGS._serialized_start=5675
  _REALTIMEARGS._serialized_end=5878
  _FEATURETABLEARGS._serialized_start=5881
  _FEATURETABLEARGS._serialized_end=6693
  _FEATUREAGGREGATION._serialized_start=6696
  _FEATUREAGGREGATION._serialized_end=7527
  _FEATUREAGGREGATION_FUNCTIONPARAMSENTRY._serialized_start=7409
  _FEATUREAGGREGATION_FUNCTIONPARAMSENTRY._serialized_end=7505
  _PARAMVALUE._serialized_start=7529
  _PARAMVALUE._serialized_end=7622
  _DATAQUALITYCONFIG._serialized_start=7625
  _DATAQUALITYCONFIG._serialized_end=7768
  _CLUSTERCONFIG._serialized_start=7771
  _CLUSTERCONFIG._serialized_end=8455
  _JSONCLUSTERCONFIG._serialized_start=8457
  _JSONCLUSTERCONFIG._serialized_end=8521
  _EXISTINGCLUSTERCONFIG._serialized_start=8523
  _EXISTINGCLUSTERCONFIG._serialized_end=8594
  _NEWCLUSTERCONFIG._serialized_start=8597
  _NEWCLUSTERCONFIG._serialized_end=9012
  _RIFTCLUSTERCONFIG._serialized_start=9014
  _RIFTCLUSTERCONFIG._serialized_end=9070
  _DEFAULTCLUSTERCONFIG._serialized_start=9073
  _DEFAULTCLUSTERCONFIG._serialized_end=9262
  _SPARKCONFIG._serialized_start=9265
  _SPARKCONFIG._serialized_end=9652
  _SPARKCONFIG_SPARKCONFENTRY._serialized_start=9592
  _SPARKCONFIG_SPARKCONFENTRY._serialized_end=9652
  _ONLINESTORECONFIG._serialized_start=9655
  _ONLINESTORECONFIG._serialized_end=9885
  _DYNAMODBONLINESTORE._serialized_start=9887
  _DYNAMODBONLINESTORE._serialized_end=9958
  _REDISONLINESTORE._serialized_start=9961
  _REDISONLINESTORE._serialized_end=10161
  _BIGTABLEONLINESTORE._serialized_start=10163
  _BIGTABLEONLINESTORE._serialized_end=10274
  _OFFLINEFEATURESTORECONFIG._serialized_start=10277
  _OFFLINEFEATURESTORECONFIG._serialized_end=10496
  _PARQUETCONFIG._serialized_start=10498
  _PARQUETCONFIG._serialized_end=10513
  _DELTACONFIG._serialized_start=10515
  _DELTACONFIG._serialized_end=10603
  _OFFLINESTORECONFIG._serialized_start=10606
  _OFFLINESTORECONFIG._serialized_end=10862
  _MONITORINGCONFIG._serialized_start=10865
  _MONITORINGCONFIG._serialized_end=11049
  _SECONDARYKEYOUTPUTCOLUMN._serialized_start=11052
  _SECONDARYKEYOUTPUTCOLUMN._serialized_end=11339
  _TIMEWINDOW._serialized_start=11342
  _TIMEWINDOW._serialized_end=11473
  _TIMEWINDOWSERIES._serialized_start=11476
  _TIMEWINDOWSERIES._serialized_end=11738
  _CACHECONFIG._serialized_start=11740
  _CACHECONFIG._serialized_end=11793
# @@protoc_insertion_point(module_scope)
