# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/auth/authorization_service__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.auditlog import metadata__client_pb2 as tecton__proto_dot_auditlog_dot_metadata____client__pb2
from tecton_proto.auth import principal__client_pb2 as tecton__proto_dot_auth_dot_principal____client__pb2
from tecton_proto.auth import resource__client_pb2 as tecton__proto_dot_auth_dot_resource____client__pb2
from tecton_proto.auth import resource_role_assignments__client_pb2 as tecton__proto_dot_auth_dot_resource__role__assignments____client__pb2
from tecton_proto.auth import service__client_pb2 as tecton__proto_dot_auth_dot_service____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5tecton_proto/auth/authorization_service__client.proto\x12\x11tecton_proto.auth\x1a,tecton_proto/auditlog/metadata__client.proto\x1a)tecton_proto/auth/principal__client.proto\x1a(tecton_proto/auth/resource__client.proto\x1a\x39tecton_proto/auth/resource_role_assignments__client.proto\x1a\'tecton_proto/auth/service__client.proto\"\x11\n\x0fGetRolesRequest\"K\n\x10GetRolesResponse\x12\x37\n\x05roles\x18\x01 \x03(\x0b\x32!.tecton_proto.auth.RoleDefinitionR\x05roles\"\x85\x03\n\x0eRoleDefinition\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12 \n\x0b\x64\x65scription\x18\x03 \x01(\tR\x0b\x64\x65scription\x12`\n\x1c\x61ssignable_on_resource_types\x18\x04 \x03(\x0e\x32\x1f.tecton_proto.auth.ResourceTypeR\x19\x61ssignableOnResourceTypes\x12\x63\n\x1d\x61ssignable_to_principal_types\x18\x05 \x03(\x0e\x32 .tecton_proto.auth.PrincipalTypeR\x1a\x61ssignableToPrincipalTypes\x12I\n\x0bpermissions\x18\x06 \x03(\x0b\x32\'.tecton_proto.auth.PermissionDefinitionR\x0bpermissions\x12\x1b\n\tlegacy_id\x18\x07 \x01(\tR\x08legacyId\"m\n\x14PermissionDefinition\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12 \n\x0b\x64\x65scription\x18\x02 \x01(\tR\x0b\x64\x65scription\x12#\n\ris_authorized\x18\x03 \x01(\x08R\x0cisAuthorized\"\x84\x02\n\x17GetAssignedRolesRequest\x12G\n\x0eprincipal_type\x18\x01 \x01(\x0e\x32 .tecton_proto.auth.PrincipalTypeR\rprincipalType\x12!\n\x0cprincipal_id\x18\x02 \x01(\tR\x0bprincipalId\x12\x44\n\rresource_type\x18\x03 \x01(\x0e\x32\x1f.tecton_proto.auth.ResourceTypeR\x0cresourceType\x12!\n\x0cresource_ids\x18\x04 \x03(\tR\x0bresourceIds\x12\x14\n\x05roles\x18\x05 \x03(\tR\x05roles\"k\n\x18GetAssignedRolesResponse\x12O\n\x0b\x61ssignments\x18\x01 \x03(\x0b\x32-.tecton_proto.auth.ResourceAndRoleAssignmentsR\x0b\x61ssignments\"\x85\x02\n\x18ListAssignedRolesRequest\x12G\n\x0eprincipal_type\x18\x01 \x01(\x0e\x32 .tecton_proto.auth.PrincipalTypeR\rprincipalType\x12!\n\x0cprincipal_id\x18\x02 \x01(\tR\x0bprincipalId\x12\x44\n\rresource_type\x18\x03 \x01(\x0e\x32\x1f.tecton_proto.auth.ResourceTypeR\x0cresourceType\x12!\n\x0cresource_ids\x18\x04 \x03(\tR\x0bresourceIds\x12\x14\n\x05roles\x18\x05 \x03(\tR\x05roles\"n\n\x19ListAssignedRolesResponse\x12Q\n\x0b\x61ssignments\x18\x01 \x03(\x0b\x32/.tecton_proto.auth.ResourceAndRoleAssignmentsV2R\x0b\x61ssignments\"\xc5\x01\n\x16GetIsAuthorizedRequest\x12G\n\x0eprincipal_type\x18\x01 \x01(\x0e\x32 .tecton_proto.auth.PrincipalTypeR\rprincipalType\x12!\n\x0cprincipal_id\x18\x02 \x01(\tR\x0bprincipalId\x12?\n\x0bpermissions\x18\x03 \x03(\x0b\x32\x1d.tecton_proto.auth.PermissionR\x0bpermissions\"Z\n\x17GetIsAuthorizedResponse\x12?\n\x0bpermissions\x18\x01 \x03(\x0b\x32\x1d.tecton_proto.auth.PermissionR\x0bpermissions\"^\n\x12\x41ssignRolesRequest\x12H\n\x0b\x61ssignments\x18\x01 \x03(\x0b\x32\x1d.tecton_proto.auth.AssignmentB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0b\x61ssignments\"\x15\n\x13\x41ssignRolesResponse\"`\n\x14UnassignRolesRequest\x12H\n\x0b\x61ssignments\x18\x01 \x03(\x0b\x32\x1d.tecton_proto.auth.AssignmentB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0b\x61ssignments\"\x17\n\x15UnassignRolesResponse\"\xa7\x02\n\nAssignment\x12M\n\rresource_type\x18\x01 \x01(\x0e\x32\x1f.tecton_proto.auth.ResourceTypeB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0cresourceType\x12(\n\x0bresource_id\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\nresourceId\x12\"\n\x04role\x18\x03 \x01(\tB\x0e\xea\x99\xbd\x46\x02\x08\x02\xea\x99\xbd\x46\x02\x10\x01R\x04role\x12P\n\x0eprincipal_type\x18\x04 \x01(\x0e\x32 .tecton_proto.auth.PrincipalTypeB\x07\xea\x99\xbd\x46\x02\x08\x02R\rprincipalType\x12*\n\x0cprincipal_id\x18\x05 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0bprincipalId\"\xb4\x02\n\x15\x41ssignRolesPutRequest\x12M\n\rresource_type\x18\x01 \x01(\x0e\x32\x1f.tecton_proto.auth.ResourceTypeB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0cresourceType\x12(\n\x0bresource_id\x18\x02 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\nresourceId\x12P\n\x0eprincipal_type\x18\x03 \x01(\x0e\x32 .tecton_proto.auth.PrincipalTypeB\x07\xea\x99\xbd\x46\x02\x08\x02R\rprincipalType\x12*\n\x0cprincipal_id\x18\x04 \x01(\tB\x07\xea\x99\xbd\x46\x02\x08\x02R\x0bprincipalId\x12$\n\x05roles\x18\x05 \x03(\tB\x0e\xea\x99\xbd\x46\x02\x08\x02\xea\x99\xbd\x46\x02\x10\x01R\x05roles\"\x18\n\x16\x41ssignRolesPutResponse\"\xe9\x01\n\x1dGetAuthorizedResourcesRequest\x12G\n\x0eprincipal_type\x18\x01 \x01(\x0e\x32 .tecton_proto.auth.PrincipalTypeR\rprincipalType\x12!\n\x0cprincipal_id\x18\x02 \x01(\tR\x0bprincipalId\x12\x44\n\rresource_type\x18\x03 \x01(\x0e\x32\x1f.tecton_proto.auth.ResourceTypeR\x0cresourceType\x12\x16\n\x06\x61\x63tion\x18\x04 \x01(\tR\x06\x61\x63tion\"{\n\x1eGetAuthorizedResourcesResponse\x12Y\n\x14\x61uthorized_resources\x18\x01 \x03(\x0b\x32&.tecton_proto.auth.AuthorizedResourcesR\x13\x61uthorizedResources\"P\n\x13\x41uthorizedResources\x12\x1f\n\x0bresource_id\x18\x01 \x01(\tR\nresourceId\x12\x18\n\x07\x61\x63tions\x18\x02 \x03(\tR\x07\x61\x63tions\"\xe6\x01\n\x1cGetAssignedPrincipalsRequest\x12\x44\n\rresource_type\x18\x01 \x01(\x0e\x32\x1f.tecton_proto.auth.ResourceTypeR\x0cresourceType\x12\x1f\n\x0bresource_id\x18\x02 \x01(\tR\nresourceId\x12\x14\n\x05roles\x18\x03 \x03(\tR\x05roles\x12I\n\x0fprincipal_types\x18\x04 \x03(\x0e\x32 .tecton_proto.auth.PrincipalTypeR\x0eprincipalTypes\"e\n\x1dGetAssignedPrincipalsResponse\x12\x44\n\x0b\x61ssignments\x18\x01 \x03(\x0b\x32\".tecton_proto.auth.AssignmentBasicR\x0b\x61ssignments\"\xe7\x01\n\x1dListAssignedPrincipalsRequest\x12\x44\n\rresource_type\x18\x01 \x01(\x0e\x32\x1f.tecton_proto.auth.ResourceTypeR\x0cresourceType\x12\x1f\n\x0bresource_id\x18\x02 \x01(\tR\nresourceId\x12\x14\n\x05roles\x18\x03 \x03(\tR\x05roles\x12I\n\x0fprincipal_types\x18\x04 \x03(\x0e\x32 .tecton_proto.auth.PrincipalTypeR\x0eprincipalTypes\"h\n\x1eListAssignedPrincipalsResponse\x12\x46\n\x0b\x61ssignments\x18\x01 \x03(\x0b\x32$.tecton_proto.auth.AssignmentBasicV2R\x0b\x61ssignments\"\xc2\x01\n\x0f\x41ssignmentBasic\x12?\n\tprincipal\x18\x01 \x01(\x0b\x32!.tecton_proto.auth.PrincipalBasicR\tprincipal\x12\x14\n\x05roles\x18\x02 \x03(\tR\x05roles\x12X\n\x10role_assignments\x18\x03 \x03(\x0b\x32-.tecton_proto.auth.ResourceAndRoleAssignmentsR\x0froleAssignments\"\xb0\x01\n\x11\x41ssignmentBasicV2\x12?\n\tprincipal\x18\x01 \x01(\x0b\x32!.tecton_proto.auth.PrincipalBasicR\tprincipal\x12Z\n\x10role_assignments\x18\x02 \x03(\x0b\x32/.tecton_proto.auth.ResourceAndRoleAssignmentsV2R\x0froleAssignments\"\xee\x01\n\x18GetAppPermissionsRequest\x12G\n\x0eprincipal_type\x18\x01 \x01(\x0e\x32 .tecton_proto.auth.PrincipalTypeR\rprincipalType\x12!\n\x0cprincipal_id\x18\x02 \x01(\tR\x0bprincipalId\x12\x66\n\x19resource_type_permissions\x18\x05 \x03(\x0b\x32*.tecton_proto.auth.ResourceTypePermissionsR\x17resourceTypePermissions\"y\n\x17ResourceTypePermissions\x12\x44\n\rresource_type\x18\x01 \x01(\x0e\x32\x1f.tecton_proto.auth.ResourceTypeR\x0cresourceType\x12\x18\n\x07\x61\x63tions\x18\x02 \x03(\tR\x07\x61\x63tions\"\x93\x01\n\x19GetAppPermissionsResponse\x12v\n\x1fresource_type_permission_values\x18\x01 \x03(\x0b\x32/.tecton_proto.auth.ResourceTypePermissionValuesR\x1cresourceTypePermissionValues\"\xb5\x01\n\x1cResourceTypePermissionValues\x12\x44\n\rresource_type\x18\x01 \x01(\x0e\x32\x1f.tecton_proto.auth.ResourceTypeR\x0cresourceType\x12O\n\x11permission_values\x18\x02 \x03(\x0b\x32\".tecton_proto.auth.PermissionValueR\x10permissionValues\"\x8b\x01\n\nPermission\x12\x44\n\rresource_type\x18\x01 \x01(\x0e\x32\x1f.tecton_proto.auth.ResourceTypeR\x0cresourceType\x12\x1f\n\x0bresource_id\x18\x02 \x01(\tR\nresourceId\x12\x16\n\x06\x61\x63tion\x18\x03 \x01(\tR\x06\x61\x63tion\"N\n\x0fPermissionValue\x12\x16\n\x06\x61\x63tion\x18\x01 \x01(\tR\x06\x61\x63tion\x12#\n\ris_authorized\x18\x02 \x01(\x08R\x0cisAuthorized\"\x97\x02\n\x1eGetWorkspacePermissionsRequest\x12G\n\x0eprincipal_type\x18\x01 \x01(\x0e\x32 .tecton_proto.auth.PrincipalTypeR\rprincipalType\x12!\n\x0cprincipal_id\x18\x02 \x01(\tR\x0bprincipalId\x12!\n\x0cworkspace_id\x18\x03 \x01(\tR\x0bworkspaceId\x12\x66\n\x19resource_type_permissions\x18\x04 \x03(\x0b\x32*.tecton_proto.auth.ResourceTypePermissionsR\x17resourceTypePermissions\"\x99\x01\n\x1fGetWorkspacePermissionsResponse\x12v\n\x1fresource_type_permission_values\x18\x01 \x03(\x0b\x32/.tecton_proto.auth.ResourceTypePermissionValuesR\x1cresourceTypePermissionValues2\x93\x0c\n\x14\x41uthorizationService\x12]\n\x08GetRoles\x12\".tecton_proto.auth.GetRolesRequest\x1a#.tecton_proto.auth.GetRolesResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02\x10\x01\x12u\n\x10GetAssignedRoles\x12*.tecton_proto.auth.GetAssignedRolesRequest\x1a+.tecton_proto.auth.GetAssignedRolesResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12x\n\x11ListAssignedRoles\x12+.tecton_proto.auth.ListAssignedRolesRequest\x1a,.tecton_proto.auth.ListAssignedRolesResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12~\n\x0b\x41ssignRoles\x12%.tecton_proto.auth.AssignRolesRequest\x1a&.tecton_proto.auth.AssignRolesResponse\" \xa2\xbc\xe6\xc0\x05\x02@\x01\xf2\x99\xbd\x46\x13\x08\x01\x12\x0c\x61ssign_roles\x1a\x01\x31\x12\x86\x01\n\rUnassignRoles\x12\'.tecton_proto.auth.UnassignRolesRequest\x1a(.tecton_proto.auth.UnassignRolesResponse\"\"\xa2\xbc\xe6\xc0\x05\x02@\x01\xf2\x99\xbd\x46\x15\x08\x01\x12\x0eunassign_roles\x1a\x01\x31\x12\x8b\x01\n\x0e\x41ssignRolesPut\x12(.tecton_proto.auth.AssignRolesPutRequest\x1a).tecton_proto.auth.AssignRolesPutResponse\"$\xa2\xbc\xe6\xc0\x05\x02@\x01\xf2\x99\xbd\x46\x17\x08\x01\x12\x10\x61ssign_roles_put\x1a\x01\x31\x12\x87\x01\n\x16GetAuthorizedResources\x12\x30.tecton_proto.auth.GetAuthorizedResourcesRequest\x1a\x31.tecton_proto.auth.GetAuthorizedResourcesResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\x84\x01\n\x15GetAssignedPrincipals\x12/.tecton_proto.auth.GetAssignedPrincipalsRequest\x1a\x30.tecton_proto.auth.GetAssignedPrincipalsResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\x87\x01\n\x16ListAssignedPrincipals\x12\x30.tecton_proto.auth.ListAssignedPrincipalsRequest\x1a\x31.tecton_proto.auth.ListAssignedPrincipalsResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12x\n\x11GetAppPermissions\x12+.tecton_proto.auth.GetAppPermissionsRequest\x1a,.tecton_proto.auth.GetAppPermissionsResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\x8a\x01\n\x17GetWorkspacePermissions\x12\x31.tecton_proto.auth.GetWorkspacePermissionsRequest\x1a\x32.tecton_proto.auth.GetWorkspacePermissionsResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x12r\n\x0fGetIsAuthorized\x12).tecton_proto.auth.GetIsAuthorizedRequest\x1a*.tecton_proto.auth.GetIsAuthorizedResponse\"\x08\xa2\xbc\xe6\xc0\x05\x02@\x01\x42V\n\x0f\x63om.tecton.authB\x19\x41uthorizationServiceProtoP\x01Z&github.com/tecton-ai/tecton_proto/auth')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.auth.authorization_service__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.authB\031AuthorizationServiceProtoP\001Z&github.com/tecton-ai/tecton_proto/auth'
  _ASSIGNROLESREQUEST.fields_by_name['assignments']._options = None
  _ASSIGNROLESREQUEST.fields_by_name['assignments']._serialized_options = b'\352\231\275F\002\010\002'
  _UNASSIGNROLESREQUEST.fields_by_name['assignments']._options = None
  _UNASSIGNROLESREQUEST.fields_by_name['assignments']._serialized_options = b'\352\231\275F\002\010\002'
  _ASSIGNMENT.fields_by_name['resource_type']._options = None
  _ASSIGNMENT.fields_by_name['resource_type']._serialized_options = b'\352\231\275F\002\010\002'
  _ASSIGNMENT.fields_by_name['resource_id']._options = None
  _ASSIGNMENT.fields_by_name['resource_id']._serialized_options = b'\352\231\275F\002\010\002'
  _ASSIGNMENT.fields_by_name['role']._options = None
  _ASSIGNMENT.fields_by_name['role']._serialized_options = b'\352\231\275F\002\010\002\352\231\275F\002\020\001'
  _ASSIGNMENT.fields_by_name['principal_type']._options = None
  _ASSIGNMENT.fields_by_name['principal_type']._serialized_options = b'\352\231\275F\002\010\002'
  _ASSIGNMENT.fields_by_name['principal_id']._options = None
  _ASSIGNMENT.fields_by_name['principal_id']._serialized_options = b'\352\231\275F\002\010\002'
  _ASSIGNROLESPUTREQUEST.fields_by_name['resource_type']._options = None
  _ASSIGNROLESPUTREQUEST.fields_by_name['resource_type']._serialized_options = b'\352\231\275F\002\010\002'
  _ASSIGNROLESPUTREQUEST.fields_by_name['resource_id']._options = None
  _ASSIGNROLESPUTREQUEST.fields_by_name['resource_id']._serialized_options = b'\352\231\275F\002\010\002'
  _ASSIGNROLESPUTREQUEST.fields_by_name['principal_type']._options = None
  _ASSIGNROLESPUTREQUEST.fields_by_name['principal_type']._serialized_options = b'\352\231\275F\002\010\002'
  _ASSIGNROLESPUTREQUEST.fields_by_name['principal_id']._options = None
  _ASSIGNROLESPUTREQUEST.fields_by_name['principal_id']._serialized_options = b'\352\231\275F\002\010\002'
  _ASSIGNROLESPUTREQUEST.fields_by_name['roles']._options = None
  _ASSIGNROLESPUTREQUEST.fields_by_name['roles']._serialized_options = b'\352\231\275F\002\010\002\352\231\275F\002\020\001'
  _AUTHORIZATIONSERVICE.methods_by_name['GetRoles']._options = None
  _AUTHORIZATIONSERVICE.methods_by_name['GetRoles']._serialized_options = b'\242\274\346\300\005\002\020\001'
  _AUTHORIZATIONSERVICE.methods_by_name['GetAssignedRoles']._options = None
  _AUTHORIZATIONSERVICE.methods_by_name['GetAssignedRoles']._serialized_options = b'\242\274\346\300\005\002@\001'
  _AUTHORIZATIONSERVICE.methods_by_name['ListAssignedRoles']._options = None
  _AUTHORIZATIONSERVICE.methods_by_name['ListAssignedRoles']._serialized_options = b'\242\274\346\300\005\002@\001'
  _AUTHORIZATIONSERVICE.methods_by_name['AssignRoles']._options = None
  _AUTHORIZATIONSERVICE.methods_by_name['AssignRoles']._serialized_options = b'\242\274\346\300\005\002@\001\362\231\275F\023\010\001\022\014assign_roles\032\0011'
  _AUTHORIZATIONSERVICE.methods_by_name['UnassignRoles']._options = None
  _AUTHORIZATIONSERVICE.methods_by_name['UnassignRoles']._serialized_options = b'\242\274\346\300\005\002@\001\362\231\275F\025\010\001\022\016unassign_roles\032\0011'
  _AUTHORIZATIONSERVICE.methods_by_name['AssignRolesPut']._options = None
  _AUTHORIZATIONSERVICE.methods_by_name['AssignRolesPut']._serialized_options = b'\242\274\346\300\005\002@\001\362\231\275F\027\010\001\022\020assign_roles_put\032\0011'
  _AUTHORIZATIONSERVICE.methods_by_name['GetAuthorizedResources']._options = None
  _AUTHORIZATIONSERVICE.methods_by_name['GetAuthorizedResources']._serialized_options = b'\242\274\346\300\005\002@\001'
  _AUTHORIZATIONSERVICE.methods_by_name['GetAssignedPrincipals']._options = None
  _AUTHORIZATIONSERVICE.methods_by_name['GetAssignedPrincipals']._serialized_options = b'\242\274\346\300\005\002@\001'
  _AUTHORIZATIONSERVICE.methods_by_name['ListAssignedPrincipals']._options = None
  _AUTHORIZATIONSERVICE.methods_by_name['ListAssignedPrincipals']._serialized_options = b'\242\274\346\300\005\002@\001'
  _AUTHORIZATIONSERVICE.methods_by_name['GetAppPermissions']._options = None
  _AUTHORIZATIONSERVICE.methods_by_name['GetAppPermissions']._serialized_options = b'\242\274\346\300\005\002@\001'
  _AUTHORIZATIONSERVICE.methods_by_name['GetWorkspacePermissions']._options = None
  _AUTHORIZATIONSERVICE.methods_by_name['GetWorkspacePermissions']._serialized_options = b'\242\274\346\300\005\002@\001'
  _AUTHORIZATIONSERVICE.methods_by_name['GetIsAuthorized']._options = None
  _AUTHORIZATIONSERVICE.methods_by_name['GetIsAuthorized']._serialized_options = b'\242\274\346\300\005\002@\001'
  _GETROLESREQUEST._serialized_start=307
  _GETROLESREQUEST._serialized_end=324
  _GETROLESRESPONSE._serialized_start=326
  _GETROLESRESPONSE._serialized_end=401
  _ROLEDEFINITION._serialized_start=404
  _ROLEDEFINITION._serialized_end=793
  _PERMISSIONDEFINITION._serialized_start=795
  _PERMISSIONDEFINITION._serialized_end=904
  _GETASSIGNEDROLESREQUEST._serialized_start=907
  _GETASSIGNEDROLESREQUEST._serialized_end=1167
  _GETASSIGNEDROLESRESPONSE._serialized_start=1169
  _GETASSIGNEDROLESRESPONSE._serialized_end=1276
  _LISTASSIGNEDROLESREQUEST._serialized_start=1279
  _LISTASSIGNEDROLESREQUEST._serialized_end=1540
  _LISTASSIGNEDROLESRESPONSE._serialized_start=1542
  _LISTASSIGNEDROLESRESPONSE._serialized_end=1652
  _GETISAUTHORIZEDREQUEST._serialized_start=1655
  _GETISAUTHORIZEDREQUEST._serialized_end=1852
  _GETISAUTHORIZEDRESPONSE._serialized_start=1854
  _GETISAUTHORIZEDRESPONSE._serialized_end=1944
  _ASSIGNROLESREQUEST._serialized_start=1946
  _ASSIGNROLESREQUEST._serialized_end=2040
  _ASSIGNROLESRESPONSE._serialized_start=2042
  _ASSIGNROLESRESPONSE._serialized_end=2063
  _UNASSIGNROLESREQUEST._serialized_start=2065
  _UNASSIGNROLESREQUEST._serialized_end=2161
  _UNASSIGNROLESRESPONSE._serialized_start=2163
  _UNASSIGNROLESRESPONSE._serialized_end=2186
  _ASSIGNMENT._serialized_start=2189
  _ASSIGNMENT._serialized_end=2484
  _ASSIGNROLESPUTREQUEST._serialized_start=2487
  _ASSIGNROLESPUTREQUEST._serialized_end=2795
  _ASSIGNROLESPUTRESPONSE._serialized_start=2797
  _ASSIGNROLESPUTRESPONSE._serialized_end=2821
  _GETAUTHORIZEDRESOURCESREQUEST._serialized_start=2824
  _GETAUTHORIZEDRESOURCESREQUEST._serialized_end=3057
  _GETAUTHORIZEDRESOURCESRESPONSE._serialized_start=3059
  _GETAUTHORIZEDRESOURCESRESPONSE._serialized_end=3182
  _AUTHORIZEDRESOURCES._serialized_start=3184
  _AUTHORIZEDRESOURCES._serialized_end=3264
  _GETASSIGNEDPRINCIPALSREQUEST._serialized_start=3267
  _GETASSIGNEDPRINCIPALSREQUEST._serialized_end=3497
  _GETASSIGNEDPRINCIPALSRESPONSE._serialized_start=3499
  _GETASSIGNEDPRINCIPALSRESPONSE._serialized_end=3600
  _LISTASSIGNEDPRINCIPALSREQUEST._serialized_start=3603
  _LISTASSIGNEDPRINCIPALSREQUEST._serialized_end=3834
  _LISTASSIGNEDPRINCIPALSRESPONSE._serialized_start=3836
  _LISTASSIGNEDPRINCIPALSRESPONSE._serialized_end=3940
  _ASSIGNMENTBASIC._serialized_start=3943
  _ASSIGNMENTBASIC._serialized_end=4137
  _ASSIGNMENTBASICV2._serialized_start=4140
  _ASSIGNMENTBASICV2._serialized_end=4316
  _GETAPPPERMISSIONSREQUEST._serialized_start=4319
  _GETAPPPERMISSIONSREQUEST._serialized_end=4557
  _RESOURCETYPEPERMISSIONS._serialized_start=4559
  _RESOURCETYPEPERMISSIONS._serialized_end=4680
  _GETAPPPERMISSIONSRESPONSE._serialized_start=4683
  _GETAPPPERMISSIONSRESPONSE._serialized_end=4830
  _RESOURCETYPEPERMISSIONVALUES._serialized_start=4833
  _RESOURCETYPEPERMISSIONVALUES._serialized_end=5014
  _PERMISSION._serialized_start=5017
  _PERMISSION._serialized_end=5156
  _PERMISSIONVALUE._serialized_start=5158
  _PERMISSIONVALUE._serialized_end=5236
  _GETWORKSPACEPERMISSIONSREQUEST._serialized_start=5239
  _GETWORKSPACEPERMISSIONSREQUEST._serialized_end=5518
  _GETWORKSPACEPERMISSIONSRESPONSE._serialized_start=5521
  _GETWORKSPACEPERMISSIONSRESPONSE._serialized_end=5674
  _AUTHORIZATIONSERVICE._serialized_start=5677
  _AUTHORIZATIONSERVICE._serialized_end=7232
# @@protoc_insertion_point(module_scope)
