# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/feature_server/configuration/feature_server_configuration__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.args import pipeline__client_pb2 as tecton__proto_dot_args_dot_pipeline____client__pb2
from tecton_proto.args import transformation__client_pb2 as tecton__proto_dot_args_dot_transformation____client__pb2
from tecton_proto.args import user_defined_function__client_pb2 as tecton__proto_dot_args_dot_user__defined__function____client__pb2
from tecton_proto.auth import acl__client_pb2 as tecton__proto_dot_auth_dot_acl____client__pb2
from tecton_proto.common import aggregation_function__client_pb2 as tecton__proto_dot_common_dot_aggregation__function____client__pb2
from tecton_proto.common import data_type__client_pb2 as tecton__proto_dot_common_dot_data__type____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import schema__client_pb2 as tecton__proto_dot_common_dot_schema____client__pb2
from tecton_proto.common import time_window__client_pb2 as tecton__proto_dot_common_dot_time__window____client__pb2
from tecton_proto.data import feature_service__client_pb2 as tecton__proto_dot_data_dot_feature__service____client__pb2
from tecton_proto.data import feature_view__client_pb2 as tecton__proto_dot_data_dot_feature__view____client__pb2
from tecton_proto.data import realtime_compute__client_pb2 as tecton__proto_dot_data_dot_realtime__compute____client__pb2
from tecton_proto.data import tecton_api_key__client_pb2 as tecton__proto_dot_data_dot_tecton__api__key____client__pb2
from tecton_proto.data import transformation__client_pb2 as tecton__proto_dot_data_dot_transformation____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nTtecton_proto/feature_server/configuration/feature_server_configuration__client.proto\x12)tecton_proto.feature_server.configuration\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a(tecton_proto/args/pipeline__client.proto\x1a.tecton_proto/args/transformation__client.proto\x1a\x35tecton_proto/args/user_defined_function__client.proto\x1a#tecton_proto/auth/acl__client.proto\x1a\x36tecton_proto/common/aggregation_function__client.proto\x1a+tecton_proto/common/data_type__client.proto\x1a$tecton_proto/common/id__client.proto\x1a(tecton_proto/common/schema__client.proto\x1a-tecton_proto/common/time_window__client.proto\x1a/tecton_proto/data/feature_service__client.proto\x1a,tecton_proto/data/feature_view__client.proto\x1a\x30tecton_proto/data/realtime_compute__client.proto\x1a.tecton_proto/data/tecton_api_key__client.proto\x1a.tecton_proto/data/transformation__client.proto\"\xea\x01\n\x0c\x46\x65\x61turesPlan\x12[\n\x0c\x66\x65\x61ture_plan\x18\x01 \x01(\x0b\x32\x36.tecton_proto.feature_server.configuration.FeaturePlanH\x00R\x0b\x66\x65\x61turePlan\x12w\n\x16realtime_features_plan\x18\x02 \x01(\x0b\x32?.tecton_proto.feature_server.configuration.RealtimeFeaturesPlanH\x00R\x14realtimeFeaturesPlanB\x04\n\x02\x66p\"\xb0\x02\n\x06\x43olumn\x12:\n\tdata_type\x18\x05 \x01(\x0b\x32\x1d.tecton_proto.common.DataTypeR\x08\x64\x61taType\x12\x35\n\x17\x66\x65\x61ture_view_space_name\x18\x02 \x01(\tR\x14\x66\x65\x61tureViewSpaceName\x12;\n\x1a\x66\x65\x61ture_service_space_name\x18\x03 \x01(\tR\x17\x66\x65\x61tureServiceSpaceName\x12,\n\x12\x66\x65\x61ture_view_index\x18\x04 \x01(\x05R\x10\x66\x65\x61tureViewIndex\x12\x42\n\x1e\x62\x61tch_table_feature_view_index\x18\x06 \x01(\x05R\x1a\x62\x61tchTableFeatureViewIndexJ\x04\x08\x01\x10\x02\"\x92\x11\n\x0b\x46\x65\x61turePlan\x12V\n\routput_column\x18\x04 \x01(\x0b\x32\x31.tecton_proto.feature_server.configuration.ColumnR\x0coutputColumn\x12V\n\rinput_columns\x18\x01 \x03(\x0b\x32\x31.tecton_proto.feature_server.configuration.ColumnR\x0cinputColumns\x12[\n\x14\x61ggregation_function\x18\x03 \x01(\x0e\x32(.tecton_proto.common.AggregationFunctionR\x13\x61ggregationFunction\x12n\n\x1b\x61ggregation_function_params\x18\x06 \x01(\x0b\x32..tecton_proto.common.AggregationFunctionParamsR\x19\x61ggregationFunctionParams\x12H\n\x12\x61ggregation_window\x18\x05 \x01(\x0b\x32\x19.google.protobuf.DurationR\x11\x61ggregationWindow\x12N\n\tjoin_keys\x18\x07 \x03(\x0b\x32\x31.tecton_proto.feature_server.configuration.ColumnR\x08joinKeys\x12_\n\x12wildcard_join_keys\x18\x08 \x03(\x0b\x32\x31.tecton_proto.feature_server.configuration.ColumnR\x10wildcardJoinKeys\x12m\n\x19\x61ggregation_secondary_key\x18\x18 \x01(\x0b\x32\x31.tecton_proto.feature_server.configuration.ColumnR\x17\x61ggregationSecondaryKey\x12\x35\n\x17is_secondary_key_output\x18\x19 \x01(\x08R\x14isSecondaryKeyOutput\x12\x1d\n\ntable_name\x18\t \x01(\tR\ttableName\x12|\n\x19\x64\x61ta_table_timestamp_type\x18\x0f \x01(\x0e\x32\x41.tecton_proto.feature_server.configuration.DataTableTimestampTypeR\x16\x64\x61taTableTimestampType\x12\x82\x01\n\x1bstatus_table_timestamp_type\x18\x10 \x01(\x0e\x32\x43.tecton_proto.feature_server.configuration.StatusTableTimestampTypeR\x18statusTableTimestampType\x12#\n\rtimestamp_key\x18\x0c \x01(\tR\x0ctimestampKey\x12<\n\x0cslide_period\x18\r \x01(\x0b\x32\x19.google.protobuf.DurationR\x0bslidePeriod\x12:\n\x0bserving_ttl\x18\x0e \x01(\x0b\x32\x19.google.protobuf.DurationR\nservingTtl\x12\x30\n\x14refresh_status_table\x18\x11 \x01(\x08R\x12refreshStatusTable\x12*\n\x11\x66\x65\x61ture_view_name\x18\x15 \x01(\tR\x0f\x66\x65\x61tureViewName\x12&\n\x0f\x66\x65\x61ture_view_id\x18\x17 \x01(\tR\rfeatureViewId\x12?\n\x1c\x66\x65\x61ture_store_format_version\x18\x12 \x01(\x05R\x19\x66\x65\x61tureStoreFormatVersion\x12T\n\x13online_store_params\x18\x13 \x01(\x0b\x32$.tecton_proto.data.OnlineStoreParamsR\x11onlineStoreParams\x12.\n\x12\x64\x65letionTimeWindow\x18\x16 \x01(\x03R\x12\x64\x65letionTimeWindow\x12@\n\x0btime_window\x18\x1d \x01(\x0b\x32\x1f.tecton_proto.common.TimeWindowR\ntimeWindow\x12\x64\n\x19\x66\x65\x61ture_view_cache_config\x18\x1b \x01(\x0b\x32).tecton_proto.data.FeatureViewCacheConfigR\x16\x66\x65\x61tureViewCacheConfig\x12\x1f\n\x0b\x63\x61\x63he_index\x18\x1c \x01(\x05R\ncacheIndex\x12\x30\n\x14table_format_version\x18\x1e \x01(\x05R\x12tableFormatVersion\x12(\n\x10\x62\x61tch_table_name\x18\x1f \x01(\tR\x0e\x62\x61tchTableName\x12\x37\n\x18\x62\x61tch_table_window_index\x18  \x01(\x05R\x15\x62\x61tchTableWindowIndex\x12*\n\x11stream_table_name\x18\" \x01(\tR\x0fstreamTableName\x12\x43\n\x05tiles\x18# \x03(\x0b\x32-.tecton_proto.common.OnlineBatchTablePartTileR\x05tiles\x12\x39\n\x19is_compacted_feature_view\x18$ \x01(\x08R\x16isCompactedFeatureView\x12\x35\n\x17\x66\x65\x61ture_set_column_hash\x18! \x01(\tR\x14\x66\x65\x61tureSetColumnHashJ\x04\x08\x02\x10\x03J\x04\x08\n\x10\x0bJ\x04\x08\x0b\x10\x0cJ\x04\x08\x14\x10\x15\"g\n\x11\x46\x65\x61tureVectorPlan\x12R\n\x08\x66\x65\x61tures\x18\x01 \x03(\x0b\x32\x36.tecton_proto.feature_server.configuration.FeaturePlanR\x08\x66\x65\x61tures\"\xbc\x06\n\x14RealtimeFeaturesPlan\x12l\n\x19\x61rgs_from_request_context\x18\x02 \x03(\x0b\x32\x31.tecton_proto.feature_server.configuration.ColumnR\x16\x61rgsFromRequestContext\x12K\n\x07outputs\x18\x03 \x03(\x0b\x32\x31.tecton_proto.feature_server.configuration.ColumnR\x07outputs\x12\x83\x01\n\x12\x66\x65\x61ture_set_inputs\x18\x05 \x03(\x0b\x32U.tecton_proto.feature_server.configuration.RealtimeFeaturesPlan.FeatureSetInputsEntryR\x10\x66\x65\x61tureSetInputs\x12\x37\n\x08pipeline\x18\x06 \x01(\x0b\x32\x1b.tecton_proto.args.PipelineR\x08pipeline\x12K\n\x0ftransformations\x18\x07 \x03(\x0b\x32!.tecton_proto.data.TransformationR\x0ftransformations\x12*\n\x11\x66\x65\x61ture_view_name\x18\x08 \x01(\tR\x0f\x66\x65\x61tureViewName\x12&\n\x0f\x66\x65\x61ture_view_id\x18\t \x01(\tR\rfeatureViewId\x12y\n\x17\x63ompact_transformations\x18\n \x03(\x0b\x32@.tecton_proto.feature_server.configuration.CompactTransformationR\x16\x63ompactTransformations\x1a\x81\x01\n\x15\x46\x65\x61tureSetInputsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12R\n\x05value\x18\x02 \x01(\x0b\x32<.tecton_proto.feature_server.configuration.FeatureVectorPlanR\x05value:\x02\x38\x01J\x04\x08\x01\x10\x02J\x04\x08\x04\x10\x05\"\xee\x01\n\x15\x43ompactTransformation\x12\x44\n\x11transformation_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x10transformationId\x12V\n\x13transformation_mode\x18\x02 \x01(\x0e\x32%.tecton_proto.args.TransformationModeR\x12transformationMode\x12\x37\n\x18user_defined_function_id\x18\x03 \x01(\tR\x15userDefinedFunctionId\"\x81\x01\n\rLoggingConfig\x12\x1f\n\x0bsample_rate\x18\x01 \x01(\x02R\nsampleRate\x12.\n\x13log_effective_times\x18\x02 \x01(\x08R\x11logEffectiveTimes\x12\x1f\n\x0b\x61vro_schema\x18\x03 \x01(\tR\navroSchema\"\xd7\x06\n\x12\x46\x65\x61tureServicePlan\x12\x45\n\x12\x66\x65\x61ture_service_id\x18\x04 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x10\x66\x65\x61tureServiceId\x12?\n\x0f\x66\x65\x61ture_view_id\x18\n \x01(\x0b\x32\x17.tecton_proto.common.IdR\rfeatureViewId\x12\x30\n\x14\x66\x65\x61ture_service_name\x18\x02 \x01(\tR\x12\x66\x65\x61tureServiceName\x12*\n\x11\x66\x65\x61ture_view_name\x18\t \x01(\tR\x0f\x66\x65\x61tureViewName\x12%\n\x0eworkspace_name\x18\x08 \x01(\tR\rworkspaceName\x12\x45\n\x12workspace_state_id\x18\x11 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x10workspaceStateId\x12^\n\x0e\x66\x65\x61tures_plans\x18\r \x03(\x0b\x32\x37.tecton_proto.feature_server.configuration.FeaturesPlanR\rfeaturesPlans\x12N\n\x11join_key_template\x18\x05 \x01(\x0b\x32\".tecton_proto.data.JoinKeyTemplateR\x0fjoinKeyTemplate\x12_\n\x0elogging_config\x18\x0c \x01(\x0b\x32\x38.tecton_proto.feature_server.configuration.LoggingConfigR\rloggingConfig\x12Y\n\x14realtime_environment\x18\x0e \x01(\x0b\x32&.tecton_proto.data.OnlineComputeConfigR\x13realtimeEnvironment\x12\x63\n\x0b\x63\x61\x63he_plans\x18\x10 \x03(\x0b\x32\x42.tecton_proto.feature_server.configuration.FeatureServiceCachePlanR\ncachePlansJ\x04\x08\x01\x10\x02J\x04\x08\x03\x10\x04J\x04\x08\x06\x10\x07J\x04\x08\x07\x10\x08J\x04\x08\x0b\x10\x0c\"\xca\x05\n\x11GlobalTableConfig\x12?\n\x0f\x66\x65\x61ture_view_id\x18\x04 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rfeatureViewId\x12*\n\x11\x66\x65\x61ture_view_name\x18\n \x01(\tR\x0f\x66\x65\x61tureViewName\x12%\n\x0eworkspace_name\x18\x0b \x01(\tR\rworkspaceName\x12<\n\x0cslide_period\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0bslidePeriod\x12\x82\x01\n\x1bstatus_table_timestamp_type\x18\x06 \x01(\x0e\x32\x43.tecton_proto.feature_server.configuration.StatusTableTimestampTypeR\x18statusTableTimestampType\x12\x30\n\x14refresh_status_table\x18\x07 \x01(\x08R\x12refreshStatusTable\x12?\n\x1c\x66\x65\x61ture_store_format_version\x18\x08 \x01(\x05R\x19\x66\x65\x61tureStoreFormatVersion\x12T\n\x13online_store_params\x18\t \x01(\x0b\x32$.tecton_proto.data.OnlineStoreParamsR\x11onlineStoreParams\x12\x30\n\x14table_format_version\x18\x0c \x01(\x05R\x12tableFormatVersion\x12Q\n\x17\x66\x65\x61ture_data_water_mark\x18\r \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x14\x66\x65\x61tureDataWaterMarkJ\x04\x08\x01\x10\x02J\x04\x08\x03\x10\x04J\x04\x08\x05\x10\x06\"\x87\x01\n\x12\x46\x65\x61tureServiceAcls\x12\x45\n\x12\x66\x65\x61ture_service_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x10\x66\x65\x61tureServiceId\x12*\n\x04\x61\x63ls\x18\x02 \x03(\x0b\x32\x16.tecton_proto.auth.AclR\x04\x61\x63ls\"b\n\rWorkspaceAcls\x12%\n\x0eworkspace_name\x18\x01 \x01(\tR\rworkspaceName\x12*\n\x04\x61\x63ls\x18\x02 \x03(\x0b\x32\x16.tecton_proto.auth.AclR\x04\x61\x63ls\"\xe1\x01\n\x0c\x43\x61naryConfig\x12\x37\n\x18\x66\x65\x61ture_server_canary_id\x18\x01 \x01(\tR\x15\x66\x65\x61tureServerCanaryId\x12\x42\n\x1e\x66\x65\x61ture_server_canary_pod_name\x18\x02 \x01(\tR\x1a\x66\x65\x61tureServerCanaryPodName\x12T\n\'feature_server_canary_follower_endpoint\x18\x03 \x01(\tR#featureServerCanaryFollowerEndpoint\"^\n\x0b\x43\x61\x63heParams\x12;\n\x05redis\x18\x02 \x01(\x0b\x32#.tecton_proto.data.RedisOnlineStoreH\x00R\x05redisB\x0c\n\ncache_typeJ\x04\x08\x01\x10\x02\"\x8c\x0f\n\x1a\x46\x65\x61tureServerConfiguration\x12?\n\rcomputed_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0c\x63omputedTime\x12h\n\x10\x66\x65\x61ture_services\x18\x01 \x03(\x0b\x32=.tecton_proto.feature_server.configuration.FeatureServicePlanR\x0f\x66\x65\x61tureServices\x12\xa0\x01\n\x1bglobal_table_config_by_name\x18\x03 \x03(\x0b\x32\x62.tecton_proto.feature_server.configuration.FeatureServerConfiguration.GlobalTableConfigByNameEntryR\x17globalTableConfigByName\x12O\n\x13\x61uthorized_api_keys\x18\x04 \x03(\x0b\x32\x1f.tecton_proto.data.TectonApiKeyR\x11\x61uthorizedApiKeys\x12o\n\x14\x66\x65\x61ture_service_acls\x18\x05 \x03(\x0b\x32=.tecton_proto.feature_server.configuration.FeatureServiceAclsR\x12\x66\x65\x61tureServiceAcls\x12_\n\x0eworkspace_acls\x18\x06 \x03(\x0b\x32\x38.tecton_proto.feature_server.configuration.WorkspaceAclsR\rworkspaceAcls\x12[\n\x17\x61ll_online_store_params\x18\x07 \x03(\x0b\x32$.tecton_proto.data.OnlineStoreParamsR\x14\x61llOnlineStoreParams\x12x\n\x1c\x66\x65\x61ture_server_canary_config\x18\x08 \x01(\x0b\x32\x37.tecton_proto.feature_server.configuration.CanaryConfigR\x19\x66\x65\x61tureServerCanaryConfig\x12\x64\n\x16remote_compute_configs\x18\t \x03(\x0b\x32..tecton_proto.data.RemoteFunctionComputeConfigR\x14remoteComputeConfigs\x12\x63\n\x1a\x61ll_online_compute_configs\x18\r \x03(\x0b\x32&.tecton_proto.data.OnlineComputeConfigR\x17\x61llOnlineComputeConfigs\x12y\n\x0c\x63\x61\x63he_groups\x18\n \x03(\x0b\x32V.tecton_proto.feature_server.configuration.FeatureServerConfiguration.CacheGroupsEntryR\x0b\x63\x61\x63heGroups\x12\x9c\x01\n\x19user_defined_function_map\x18\x0b \x03(\x0b\x32\x61.tecton_proto.feature_server.configuration.FeatureServerConfiguration.UserDefinedFunctionMapEntryR\x16userDefinedFunctionMap\x12\x63\n\x04jwks\x18\x0c \x03(\x0b\x32O.tecton_proto.feature_server.configuration.FeatureServerConfiguration.JwksEntryR\x04jwks\x1a\x88\x01\n\x1cGlobalTableConfigByNameEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12R\n\x05value\x18\x02 \x01(\x0b\x32<.tecton_proto.feature_server.configuration.GlobalTableConfigR\x05value:\x02\x38\x01\x1au\n\x10\x43\x61\x63heGroupsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12K\n\x05value\x18\x02 \x01(\x0b\x32\x35.tecton_proto.feature_server.configuration.CacheGroupR\x05value:\x02\x38\x01\x1aq\n\x1bUserDefinedFunctionMapEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12<\n\x05value\x18\x02 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x05value:\x02\x38\x01\x1ag\n\tJwksEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x44\n\x05value\x18\x02 \x01(\x0b\x32..tecton_proto.feature_server.configuration.JwkR\x05value:\x02\x38\x01\"7\n\x03Jwk\x12\x0c\n\x01n\x18\x01 \x01(\tR\x01n\x12\x10\n\x03\x61lg\x18\x02 \x01(\tR\x03\x61lg\x12\x10\n\x03use\x18\x03 \x01(\tR\x03use\"/\n\x10RemappedJoinKeys\x12\x1b\n\tjoin_keys\x18\x01 \x03(\tR\x08joinKeys\"\x9c\x02\n\x17\x46\x65\x61tureServiceCachePlan\x12(\n\x10\x66\x65\x61ture_view_ids\x18\x01 \x03(\tR\x0e\x66\x65\x61tureViewIds\x12(\n\x10\x63\x61\x63he_group_name\x18\x02 \x01(\tR\x0e\x63\x61\x63heGroupName\x12r\n\x17remapped_join_key_lists\x18\x03 \x03(\x0b\x32;.tecton_proto.feature_server.configuration.RemappedJoinKeysR\x14remappedJoinKeyLists\x12\x39\n\x19\x66\x65\x61ture_set_column_hashes\x18\x04 \x03(\tR\x16\x66\x65\x61tureSetColumnHashes\"\xd5\x01\n\nCacheGroup\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x1b\n\tjoin_keys\x18\x02 \x03(\tR\x08joinKeys\x12\x32\n\x07key_ttl\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\x06keyTtl\x12\x38\n\nkey_jitter\x18\x04 \x01(\x0b\x32\x19.google.protobuf.DurationR\tkeyJitter\x12(\n\x10\x66\x65\x61ture_view_ids\x18\x05 \x03(\tR\x0e\x66\x65\x61tureViewIds*\x90\x01\n\x16\x44\x61taTableTimestampType\x12%\n!DATA_TABLE_TIMESTAMP_TYPE_UNKNOWN\x10\x00\x12&\n\"DATA_TABLE_TIMESTAMP_TYPE_SORT_KEY\x10\x01\x12\'\n#DATA_TABLE_TIMESTAMP_TYPE_ATTRIBUTE\x10\x02*\xc9\x01\n\x18StatusTableTimestampType\x12\'\n#STATUS_TABLE_TIMESTAMP_TYPE_UNKNOWN\x10\x00\x12(\n$STATUS_TABLE_TIMESTAMP_TYPE_SORT_KEY\x10\x01\x12)\n%STATUS_TABLE_TIMESTAMP_TYPE_ATTRIBUTE\x10\x02\x12/\n+STATUS_TABLE_TIMESTAMP_CONTINUOUS_AGGREGATE\x10\x03\x42l\n(com.tecton.feature_service.configurationP\x01Z>github.com/tecton-ai/tecton_proto/feature_server/configuration')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.feature_server.configuration.feature_server_configuration__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n(com.tecton.feature_service.configurationP\001Z>github.com/tecton-ai/tecton_proto/feature_server/configuration'
  _REALTIMEFEATURESPLAN_FEATURESETINPUTSENTRY._options = None
  _REALTIMEFEATURESPLAN_FEATURESETINPUTSENTRY._serialized_options = b'8\001'
  _FEATURESERVERCONFIGURATION_GLOBALTABLECONFIGBYNAMEENTRY._options = None
  _FEATURESERVERCONFIGURATION_GLOBALTABLECONFIGBYNAMEENTRY._serialized_options = b'8\001'
  _FEATURESERVERCONFIGURATION_CACHEGROUPSENTRY._options = None
  _FEATURESERVERCONFIGURATION_CACHEGROUPSENTRY._serialized_options = b'8\001'
  _FEATURESERVERCONFIGURATION_USERDEFINEDFUNCTIONMAPENTRY._options = None
  _FEATURESERVERCONFIGURATION_USERDEFINEDFUNCTIONMAPENTRY._serialized_options = b'8\001'
  _FEATURESERVERCONFIGURATION_JWKSENTRY._options = None
  _FEATURESERVERCONFIGURATION_JWKSENTRY._serialized_options = b'8\001'
  _DATATABLETIMESTAMPTYPE._serialized_start=9579
  _DATATABLETIMESTAMPTYPE._serialized_end=9723
  _STATUSTABLETIMESTAMPTYPE._serialized_start=9726
  _STATUSTABLETIMESTAMPTYPE._serialized_end=9927
  _FEATURESPLAN._serialized_start=848
  _FEATURESPLAN._serialized_end=1082
  _COLUMN._serialized_start=1085
  _COLUMN._serialized_end=1389
  _FEATUREPLAN._serialized_start=1392
  _FEATUREPLAN._serialized_end=3586
  _FEATUREVECTORPLAN._serialized_start=3588
  _FEATUREVECTORPLAN._serialized_end=3691
  _REALTIMEFEATURESPLAN._serialized_start=3694
  _REALTIMEFEATURESPLAN._serialized_end=4522
  _REALTIMEFEATURESPLAN_FEATURESETINPUTSENTRY._serialized_start=4381
  _REALTIMEFEATURESPLAN_FEATURESETINPUTSENTRY._serialized_end=4510
  _COMPACTTRANSFORMATION._serialized_start=4525
  _COMPACTTRANSFORMATION._serialized_end=4763
  _LOGGINGCONFIG._serialized_start=4766
  _LOGGINGCONFIG._serialized_end=4895
  _FEATURESERVICEPLAN._serialized_start=4898
  _FEATURESERVICEPLAN._serialized_end=5753
  _GLOBALTABLECONFIG._serialized_start=5756
  _GLOBALTABLECONFIG._serialized_end=6470
  _FEATURESERVICEACLS._serialized_start=6473
  _FEATURESERVICEACLS._serialized_end=6608
  _WORKSPACEACLS._serialized_start=6610
  _WORKSPACEACLS._serialized_end=6708
  _CANARYCONFIG._serialized_start=6711
  _CANARYCONFIG._serialized_end=6936
  _CACHEPARAMS._serialized_start=6938
  _CACHEPARAMS._serialized_end=7032
  _FEATURESERVERCONFIGURATION._serialized_start=7035
  _FEATURESERVERCONFIGURATION._serialized_end=8967
  _FEATURESERVERCONFIGURATION_GLOBALTABLECONFIGBYNAMEENTRY._serialized_start=8492
  _FEATURESERVERCONFIGURATION_GLOBALTABLECONFIGBYNAMEENTRY._serialized_end=8628
  _FEATURESERVERCONFIGURATION_CACHEGROUPSENTRY._serialized_start=8630
  _FEATURESERVERCONFIGURATION_CACHEGROUPSENTRY._serialized_end=8747
  _FEATURESERVERCONFIGURATION_USERDEFINEDFUNCTIONMAPENTRY._serialized_start=8749
  _FEATURESERVERCONFIGURATION_USERDEFINEDFUNCTIONMAPENTRY._serialized_end=8862
  _FEATURESERVERCONFIGURATION_JWKSENTRY._serialized_start=8864
  _FEATURESERVERCONFIGURATION_JWKSENTRY._serialized_end=8967
  _JWK._serialized_start=8969
  _JWK._serialized_end=9024
  _REMAPPEDJOINKEYS._serialized_start=9026
  _REMAPPEDJOINKEYS._serialized_end=9073
  _FEATURESERVICECACHEPLAN._serialized_start=9076
  _FEATURESERVICECACHEPLAN._serialized_end=9360
  _CACHEGROUP._serialized_start=9363
  _CACHEGROUP._serialized_end=9576
# @@protoc_insertion_point(module_scope)
