# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/common/compute_mode__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.tecton_proto/common/compute_mode__client.proto\x12\x13tecton_proto.common*\x93\x01\n\x10\x42\x61tchComputeMode\x12\"\n\x1e\x42\x41TCH_COMPUTE_MODE_UNSPECIFIED\x10\x00\x12\x1c\n\x18\x42\x41TCH_COMPUTE_MODE_SPARK\x10\x01\x12 \n\x1c\x42\x41TCH_COMPUTE_MODE_SNOWFLAKE\x10\x02\x12\x1b\n\x17\x42\x41TCH_COMPUTE_MODE_RIFT\x10\x03\x42\x15\n\x11\x63om.tecton.commonP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.common.compute_mode__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021com.tecton.commonP\001'
  _BATCHCOMPUTEMODE._serialized_start=72
  _BATCHCOMPUTEMODE._serialized_end=219
# @@protoc_insertion_point(module_scope)
