# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/transformation__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.args import transformation__client_pb2 as tecton__proto_dot_args_dot_transformation____client__pb2
from tecton_proto.args import user_defined_function__client_pb2 as tecton__proto_dot_args_dot_user__defined__function____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.data import fco_metadata__client_pb2 as tecton__proto_dot_data_dot_fco__metadata____client__pb2
from tecton_proto.validation import validator__client_pb2 as tecton__proto_dot_validation_dot_validator____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.tecton_proto/data/transformation__client.proto\x12\x11tecton_proto.data\x1a.tecton_proto/args/transformation__client.proto\x1a\x35tecton_proto/args/user_defined_function__client.proto\x1a$tecton_proto/common/id__client.proto\x1a,tecton_proto/data/fco_metadata__client.proto\x1a/tecton_proto/validation/validator__client.proto\"\xa4\x04\n\x0eTransformation\x12\x44\n\x11transformation_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x10transformationId\x12\x41\n\x0c\x66\x63o_metadata\x18\x02 \x01(\x0b\x32\x1e.tecton_proto.data.FcoMetadataR\x0b\x66\x63oMetadata\x12V\n\x13transformation_mode\x18\x03 \x01(\x0e\x32%.tecton_proto.args.TransformationModeR\x12transformationMode\x12K\n\ruser_function\x18\x04 \x01(\x0b\x32&.tecton_proto.args.UserDefinedFunctionR\x0cuserFunction\x12^\n\x0fvalidation_args\x18\x05 \x01(\x0b\x32\x35.tecton_proto.validation.TransformationValidationArgsR\x0evalidationArgs\x12H\n\x07options\x18\x06 \x03(\x0b\x32..tecton_proto.data.Transformation.OptionsEntryR\x07options\x1a:\n\x0cOptionsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x42\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.transformation__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _TRANSFORMATION_OPTIONSENTRY._options = None
  _TRANSFORMATION_OPTIONSENTRY._serialized_options = b'8\001'
  _TRANSFORMATION._serialized_start=306
  _TRANSFORMATION._serialized_end=854
  _TRANSFORMATION_OPTIONSENTRY._serialized_start=796
  _TRANSFORMATION_OPTIONSENTRY._serialized_end=854
# @@protoc_insertion_point(module_scope)
