# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/data_source_config__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2tecton_proto/args/data_source_config__client.proto\x12\x11tecton_proto.args\x1a\x1egoogle/protobuf/duration.proto\"\x13\n\x0b\x42\x61tchConfigJ\x04\x08\x01\x10\x02\"\xc7\x01\n\x0cStreamConfig\x12`\n\x17initial_stream_position\x18\x01 \x01(\x0e\x32(.tecton_proto.args.InitialStreamPositionR\x15initialStreamPosition\x12U\n\x19watermark_delay_threshold\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationR\x17watermarkDelayThreshold*\x94\x01\n\x15InitialStreamPosition\x12\'\n#INITIAL_STREAM_POSITION_UNSPECIFIED\x10\x00\x12\"\n\x1eINITIAL_STREAM_POSITION_LATEST\x10\x01\x12(\n$INITIAL_STREAM_POSITION_TRIM_HORIZON\x10\x03\"\x04\x08\x02\x10\x02\x42\x13\n\x0f\x63om.tecton.argsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.data_source_config__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001'
  _INITIALSTREAMPOSITION._serialized_start=329
  _INITIALSTREAMPOSITION._serialized_end=477
  _BATCHCONFIG._serialized_start=105
  _BATCHCONFIG._serialized_end=124
  _STREAMCONFIG._serialized_start=127
  _STREAMCONFIG._serialized_end=326
# @@protoc_insertion_point(module_scope)
