# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/realtime_compute__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0tecton_proto/data/realtime_compute__client.proto\x12\x11tecton_proto.data\"\x18\n\x16\x43olocatedComputeConfig\"d\n\x1bRemoteFunctionComputeConfig\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12!\n\x0c\x66unction_uri\x18\x03 \x01(\tR\x0b\x66unctionUri\"\xdb\x01\n\x17\x41WSComputeInstanceGroup\x12\x32\n\x15\x61utoscaling_group_arn\x18\x01 \x01(\tR\x13\x61utoscalingGroupArn\x12\x34\n\x16\x61utoscaling_group_name\x18\x02 \x01(\tR\x14\x61utoscalingGroupName\x12\x16\n\x06region\x18\x03 \x01(\tR\x06region\x12\x12\n\x04port\x18\x04 \x01(\x05R\x04port\x12*\n\x11health_check_path\x18\x05 \x01(\tR\x0fhealthCheckPath\"\x98\x01\n\x0e\x41WSTargetGroup\x12\x10\n\x03\x61rn\x18\x01 \x01(\tR\x03\x61rn\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12`\n\x16\x63ompute_instance_group\x18\x03 \x01(\x0b\x32*.tecton_proto.data.AWSComputeInstanceGroupR\x14\x63omputeInstanceGroup\"p\n\x1fGoogleCloudComputeInstanceGroup\x12\x18\n\x07project\x18\x01 \x01(\tR\x07project\x12\x16\n\x06region\x18\x02 \x01(\tR\x06region\x12\x1b\n\ttarget_id\x18\x03 \x01(\tR\x08targetId\"\xd4\x01\n\x19GoogleCloudBackendService\x12\x1b\n\ttarget_id\x18\x01 \x01(\tR\x08targetId\x12\x18\n\x07project\x18\x02 \x01(\tR\x07project\x12\x16\n\x06region\x18\x03 \x01(\tR\x06region\x12h\n\x16\x63ompute_instance_group\x18\x04 \x01(\x0b\x32\x32.tecton_proto.data.GoogleCloudComputeInstanceGroupR\x14\x63omputeInstanceGroup\"\xd3\x01\n\x12LoadBalancerTarget\x12M\n\x10\x61ws_target_group\x18\x01 \x01(\x0b\x32!.tecton_proto.data.AWSTargetGroupH\x00R\x0e\x61wsTargetGroup\x12\x64\n\x16google_backend_service\x18\x02 \x01(\x0b\x32,.tecton_proto.data.GoogleCloudBackendServiceH\x00R\x14googleBackendServiceB\x08\n\x06target\"\xb7\x03\n\x13OnlineComputeConfig\x12X\n\x11\x63olocated_compute\x18\x01 \x01(\x0b\x32).tecton_proto.data.ColocatedComputeConfigH\x00R\x10\x63olocatedCompute\x12W\n\x0eremote_compute\x18\x02 \x01(\x0b\x32..tecton_proto.data.RemoteFunctionComputeConfigH\x00R\rremoteCompute\x12i\n\x1a\x61ws_compute_instance_group\x18\x03 \x01(\x0b\x32*.tecton_proto.data.AWSComputeInstanceGroupH\x00R\x17\x61wsComputeInstanceGroup\x12w\n\x1dgoogle_compute_instance_group\x18\x04 \x01(\x0b\x32\x32.tecton_proto.data.GoogleCloudComputeInstanceGroupH\x00R\x1agoogleComputeInstanceGroupB\t\n\x07\x63omputeB\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.realtime_compute__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _COLOCATEDCOMPUTECONFIG._serialized_start=71
  _COLOCATEDCOMPUTECONFIG._serialized_end=95
  _REMOTEFUNCTIONCOMPUTECONFIG._serialized_start=97
  _REMOTEFUNCTIONCOMPUTECONFIG._serialized_end=197
  _AWSCOMPUTEINSTANCEGROUP._serialized_start=200
  _AWSCOMPUTEINSTANCEGROUP._serialized_end=419
  _AWSTARGETGROUP._serialized_start=422
  _AWSTARGETGROUP._serialized_end=574
  _GOOGLECLOUDCOMPUTEINSTANCEGROUP._serialized_start=576
  _GOOGLECLOUDCOMPUTEINSTANCEGROUP._serialized_end=688
  _GOOGLECLOUDBACKENDSERVICE._serialized_start=691
  _GOOGLECLOUDBACKENDSERVICE._serialized_end=903
  _LOADBALANCERTARGET._serialized_start=906
  _LOADBALANCERTARGET._serialized_end=1117
  _ONLINECOMPUTECONFIG._serialized_start=1120
  _ONLINECOMPUTECONFIG._serialized_end=1559
# @@protoc_insertion_point(module_scope)
