# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/modelartifactservice/model_artifact_service__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.auth import service__client_pb2 as tecton__proto_dot_auth_dot_service____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import schema__client_pb2 as tecton__proto_dot_common_dot_schema____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nFtecton_proto/modelartifactservice/model_artifact_service__client.proto\x12!tecton_proto.modelartifactservice\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\'tecton_proto/auth/service__client.proto\x1a$tecton_proto/common/id__client.proto\x1a(tecton_proto/common/schema__client.proto\"\xfa\x06\n\x11ModelArtifactInfo\x12\'\n\x02id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x02id\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12@\n\x04type\x18\x03 \x01(\x0e\x32,.tecton_proto.modelartifactservice.ModelTypeR\x04type\x12 \n\x0b\x64\x65scription\x18\x04 \x01(\tR\x0b\x64\x65scription\x12\x65\n\x0b\x66ile_hashes\x18\x05 \x03(\x0b\x32\x44.tecton_proto.modelartifactservice.ModelArtifactInfo.FileHashesEntryR\nfileHashes\x12&\n\x0fmodel_file_path\x18\r \x01(\tR\rmodelFilePath\x12>\n\x0cinput_schema\x18\x06 \x01(\x0b\x32\x1b.tecton_proto.common.SchemaR\x0binputSchema\x12@\n\routput_schema\x18\x07 \x01(\x0b\x32\x1b.tecton_proto.common.SchemaR\x0coutputSchema\x12R\n\x04tags\x18\t \x03(\x0b\x32>.tecton_proto.modelartifactservice.ModelArtifactInfo.TagsEntryR\x04tags\x12!\n\x0cstorage_path\x18\x08 \x01(\tR\x0bstoragePath\x12N\n\x06status\x18\n \x01(\x0e\x32\x36.tecton_proto.modelartifactservice.ModelArtifactStatusR\x06status\x12\x39\n\ncreated_at\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x1a=\n\x0f\x46ileHashesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x1a\x37\n\tTagsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"X\n\x19ListModelArtifactsRequest\x12\'\n\x02id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x02id\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\"j\n\x1aListModelArtifactsResponse\x12L\n\x06models\x18\x01 \x03(\x0b\x32\x34.tecton_proto.modelartifactservice.ModelArtifactInfoR\x06models\"[\n\x1c\x44\x65scribeModelArtifactRequest\x12\'\n\x02id\x18\x02 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x02id\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\"\x85\x01\n\x1d\x44\x65scribeModelArtifactResponse\x12\x64\n\x13model_artifact_info\x18\x01 \x01(\x0b\x32\x34.tecton_proto.modelartifactservice.ModelArtifactInfoR\x11modelArtifactInfo\"X\n\x19\x46\x65tchModelArtifactRequest\x12\'\n\x02id\x18\x02 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x02id\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\"?\n\x1a\x46\x65tchModelArtifactResponse\x12!\n\x0c\x64ownload_url\x18\x01 \x01(\tR\x0b\x64ownloadUrl\"\x83\x05\n\x1a\x43reateModelArtifactRequest\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12@\n\x04type\x18\x02 \x01(\x0e\x32,.tecton_proto.modelartifactservice.ModelTypeR\x04type\x12 \n\x0b\x64\x65scription\x18\x03 \x01(\tR\x0b\x64\x65scription\x12n\n\x0b\x66ile_hashes\x18\x04 \x03(\x0b\x32M.tecton_proto.modelartifactservice.CreateModelArtifactRequest.FileHashesEntryR\nfileHashes\x12&\n\x0fmodel_file_path\x18\x08 \x01(\tR\rmodelFilePath\x12>\n\x0cinput_schema\x18\x05 \x01(\x0b\x32\x1b.tecton_proto.common.SchemaR\x0binputSchema\x12@\n\routput_schema\x18\x06 \x01(\x0b\x32\x1b.tecton_proto.common.SchemaR\x0coutputSchema\x12[\n\x04tags\x18\x07 \x03(\x0b\x32G.tecton_proto.modelartifactservice.CreateModelArtifactRequest.TagsEntryR\x04tags\x1a=\n\x0f\x46ileHashesEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\x1a\x37\n\tTagsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\x83\x01\n\x1b\x43reateModelArtifactResponse\x12\x64\n\x13model_artifact_info\x18\x01 \x01(\x0b\x32\x34.tecton_proto.modelartifactservice.ModelArtifactInfoR\x11modelArtifactInfo\"g\n GetModelArtifactUploadUrlRequest\x12\x43\n\x11model_artifact_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x0fmodelArtifactId\"\x85\x01\n!GetModelArtifactUploadUrlResponse\x12\x43\n\x11model_artifact_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x0fmodelArtifactId\x12\x1b\n\tupload_id\x18\x02 \x01(\tR\x08uploadId\"\xb0\x01\n\x1eUploadModelArtifactPartRequest\x12\x43\n\x11model_artifact_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x0fmodelArtifactId\x12(\n\x10parent_upload_id\x18\x02 \x01(\tR\x0eparentUploadId\x12\x1f\n\x0bpart_number\x18\x03 \x01(\x05R\npartNumber\"@\n\x1fUploadModelArtifactPartResponse\x12\x1d\n\nupload_url\x18\x01 \x01(\tR\tuploadUrl\"\xb9\x02\n\"CompleteModelArtifactUploadRequest\x12\x43\n\x11model_artifact_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x0fmodelArtifactId\x12\x1b\n\tupload_id\x18\x02 \x01(\tR\x08uploadId\x12s\n\npart_etags\x18\x03 \x03(\x0b\x32T.tecton_proto.modelartifactservice.CompleteModelArtifactUploadRequest.PartEtagsEntryR\tpartEtags\x1a<\n\x0ePartEtagsEntry\x12\x10\n\x03key\x18\x01 \x01(\x05R\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"7\n#CompleteModelArtifactUploadResponse\x12\x10\n\x03key\x18\x02 \x01(\tR\x03key\"Y\n\x1a\x44\x65leteModelArtifactRequest\x12\'\n\x02id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x02id\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\"\x1d\n\x1b\x44\x65leteModelArtifactResponse*t\n\x13ModelArtifactStatus\x12\x16\n\x12STATUS_UNSPECIFIED\x10\x00\x12\x10\n\x0cPENDING_FILE\x10\x01\x12\x10\n\x0cPENDING_SCAN\x10\x02\x12\t\n\x05READY\x10\x03\x12\t\n\x05\x45RROR\x10\x04\x12\x0b\n\x07\x44\x45LETED\x10\x05*O\n\tModelType\x12\x1a\n\x16MODEL_TYPE_UNSPECIFIED\x10\x00\x12\x19\n\x15TECTON_TEXT_EMBEDDING\x10\x01\x12\x0b\n\x07PYTORCH\x10\x02\x32\xc5\x0b\n\x14ModelArtifactService\x12\xc6\x01\n\x13\x43reateModelArtifact\x12=.tecton_proto.modelartifactservice.CreateModelArtifactRequest\x1a>.tecton_proto.modelartifactservice.CreateModelArtifactResponse\"0\xa2\xbc\xe6\xc0\x05**(tecton/organization#scale_feature_server\x12\xc3\x01\n\x12ListModelArtifacts\x12<.tecton_proto.modelartifactservice.ListModelArtifactsRequest\x1a=.tecton_proto.modelartifactservice.ListModelArtifactsResponse\"0\xa2\xbc\xe6\xc0\x05**(tecton/organization#scale_feature_server\x12\xc3\x01\n\x12\x46\x65tchModelArtifact\x12<.tecton_proto.modelartifactservice.FetchModelArtifactRequest\x1a=.tecton_proto.modelartifactservice.FetchModelArtifactResponse\"0\xa2\xbc\xe6\xc0\x05**(tecton/organization#scale_feature_server\x12\xd8\x01\n\x19GetModelArtifactUploadUrl\x12\x43.tecton_proto.modelartifactservice.GetModelArtifactUploadUrlRequest\x1a\x44.tecton_proto.modelartifactservice.GetModelArtifactUploadUrlResponse\"0\xa2\xbc\xe6\xc0\x05**(tecton/organization#scale_feature_server\x12\xd2\x01\n\x17UploadModelArtifactPart\x12\x41.tecton_proto.modelartifactservice.UploadModelArtifactPartRequest\x1a\x42.tecton_proto.modelartifactservice.UploadModelArtifactPartResponse\"0\xa2\xbc\xe6\xc0\x05**(tecton/organization#scale_feature_server\x12\xde\x01\n\x1b\x43ompleteModelArtifactUpload\x12\x45.tecton_proto.modelartifactservice.CompleteModelArtifactUploadRequest\x1a\x46.tecton_proto.modelartifactservice.CompleteModelArtifactUploadResponse\"0\xa2\xbc\xe6\xc0\x05**(tecton/organization#scale_feature_server\x12\xc6\x01\n\x13\x44\x65leteModelArtifact\x12=.tecton_proto.modelartifactservice.DeleteModelArtifactRequest\x1a>.tecton_proto.modelartifactservice.DeleteModelArtifactResponse\"0\xa2\xbc\xe6\xc0\x05**(tecton/organization#scale_feature_serverBv\n\x1f\x63om.tecton.modelartifactserviceB\x19ModelArtifactServiceProtoP\x01Z6github.com/tecton-ai/tecton_proto/modelartifactservice')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.modelartifactservice.model_artifact_service__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\037com.tecton.modelartifactserviceB\031ModelArtifactServiceProtoP\001Z6github.com/tecton-ai/tecton_proto/modelartifactservice'
  _MODELARTIFACTINFO_FILEHASHESENTRY._options = None
  _MODELARTIFACTINFO_FILEHASHESENTRY._serialized_options = b'8\001'
  _MODELARTIFACTINFO_TAGSENTRY._options = None
  _MODELARTIFACTINFO_TAGSENTRY._serialized_options = b'8\001'
  _CREATEMODELARTIFACTREQUEST_FILEHASHESENTRY._options = None
  _CREATEMODELARTIFACTREQUEST_FILEHASHESENTRY._serialized_options = b'8\001'
  _CREATEMODELARTIFACTREQUEST_TAGSENTRY._options = None
  _CREATEMODELARTIFACTREQUEST_TAGSENTRY._serialized_options = b'8\001'
  _COMPLETEMODELARTIFACTUPLOADREQUEST_PARTETAGSENTRY._options = None
  _COMPLETEMODELARTIFACTUPLOADREQUEST_PARTETAGSENTRY._serialized_options = b'8\001'
  _MODELARTIFACTSERVICE.methods_by_name['CreateModelArtifact']._options = None
  _MODELARTIFACTSERVICE.methods_by_name['CreateModelArtifact']._serialized_options = b'\242\274\346\300\005**(tecton/organization#scale_feature_server'
  _MODELARTIFACTSERVICE.methods_by_name['ListModelArtifacts']._options = None
  _MODELARTIFACTSERVICE.methods_by_name['ListModelArtifacts']._serialized_options = b'\242\274\346\300\005**(tecton/organization#scale_feature_server'
  _MODELARTIFACTSERVICE.methods_by_name['FetchModelArtifact']._options = None
  _MODELARTIFACTSERVICE.methods_by_name['FetchModelArtifact']._serialized_options = b'\242\274\346\300\005**(tecton/organization#scale_feature_server'
  _MODELARTIFACTSERVICE.methods_by_name['GetModelArtifactUploadUrl']._options = None
  _MODELARTIFACTSERVICE.methods_by_name['GetModelArtifactUploadUrl']._serialized_options = b'\242\274\346\300\005**(tecton/organization#scale_feature_server'
  _MODELARTIFACTSERVICE.methods_by_name['UploadModelArtifactPart']._options = None
  _MODELARTIFACTSERVICE.methods_by_name['UploadModelArtifactPart']._serialized_options = b'\242\274\346\300\005**(tecton/organization#scale_feature_server'
  _MODELARTIFACTSERVICE.methods_by_name['CompleteModelArtifactUpload']._options = None
  _MODELARTIFACTSERVICE.methods_by_name['CompleteModelArtifactUpload']._serialized_options = b'\242\274\346\300\005**(tecton/organization#scale_feature_server'
  _MODELARTIFACTSERVICE.methods_by_name['DeleteModelArtifact']._options = None
  _MODELARTIFACTSERVICE.methods_by_name['DeleteModelArtifact']._serialized_options = b'\242\274\346\300\005**(tecton/organization#scale_feature_server'
  _MODELARTIFACTSTATUS._serialized_start=3499
  _MODELARTIFACTSTATUS._serialized_end=3615
  _MODELTYPE._serialized_start=3617
  _MODELTYPE._serialized_end=3696
  _MODELARTIFACTINFO._serialized_start=264
  _MODELARTIFACTINFO._serialized_end=1154
  _MODELARTIFACTINFO_FILEHASHESENTRY._serialized_start=1036
  _MODELARTIFACTINFO_FILEHASHESENTRY._serialized_end=1097
  _MODELARTIFACTINFO_TAGSENTRY._serialized_start=1099
  _MODELARTIFACTINFO_TAGSENTRY._serialized_end=1154
  _LISTMODELARTIFACTSREQUEST._serialized_start=1156
  _LISTMODELARTIFACTSREQUEST._serialized_end=1244
  _LISTMODELARTIFACTSRESPONSE._serialized_start=1246
  _LISTMODELARTIFACTSRESPONSE._serialized_end=1352
  _DESCRIBEMODELARTIFACTREQUEST._serialized_start=1354
  _DESCRIBEMODELARTIFACTREQUEST._serialized_end=1445
  _DESCRIBEMODELARTIFACTRESPONSE._serialized_start=1448
  _DESCRIBEMODELARTIFACTRESPONSE._serialized_end=1581
  _FETCHMODELARTIFACTREQUEST._serialized_start=1583
  _FETCHMODELARTIFACTREQUEST._serialized_end=1671
  _FETCHMODELARTIFACTRESPONSE._serialized_start=1673
  _FETCHMODELARTIFACTRESPONSE._serialized_end=1736
  _CREATEMODELARTIFACTREQUEST._serialized_start=1739
  _CREATEMODELARTIFACTREQUEST._serialized_end=2382
  _CREATEMODELARTIFACTREQUEST_FILEHASHESENTRY._serialized_start=1036
  _CREATEMODELARTIFACTREQUEST_FILEHASHESENTRY._serialized_end=1097
  _CREATEMODELARTIFACTREQUEST_TAGSENTRY._serialized_start=1099
  _CREATEMODELARTIFACTREQUEST_TAGSENTRY._serialized_end=1154
  _CREATEMODELARTIFACTRESPONSE._serialized_start=2385
  _CREATEMODELARTIFACTRESPONSE._serialized_end=2516
  _GETMODELARTIFACTUPLOADURLREQUEST._serialized_start=2518
  _GETMODELARTIFACTUPLOADURLREQUEST._serialized_end=2621
  _GETMODELARTIFACTUPLOADURLRESPONSE._serialized_start=2624
  _GETMODELARTIFACTUPLOADURLRESPONSE._serialized_end=2757
  _UPLOADMODELARTIFACTPARTREQUEST._serialized_start=2760
  _UPLOADMODELARTIFACTPARTREQUEST._serialized_end=2936
  _UPLOADMODELARTIFACTPARTRESPONSE._serialized_start=2938
  _UPLOADMODELARTIFACTPARTRESPONSE._serialized_end=3002
  _COMPLETEMODELARTIFACTUPLOADREQUEST._serialized_start=3005
  _COMPLETEMODELARTIFACTUPLOADREQUEST._serialized_end=3318
  _COMPLETEMODELARTIFACTUPLOADREQUEST_PARTETAGSENTRY._serialized_start=3258
  _COMPLETEMODELARTIFACTUPLOADREQUEST_PARTETAGSENTRY._serialized_end=3318
  _COMPLETEMODELARTIFACTUPLOADRESPONSE._serialized_start=3320
  _COMPLETEMODELARTIFACTUPLOADRESPONSE._serialized_end=3375
  _DELETEMODELARTIFACTREQUEST._serialized_start=3377
  _DELETEMODELARTIFACTREQUEST._serialized_end=3466
  _DELETEMODELARTIFACTRESPONSE._serialized_start=3468
  _DELETEMODELARTIFACTRESPONSE._serialized_end=3497
  _MODELARTIFACTSERVICE._serialized_start=3699
  _MODELARTIFACTSERVICE._serialized_end=5176
# @@protoc_insertion_point(module_scope)
