BUILD_STATUS = {'BUILD_ID': 'uncommitted-buildkite-agent-1720474544', 'BUILD_TIMESTAMP': '1720474544', 'FORMATTED_DATE': '2024 Jul 08 21 35 44 Mon', 'GIT_COMMIT': '23555ae47e66c70b59532fd618d87d6f83a4a925', 'BUILD_EMBED_LABEL': '', 'BUILD_HOST': 'buildkite-agent-medium-5bcd4bb9c9-9rw7t', 'BUILD_USER': 'buildkite-agent'}
