# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/online_store_writer/copier__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.data import feature_view__client_pb2 as tecton__proto_dot_data_dot_feature__view____client__pb2
from tecton_proto.online_store_writer import config__client_pb2 as tecton__proto_dot_online__store__writer_dot_config____client__pb2
from tecton_proto.snowflake import snowflake_credentials__client_pb2 as tecton__proto_dot_snowflake_dot_snowflake__credentials____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5tecton_proto/online_store_writer/copier__client.proto\x12 tecton_proto.online_store_writer\x1a\x1fgoogle/protobuf/timestamp.proto\x1a$tecton_proto/common/id__client.proto\x1a,tecton_proto/data/feature_view__client.proto\x1a\x35tecton_proto/online_store_writer/config__client.proto\x1a:tecton_proto/snowflake/snowflake_credentials__client.proto\"\xcd\x07\n\x18OnlineStoreCopierRequest\x12\x8b\x01\n!online_store_writer_configuration\x18\x01 \x01(\x0b\x32@.tecton_proto.online_store_writer.OnlineStoreWriterConfigurationR\x1eonlineStoreWriterConfiguration\x12\x41\n\x0c\x66\x65\x61ture_view\x18\x02 \x01(\x0b\x32\x1e.tecton_proto.data.FeatureViewR\x0b\x66\x65\x61tureView\x12^\n\x1fmaterialization_task_attempt_id\x18\x05 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x1cmaterializationTaskAttemptId\x12\x19\n\x08task_key\x18\t \x01(\tR\x07taskKey\x12#\n\rattempt_index\x18\x08 \x01(\x05R\x0c\x61ttemptIndex\x12\x17\n\x07sqs_url\x18\n \x01(\tR\x06sqsUrl\x12\x91\x01\n#online_store_metadata_configuration\x18\x0b \x01(\x0b\x32\x42.tecton_proto.online_store_writer.OnlineStoreMetadataConfigurationR onlineStoreMetadataConfiguration\x12\x43\n\x1e\x65nable_new_consumption_metrics\x18\x0c \x01(\x08R\x1b\x65nableNewConsumptionMetrics\x12\x65\n\x13object_copy_request\x18\x06 \x01(\x0b\x32\x33.tecton_proto.online_store_writer.ObjectCopyRequestH\x00R\x11objectCopyRequest\x12k\n\x15status_update_request\x18\x07 \x01(\x0b\x32\x35.tecton_proto.online_store_writer.StatusUpdateRequestH\x00R\x13statusUpdateRequest\x12^\n\x10\x64\x65letion_request\x18\r \x01(\x0b\x32\x31.tecton_proto.online_store_writer.DeletionRequestH\x00R\x0f\x64\x65letionRequestB\x0e\n\x0crequest_typeJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05\"3\n\x07S3Stage\x12\x16\n\x06\x62ucket\x18\x01 \x01(\tR\x06\x62ucket\x12\x10\n\x03key\x18\x02 \x01(\tR\x03key\"6\n\x08GCSStage\x12\x16\n\x06\x62ucket\x18\x01 \x01(\tR\x06\x62ucket\x12\x12\n\x04\x62lob\x18\x02 \x01(\tR\x04\x62lob\"\xc6\x01\n\x16SnowflakeInternalStage\x12@\n\x1csnowflake_account_identifier\x18\x01 \x01(\tR\x1asnowflakeAccountIdentifier\x12N\n\x0b\x63redentials\x18\x02 \x01(\x0b\x32,.tecton_proto.snowflake.SnowflakeCredentialsR\x0b\x63redentials\x12\x1a\n\x08location\x18\x03 \x01(\tR\x08location\",\n\x0eLocalFileStage\x12\x1a\n\x08location\x18\x01 \x01(\tR\x08location\"\x8b\x04\n\x11ObjectCopyRequest\x12\x46\n\x08s3_stage\x18\x05 \x01(\x0b\x32).tecton_proto.online_store_writer.S3StageH\x00R\x07s3Stage\x12t\n\x18snowflake_internal_stage\x18\x06 \x01(\x0b\x32\x38.tecton_proto.online_store_writer.SnowflakeInternalStageH\x00R\x16snowflakeInternalStage\x12\\\n\x10local_file_stage\x18\x07 \x01(\x0b\x32\x30.tecton_proto.online_store_writer.LocalFileStageH\x00R\x0elocalFileStage\x12I\n\tgcs_stage\x18\t \x01(\x0b\x32*.tecton_proto.online_store_writer.GCSStageH\x00R\x08gcsStage\x12\x1b\n\tskip_rows\x18\x03 \x01(\x03R\x08skipRows\x12X\n\x0ftimestamp_units\x18\x08 \x01(\x0e\x32/.tecton_proto.online_store_writer.TimestampUnitR\x0etimestampUnitsB\x0c\n\nstage_typeJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03\"\xc3\x01\n\x13StatusUpdateRequest\x12=\n\x0b\x61nchor_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00R\nanchorTime\x12`\n\x1ematerialized_raw_data_end_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00R\x1amaterializedRawDataEndTimeB\x0b\n\ttime_type\"\x90\x01\n\x0f\x44\x65letionRequest\x12\x33\n\x15online_join_keys_path\x18\x01 \x01(\tH\x00R\x12onlineJoinKeysPath\x12<\n\x1aonline_join_keys_full_path\x18\x02 \x01(\tH\x00R\x16onlineJoinKeysFullPathB\n\n\x08location*8\n\rTimestampUnit\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\n\n\x06MILLIS\x10\x01\x12\n\n\x06MICROS\x10\x02\x42 \n\x1c\x63om.tecton.onlinestorewriterP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.online_store_writer.copier__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\034com.tecton.onlinestorewriterP\001'
  _TIMESTAMPUNIT._serialized_start=2526
  _TIMESTAMPUNIT._serialized_end=2582
  _ONLINESTORECOPIERREQUEST._serialized_start=324
  _ONLINESTORECOPIERREQUEST._serialized_end=1297
  _S3STAGE._serialized_start=1299
  _S3STAGE._serialized_end=1350
  _GCSSTAGE._serialized_start=1352
  _GCSSTAGE._serialized_end=1406
  _SNOWFLAKEINTERNALSTAGE._serialized_start=1409
  _SNOWFLAKEINTERNALSTAGE._serialized_end=1607
  _LOCALFILESTAGE._serialized_start=1609
  _LOCALFILESTAGE._serialized_end=1653
  _OBJECTCOPYREQUEST._serialized_start=1656
  _OBJECTCOPYREQUEST._serialized_end=2179
  _STATUSUPDATEREQUEST._serialized_start=2182
  _STATUSUPDATEREQUEST._serialized_end=2377
  _DELETIONREQUEST._serialized_start=2380
  _DELETIONREQUEST._serialized_end=2524
# @@protoc_insertion_point(module_scope)
