# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/databricks_api/instance_profiles__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n;tecton_proto/databricks_api/instance_profiles__client.proto\x12\x1btecton_proto.databricks_api\"y\n\x1cInstanceProfilesListResponse\x12Y\n\x11instance_profiles\x18\x01 \x03(\x0b\x32,.tecton_proto.databricks_api.InstanceProfileR\x10instanceProfiles\"C\n\x0fInstanceProfile\x12\x30\n\x14instance_profile_arn\x18\x01 \x01(\tR\x12instanceProfileArnB\x1d\n\x19\x63om.tecton.databricks_apiP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.databricks_api.instance_profiles__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\031com.tecton.databricks_apiP\001'
  _INSTANCEPROFILESLISTRESPONSE._serialized_start=92
  _INSTANCEPROFILESLISTRESPONSE._serialized_end=213
  _INSTANCEPROFILE._serialized_start=215
  _INSTANCEPROFILE._serialized_end=282
# @@protoc_insertion_point(module_scope)
