# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/common/schema__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from tecton_proto.common import column_type__client_pb2 as tecton__proto_dot_common_dot_column__type____client__pb2
from tecton_proto.common import data_type__client_pb2 as tecton__proto_dot_common_dot_data__type____client__pb2
from tecton_proto.common import time_window__client_pb2 as tecton__proto_dot_common_dot_time__window____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n(tecton_proto/common/schema__client.proto\x12\x13tecton_proto.common\x1a\x1egoogle/protobuf/duration.proto\x1a-tecton_proto/common/column_type__client.proto\x1a+tecton_proto/common/data_type__client.proto\x1a-tecton_proto/common/time_window__client.proto\"\xe4\x02\n\x06\x43olumn\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12I\n\x11offline_data_type\x18\x05 \x01(\x0b\x32\x1d.tecton_proto.common.DataTypeR\x0fofflineDataType\x12V\n\x18\x66\x65\x61ture_server_data_type\x18\x06 \x01(\x0b\x32\x1d.tecton_proto.common.DataTypeR\x15\x66\x65\x61tureServerDataType\x12$\n\x0eraw_spark_type\x18\x02 \x01(\tR\x0crawSparkType\x12,\n\x12raw_snowflake_type\x18\x04 \x01(\tR\x10rawSnowflakeType\x12O\n\x13\x66\x65\x61ture_server_type\x18\x03 \x01(\x0e\x32\x1f.tecton_proto.common.ColumnTypeR\x11\x66\x65\x61tureServerType\"E\n\x06Schema\x12\x35\n\x07\x63olumns\x18\x02 \x03(\x0b\x32\x1b.tecton_proto.common.ColumnR\x07\x63olumnsJ\x04\x08\x01\x10\x02\"|\n\x16OnlineBatchTableFormat\x12\x62\n\x18online_batch_table_parts\x18\x03 \x03(\x0b\x32).tecton_proto.common.OnlineBatchTablePartR\x15onlineBatchTableParts\"\xf5\x01\n\x14OnlineBatchTablePart\x12!\n\x0cwindow_index\x18\x01 \x01(\x05R\x0bwindowIndex\x12@\n\x0btime_window\x18\x02 \x01(\x0b\x32\x1f.tecton_proto.common.TimeWindowR\ntimeWindow\x12\x33\n\x06schema\x18\x03 \x01(\x0b\x32\x1b.tecton_proto.common.SchemaR\x06schema\x12\x43\n\x05tiles\x18\x04 \x03(\x0b\x32-.tecton_proto.common.OnlineBatchTablePartTileR\x05tiles\"\xd2\x01\n\x18OnlineBatchTablePartTile\x12\\\n\x1drelative_start_time_inclusive\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationR\x1arelativeStartTimeInclusive\x12X\n\x1brelative_end_time_exclusive\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationR\x18relativeEndTimeExclusiveB\x15\n\x11\x63om.tecton.commonP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.common.schema__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021com.tecton.commonP\001'
  _COLUMN._serialized_start=237
  _COLUMN._serialized_end=593
  _SCHEMA._serialized_start=595
  _SCHEMA._serialized_end=664
  _ONLINEBATCHTABLEFORMAT._serialized_start=666
  _ONLINEBATCHTABLEFORMAT._serialized_end=790
  _ONLINEBATCHTABLEPART._serialized_start=793
  _ONLINEBATCHTABLEPART._serialized_end=1038
  _ONLINEBATCHTABLEPARTTILE._serialized_start=1041
  _ONLINEBATCHTABLEPARTTILE._serialized_end=1251
# @@protoc_insertion_point(module_scope)
