# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/api/featureservice/feature_service__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.api.featureservice import feature_service_request__client_pb2 as tecton__proto_dot_api_dot_featureservice_dot_feature__service__request____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n=tecton_proto/api/featureservice/feature_service__client.proto\x12 tecton_proto.api.feature_service\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x45tecton_proto/api/featureservice/feature_service_request__client.proto\"\xcf\x01\n\x0bNameAndType\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12[\n\tdata_type\x18\x03 \x01(\x0b\x32>.tecton_proto.api.feature_service.FeatureServerComplexDataTypeR\x08\x64\x61taType\x12O\n\x04type\x18\x02 \x01(\x0e\x32\x37.tecton_proto.api.feature_service.FeatureServerDataTypeB\x02\x18\x01R\x04type\"\x94\x01\n\x11GetFeaturesResult\x12\x32\n\x08\x66\x65\x61tures\x18\x05 \x03(\x0b\x32\x16.google.protobuf.ValueR\x08\x66\x65\x61tures\x12\x33\n\tjoin_keys\x18\x06 \x03(\x0b\x32\x16.google.protobuf.ValueR\x08joinKeysJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05\"\xdc\x03\n\x1c\x46\x65\x61tureServerComplexDataType\x12K\n\x04type\x18\x01 \x01(\x0e\x32\x37.tecton_proto.api.feature_service.FeatureServerDataTypeR\x04type\x12\x61\n\x0c\x65lement_type\x18\x02 \x01(\x0b\x32>.tecton_proto.api.feature_service.FeatureServerComplexDataTypeR\x0b\x65lementType\x12R\n\x06\x66ields\x18\x03 \x03(\x0b\x32:.tecton_proto.api.feature_service.FeatureServerStructFieldR\x06\x66ields\x12Y\n\x08key_type\x18\x04 \x01(\x0b\x32>.tecton_proto.api.feature_service.FeatureServerComplexDataTypeR\x07keyType\x12]\n\nvalue_type\x18\x05 \x01(\x0b\x32>.tecton_proto.api.feature_service.FeatureServerComplexDataTypeR\tvalueType\"\x8b\x01\n\x18\x46\x65\x61tureServerStructField\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12[\n\tdata_type\x18\x02 \x01(\x0b\x32>.tecton_proto.api.feature_service.FeatureServerComplexDataTypeR\x08\x64\x61taType\"\xdf\x03\n\x07SloInfo\x12!\n\x0cslo_eligible\x18\x01 \x01(\x08R\x0bsloEligible\x12\x35\n\x17slo_server_time_seconds\x18\x05 \x01(\x01R\x14sloServerTimeSeconds\x12t\n\x19slo_ineligibility_reasons\x18\x02 \x03(\x0e\x32\x38.tecton_proto.api.feature_service.SloIneligibilityReasonR\x17sloIneligibilityReasons\x12?\n\x1c\x64ynamodb_response_size_bytes\x18\x03 \x01(\x05R\x19\x64ynamodbResponseSizeBytes\x12.\n\x13server_time_seconds\x18\x04 \x01(\x01R\x11serverTimeSeconds\x12,\n\x12store_time_seconds\x18\x06 \x01(\x01R\x10storeTimeSeconds\x12*\n\x11store_max_latency\x18\x07 \x01(\x01R\x0fstoreMaxLatency\x12\x39\n\x19store_response_size_bytes\x18\x08 \x01(\x05R\x16storeResponseSizeBytes\"\xea\x04\n\x08Metadata\x12K\n\x08\x66\x65\x61tures\x18\x01 \x03(\x0b\x32/.tecton_proto.api.feature_service.Metadata.ItemR\x08\x66\x65\x61tures\x12L\n\tjoin_keys\x18\x02 \x03(\x0b\x32/.tecton_proto.api.feature_service.Metadata.ItemR\x08joinKeys\x12\x44\n\x08slo_info\x18\x03 \x01(\x0b\x32).tecton_proto.api.feature_service.SloInfoR\x07sloInfo\x12\'\n\x0fpartial_results\x18\x04 \x01(\x08R\x0epartialResults\x1a\xd3\x02\n\x04Item\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x41\n\x0e\x65\x66\x66\x65\x63tive_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\reffectiveTime\x12O\n\x04type\x18\x03 \x01(\x0e\x32\x37.tecton_proto.api.feature_service.FeatureServerDataTypeB\x02\x18\x01R\x04type\x12[\n\tdata_type\x18\x04 \x01(\x0b\x32>.tecton_proto.api.feature_service.FeatureServerComplexDataTypeR\x08\x64\x61taType\x12@\n\x06status\x18\x06 \x01(\x0e\x32(.tecton_proto.api.feature_service.StatusR\x06statusJ\x04\x08\x05\x10\x06\"\xaa\x01\n\x13GetFeaturesResponse\x12K\n\x06result\x18\x02 \x01(\x0b\x32\x33.tecton_proto.api.feature_service.GetFeaturesResultR\x06result\x12\x46\n\x08metadata\x18\x03 \x01(\x0b\x32*.tecton_proto.api.feature_service.MetadataR\x08metadata\"\xba\x01\n\x18GetFeaturesBatchResponse\x12K\n\x06result\x18\x02 \x03(\x0b\x32\x33.tecton_proto.api.feature_service.GetFeaturesResultR\x06result\x12K\n\x08metadata\x18\x04 \x01(\x0b\x32/.tecton_proto.api.feature_service.BatchMetadataR\x08metadataJ\x04\x08\x03\x10\x04\"\xba\x02\n\x0c\x42\x61tchSloInfo\x12!\n\x0cslo_eligible\x18\x01 \x01(\x08R\x0bsloEligible\x12\x35\n\x17slo_server_time_seconds\x18\x02 \x01(\x01R\x14sloServerTimeSeconds\x12t\n\x19slo_ineligibility_reasons\x18\x03 \x03(\x0e\x32\x38.tecton_proto.api.feature_service.SloIneligibilityReasonR\x17sloIneligibilityReasons\x12.\n\x13server_time_seconds\x18\x04 \x01(\x01R\x11serverTimeSeconds\x12*\n\x11store_max_latency\x18\x05 \x01(\x01R\x0fstoreMaxLatency\"\xd5\x04\n\rBatchMetadata\x12P\n\x08\x66\x65\x61tures\x18\x01 \x03(\x0b\x32\x34.tecton_proto.api.feature_service.BatchMetadata.ItemR\x08\x66\x65\x61tures\x12Q\n\tjoin_keys\x18\x02 \x03(\x0b\x32\x34.tecton_proto.api.feature_service.BatchMetadata.ItemR\x08joinKeys\x12\x44\n\x08slo_info\x18\x03 \x03(\x0b\x32).tecton_proto.api.feature_service.SloInfoR\x07sloInfo\x12T\n\x0e\x62\x61tch_slo_info\x18\x04 \x01(\x0b\x32..tecton_proto.api.feature_service.BatchSloInfoR\x0c\x62\x61tchSloInfo\x1a\x82\x02\n\x04Item\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x41\n\x0e\x65\x66\x66\x65\x63tive_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\reffectiveTime\x12[\n\tdata_type\x18\x03 \x01(\x0b\x32>.tecton_proto.api.feature_service.FeatureServerComplexDataTypeR\x08\x64\x61taType\x12@\n\x06status\x18\x05 \x03(\x0e\x32(.tecton_proto.api.feature_service.StatusR\x06statusJ\x04\x08\x04\x10\x05\"i\n\x16ServiceMetadataRequest\x12O\n\x06params\x18\x01 \x01(\x0b\x32\x37.tecton_proto.api.feature_service.FeatureServiceLocatorR\x06params\"\xf3\x03\n\x17ServiceMetadataResponse\x12\x66\n\x14\x66\x65\x61ture_service_type\x18\x01 \x01(\x0e\x32\x34.tecton_proto.api.feature_service.FeatureServiceTypeR\x12\x66\x65\x61tureServiceType\x12U\n\x0finput_join_keys\x18\x02 \x03(\x0b\x32-.tecton_proto.api.feature_service.NameAndTypeR\rinputJoinKeys\x12j\n\x1ainput_request_context_keys\x18\x03 \x03(\x0b\x32-.tecton_proto.api.feature_service.NameAndTypeR\x17inputRequestContextKeys\x12W\n\x10output_join_keys\x18\x04 \x03(\x0b\x32-.tecton_proto.api.feature_service.NameAndTypeR\x0eoutputJoinKeys\x12T\n\x0e\x66\x65\x61ture_values\x18\x05 \x03(\x0b\x32-.tecton_proto.api.feature_service.NameAndTypeR\rfeatureValues\"g\n\x14QueryFeaturesRequest\x12O\n\x06params\x18\x01 \x01(\x0b\x32\x37.tecton_proto.api.feature_service.GetFeaturesParametersR\x06params\"\xae\x01\n\x15QueryFeaturesResponse\x12M\n\x07results\x18\x01 \x03(\x0b\x32\x33.tecton_proto.api.feature_service.GetFeaturesResultR\x07results\x12\x46\n\x08metadata\x18\x02 \x01(\x0b\x32*.tecton_proto.api.feature_service.MetadataR\x08metadata\"\xe9\x02\n\x15\x46\x65\x61tureServiceLocator\x12.\n\x12\x66\x65\x61ture_service_id\x18\x01 \x01(\tH\x00R\x10\x66\x65\x61tureServiceId\x12\x32\n\x14\x66\x65\x61ture_service_name\x18\x02 \x01(\tH\x00R\x12\x66\x65\x61tureServiceName\x12.\n\x12\x66\x65\x61ture_package_id\x18\x04 \x01(\tH\x00R\x10\x66\x65\x61turePackageId\x12\x32\n\x14\x66\x65\x61ture_package_name\x18\x05 \x01(\tH\x00R\x12\x66\x65\x61turePackageName\x12(\n\x0f\x66\x65\x61ture_view_id\x18\x06 \x01(\tH\x00R\rfeatureViewId\x12,\n\x11\x66\x65\x61ture_view_name\x18\x07 \x01(\tH\x00R\x0f\x66\x65\x61tureViewName\x12%\n\x0eworkspace_name\x18\x03 \x01(\tR\rworkspaceNameB\t\n\x07locator\"\x90\x01\n\x1dGetFeatureServiceStateRequest\x12o\n\x17\x66\x65\x61ture_service_locator\x18\x01 \x01(\x0b\x32\x37.tecton_proto.api.feature_service.FeatureServiceLocatorR\x15\x66\x65\x61tureServiceLocator\"&\n\x1eGetFeatureServiceStateResponseJ\x04\x08\x01\x10\x02*/\n\x12\x46\x65\x61tureServiceType\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x0c\n\x08WILDCARD\x10\x01*\xa8\x01\n\x15\x46\x65\x61tureServerDataType\x12\x10\n\x0cmissing_type\x10\x00\x12\x0b\n\x07\x62oolean\x10\x01\x12\x0b\n\x07\x66loat64\x10\x02\x12\t\n\x05int64\x10\x03\x12\n\n\x06string\x10\x04\x12\x10\n\x0cstring_array\x10\x05\x12\t\n\x05\x61rray\x10\x06\x12\x0b\n\x07\x66loat32\x10\x07\x12\n\n\x06struct\x10\x08\x12\x07\n\x03map\x10\t\x12\r\n\ttimestamp\x10\n*|\n\x06Status\x12\x0b\n\x07PRESENT\x10\x00\x12\x10\n\x0cMISSING_DATA\x10\x01\x12\x12\n\x0eUNKNOWN_STATUS\x10\x02\x12\x12\n\x0e\x43\x41\x43HED_UNKNOWN\x10\x03\x12\x17\n\x13\x43\x41\x43HED_MISSING_DATA\x10\x04\x12\x12\n\x0e\x43\x41\x43HED_PRESENT\x10\x05*\x9a\x01\n\x16SloIneligibilityReason\x12\x0b\n\x07UNKNOWN\x10\x00\x12)\n%DYNAMODB_RESPONSE_SIZE_LIMIT_EXCEEDED\x10\x01\x12&\n\"REDIS_RESPONSE_SIZE_LIMIT_EXCEEDED\x10\x02\x12 \n\x1cREDIS_LATENCY_LIMIT_EXCEEDED\x10\x03\x32\xc2\x05\n\x0e\x46\x65\x61tureService\x12z\n\x0bGetFeatures\x12\x34.tecton_proto.api.feature_service.GetFeaturesRequest\x1a\x35.tecton_proto.api.feature_service.GetFeaturesResponse\x12\x89\x01\n\x10GetFeaturesBatch\x12\x39.tecton_proto.api.feature_service.GetFeaturesBatchRequest\x1a:.tecton_proto.api.feature_service.GetFeaturesBatchResponse\x12\x80\x01\n\rQueryFeatures\x12\x36.tecton_proto.api.feature_service.QueryFeaturesRequest\x1a\x37.tecton_proto.api.feature_service.QueryFeaturesResponse\x12\x9b\x01\n\x16GetFeatureServiceState\x12?.tecton_proto.api.feature_service.GetFeatureServiceStateRequest\x1a@.tecton_proto.api.feature_service.GetFeatureServiceStateResponse\x12\x86\x01\n\x0fServiceMetadata\x12\x38.tecton_proto.api.feature_service.ServiceMetadataRequest\x1a\x39.tecton_proto.api.feature_service.ServiceMetadataResponseB6Z4github.com/tecton-ai/tecton_proto/api/featureservice')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.api.featureservice.feature_service__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z4github.com/tecton-ai/tecton_proto/api/featureservice'
  _NAMEANDTYPE.fields_by_name['type']._options = None
  _NAMEANDTYPE.fields_by_name['type']._serialized_options = b'\030\001'
  _METADATA_ITEM.fields_by_name['type']._options = None
  _METADATA_ITEM.fields_by_name['type']._serialized_options = b'\030\001'
  _FEATURESERVICETYPE._serialized_start=5039
  _FEATURESERVICETYPE._serialized_end=5086
  _FEATURESERVERDATATYPE._serialized_start=5089
  _FEATURESERVERDATATYPE._serialized_end=5257
  _STATUS._serialized_start=5259
  _STATUS._serialized_end=5383
  _SLOINELIGIBILITYREASON._serialized_start=5386
  _SLOINELIGIBILITYREASON._serialized_end=5540
  _NAMEANDTYPE._serialized_start=234
  _NAMEANDTYPE._serialized_end=441
  _GETFEATURESRESULT._serialized_start=444
  _GETFEATURESRESULT._serialized_end=592
  _FEATURESERVERCOMPLEXDATATYPE._serialized_start=595
  _FEATURESERVERCOMPLEXDATATYPE._serialized_end=1071
  _FEATURESERVERSTRUCTFIELD._serialized_start=1074
  _FEATURESERVERSTRUCTFIELD._serialized_end=1213
  _SLOINFO._serialized_start=1216
  _SLOINFO._serialized_end=1695
  _METADATA._serialized_start=1698
  _METADATA._serialized_end=2316
  _METADATA_ITEM._serialized_start=1977
  _METADATA_ITEM._serialized_end=2316
  _GETFEATURESRESPONSE._serialized_start=2319
  _GETFEATURESRESPONSE._serialized_end=2489
  _GETFEATURESBATCHRESPONSE._serialized_start=2492
  _GETFEATURESBATCHRESPONSE._serialized_end=2678
  _BATCHSLOINFO._serialized_start=2681
  _BATCHSLOINFO._serialized_end=2995
  _BATCHMETADATA._serialized_start=2998
  _BATCHMETADATA._serialized_end=3595
  _BATCHMETADATA_ITEM._serialized_start=3337
  _BATCHMETADATA_ITEM._serialized_end=3595
  _SERVICEMETADATAREQUEST._serialized_start=3597
  _SERVICEMETADATAREQUEST._serialized_end=3702
  _SERVICEMETADATARESPONSE._serialized_start=3705
  _SERVICEMETADATARESPONSE._serialized_end=4204
  _QUERYFEATURESREQUEST._serialized_start=4206
  _QUERYFEATURESREQUEST._serialized_end=4309
  _QUERYFEATURESRESPONSE._serialized_start=4312
  _QUERYFEATURESRESPONSE._serialized_end=4486
  _FEATURESERVICELOCATOR._serialized_start=4489
  _FEATURESERVICELOCATOR._serialized_end=4850
  _GETFEATURESERVICESTATEREQUEST._serialized_start=4853
  _GETFEATURESERVICESTATEREQUEST._serialized_end=4997
  _GETFEATURESERVICESTATERESPONSE._serialized_start=4999
  _GETFEATURESERVICESTATERESPONSE._serialized_end=5037
  _FEATURESERVICE._serialized_start=5543
  _FEATURESERVICE._serialized_end=6249
# @@protoc_insertion_point(module_scope)
