# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/common/container_image__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n1tecton_proto/common/container_image__client.proto\x12\x13tecton_proto.common\"\x96\x01\n\x0e\x43ontainerImage\x12\'\n\x0frepository_name\x18\x01 \x01(\tR\x0erepositoryName\x12\x1b\n\timage_uri\x18\x02 \x01(\tR\x08imageUri\x12!\n\x0cimage_digest\x18\x03 \x01(\tR\x0bimageDigest\x12\x1b\n\timage_tag\x18\x04 \x01(\tR\x08imageTagB\x15\n\x11\x63om.tecton.commonP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.common.container_image__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021com.tecton.commonP\001'
  _CONTAINERIMAGE._serialized_start=75
  _CONTAINERIMAGE._serialized_end=225
# @@protoc_insertion_point(module_scope)
