# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/databricks_api/error__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n/tecton_proto/databricks_api/error__client.proto\x12\x1btecton_proto.databricks_api\"^\n\rErrorResponse\x12\x1d\n\nerror_code\x18\x01 \x01(\tR\terrorCode\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message\x12\x14\n\x05\x65rror\x18\x03 \x01(\tR\x05\x65rrorB\x1d\n\x19\x63om.tecton.databricks_apiP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.databricks_api.error__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\031com.tecton.databricks_apiP\001'
  _ERRORRESPONSE._serialized_start=80
  _ERRORRESPONSE._serialized_end=174
# @@protoc_insertion_point(module_scope)
