# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/materialization_roles_allowlists__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n@tecton_proto/data/materialization_roles_allowlists__client.proto\x12\x11tecton_proto.data\"\x96\x01\n\x1eMaterializationRolesAllowlists\x12t\n\x18workspace_configurations\x18\x01 \x03(\x0b\x32\x39.tecton_proto.data.WorkspaceMaterializationRolesAllowlistR\x17workspaceConfigurations\"\x93\x01\n&WorkspaceMaterializationRolesAllowlist\x12%\n\x0eworkspace_name\x18\x01 \x01(\tR\rworkspaceName\x12\x42\n\x1d\x61llowed_materialization_roles\x18\x02 \x03(\tR\x1b\x61llowedMaterializationRolesB\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.materialization_roles_allowlists__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _MATERIALIZATIONROLESALLOWLISTS._serialized_start=88
  _MATERIALIZATIONROLESALLOWLISTS._serialized_end=238
  _WORKSPACEMATERIALIZATIONROLESALLOWLIST._serialized_start=241
  _WORKSPACEMATERIALIZATIONROLESALLOWLIST._serialized_end=388
# @@protoc_insertion_point(module_scope)
