# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/metadataservice/http_over_grpc__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.auth import service__client_pb2 as tecton__proto_dot_auth_dot_service____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n9tecton_proto/metadataservice/http_over_grpc__client.proto\x12\x1ctecton_proto.metadataservice\x1a\'tecton_proto/auth/service__client.proto\"\x93\x01\n\x0bHttpRequest\x12\x16\n\x06method\x18\x01 \x01(\tR\x06method\x12\x10\n\x03url\x18\x02 \x01(\tR\x03url\x12\x12\n\x04\x62ody\x18\x03 \x01(\x0cR\x04\x62ody\x12 \n\x0b\x63ontentType\x18\x04 \x01(\tR\x0b\x63ontentType\x12$\n\rcontentLength\x18\x05 \x01(\x03R\rcontentLength\"u\n\x13HttpResponseHeaders\x12\x16\n\x06status\x18\x01 \x01(\x05R\x06status\x12 \n\x0b\x63ontentType\x18\x02 \x01(\tR\x0b\x63ontentType\x12$\n\rcontentLength\x18\x03 \x01(\x03R\rcontentLength\"\x93\x01\n\x11HttpResponsePiece\x12M\n\x07headers\x18\x01 \x01(\x0b\x32\x31.tecton_proto.metadataservice.HttpResponseHeadersH\x00R\x07headers\x12\x1f\n\nbody_piece\x18\x02 \x01(\x0cH\x00R\tbodyPieceB\x0e\n\x0cmessage_type2\xac\x01\n\x13HttpOverGrpcService\x12\x94\x01\n\x05Proxy\x12).tecton_proto.metadataservice.HttpRequest\x1a/.tecton_proto.metadataservice.HttpResponsePiece\"-\xa2\xbc\xe6\xc0\x05\'*%tecton/organization#read_organization0\x01\x42_\n\x1a\x63om.tecton.metadataserviceB\x0cHttpOverGrpcP\x01Z1github.com/tecton-ai/tecton_proto/metadataservice')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.metadataservice.http_over_grpc__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\032com.tecton.metadataserviceB\014HttpOverGrpcP\001Z1github.com/tecton-ai/tecton_proto/metadataservice'
  _HTTPOVERGRPCSERVICE.methods_by_name['Proxy']._options = None
  _HTTPOVERGRPCSERVICE.methods_by_name['Proxy']._serialized_options = b'\242\274\346\300\005\'*%tecton/organization#read_organization'
  _HTTPREQUEST._serialized_start=133
  _HTTPREQUEST._serialized_end=280
  _HTTPRESPONSEHEADERS._serialized_start=282
  _HTTPRESPONSEHEADERS._serialized_end=399
  _HTTPRESPONSEPIECE._serialized_start=402
  _HTTPRESPONSEPIECE._serialized_end=549
  _HTTPOVERGRPCSERVICE._serialized_start=552
  _HTTPOVERGRPCSERVICE._serialized_end=724
# @@protoc_insertion_point(module_scope)
