# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/virtual_data_source__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.common import data_source_type__client_pb2 as tecton__proto_dot_common_dot_data__source__type____client__pb2
from tecton_proto.common import id__client_pb2 as tecton__proto_dot_common_dot_id____client__pb2
from tecton_proto.common import schema_container__client_pb2 as tecton__proto_dot_common_dot_schema__container____client__pb2
from tecton_proto.data import batch_data_source__client_pb2 as tecton__proto_dot_data_dot_batch__data__source____client__pb2
from tecton_proto.data import fco_metadata__client_pb2 as tecton__proto_dot_data_dot_fco__metadata____client__pb2
from tecton_proto.data import stream_data_source__client_pb2 as tecton__proto_dot_data_dot_stream__data__source____client__pb2
from tecton_proto.validation import validator__client_pb2 as tecton__proto_dot_validation_dot_validator____client__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n3tecton_proto/data/virtual_data_source__client.proto\x12\x11tecton_proto.data\x1a\x32tecton_proto/common/data_source_type__client.proto\x1a$tecton_proto/common/id__client.proto\x1a\x32tecton_proto/common/schema_container__client.proto\x1a\x31tecton_proto/data/batch_data_source__client.proto\x1a,tecton_proto/data/fco_metadata__client.proto\x1a\x32tecton_proto/data/stream_data_source__client.proto\x1a/tecton_proto/validation/validator__client.proto\"\xd2\x05\n\x11VirtualDataSource\x12L\n\x16virtual_data_source_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x13virtualDataSourceId\x12N\n\x11\x62\x61tch_data_source\x18\x04 \x01(\x0b\x32\".tecton_proto.data.BatchDataSourceR\x0f\x62\x61tchDataSource\x12Q\n\x12stream_data_source\x18\x05 \x01(\x0b\x32#.tecton_proto.data.StreamDataSourceR\x10streamDataSource\x12M\n\x10\x64\x61ta_source_type\x18\x06 \x01(\x0e\x32#.tecton_proto.common.DataSourceTypeR\x0e\x64\x61taSourceType\x12\x41\n\x0c\x66\x63o_metadata\x18\x07 \x01(\x0b\x32\x1e.tecton_proto.data.FcoMetadataR\x0b\x66\x63oMetadata\x12<\n\x06schema\x18\t \x01(\x0b\x32$.tecton_proto.common.SchemaContainerR\x06schema\x12\x61\n\x0fvalidation_args\x18\n \x01(\x0b\x32\x38.tecton_proto.validation.VirtualDataSourceValidationArgsR\x0evalidationArgs\x12K\n\x07options\x18\x0b \x03(\x0b\x32\x31.tecton_proto.data.VirtualDataSource.OptionsEntryR\x07options\x1a:\n\x0cOptionsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01J\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04J\x04\x08\x08\x10\tB\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.virtual_data_source__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _VIRTUALDATASOURCE_OPTIONSENTRY._options = None
  _VIRTUALDATASOURCE_OPTIONSENTRY._serialized_options = b'8\001'
  _VIRTUALDATASOURCE._serialized_start=415
  _VIRTUALDATASOURCE._serialized_end=1137
  _VIRTUALDATASOURCE_OPTIONSENTRY._serialized_start=1061
  _VIRTUALDATASOURCE_OPTIONSENTRY._serialized_end=1119
# @@protoc_insertion_point(module_scope)
