# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/snowflake/snowflake_credentials__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n:tecton_proto/snowflake/snowflake_credentials__client.proto\x12\x16tecton_proto.snowflake\"\x9f\x01\n\x14SnowflakeCredentials\x12\x12\n\x04user\x18\x01 \x01(\tR\x04user\x12\x32\n\x14password_secret_name\x18\x02 \x01(\tH\x00R\x12passwordSecretName\x12,\n\x11private_key_alias\x18\x03 \x01(\tH\x00R\x0fprivateKeyAliasB\x11\n\x0f\x63redential_typeB\x18\n\x14\x63om.tecton.snowflakeP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.snowflake.snowflake_credentials__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024com.tecton.snowflakeP\001'
  _SNOWFLAKECREDENTIALS._serialized_start=87
  _SNOWFLAKECREDENTIALS._serialized_end=246
# @@protoc_insertion_point(module_scope)
