# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/amplitude/client_logging__client.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n3tecton_proto/amplitude/client_logging__client.proto\x12\x16tecton_proto.amplitude\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"7\n\x0bLoggedValue\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value\"|\n\x08\x45rrorLog\x12\x18\n\x07message\x18\x01 \x01(\tR\x07message\x12\x1e\n\nstacktrace\x18\x02 \x01(\tR\nstacktrace\x12\x36\n\x05\x63\x61use\x18\x03 \x01(\x0b\x32 .tecton_proto.amplitude.ErrorLogR\x05\x63\x61use\"\xf0\x04\n\x13SDKMethodInvocation\x12.\n\x04time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x04time\x12\x17\n\x07user_id\x18\x02 \x01(\tR\x06userId\x12\x19\n\x08trace_id\x18\x03 \x01(\tR\x07traceId\x12\x1b\n\tlog_level\x18\x04 \x01(\tR\x08logLevel\x12:\n\x04type\x18\x05 \x01(\x0e\x32&.tecton_proto.amplitude.LogMessageTypeR\x04type\x12\x1d\n\nclass_name\x18\x06 \x01(\tR\tclassName\x12\x1f\n\x0bmethod_name\x18\x07 \x01(\tR\nmethodName\x12@\n\x0e\x65xecution_time\x18\x08 \x01(\x0b\x32\x19.google.protobuf.DurationR\rexecutionTime\x12Z\n\x17params_or_return_values\x18\t \x03(\x0b\x32#.tecton_proto.amplitude.LoggedValueR\x14paramsOrReturnValues\x12\x36\n\x05\x65rror\x18\n \x01(\x0b\x32 .tecton_proto.amplitude.ErrorLogR\x05\x65rror\x12\x1c\n\tworkspace\x18\x0b \x01(\tR\tworkspace\x12\x1f\n\x0bsdk_version\x18\x0c \x01(\tR\nsdkVersion\x12%\n\x0epython_version\x18\r \x01(\tR\rpythonVersion\x12 \n\x0cis_local_fco\x18\x0e \x01(\x08R\nisLocalFco*\x8a\x01\n\x0eLogMessageType\x12#\n\x1f\x43LIENT_LOG_MESSAGE_TYPE_UNKNOWN\x10\x00\x12(\n$CLIENT_LOG_MESSAGE_TYPE_METHOD_ENTRY\x10\x01\x12)\n%CLIENT_LOG_MESSAGE_TYPE_METHOD_RETURN\x10\x02\x42,\n\x14\x63om.tecton.amplitudeB\x12\x43lientLoggingProtoP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.amplitude.client_logging__client_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024com.tecton.amplitudeB\022ClientLoggingProtoP\001'
  _LOGMESSAGETYPE._serialized_start=955
  _LOGMESSAGETYPE._serialized_end=1093
  _LOGGEDVALUE._serialized_start=144
  _LOGGEDVALUE._serialized_end=199
  _ERRORLOG._serialized_start=201
  _ERRORLOG._serialized_end=325
  _SDKMETHODINVOCATION._serialized_start=328
  _SDKMETHODINVOCATION._serialized_end=952
# @@protoc_insertion_point(module_scope)
